@@ -61,10 +61,13 @@
 
 #include "vdef.h"
 
+#include "vqueue.h"
+
 #include "vapi/vsig.h"
 #include "vapi/vsm.h"
 #include "vas.h"
 #include "vcli.h"
+#include "vjsn.h"
 #include "vtcp.h"
 
 #define RL_EXIT(status) \
@@ -73,9 +76,14 @@
 		exit(status); \
 	} while (0)
 
+enum pass_mode_e {
+	pass_script,
+	pass_interactive,
+};
 
-static const double timeout = 5;	// XXX should be settable by arg ?
+static double timeout = 5;
 static int p_arg = 0;
+static int line_sock;
 
 static void
 cli_write(int sock, const char *s)
@@ -157,7 +165,7 @@ cli_sock(const char *T_arg, const char *S_arg)
 }
 
 static unsigned
-pass_answer(int fd)
+pass_answer(int fd, enum pass_mode_e mode)
 {
 	unsigned u, status;
 	char *answer = NULL;
@@ -176,7 +184,8 @@ pass_answer(int fd)
 	} else if (p_arg) {
 		printf("%-3u %-8u\n", status, 0U);
 	} else {
-		printf("%u\n", status);
+		if (mode == pass_interactive)
+			printf("%u\n", status);
 		if (answer != NULL)
 			printf("%s\n", answer);
 		if (status == CLIS_TRUNCATED)
@@ -200,7 +209,7 @@ do_args(int sock, int argc, char * const *argv)
 		cli_write(sock, argv[i]);
 	}
 	cli_write(sock, "\n");
-	status = pass_answer(sock);
+	status = pass_answer(sock, pass_script);
 	closefd(&sock);
 	if (status == CLIS_OK || status == CLIS_TRUNCATED)
 		exit(0);
@@ -212,13 +221,12 @@ do_args(int sock, int argc, char * const *argv)
 /* Callback for readline, doesn't take a private pointer, so we need
  * to have a global variable.
  */
-static int _line_sock;
 static void v_matchproto_()
 send_line(char *l)
 {
 	if (l) {
-		cli_write(_line_sock, l);
-		cli_write(_line_sock, "\n");
+		cli_write(line_sock, l);
+		cli_write(line_sock, "\n");
 		if (*l)
 			add_history(l);
 		rl_callback_handler_install("varnish> ", send_line);
@@ -227,27 +235,49 @@ send_line(char *l)
 	}
 }
 
-static char *commands[256];
 static char *
 command_generator (const char *text, int state)
 {
-	static int list_index, len;
-	const char *name;
+	static struct vjsn *jsn_cmds;
+	static const struct vjsn_val *jv;
+	struct vjsn_val *jv2;
+	unsigned u;
+	char *answer = NULL;
+	const char *err;
 
-	/* If this is a new word to complete, initialize now.  This
-	   includes saving the length of TEXT for efficiency, and
-	   initializing the index variable to 0. */
 	if (!state) {
-		list_index = 0;
-		len = strlen(text);
+		cli_write(line_sock, "help -j\n");
+		u = VCLI_ReadResult(line_sock, NULL, &answer, timeout);
+		if (u) {
+			free(answer);
+			return (NULL);
+		}
+		jsn_cmds = vjsn_parse(answer, &err);
+		free(answer);
+		if (err != NULL)
+			return (NULL);
+		AN(jsn_cmds);
+		AN(jsn_cmds->value);
+		assert (vjsn_is_array(jsn_cmds->value));
+		jv = VTAILQ_FIRST(&jsn_cmds->value->children);
+		assert (vjsn_is_number(jv));
+		jv = VTAILQ_NEXT(jv, list);
+		assert (vjsn_is_array(jv));
+		jv = VTAILQ_NEXT(jv, list);
+		assert (vjsn_is_number(jv));
+		jv = VTAILQ_NEXT(jv, list);
 	}
-
-	while ((name = commands[list_index]) != NULL) {
-		list_index++;
-		if (strncmp (name, text, len) == 0)
-			return (strdup(name));
+	while (jv != NULL) {
+		assert (vjsn_is_object(jv));
+		jv2 = VTAILQ_FIRST(&jv->children);
+		AN(jv2);
+		jv = VTAILQ_NEXT(jv, list);
+		assert (vjsn_is_string(jv2));
+		assert (!strcmp(jv2->name, "request"));
+		if (!strncmp(text, jv2->value, strlen(text)))
+			return (strdup(jv2->value));
 	}
-	/* If no names matched, then return NULL. */
+	vjsn_delete(&jsn_cmds);
 	return (NULL);
 }
 
@@ -272,9 +302,7 @@ interactive(int sock)
 {
 	struct pollfd fds[2];
 	int i;
-	char *answer = NULL;
-	unsigned u, status;
-	_line_sock = sock;
+	line_sock = sock;
 	rl_already_prompted = 1;
 	rl_callback_handler_install("varnish> ", send_line);
 	rl_attempted_completion_function = varnishadm_completion;
@@ -284,34 +312,6 @@ interactive(int sock)
 	fds[1].fd = 0;
 	fds[1].events = POLLIN;
 
-	/* Grab the commands, for completion */
-	cli_write(sock, "help\n");
-	u = VCLI_ReadResult(fds[0].fd, &status, &answer, timeout);
-	if (!u) {
-		char *t, c[128];
-		if (status == CLIS_COMMS) {
-			RL_EXIT(0);
-		}
-		t = answer;
-
-		i = 0;
-		while (*t) {
-			if (sscanf(t, "%127s", c) == 1) {
-				commands[i++] = strdup(c);
-				while (*t != '\n' && *t != '\0')
-					t++;
-				if (*t == '\n')
-					t++;
-			} else {
-				/* what? */
-				fprintf(stderr, "Unknown command '%s' parsing "
-					"help output. Tab completion may be "
-					"broken\n", t);
-				break;
-			}
-		}
-	}
-	free(answer);
 	cli_write(sock, "banner\n");
 	while (1) {
 		i = poll(fds, 2, -1);
@@ -322,7 +322,7 @@ interactive(int sock)
 		if (fds[0].revents & POLLIN) {
 			/* Get rid of the prompt, kinda hackish */
 			printf("\r           \r");
-			(void)pass_answer(fds[0].fd);
+			(void)pass_answer(fds[0].fd, pass_interactive);
 			rl_forced_update_display();
 		}
 		if (fds[1].revents & POLLIN) {
@@ -341,6 +341,7 @@ pass(int sock)
 	char buf[1024];
 	int i;
 	ssize_t n;
+	int busy = 0;
 
 	fds[0].fd = sock;
 	fds[0].events = POLLIN;
@@ -352,16 +353,22 @@ pass(int sock)
 			continue;
 		}
 		assert(i > 0);
-		if (fds[0].revents & POLLIN)
-			(void)pass_answer(fds[0].fd);
+		if (fds[0].revents & POLLIN) {
+			(void)pass_answer(fds[0].fd, pass_script);
+			busy = 0;
+			if (fds[1].fd < 0)
+				RL_EXIT(0);
+		}
 		if (fds[1].revents & POLLIN || fds[1].revents & POLLHUP) {
 			n = read(fds[1].fd, buf, sizeof buf - 1);
 			if (n == 0) {
-				AZ(shutdown(sock, SHUT_WR));
+				if (!busy)
+					RL_EXIT(0);
 				fds[1].fd = -1;
 			} else if (n < 0) {
 				RL_EXIT(0);
 			} else {
+				busy = 1;
 				buf[n] = '\0';
 				cli_write(sock, buf);
 			}
@@ -374,8 +381,8 @@ static void v_noreturn_
 usage(int status)
 {
 	fprintf(stderr,
-	    "Usage: varnishadm [-h] [-n ident] [-t timeout] [-S secretfile] "
-	    "[-T [address]:port] [command [...]]\n");
+	    "Usage: varnishadm [-h] [-n ident] [-p] [-S secretfile] "
+	    "[-T [address]:port] [-t timeout] [command [...]]\n");
 	fprintf(stderr, "\t-n is mutually exclusive with -S and -T\n");
 	exit(status);
 }
@@ -423,6 +430,21 @@ n_arg_sock(const char *n_arg, const char *t_arg)
 	return (sock);
 }
 
+static int
+t_arg_timeout(const char *t_arg)
+{
+	char *p = NULL;
+
+	AN(t_arg);
+	timeout = strtod(t_arg, &p);
+	if ((p != NULL && *p != '\0') ||
+	    !isfinite(timeout) || timeout < 0) {
+		fprintf(stderr, "-t: Invalid argument: %s", t_arg);
+		return (-1);
+	}
+	return (1);
+}
+
 #define OPTARG "hn:pS:T:t:"
 
 int
@@ -486,6 +508,9 @@ main(int argc, char * const *argv)
 	if (sock < 0)
 		exit(2);
 
+	if (t_arg != NULL && t_arg_timeout(t_arg) < 0)
+		exit(2);
+
 	if (argc > 0) {
 		VSIG_Arm_int();
 		VSIG_Arm_term();
