@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -44,7 +46,7 @@
 #include "vdef.h"
 #include "vrt.h"
 
-#define CACHE_H_INCLUDED
+#define CACHE_H_INCLUDED	// After vrt.h include.
 
 #include "miniobj.h"
 #include "vas.h"
@@ -55,25 +57,35 @@
 
 /*--------------------------------------------------------------------*/
 
-enum body_status {
-#define BODYSTATUS(U,l)	BS_##U,
+struct body_status {
+	const char		*name;
+	int			nbr;
+	int			avail;
+	int			length_known;
+};
+
+#define BODYSTATUS(U, l, n, a, k) extern const struct body_status BS_##U[1];
 #include "tbl/body_status.h"
-};
+
+typedef const struct body_status *body_status_t;
+
+typedef const char *hdr_t;
 
 /*--------------------------------------------------------------------*/
 
-enum req_body_state_e {
-#define REQ_BODY(U)	REQ_BODY_##U,
-#include "tbl/req_body.h"
+struct stream_close {
+	unsigned		magic;
+#define STREAM_CLOSE_MAGIC	0xc879c93d
+	int			idx;
+	unsigned		is_err;
+	const char		*name;
+	const char		*desc;
 };
-
-/*--------------------------------------------------------------------*/
-
-enum sess_close {
-	SC_NULL = 0,
-#define SESS_CLOSE(nm, stat, err, desc)	SC_##nm,
+    extern const struct stream_close SC_NULL[1];
+#define SESS_CLOSE(nm, stat, err, desc) \
+    extern const struct stream_close SC_##nm[1];
 #include "tbl/sess_close.h"
-};
+
 
 /*--------------------------------------------------------------------
  * Indices into http->hd[]
@@ -85,59 +97,41 @@ enum {
 
 /*--------------------------------------------------------------------*/
 
-struct VSC_lck;
-struct VSC_main;
-struct VSC_main_wrk;
-struct backend;
 struct ban;
 struct ban_proto;
 struct cli;
 struct http_conn;
+struct listen_sock;
 struct mempool;
 struct objcore;
 struct objhead;
 struct pool;
+struct req_step;
 struct sess;
 struct transport;
-struct worker;
-struct listen_sock;
 struct vcf;
+struct VSC_lck;
+struct VSC_main;
+struct VSC_main_wrk;
+struct worker;
+struct worker_priv;
 
 #define DIGEST_LEN		32
 
 /*--------------------------------------------------------------------*/
 
-typedef struct {
-	const char		*b;
-	const char		*e;
-} txt;
-
-/*--------------------------------------------------------------------*/
-
-enum req_step {
-	R_STP_NONE = 0,
-#define REQ_STEP(l, u, arg)	R_STP_##u,
-#include "tbl/steps.h"
-};
-
-enum fetch_step {
-	F_STP_NONE = 0,
-#define FETCH_STEP(l, U, arg)	F_STP_##U,
-#include "tbl/steps.h"
-};
-
-/*--------------------------------------------------------------------*/
-
 struct lock { void *priv; };	// Opaque
 
 /*--------------------------------------------------------------------
  * Workspace structure for quick memory allocation.
  */
 
+#define WS_ID_SIZE 4
+
 struct ws {
 	unsigned		magic;
 #define WS_MAGIC		0x35fac554
-	char			id[4];		/* identity */
+	char			id[WS_ID_SIZE];	/* identity */
 	char			*s;		/* (S)tart of buffer */
 	char			*f;		/* (F)ree/front pointer */
 	char			*r;		/* (R)eserved length */
@@ -192,20 +186,7 @@ struct vsl_log {
 
 /*--------------------------------------------------------------------*/
 
-struct vxid_pool {
-	uint32_t		next;
-	uint32_t		count;
-};
-
-/*--------------------------------------------------------------------*/
-
-VRBT_HEAD(vrt_priv_tree,vrt_priv);
-
-struct vrt_privs {
-	unsigned		magic;
-#define VRT_PRIVS_MAGIC		0x03ba7502
-	struct vrt_priv_tree	privs;
-};
+VRBT_HEAD(vrt_privs, vrt_priv);
 
 /* Worker pool stuff -------------------------------------------------*/
 
@@ -220,24 +201,23 @@ struct pool_task {
 /*
  * tasks are taken off the queues in this order
  *
- * prios up to TASK_QUEUE_RESERVE are run from the reserve
- *
  * TASK_QUEUE_{REQ|STR} are new req's (H1/H2), and subject to queue limit.
  *
- * TASK_QUEUE_RUSH is req's returning from waiting list, they are
- * not subject to TASK_QUEUE_CLIENT because we cannot safely clean
- * them up if scheduling them fails.
+ * TASK_QUEUE_RUSH is req's returning from waiting list
+ *
+ * NOTE: When changing the number of classes, update places marked with
+ * TASK_QUEUE__END in mgt_pool.c
  */
 enum task_prio {
 	TASK_QUEUE_BO,
-#define TASK_QUEUE_RESERVE	TASK_QUEUE_BO
 	TASK_QUEUE_RUSH,
 	TASK_QUEUE_REQ,
 	TASK_QUEUE_STR,
 	TASK_QUEUE_VCA,
-	TASK_QUEUE_END
+	TASK_QUEUE__END
 };
 
+#define TASK_QUEUE_HIGHEST_PRIORITY TASK_QUEUE_BO
 #define TASK_QUEUE_CLIENT(prio) \
 	(prio == TASK_QUEUE_REQ || prio == TASK_QUEUE_STR)
 
@@ -246,28 +226,22 @@ enum task_prio {
 struct worker {
 	unsigned		magic;
 #define WORKER_MAGIC		0x6391adcf
+	int			strangelove;
+	struct worker_priv	*wpriv;
 	struct pool		*pool;
-	struct objhead		*nobjhead;
-	struct objcore		*nobjcore;
-	void			*nhashpriv;
 	struct VSC_main_wrk	*stats;
 	struct vsl_log		*vsl;		// borrowed from req/bo
 
-	struct pool_task	task;
+	struct pool_task	task[1];
 
 	vtim_real		lastused;
-	int			strangelove;
 
 	struct v1l		*v1l;
 
 	pthread_cond_t		cond;
 
-	struct vcl		*vcl;
-
 	struct ws		aws[1];
 
-	struct vxid_pool	vxid_pool;
-
 	unsigned		cur_method;
 	unsigned		seen_methods;
 	unsigned		handling;
@@ -394,27 +368,30 @@ struct busyobj {
 	 * All fields from retries and down are zeroed when the busyobj
 	 * is recycled.
 	 */
-	int			retries;
+	unsigned		retries;
 	struct req		*req;
 	struct sess		*sp;
 	struct worker		*wrk;
 
 	struct vfp_ctx		*vfc;
-	const char		*filter_list;
+	const char		*vfp_filter_list;
 
 	struct ws		ws[1];
 	uintptr_t		ws_bo;
 	struct http		*bereq0;
 	struct http		*bereq;
 	struct http		*beresp;
+	struct objcore		*bereq_body;
 	struct objcore		*stale_oc;
 	struct objcore		*fetch_objcore;
 
+	const char		*no_retry;
+
 	struct http_conn	*htc;
 
-	struct pool_task	fetch_task;
+	struct pool_task	fetch_task[1];
 
-#define BO_FLAG(l, r, w, d) unsigned	l:1;
+#define BO_FLAG(l, r, rr, rw, f, d) unsigned	l:1;
 #include "tbl/bo_flags.h"
 
 	/* Timeouts */
@@ -424,6 +401,7 @@ struct busyobj {
 
 	/* Timers */
 	vtim_real		t_first;	/* First timestamp logged */
+	vtim_real		t_resp;		/* response received */
 	vtim_real		t_prev;		/* Previous timestamp logged */
 
 	/* Acct */
@@ -439,23 +417,40 @@ struct busyobj {
 
 	uint8_t			digest[DIGEST_LEN];
 	struct vrt_privs	privs[1];
+
+	uint16_t		err_code;
+	const char		*err_reason;
+
+	const char		*client_identity;
 };
 
 
 /*--------------------------------------------------------------------*/
 
+struct reqtop {
+	unsigned		magic;
+#define REQTOP_MAGIC		0x57fbda52
+	struct req		*topreq;
+	struct vcl		*vcl0;
+	struct vrt_privs	privs[1];
+};
 
 struct req {
 	unsigned		magic;
-#define REQ_MAGIC		0x2751aaa1
+#define REQ_MAGIC		0xfb4abf6d
 
-	enum req_step		req_step;
-	volatile enum req_body_state_e	req_body_status;
-	enum sess_close		doclose;
-	int			restarts;
-	int			esi_level;
-	struct req		*topreq;	/* esi_level == 0 request */
-	struct vcl		*vcl0;
+	body_status_t		req_body_status;
+	stream_close_t		doclose;
+	unsigned		restarts;
+	unsigned		esi_level;
+
+	/* Delivery mode */
+	unsigned		res_mode;
+#define RES_ESI			(1<<4)
+#define RES_PIPE		(1<<7)
+
+	const struct req_step	*req_step;
+	struct reqtop		*top;	/* esi_level == 0 request */
 
 #define REQ_FLAG(l, r, w, d) unsigned	l:1;
 #include "tbl/req_flags.h"
@@ -465,7 +460,7 @@ struct req {
 
 	struct sess		*sp;
 	struct worker		*wrk;
-	struct pool_task	task;
+	struct pool_task	task[1];
 
 	const struct transport	*transport;
 	void			*transport_priv;
@@ -498,6 +493,7 @@ struct req {
 	vtim_real		t_first;	/* First timestamp logged */
 	vtim_real		t_prev;		/* Previous timestamp logged */
 	vtim_real		t_req;		/* Headers complete */
+	vtim_real		t_resp;		/* Entry to last deliver/synth */
 
 	struct http_conn	*htc;
 	struct vfp_ctx		*vfc;
@@ -519,11 +515,6 @@ struct req {
 	struct vdp_ctx		*vdc;
 	const char		*filter_list;
 
-	/* Delivery mode */
-	unsigned		res_mode;
-#define RES_ESI			(1<<4)
-#define RES_PIPE		(1<<7)
-
 	/* Transaction VSL buffer */
 	struct vsl_log		vsl[1];
 
@@ -535,7 +526,7 @@ struct req {
 	struct vcf		*vcf;
 };
 
-#define IS_TOPREQ(req) ((req)->topreq == (req))
+#define IS_TOPREQ(req) ((req)->top->topreq == (req))
 
 /*--------------------------------------------------------------------
  * Struct sess is a high memory-load structure because sessions typically
@@ -549,7 +540,6 @@ struct req {
 
 enum sess_attr {
 #define SESS_ATTR(UP, low, typ, len)	SA_##UP,
-	SA_TRANSPORT,
 #include "tbl/sess_attr.h"
 	SA_LAST
 };
@@ -572,9 +562,15 @@ struct sess {
 
 	vtim_real		t_open;		/* fd accepted */
 	vtim_real		t_idle;		/* fd accepted or resp sent */
-
+	vtim_dur		timeout_idle;
+	vtim_dur		timeout_linger;
+	vtim_dur		send_timeout;
+	vtim_dur		idle_send_timeout;
 };
 
+#define SESS_TMO(sp, tmo)					\
+	(isnan((sp)->tmo) ? cache_param->tmo : (sp)->tmo)
+
 /* Prototypes etc ----------------------------------------------------*/
 
 
@@ -589,7 +585,7 @@ void BAN_Abandon(struct ban_proto *b);
 
 /* cache_cli.c [CLI] */
 extern pthread_t cli_thread;
-#define ASSERT_CLI() do {assert(pthread_self() == cli_thread);} while (0)
+#define ASSERT_CLI() do {assert(pthread_equal(pthread_self(), cli_thread));} while (0)
 
 /* cache_http.c */
 unsigned HTTP_estimate(unsigned nhttp);
@@ -597,52 +593,57 @@ void HTTP_Clone(struct http *to, const struct http * const fm);
 void HTTP_Dup(struct http *to, const struct http * const fm);
 struct http *HTTP_create(void *p, uint16_t nhttp, unsigned);
 const char *http_Status2Reason(unsigned, const char **);
+int http_IsHdr(const txt *hh, hdr_t hdr);
 unsigned http_EstimateWS(const struct http *fm, unsigned how);
 void http_PutResponse(struct http *to, const char *proto, uint16_t status,
     const char *response);
 void http_FilterReq(struct http *to, const struct http *fm, unsigned how);
 void HTTP_Encode(const struct http *fm, uint8_t *, unsigned len, unsigned how);
 int HTTP_Decode(struct http *to, const uint8_t *fm);
-void http_ForceHeader(struct http *to, const char *hdr, const char *val);
+void http_ForceHeader(struct http *to, hdr_t, const char *val);
 void http_PrintfHeader(struct http *to, const char *fmt, ...)
     v_printflike_(2, 3);
 void http_TimeHeader(struct http *to, const char *fmt, vtim_real now);
 void http_Proto(struct http *to);
-void http_SetHeader(struct http *to, const char *hdr);
-void http_SetH(struct http *to, unsigned n, const char *fm);
+void http_SetHeader(struct http *to, const char *header);
+void http_SetH(struct http *to, unsigned n, const char *header);
 void http_ForceField(struct http *to, unsigned n, const char *t);
 void HTTP_Setup(struct http *, struct ws *, struct vsl_log *, enum VSL_tag_e);
 void http_Teardown(struct http *ht);
-int http_GetHdr(const struct http *hp, const char *hdr, const char **ptr);
-int http_GetHdrToken(const struct http *hp, const char *hdr,
+int http_GetHdr(const struct http *hp, hdr_t, const char **ptr);
+int http_GetHdrToken(const struct http *hp, hdr_t,
     const char *token, const char **pb, const char **pe);
-int http_GetHdrField(const struct http *hp, const char *hdr,
+int http_GetHdrField(const struct http *hp, hdr_t,
     const char *field, const char **ptr);
-double http_GetHdrQ(const struct http *hp, const char *hdr, const char *field);
+double http_GetHdrQ(const struct http *hp, hdr_t, const char *field);
 ssize_t http_GetContentLength(const struct http *hp);
+ssize_t http_GetContentRange(const struct http *hp, ssize_t *lo, ssize_t *hi);
+const char * http_GetRange(const struct http *hp, ssize_t *lo, ssize_t *hi);
 uint16_t http_GetStatus(const struct http *hp);
 int http_IsStatus(const struct http *hp, int);
-void http_SetStatus(struct http *to, uint16_t status);
+void http_SetStatus(struct http *to, uint16_t status, const char *reason);
 const char *http_GetMethod(const struct http *hp);
-int http_HdrIs(const struct http *hp, const char *hdr, const char *val);
+int http_HdrIs(const struct http *hp, hdr_t, const char *val);
 void http_CopyHome(const struct http *hp);
-void http_Unset(struct http *hp, const char *hdr);
-unsigned http_CountHdr(const struct http *hp, const char *hdr);
-void http_CollectHdr(struct http *hp, const char *hdr);
-void http_CollectHdrSep(struct http *hp, const char *hdr, const char *sep);
+void http_Unset(struct http *hp, hdr_t);
+unsigned http_CountHdr(const struct http *hp, hdr_t);
+void http_CollectHdr(struct http *hp, hdr_t);
+void http_CollectHdrSep(struct http *hp, hdr_t, const char *sep);
 void http_VSL_log(const struct http *hp);
 void HTTP_Merge(struct worker *, struct objcore *, struct http *to);
 uint16_t HTTP_GetStatusPack(struct worker *, struct objcore *oc);
 int HTTP_IterHdrPack(struct worker *, struct objcore *, const char **);
 #define HTTP_FOREACH_PACK(wrk, oc, ptr) \
 	 for ((ptr) = NULL; HTTP_IterHdrPack(wrk, oc, &(ptr));)
-const char *HTTP_GetHdrPack(struct worker *, struct objcore *, const char *hdr);
-enum sess_close http_DoConnection(struct http *hp);
+const char *HTTP_GetHdrPack(struct worker *, struct objcore *, hdr_t);
+stream_close_t http_DoConnection(struct http *hp, stream_close_t sc_close);
+int http_IsFiltered(const struct http *hp, unsigned u, unsigned how);
 
-#define HTTPH_R_PASS	(1 << 0)	/* Request (c->b) in pass mode */
-#define HTTPH_R_FETCH	(1 << 1)	/* Request (c->b) for fetch */
-#define HTTPH_A_INS	(1 << 2)	/* Response (b->o) for insert */
-#define HTTPH_A_PASS	(1 << 3)	/* Response (b->o) for pass */
+#define HTTPH_R_PASS		(1 << 0)	/* Request (c->b) in pass mode */
+#define HTTPH_R_FETCH		(1 << 1)	/* Request (c->b) for fetch */
+#define HTTPH_A_INS		(1 << 2)	/* Response (b->o) for insert */
+#define HTTPH_A_PASS		(1 << 3)	/* Response (b->o) for pass */
+#define HTTPH_C_SPECIFIC	(1 << 4)	/* Connection-specific */
 
 #define HTTPH(a, b, c) extern char b[];
 #include "tbl/http_headers.h"
@@ -651,9 +652,39 @@ extern const char H__Status[];
 extern const char H__Proto[];
 extern const char H__Reason[];
 
+// rfc7233,l,1207,1208
+#define http_tok_eq(s1, s2)		(!vct_casecmp(s1, s2))
+#define http_tok_at(s1, s2, l)		(!vct_caselencmp(s1, s2, l))
+#define http_ctok_at(s, cs)		(!vct_caselencmp(s, cs, sizeof(cs) - 1))
+
+// rfc7230,l,1037,1038
+#define http_scheme_at(str, tok)	http_ctok_at(str, #tok "://")
+
+// rfc7230,l,1144,1144
+// rfc7231,l,1156,1158
+#define http_method_eq(str, tok)	(!strcmp(str, #tok))
+
+// rfc7230,l,1222,1222
+// rfc7230,l,2848,2848
+// rfc7231,l,3883,3885
+// rfc7234,l,1339,1340
+// rfc7234,l,1418,1419
+#define http_hdr_eq(s1, s2)		http_tok_eq(s1, s2)
+#define http_hdr_at(s1, s2, l)		http_tok_at(s1, s2, l)
+
+// rfc7230,l,1952,1952
+// rfc7231,l,604,604
+#define http_coding_eq(str, tok)	http_tok_eq(str, #tok)
+
+// rfc7231,l,1864,1864
+#define http_expect_eq(str, tok)	http_tok_eq(str, #tok)
+
+// rfc7233,l,1207,1208
+#define http_range_at(str, tok, l)	http_tok_at(str, #tok, l)
+
 /* cache_main.c */
 #define VXID(u) ((u) & VSL_IDENTMASK)
-uint32_t VXID_Get(struct worker *, uint32_t marker);
+uint32_t VXID_Get(const struct worker *, uint32_t marker);
 extern pthread_key_t witness_key;
 
 /* cache_lck.c */
@@ -665,10 +696,13 @@ int Lck__Trylock(struct lock *lck, const char *p,  int l);
 void Lck__New(struct lock *lck, struct VSC_lck *, const char *);
 int Lck__Held(const struct lock *lck);
 int Lck__Owned(const struct lock *lck);
+extern pthread_mutexattr_t mtxattr_errorcheck;
 
 /* public interface: */
 void Lck_Delete(struct lock *lck);
-int Lck_CondWait(pthread_cond_t *cond, struct lock *lck, vtim_real);
+int Lck_CondWaitUntil(pthread_cond_t *, struct lock *, vtim_real when);
+int Lck_CondWait(pthread_cond_t *, struct lock *);
+int Lck_CondWaitTimeout(pthread_cond_t *, struct lock *, vtim_dur timeout);
 
 #define Lck_New(a, b) Lck__New(a, b, #b)
 #define Lck_Lock(a) Lck__Lock(a, __func__, __LINE__)
@@ -695,7 +729,7 @@ const void *ObjGetAttr(struct worker *, struct objcore *, enum obj_attr,
 typedef int objiterate_f(void *priv, unsigned flush,
     const void *ptr, ssize_t len);
 #define OBJ_ITER_FLUSH	0x01
-#define OBJ_ITER_FINAL	0x02
+#define OBJ_ITER_END	0x02
 
 int ObjIterate(struct worker *, struct objcore *,
     void *priv, objiterate_f *func, int final);
@@ -706,6 +740,11 @@ int ObjGetDouble(struct worker *, struct objcore *, enum obj_attr, double *);
 int ObjGetU32(struct worker *, struct objcore *, enum obj_attr, uint32_t *);
 int ObjGetU64(struct worker *, struct objcore *, enum obj_attr, uint64_t *);
 int ObjCheckFlag(struct worker *, struct objcore *, enum obj_flags of);
+
+/* cache_req_body.c */
+ssize_t VRB_Iterate(struct worker *, struct vsl_log *, struct req *,
+    objiterate_f *func, void *priv);
+
 /* cache_session.c [SES] */
 
 #define SESS_ATTR(UP, low, typ, len)					\
@@ -717,10 +756,12 @@ const char *SES_Get_String_Attr(const struct sess *sp, enum sess_attr a);
 void VSLv(enum VSL_tag_e tag, uint32_t vxid, const char *fmt, va_list va);
 void VSL(enum VSL_tag_e tag, uint32_t vxid, const char *fmt, ...)
     v_printflike_(3, 4);
+void VSLs(enum VSL_tag_e tag, uint32_t vxid, const struct strands *s);
 void VSLbv(struct vsl_log *, enum VSL_tag_e tag, const char *fmt, va_list va);
 void VSLb(struct vsl_log *, enum VSL_tag_e tag, const char *fmt, ...)
     v_printflike_(3, 4);
 void VSLbt(struct vsl_log *, enum VSL_tag_e tag, txt t);
+void VSLbs(struct vsl_log *, enum VSL_tag_e tag, const struct strands *s);
 void VSLb_ts(struct vsl_log *, const char *event, vtim_real first,
     vtim_real *pprev, vtim_real now);
 void VSLb_bin(struct vsl_log *, enum VSL_tag_e, ssize_t, const void*);
@@ -746,15 +787,6 @@ VSLb_ts_busyobj(struct busyobj *bo, const char *event, vtim_real now)
 /* cache_vcl.c */
 const char *VCL_Name(const struct vcl *);
 
-/* cache_vrt.c */
-/*
- * These prototypes go here, because we do not want to pollute vrt.h
- * with va_list.  VCC never generates direct calls to them.
- * XXX: We should deprecate these (ref: STRANDS)
- */
-const char *VRT_String(struct ws *ws, const char *h, const char *p, va_list ap);
-char *VRT_StringList(char *d, unsigned dl, const char *p, va_list ap);
-
 /* cache_wrk.c */
 
 typedef void *bgthread_t(struct worker *, void *priv);
@@ -762,11 +794,10 @@ void WRK_BgThread(pthread_t *thr, const char *name, bgthread_t *func,
     void *priv);
 
 /* cache_ws.c */
-
 void WS_Init(struct ws *ws, const char *id, void *space, unsigned len);
-unsigned WS_Reserve(struct ws *ws, unsigned bytes);
-unsigned WS_ReserveLumps(struct ws *ws, size_t sz);
-void WS_MarkOverflow(struct ws *ws);
+
+unsigned WS_ReserveSize(struct ws *, unsigned);
+unsigned WS_ReserveAll(struct ws *);
 void WS_Release(struct ws *ws, unsigned bytes);
 void WS_ReleaseP(struct ws *ws, const char *ptr);
 void WS_Assert(const struct ws *ws);
@@ -774,17 +805,44 @@ void WS_Reset(struct ws *ws, uintptr_t);
 void *WS_Alloc(struct ws *ws, unsigned bytes);
 void *WS_Copy(struct ws *ws, const void *str, int len);
 uintptr_t WS_Snapshot(struct ws *ws);
-int WS_Overflowed(const struct ws *ws);
-void *WS_Printf(struct ws *ws, const char *fmt, ...) v_printflike_(2, 3);
-int WS_Inside(const struct ws *, const void *, const void *);
-void WS_Assert_Allocated(const struct ws *ws, const void *ptr, ssize_t len);
+int WS_Allocated(const struct ws *ws, const void *ptr, ssize_t len);
+unsigned WS_Dump(const struct ws *ws, char, size_t off, void *buf, size_t len);
 
-static inline char*
-WS_Front(const struct ws *ws)
+static inline void *
+WS_Reservation(const struct ws *ws)
 {
-	return ws->f;
+
+	WS_Assert(ws);
+	AN(ws->r);
+	AN(ws->f);
+	return (ws->f);
 }
 
+static inline unsigned
+WS_ReservationSize(const struct ws *ws)
+{
+
+	AN(ws->r);
+	return (ws->r - ws->f);
+}
+
+static inline unsigned
+WS_ReserveLumps(struct ws *ws, size_t sz)
+{
+
+	AN(sz);
+	return (WS_ReserveAll(ws) / sz);
+}
+
+/* cache_ws_common.c */
+void WS_MarkOverflow(struct ws *ws);
+int WS_Overflowed(const struct ws *ws);
+
+const char *WS_Printf(struct ws *ws, const char *fmt, ...) v_printflike_(2, 3);
+
+void WS_VSB_new(struct vsb *, struct ws *);
+char *WS_VSB_finish(struct vsb *, struct ws *, size_t *);
+
 /* cache_rfc2616.c */
 void RFC2616_Ttl(struct busyobj *, vtim_real now, vtim_real *t_origin,
     float *ttl, float *grace, float *keep);
@@ -792,50 +850,9 @@ unsigned RFC2616_Req_Gzip(const struct http *);
 int RFC2616_Do_Cond(const struct req *sp);
 void RFC2616_Weaken_Etag(struct http *hp);
 void RFC2616_Vary_AE(struct http *hp);
-void RFC2616_Response_Body(const struct worker *, const struct busyobj *);
-
-/*
- * A normal pointer difference is signed, but we never want a negative value
- * so this little tool will make sure we don't get that.
- */
-
-static inline unsigned
-pdiff(const void *b, const void *e)
-{
-
-	assert(b <= e);
-	return
-	    ((unsigned)((const unsigned char *)e - (const unsigned char *)b));
-}
-
-#define Tcheck(t) do {						\
-		AN((t).b);					\
-		AN((t).e);					\
-		assert((t).b <= (t).e);				\
-	} while(0)
-
-/*
- * unsigned length of a txt
- */
-
-static inline unsigned
-Tlen(const txt t)
-{
-
-	Tcheck(t);
-	return ((unsigned)(t.e - t.b));
-}
 
 /*
  * We want to cache the most recent timestamp in wrk->lastused to avoid
  * extra timestamps in cache_pool.c.  Hide this detail with a macro
  */
 #define W_TIM_real(w) ((w)->lastused = VTIM_real())
-
-#define PAN_CheckMagic(vsb, ptr, exp)					\
-	do {								\
-		if ((ptr)->magic != (exp))				\
-			VSB_printf((vsb),				\
-			    "MAGIC at %p is 0x%08x (Should be: %s/0x%08x)\n", \
-			    ptr, (ptr)->magic, #exp, exp);		\
-	} while(0)
