@@ -38,7 +38,6 @@
 #include <stdlib.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
-#include <sys/socket.h>
 
 #include "cache_varnishd.h"
 
@@ -81,47 +80,54 @@ struct poolsock {
  * TCP options we want to control
  */
 
-static struct tcp_opt {
+union sock_arg {
+	struct linger	lg;
+	struct timeval	tv;
+	int		i;
+};
+
+static struct sock_opt {
 	int		level;
 	int		optname;
 	const char	*strname;
+	unsigned	mod;
 	socklen_t	sz;
-	void		*ptr;
-	int		need;
-	int		iponly;
-} tcp_opts[] = {
-#define TCPO(lvl, nam, sz, ip) { lvl, nam, #nam, sizeof(sz), 0, 0, ip},
+	union sock_arg	arg[1];
+} sock_opts[] = {
+	/* Note: Setting the mod counter to something not-zero is needed
+	 * to force the setsockopt() calls on startup */
+#define SOCK_OPT(lvl, nam, typ) { lvl, nam, #nam, 1, sizeof(typ) },
 
-	TCPO(SOL_SOCKET, SO_LINGER, struct linger, 0)
-	TCPO(SOL_SOCKET, SO_KEEPALIVE, int, 0)
-	TCPO(IPPROTO_TCP, TCP_NODELAY, int, 1)
+	SOCK_OPT(SOL_SOCKET, SO_LINGER, struct linger)
+	SOCK_OPT(SOL_SOCKET, SO_KEEPALIVE, int)
+	SOCK_OPT(SOL_SOCKET, SO_SNDTIMEO, struct timeval)
+	SOCK_OPT(SOL_SOCKET, SO_RCVTIMEO, struct timeval)
 
-#ifdef SO_SNDTIMEO_WORKS
-	TCPO(SOL_SOCKET, SO_SNDTIMEO, struct timeval, 0)
-#endif
-
-#ifdef SO_RCVTIMEO_WORKS
-	TCPO(SOL_SOCKET, SO_RCVTIMEO, struct timeval, 0)
-#endif
+	SOCK_OPT(IPPROTO_TCP, TCP_NODELAY, int)
 
 #ifdef HAVE_TCP_KEEP
-	TCPO(IPPROTO_TCP, TCP_KEEPIDLE, int, 1)
-	TCPO(IPPROTO_TCP, TCP_KEEPCNT, int, 1)
-	TCPO(IPPROTO_TCP, TCP_KEEPINTVL, int, 1)
+	SOCK_OPT(IPPROTO_TCP, TCP_KEEPIDLE, int)
+	SOCK_OPT(IPPROTO_TCP, TCP_KEEPCNT, int)
+	SOCK_OPT(IPPROTO_TCP, TCP_KEEPINTVL, int)
 #endif
 
-#undef TCPO
+#undef SOCK_OPT
 };
 
-static const int n_tcp_opts = sizeof tcp_opts / sizeof tcp_opts[0];
+static const int n_sock_opts = sizeof sock_opts / sizeof sock_opts[0];
+
+struct conn_heritage {
+	unsigned	sess_set;
+	unsigned	listen_mod;
+};
 
 /*--------------------------------------------------------------------
  * We want to get out of any kind of trouble-hit TCP connections as fast
- * as absolutely possible, so we set them LINGER enabled with zero timeout,
- * so that even if there are outstanding write data on the socket, a close(2)
- * will return immediately.
+ * as absolutely possible, so we set them LINGER disabled, so that even if
+ * there are outstanding write data on the socket, a close(2) will return
+ * immediately.
  */
-static const struct linger linger = {
+static const struct linger disable_so_linger = {
 	.l_onoff	=	0,
 };
 
@@ -130,7 +136,12 @@ static const struct linger linger = {
  * hung up on connections returning from waitinglists
  */
 
-static unsigned		need_test;
+static const unsigned enable_so_keepalive = 1;
+
+/* We disable Nagle's algorithm in favor of low latency setups.
+ */
+
+static const unsigned enable_tcp_nodelay = 1;
 
 /*--------------------------------------------------------------------
  * lacking a better place, we put some generic periodic updates
@@ -154,111 +165,147 @@ vca_periodic(vtim_real t0)
  */
 
 static int
-vca_tcp_opt_init(void)
+vca_sock_opt_init(void)
 {
-	int n;
-	int one = 1;
-	struct tcp_opt *to;
-	struct timeval tv;
-	int chg = 0;
-	int x;
+	struct sock_opt *so;
+	union sock_arg tmp;
+	int n, chg = 0;
+	size_t sz;
 
-	memset(&tv, 0, sizeof tv);
-	memset(&x, 0, sizeof x);
+	memset(&tmp, 0, sizeof tmp);
 
-	for (n = 0; n < n_tcp_opts; n++) {
-		to = &tcp_opts[n];
-		if (to->ptr == NULL)
-			to->ptr = calloc(1, to->sz);
-		AN(to->ptr);
-		if (!strcmp(to->strname, "SO_LINGER")) {
-			assert(to->sz == sizeof linger);
-			memcpy(to->ptr, &linger, sizeof linger);
-			to->need = 1;
-		} else if (!strcmp(to->strname, "TCP_NODELAY")) {
-			assert(to->sz == sizeof one);
-			memcpy(to->ptr, &one, sizeof one);
-			to->need = 1;
-		} else if (!strcmp(to->strname, "SO_KEEPALIVE")) {
-			assert(to->sz == sizeof one);
-			memcpy(to->ptr, &one, sizeof one);
-			to->need = 1;
-#define NEW_VAL(to, xx)						\
-	do {							\
-		assert(to->sz == sizeof xx);			\
-		if (memcmp(to->ptr, &(xx), sizeof xx)) {	\
-			memcpy(to->ptr, &(xx), sizeof xx);	\
-			to->need = 1;				\
-			chg = 1;				\
-			need_test = 1;				\
-		}						\
+	for (n = 0; n < n_sock_opts; n++) {
+		so = &sock_opts[n];
+
+#define SET_VAL(nm, so, fld, val)					\
+	do {								\
+		if (!strcmp(#nm, so->strname)) {			\
+			assert(so->sz == sizeof so->arg->fld);		\
+			so->arg->fld = (val);				\
+		}							\
 	} while (0)
 
-#ifdef SO_SNDTIMEO_WORKS
-		} else if (!strcmp(to->strname, "SO_SNDTIMEO")) {
-			tv = VTIM_timeval(cache_param->idle_send_timeout);
-			NEW_VAL(to, tv);
-#endif
-#ifdef SO_RCVTIMEO_WORKS
-		} else if (!strcmp(to->strname, "SO_RCVTIMEO")) {
-			tv = VTIM_timeval(cache_param->timeout_idle);
-			NEW_VAL(to, tv);
-#endif
+#define NEW_VAL(nm, so, fld, val)					\
+	do {								\
+		if (!strcmp(#nm, so->strname)) {			\
+			sz = sizeof tmp.fld;				\
+			assert(so->sz == sz);				\
+			tmp.fld = (val);				\
+			if (memcmp(&so->arg->fld, &(tmp.fld), sz)) {	\
+				memcpy(&so->arg->fld, &(tmp.fld), sz);	\
+				so->mod++;				\
+				chg = 1;				\
+			}						\
+		}							\
+	} while (0)
+
+		SET_VAL(SO_LINGER, so, lg, disable_so_linger);
+		SET_VAL(SO_KEEPALIVE, so, i, enable_so_keepalive);
+		NEW_VAL(SO_SNDTIMEO, so, tv,
+		    VTIM_timeval(cache_param->idle_send_timeout));
+		NEW_VAL(SO_RCVTIMEO, so, tv,
+		    VTIM_timeval(cache_param->timeout_idle));
+		SET_VAL(TCP_NODELAY, so, i, enable_tcp_nodelay);
 #ifdef HAVE_TCP_KEEP
-		} else if (!strcmp(to->strname, "TCP_KEEPIDLE")) {
-			x = (int)(cache_param->tcp_keepalive_time);
-			NEW_VAL(to, x);
-		} else if (!strcmp(to->strname, "TCP_KEEPCNT")) {
-			x = (int)(cache_param->tcp_keepalive_probes);
-			NEW_VAL(to, x);
-		} else if (!strcmp(to->strname, "TCP_KEEPINTVL")) {
-			x = (int)(cache_param->tcp_keepalive_intvl);
-			NEW_VAL(to, x);
+		NEW_VAL(TCP_KEEPIDLE, so, i,
+		    (int)cache_param->tcp_keepalive_time);
+		NEW_VAL(TCP_KEEPCNT, so, i,
+		    (int)cache_param->tcp_keepalive_probes);
+		NEW_VAL(TCP_KEEPINTVL, so, i,
+		    (int)cache_param->tcp_keepalive_intvl);
 #endif
-		}
 	}
 	return (chg);
 }
 
 static void
-vca_tcp_opt_test(const int sock, const unsigned uds)
+vca_sock_opt_test(const struct listen_sock *ls, const struct sess *sp)
 {
-	int i, n;
-	struct tcp_opt *to;
+	struct conn_heritage *ch;
+	struct sock_opt *so;
+	union sock_arg tmp;
 	socklen_t l;
-	void *ptr;
+	int i, n;
 
-	for (n = 0; n < n_tcp_opts; n++) {
-		to = &tcp_opts[n];
-		if (to->iponly && uds)
+	CHECK_OBJ_NOTNULL(ls, LISTEN_SOCK_MAGIC);
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+
+	for (n = 0; n < n_sock_opts; n++) {
+		so = &sock_opts[n];
+		ch = &ls->conn_heritage[n];
+		if (ch->sess_set) {
+			VSL(SLT_Debug, sp->vxid,
+			    "sockopt: Not testing nonhereditary %s for %s=%s",
+			    so->strname, ls->name, ls->endpoint);
 			continue;
-		to->need = 1;
-		ptr = calloc(1, to->sz);
-		AN(ptr);
-		l = to->sz;
-		i = getsockopt(sock, to->level, to->optname, ptr, &l);
-		if (i == 0 && !memcmp(ptr, to->ptr, to->sz))
-			to->need = 0;
-		free(ptr);
+		}
+		if (so->level == IPPROTO_TCP && ls->uds) {
+			VSL(SLT_Debug, sp->vxid,
+			    "sockopt: Not testing incompatible %s for %s=%s",
+			    so->strname, ls->name, ls->endpoint);
+			continue;
+		}
+		memset(&tmp, 0, sizeof tmp);
+		l = so->sz;
+		i = getsockopt(sp->fd, so->level, so->optname, &tmp, &l);
+		if (i == 0 && memcmp(&tmp, so->arg, so->sz)) {
+			VSL(SLT_Debug, sp->vxid,
+			    "sockopt: Test confirmed %s non heredity for %s=%s",
+			    so->strname, ls->name, ls->endpoint);
+			ch->sess_set = 1;
+		}
 		if (i && errno != ENOPROTOOPT)
 			VTCP_Assert(i);
 	}
 }
 
 static void
-vca_tcp_opt_set(const int sock, const unsigned uds, const int force)
+vca_sock_opt_set(const struct listen_sock *ls, const struct sess *sp)
 {
-	int n;
-	struct tcp_opt *to;
+	struct conn_heritage *ch;
+	struct sock_opt *so;
+	unsigned vxid;
+	int n, sock;
 
-	for (n = 0; n < n_tcp_opts; n++) {
-		to = &tcp_opts[n];
-		if (to->iponly && uds)
+	CHECK_OBJ_NOTNULL(ls, LISTEN_SOCK_MAGIC);
+
+	if (sp != NULL) {
+		CHECK_OBJ(sp, SESS_MAGIC);
+		sock = sp->fd;
+		vxid = sp->vxid;
+	} else {
+		sock = ls->sock;
+		vxid = 0;
+	}
+
+	for (n = 0; n < n_sock_opts; n++) {
+		so = &sock_opts[n];
+		ch = &ls->conn_heritage[n];
+		if (so->level == IPPROTO_TCP && ls->uds) {
+			VSL(SLT_Debug, vxid,
+			    "sockopt: Not setting incompatible %s for %s=%s",
+			    so->strname, ls->name, ls->endpoint);
 			continue;
-		if (to->need || force) {
-			VTCP_Assert(setsockopt(sock,
-			    to->level, to->optname, to->ptr, to->sz));
 		}
+		if (sp == NULL && ch->listen_mod == so->mod) {
+			VSL(SLT_Debug, vxid,
+			    "sockopt: Not setting unmodified %s for %s=%s",
+			    so->strname, ls->name, ls->endpoint);
+			continue;
+		}
+		if  (sp != NULL && !ch->sess_set) {
+			VSL(SLT_Debug, sp->vxid,
+			    "sockopt: %s may be inherited for %s=%s",
+			    so->strname, ls->name, ls->endpoint);
+			continue;
+		}
+		VSL(SLT_Debug, vxid,
+		    "sockopt: Setting %s for %s=%s",
+		    so->strname, ls->name, ls->endpoint);
+		VTCP_Assert(setsockopt(sock,
+		    so->level, so->optname, so->arg, so->sz));
+		if (sp == NULL)
+			ch->listen_mod = so->mod;
 	}
 }
 
@@ -316,10 +363,10 @@ vca_mk_tcp(const struct wrk_accept *wa,
     struct sess *sp, char *laddr, char *lport, char *raddr, char *rport)
 {
 	struct suckaddr *sa;
-	struct sockaddr_storage ss;
-	socklen_t sl;
+	ssize_t sz;
 
-	AN(SES_Reserve_remote_addr(sp, &sa));
+	AN(SES_Reserve_remote_addr(sp, &sa, &sz));
+	assert(sz == vsa_suckaddr_len);
 	AN(VSA_Build(sa, &wa->acceptaddr, wa->acceptaddrlen));
 	sp->sattr[SA_CLIENT_ADDR] = sp->sattr[SA_REMOTE_ADDR];
 
@@ -327,10 +374,9 @@ vca_mk_tcp(const struct wrk_accept *wa,
 	AN(SES_Set_String_Attr(sp, SA_CLIENT_IP, raddr));
 	AN(SES_Set_String_Attr(sp, SA_CLIENT_PORT, rport));
 
-	sl = sizeof ss;
-	AZ(getsockname(sp->fd, (void*)&ss, &sl));
-	AN(SES_Reserve_local_addr(sp, &sa));
-	AN(VSA_Build(sa, &ss, sl));
+
+	AN(SES_Reserve_local_addr(sp, &sa, &sz));
+	AN(VSA_getsockname(sp->fd, sa, sz));
 	sp->sattr[SA_SERVER_ADDR] = sp->sattr[SA_LOCAL_ADDR];
 	VTCP_name(sa, laddr, VTCP_ADDRBUFSIZE, lport, VTCP_PORTBUFSIZE);
 }
@@ -340,9 +386,11 @@ vca_mk_uds(struct wrk_accept *wa, struct sess *sp, char *laddr, char *lport,
 	   char *raddr, char *rport)
 {
 	struct suckaddr *sa;
+	ssize_t sz;
 
 	(void) wa;
-	AN(SES_Reserve_remote_addr(sp, &sa));
+	AN(SES_Reserve_remote_addr(sp, &sa, &sz));
+	assert(sz == vsa_suckaddr_len);
 	AZ(SES_Set_remote_addr(sp, bogo_ip));
 	sp->sattr[SA_CLIENT_ADDR] = sp->sattr[SA_REMOTE_ADDR];
 	sp->sattr[SA_LOCAL_ADDR] = sp->sattr[SA_REMOTE_ADDR];
@@ -400,22 +448,21 @@ vca_make_session(struct worker *wrk, void *arg)
 	    raddr, rport, wa->acceptlsock->name, laddr, lport,
 	    sp->t_open, sp->fd);
 
-	WS_Release(wrk->aws, 0);
-
 	vca_pace_good();
 	wrk->stats->sess_conn++;
 
-	if (need_test) {
-		vca_tcp_opt_test(sp->fd, wa->acceptlsock->uds);
-		need_test = 0;
+	if (wa->acceptlsock->test_heritage) {
+		vca_sock_opt_test(wa->acceptlsock, sp);
+		wa->acceptlsock->test_heritage = 0;
 	}
-	vca_tcp_opt_set(sp->fd, wa->acceptlsock->uds, 0);
+	vca_sock_opt_set(wa->acceptlsock, sp);
 
-	req = Req_New(wrk, sp);
+	req = Req_New(sp);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	req->htc->rfd = &sp->fd;
 
 	SES_SetTransport(wrk, sp, req, wa->acceptlsock->transport);
+	WS_Release(wrk->aws, 0);
 }
 
 /*--------------------------------------------------------------------
@@ -445,8 +492,8 @@ vca_accept_task(struct worker *wrk, void *arg)
 		VTIM_sleep(.1);
 
 	/* Dont hold on to (possibly) discarded VCLs */
-	if (wrk->vcl != NULL)
-		VCL_Rel(&wrk->vcl);
+	if (wrk->wpriv->vcl != NULL)
+		VCL_Rel(&wrk->wpriv->vcl);
 
 	while (!ps->pool->die) {
 		INIT_OBJ(&wa, WRK_ACCEPT_MAGIC);
@@ -509,14 +556,14 @@ vca_accept_task(struct worker *wrk, void *arg)
 
 			VSL(SLT_SessError, 0, "%s %s %s %d %d \"%s\"",
 			    wa.acceptlsock->name, laddr, lport,
-			    ls->sock, i, vstrerror(i));
+			    ls->sock, i, VAS_errtxt(i));
 			(void)Pool_TrySumstat(wrk);
 			continue;
 		}
 
 		wa.acceptsock = i;
 
-		if (!Pool_Task_Arg(wrk, TASK_QUEUE_VCA,
+		if (!Pool_Task_Arg(wrk, TASK_QUEUE_REQ,
 		    vca_make_session, &wa, sizeof wa)) {
 			/*
 			 * We couldn't get another thread, so we will handle
@@ -594,13 +641,24 @@ vca_acct(void *arg)
 
 	while (1) {
 		(void)sleep(1);
-		if (vca_tcp_opt_init()) {
+		if (vca_sock_opt_init()) {
 			AZ(pthread_mutex_lock(&shut_mtx));
 			VTAILQ_FOREACH(ls, &heritage.socks, list) {
 				if (ls->sock == -2)
 					continue;	// VCA_Shutdown
 				assert (ls->sock > 0);
-				vca_tcp_opt_set(ls->sock, ls->uds, 1);
+				vca_sock_opt_set(ls, NULL);
+				/* If one of the options on a socket has
+				 * changed, also force a retest of whether
+				 * the values are inherited to the
+				 * accepted sockets. This should then
+				 * catch any false positives from previous
+				 * tests that could happen if the set
+				 * value of an option happened to just be
+				 * the OS default for that value, and
+				 * wasn't actually inherited from the
+				 * listening socket. */
+				ls->test_heritage = 1;
 			}
 			AZ(pthread_mutex_unlock(&shut_mtx));
 		}
@@ -620,7 +678,7 @@ ccf_start(struct cli *cli, const char * const *av, void *priv)
 	(void)av;
 	(void)priv;
 
-	(void)vca_tcp_opt_init();
+	(void)vca_sock_opt_init();
 
 	VTAILQ_FOREACH(ls, &heritage.socks, list) {
 		CHECK_OBJ_NOTNULL(ls->transport, TRANSPORT_MAGIC);
@@ -629,22 +687,25 @@ ccf_start(struct cli *cli, const char * const *av, void *priv)
 		    VTCP_fastopen(ls->sock, cache_param->listen_depth))
 			VSL(SLT_Error, 0,
 			    "Kernel TCP Fast Open: sock=%d, errno=%d %s",
-			    ls->sock, errno, vstrerror(errno));
+			    ls->sock, errno, VAS_errtxt(errno));
 		if (listen(ls->sock, cache_param->listen_depth)) {
 			VCLI_SetResult(cli, CLIS_CANT);
 			VCLI_Out(cli, "Listen failed on socket '%s': %s",
-			    ls->endpoint, vstrerror(errno));
+			    ls->endpoint, VAS_errtxt(errno));
 			return;
 		}
-		vca_tcp_opt_set(ls->sock, ls->uds, 1);
+		AZ(ls->conn_heritage);
+		ls->conn_heritage = calloc(n_sock_opts,
+		    sizeof *ls->conn_heritage);
+		AN(ls->conn_heritage);
+		ls->test_heritage = 1;
+		vca_sock_opt_set(ls, NULL);
 		if (cache_param->accept_filter && VTCP_filter_http(ls->sock))
 			VSL(SLT_Error, 0,
 			    "Kernel filtering: sock=%d, errno=%d %s",
-			    ls->sock, errno, vstrerror(errno));
+			    ls->sock, errno, VAS_errtxt(errno));
 	}
 
-	need_test = 1;
-
 	AZ(pthread_create(&VCA_thread, NULL, vca_acct, NULL));
 }
 
@@ -654,7 +715,7 @@ static void v_matchproto_(cli_func_t)
 ccf_listen_address(struct cli *cli, const char * const *av, void *priv)
 {
 	struct listen_sock *ls;
-	char h[32], p[32];
+	char h[VTCP_ADDRBUFSIZE], p[VTCP_PORTBUFSIZE];
 
 	(void)cli;
 	(void)av;
@@ -673,10 +734,10 @@ ccf_listen_address(struct cli *cli, const char * const *av, void *priv)
 	VTAILQ_FOREACH(ls, &heritage.socks, list) {
 		if (!ls->uds) {
 			VTCP_myname(ls->sock, h, sizeof h, p, sizeof p);
-			VCLI_Out(cli, "%s %s\n", h, p);
+			VCLI_Out(cli, "%s %s %s\n", ls->name, h, p);
 		}
 		else
-			VCLI_Out(cli, "%s -\n", ls->endpoint);
+			VCLI_Out(cli, "%s %s -\n", ls->name, ls->endpoint);
 	}
 	AZ(pthread_mutex_unlock(&shut_mtx));
 }
@@ -720,21 +781,28 @@ VCA_Shutdown(void)
 static VTAILQ_HEAD(,transport)	transports =
     VTAILQ_HEAD_INITIALIZER(transports);
 
+static uint16_t next_xport;
+
+static void
+XPORT_Register(struct transport *xp)
+{
+
+	CHECK_OBJ_NOTNULL(xp, TRANSPORT_MAGIC);
+	AZ(xp->number);
+
+	xp->number = ++next_xport;
+	VTAILQ_INSERT_TAIL(&transports, xp, list);
+}
+
 void
 XPORT_Init(void)
 {
-	uint16_t n;
-	struct transport *xp;
 
 	ASSERT_MGT();
 
-	VTAILQ_INSERT_TAIL(&transports, &PROXY_transport, list);
-	VTAILQ_INSERT_TAIL(&transports, &HTTP1_transport, list);
-	VTAILQ_INSERT_TAIL(&transports, &H2_transport, list);
-
-	n = 0;
-	VTAILQ_FOREACH(xp, &transports, list)
-		xp->number = ++n;
+#define TRANSPORT_MACRO(name) XPORT_Register(&name##_transport);
+	TRANSPORTS
+#undef TRANSPORT_MACRO
 }
 
 const struct transport *
