@@ -54,9 +54,6 @@ struct backend {
 
 	unsigned		n_conn;
 
-	VTAILQ_ENTRY(backend)	list;
-	struct lock		mtx;
-
 	struct vrt_endpoint	*endpoint;
 
 	VRT_BACKEND_FIELDS()
@@ -72,17 +69,12 @@ struct backend {
 	struct conn_pool	*conn_pool;
 
 	VCL_BACKEND		director;
-
-	vtim_real		cooled;
 };
 
 /*---------------------------------------------------------------------
  * Prototypes
  */
 
-/* cache_backend_cfg.c */
-void VBE_SetHappy(const struct backend *, uint64_t);
-
 /* cache_backend_probe.c */
 void VBP_Update_Backend(struct vbp_target *vt);
 void VBP_Insert(struct backend *b, struct vrt_backend_probe const *p,
