@@ -54,6 +54,8 @@
 #include "cache_backend.h"
 #include "cache_conn_pool.h"
 
+#include "VSC_vbe.h"
+
 /* Default averaging rate, we want something pretty responsive */
 #define AVG_RATE			4
 
@@ -100,6 +102,8 @@ static const unsigned char vbp_proxy_local[] = {
 static void
 vbp_delete(struct vbp_target *vt)
 {
+	CHECK_OBJ_NOTNULL(vt, VBP_TARGET_MAGIC);
+
 #define DN(x)	/**/
 	VRT_BACKEND_PROBE_HANDLE();
 #undef DN
@@ -153,7 +157,6 @@ vbp_has_poked(struct vbp_target *vt)
 void
 VBP_Update_Backend(struct vbp_target *vt)
 {
-	const struct director *dir;
 	unsigned i = 0, chg;
 	char bits[10];
 
@@ -171,28 +174,19 @@ VBP_Update_Backend(struct vbp_target *vt)
 		return;
 	}
 
-	dir = vt->backend->director;
-	if (dir == NULL) {
-		Lck_Unlock(&vbp_mtx);
-		return;
-	}
-
 	i = (vt->good < vt->threshold);
 	chg = (i != vt->backend->sick);
 	vt->backend->sick = i;
 
-	AN(dir->vcl_name);
+	AN(vt->backend->vcl_name);
 	VSL(SLT_Backend_health, 0, "%s %s %s %s %u %u %u %.6f %.6f \"%s\"",
-	    dir->vcl_name, chg ? "Went" : "Still",
+	    vt->backend->vcl_name, chg ? "Went" : "Still",
 	    i ? "sick" : "healthy", bits,
 	    vt->good, vt->threshold, vt->window,
 	    vt->last, vt->avg, vt->resp_buf);
-	VBE_SetHappy(vt->backend, vt->happy);
-
-	if (chg) {
+	vt->backend->vsc->happy = vt->happy;
+	if (chg)
 		vt->backend->changed = VTIM_real();
-		VRT_SetChanged(dir, vt->backend->changed);
-	}
 	Lck_Unlock(&vbp_mtx);
 }
 
@@ -233,11 +227,11 @@ vbp_write(struct vbp_target *vt, int *sock, const void *buf, size_t len)
 		if (i < 0) {
 			vt->err_xmit |= 1;
 			bprintf(vt->resp_buf, "Write error %d (%s)",
-				errno, vstrerror(errno));
+				errno, VAS_errtxt(errno));
 		} else {
 			bprintf(vt->resp_buf,
 				"Short write (%d/%zu) error %d (%s)",
-				i, len, errno, vstrerror(errno));
+				i, len, errno, VAS_errtxt(errno));
 		}
 		VTCP_close(sock);
 		return (-1);
@@ -290,7 +284,7 @@ vbp_poke(struct vbp_target *vt)
 
 	s = VCP_Open(vt->conn_pool, t_end - t_now, &sa, &err);
 	if (s < 0) {
-		bprintf(vt->resp_buf, "Open error %d (%s)", err, vstrerror(err));
+		bprintf(vt->resp_buf, "Open error %d (%s)", err, VAS_errtxt(err));
 		Lck_Lock(&vbp_mtx);
 		if (vt->backend)
 			VBE_Connect_Error(vt->backend->vsc, err);
@@ -365,7 +359,7 @@ vbp_poke(struct vbp_target *vt)
 			if (!i)
 				errno = ETIMEDOUT;
 			bprintf(vt->resp_buf, "Poll error %d (%s)",
-			    errno, vstrerror(errno));
+			    errno, VAS_errtxt(errno));
 			i = -1;
 			break;
 		}
@@ -378,7 +372,7 @@ vbp_poke(struct vbp_target *vt)
 		if (i <= 0) {
 			if (i < 0)
 				bprintf(vt->resp_buf, "Read error %d (%s)",
-					errno, vstrerror(errno));
+					errno, VAS_errtxt(errno));
 			break;
 		}
 		rlen += i;
@@ -481,11 +475,11 @@ vbp_thread(struct worker *wrk, void *priv)
 		vt = VBH_root(vbp_heap);
 		if (vt == NULL) {
 			nxt = 8.192 + now;
-			(void)Lck_CondWait(&vbp_cond, &vbp_mtx, nxt);
+			(void)Lck_CondWaitUntil(&vbp_cond, &vbp_mtx, nxt);
 		} else if (vt->due > now) {
 			nxt = vt->due;
 			vt = NULL;
-			(void)Lck_CondWait(&vbp_cond, &vbp_mtx, nxt);
+			(void)Lck_CondWaitUntil(&vbp_cond, &vbp_mtx, nxt);
 		} else {
 			VBH_delete(vbp_heap, vt->heap_idx);
 			vt->due = now + vt->interval;
@@ -639,8 +633,7 @@ vbp_set_defaults(struct vbp_target *vt, const struct vrt_backend_probe *vp)
 	if (vt->initial == ~0U)
 		vt->initial = vt->threshold - 1;
 
-	if (vt->initial > vt->threshold)
-		vt->initial = vt->threshold;
+	vt->initial = vmin(vt->initial, vt->threshold);
 }
 
 /*--------------------------------------------------------------------
