@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -30,7 +32,7 @@
 
 #include "config.h"
 
-#include <pcre.h>
+#include <stdlib.h>
 #include <stdio.h>
 
 #include "cache_varnishd.h"
@@ -70,8 +72,6 @@ static const char * const arg_name[BAN_ARGARRSZ + 1] = {
 	[BAN_ARGARRSZ] = NULL
 };
 
-extern const char * const oper[BAN_OPERARRSZ + 1];
-
 /*--------------------------------------------------------------------
  * Storage handling of bans
  */
@@ -136,9 +136,12 @@ vtim_real
 ban_time(const uint8_t *banspec)
 {
 	vtim_real t;
+	uint64_t u;
 
+	assert(sizeof t == sizeof u);
 	assert(sizeof t == (BANS_LENGTH - BANS_TIMESTAMP));
-	memcpy(&t, banspec, sizeof t);
+	u = vbe64dec(banspec + BANS_TIMESTAMP);
+	memcpy(&t, &u, sizeof t);
 	return (t);
 }
 
@@ -159,8 +162,8 @@ ban_equal(const uint8_t *bs1, const uint8_t *bs2)
 	/*
 	 * Compare two ban-strings.
 	 */
-	u = vbe32dec(bs1 + BANS_LENGTH);
-	if (u != vbe32dec(bs2 + BANS_LENGTH))
+	u = ban_len(bs1);
+	if (u != ban_len(bs2))
 		return (0);
 	if (bs1[BANS_FLAGS] & BANS_FLAG_NODEDUP)
 		return (0);
@@ -266,6 +269,8 @@ BAN_DestroyObj(struct objcore *oc)
 {
 
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+	if (oc->ban == NULL)
+		return;
 	Lck_Lock(&ban_mtx);
 	CHECK_OBJ_ORNULL(oc->ban, BAN_MAGIC);
 	if (oc->ban != NULL) {
@@ -490,6 +495,7 @@ ban_evaluate(struct worker *wrk, const uint8_t *bsarg, struct objcore *oc,
 	const char *p;
 	const char *arg1;
 	double darg1, darg2;
+	int rv;
 
 	/*
 	 * for ttl and age, fix the point in time such that banning refers to
@@ -562,15 +568,19 @@ ban_evaluate(struct worker *wrk, const uint8_t *bsarg, struct objcore *oc,
 			}
 			break;
 		case BANS_OPER_MATCH:
-			if (arg1 == NULL ||
-			    pcre_exec(bt.arg2_spec, NULL, arg1, strlen(arg1),
-			    0, 0, NULL, 0) < 0)
+			if (arg1 == NULL)
+				return (0);
+			rv = VRE_match(bt.arg2_spec, arg1, 0, 0, NULL);
+			xxxassert(rv >= -1);
+			if (rv < 0)
 				return (0);
 			break;
 		case BANS_OPER_NMATCH:
-			if (arg1 != NULL &&
-			    pcre_exec(bt.arg2_spec, NULL, arg1, strlen(arg1),
-			    0, 0, NULL, 0) >= 0)
+			if (arg1 == NULL)
+				return (0);
+			rv = VRE_match(bt.arg2_spec, arg1, 0, 0, NULL);
+			xxxassert(rv >= -1);
+			if (rv >= 0)
 				return (0);
 			break;
 		case BANS_OPER_GT:
@@ -798,13 +808,13 @@ ban_render(struct cli *cli, const uint8_t *bs, int quote)
 		ASSERT_BAN_OPER(bt.oper);
 
 		if (BANS_HAS_ARG1_SPEC(bt.arg1))
-			VCLI_Out(cli, "%s.%.*s",
+			VCLI_Out(cli, "%s%.*s",
 			    arg_name[BAN_ARGIDX(bt.arg1)],
 			    bt.arg1_spec[0] - 1, bt.arg1_spec + 1);
 		else
 			VCLI_Out(cli, "%s", arg_name[BAN_ARGIDX(bt.arg1)]);
 
-		VCLI_Out(cli, " %s ", oper[BAN_OPERIDX(bt.oper)]);
+		VCLI_Out(cli, " %s ", ban_oper[BAN_OPERIDX(bt.oper)]);
 
 		if (BANS_HAS_ARG2_DOUBLE(bt.arg1)) {
 			vdur_render(buf, bt.arg2_double);
