@@ -32,7 +32,7 @@
 
 #include "config.h"
 
-#include <pcre.h>
+#include <stdlib.h>
 
 #include "cache_varnishd.h"
 #include "cache_ban.h"
@@ -40,6 +40,7 @@
 #include "vend.h"
 #include "vtim.h"
 #include "vnum.h"
+#include "vre.h"
 
 void BAN_Build_Init(void);
 void BAN_Build_Fini(void);
@@ -187,18 +188,26 @@ ban_parse_http(const struct ban_proto *bp, const char *a1)
 static const char *
 ban_parse_regexp(struct ban_proto *bp, const char *a3)
 {
-	const char *error;
-	int erroroffset, rc;
+	struct vsb vsb[1];
+	char errbuf[VRE_ERROR_LEN];
+	int errorcode, erroroffset;
 	size_t sz;
-	pcre *re;
+	vre_t *re, *rex;
 
-	re = pcre_compile(a3, 0, &error, &erroroffset, NULL);
-	if (re == NULL)
-		return (ban_error(bp, "Regex compile error: %s", error));
-	rc = pcre_fullinfo(re, NULL, PCRE_INFO_SIZE, &sz);
-	AZ(rc);
-	ban_add_lump(bp, re, sz);
-	pcre_free(re);
+	re = VRE_compile(a3, 0, &errorcode, &erroroffset, 0);
+	if (re == NULL) {
+		AN(VSB_init(vsb, errbuf, sizeof errbuf));
+		AZ(VRE_error(vsb, errorcode));
+		AZ(VSB_finish(vsb));
+		VSB_fini(vsb);
+		return (ban_error(bp, "Regex compile error: %s", errbuf));
+	}
+
+	rex = VRE_export(re, &sz);
+	AN(rex);
+	ban_add_lump(bp, rex, sz);
+	VRE_free(&rex);
+	VRE_free(&re);
 	return (0);
 }
 
