@@ -49,7 +49,7 @@ void
 VBO_Init(void)
 {
 
-	vbopool = MPL_New("busyobj", &cache_param->vbo_pool,
+	vbopool = MPL_New("busyobj", &cache_param->pool_vbo,
 	    &cache_param->workspace_backend);
 	AN(vbopool);
 }
@@ -82,7 +82,7 @@ vbo_Free(struct busyobj **bop)
 }
 
 struct busyobj *
-VBO_GetBusyObj(struct worker *wrk, const struct req *req)
+VBO_GetBusyObj(const struct worker *wrk, const struct req *req)
 {
 	struct busyobj *bo;
 	uint16_t nhttp;
@@ -132,7 +132,12 @@ VBO_GetBusyObj(struct worker *wrk, const struct req *req)
 
 	bo->do_stream = 1;
 
-	bo->director_req = req->director_hint;
+	if (req->client_identity != NULL) {
+		bo->client_identity = WS_Copy(bo->ws, req->client_identity, -1);
+		XXXAN(bo->client_identity);
+	}
+
+	VRT_Assign_Backend(&bo->director_req, req->director_hint);
 	bo->vcl = req->vcl;
 	VCL_Ref(bo->vcl);
 
@@ -175,7 +180,12 @@ VBO_ReleaseBusyObj(struct worker *wrk, struct busyobj **pbo)
 		    HSH_RUSH_POLICY);
 	}
 
+	VRT_Assign_Backend(&bo->director_req, NULL);
+	VRT_Assign_Backend(&bo->director_resp, NULL);
 	VCL_Rel(&bo->vcl);
+#ifdef ENABLE_WORKSPACE_EMULATOR
+	WS_Rollback(bo->ws, 0);
+#endif
 
 	memset(&bo->retries, 0,
 	    sizeof *bo - offsetof(struct busyobj, retries));
