@@ -156,6 +156,7 @@ vcp_handle(struct waited *w, enum wait_event ev, vtim_real now)
 	struct pfd *pfd;
 	struct conn_pool *cp;
 
+	CHECK_OBJ_NOTNULL(w, WAITED_MAGIC);
 	CAST_OBJ_NOTNULL(pfd, w->priv1, PFD_MAGIC);
 	(void)ev;
 	(void)now;
@@ -487,7 +488,7 @@ VCP_Get(struct conn_pool *cp, vtim_dur tmo, struct worker *wrk,
  */
 
 int
-VCP_Wait(struct worker *wrk, struct pfd *pfd, vtim_real tmo)
+VCP_Wait(struct worker *wrk, struct pfd *pfd, vtim_real when)
 {
 	struct conn_pool *cp;
 	int r;
@@ -499,7 +500,7 @@ VCP_Wait(struct worker *wrk, struct pfd *pfd, vtim_real tmo)
 	assert(pfd->cond == &wrk->cond);
 	Lck_Lock(&cp->mtx);
 	while (pfd->state == PFD_STATE_STOLEN) {
-		r = Lck_CondWait(&wrk->cond, &cp->mtx, tmo);
+		r = Lck_CondWaitUntil(&wrk->cond, &cp->mtx, when);
 		if (r != 0) {
 			if (r == EINTR)
 				continue;
@@ -730,6 +731,7 @@ VCP_Ref(const struct vrt_endpoint *vep, const char *ident)
 	cp->refcnt = 1;
 	cp->holddown = 0;
 	cp->endpoint = VRT_Endpoint_Clone(vep);
+	CHECK_OBJ_NOTNULL(cp->endpoint, VRT_ENDPOINT_MAGIC);
 	memcpy(cp->ident, digest, sizeof cp->ident);
 	if (vep->uds_path != NULL)
 		cp->methods = &vus_methods;
