@@ -34,6 +34,48 @@
 #include "cache_varnishd.h"
 #include "cache_filter.h"
 
+void
+VDP_Panic(struct vsb *vsb, const struct vdp_ctx *vdc)
+{
+	struct vdp_entry *vde;
+
+	if (PAN_dump_struct(vsb, vdc, VDP_CTX_MAGIC, "vdc"))
+		return;
+	VSB_printf(vsb, "nxt = %p,\n", vdc->nxt);
+	VSB_printf(vsb, "retval = %d,\n", vdc->retval);
+
+	if (!VTAILQ_EMPTY(&vdc->vdp)) {
+		VSB_cat(vsb, "filters = {\n");
+		VSB_indent(vsb, 2);
+		VTAILQ_FOREACH(vde, &vdc->vdp, list)
+			VSB_printf(vsb, "%s = %p { priv = %p }\n",
+			    vde->vdp->name, vde, vde->priv);
+		VSB_indent(vsb, -2);
+		VSB_cat(vsb, "},\n");
+	}
+
+	VSB_indent(vsb, -2);
+	VSB_cat(vsb, "},\n");
+}
+
+
+
+void
+VDP_Init(struct vdp_ctx *vdc, struct worker *wrk, struct vsl_log *vsl,
+    struct req *req)
+{
+	AN(vdc);
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	AN(vsl);
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+
+	INIT_OBJ(vdc, VDP_CTX_MAGIC);
+	VTAILQ_INIT(&vdc->vdp);
+	vdc->wrk = wrk;
+	vdc->vsl = vsl;
+	vdc->req = req;
+}
+
 /* VDP_bytes
  *
  * Pushes len bytes at ptr down the delivery processor list.
@@ -57,15 +99,15 @@
  * r == 0: Continue
  * r > 0:  Stop, breaks out early without error condition
  */
+
 int
-VDP_bytes(struct req *req, enum vdp_action act, const void *ptr, ssize_t len)
+VDP_bytes(struct vdp_ctx *vdc, enum vdp_action act,
+    const void *ptr, ssize_t len)
 {
 	int retval;
 	struct vdp_entry *vdpe;
-	struct vdp_ctx *vdc;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	vdc = req->vdc;
+	CHECK_OBJ_NOTNULL(vdc, VDP_CTX_MAGIC);
 	if (vdc->retval)
 		return (vdc->retval);
 	vdpe = vdc->nxt;
@@ -83,7 +125,10 @@ VDP_bytes(struct req *req, enum vdp_action act, const void *ptr, ssize_t len)
 
 	/* Call the present layer, while pointing to the next layer down */
 	vdc->nxt = VTAILQ_NEXT(vdpe, list);
-	retval = vdpe->vdp->bytes(req, act, &vdpe->priv, ptr, len);
+	vdpe->calls++;
+	vdc->bytes_done = len;
+	retval = vdpe->vdp->bytes(vdc, act, &vdpe->priv, ptr, len);
+	vdpe->bytes_in += vdc->bytes_done;
 	if (retval && (vdc->retval == 0 || retval < vdc->retval))
 		vdc->retval = retval; /* Latch error value */
 	vdc->nxt = vdpe;
@@ -91,13 +136,14 @@ VDP_bytes(struct req *req, enum vdp_action act, const void *ptr, ssize_t len)
 }
 
 int
-VDP_Push(struct req *req, const struct vdp *vdp, void *priv)
+VDP_Push(VRT_CTX, struct vdp_ctx *vdc, struct ws *ws, const struct vdp *vdp,
+    void *priv)
 {
 	struct vdp_entry *vdpe;
-	struct vdp_ctx *vdc;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	vdc = req->vdc;
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(vdc, VDP_CTX_MAGIC);
+	AN(ws);
 	AN(vdp);
 	AN(vdp->name);
 	AN(vdp->bytes);
@@ -106,9 +152,9 @@ VDP_Push(struct req *req, const struct vdp *vdp, void *priv)
 		return (vdc->retval);
 
 	if (DO_DEBUG(DBG_PROCESSORS))
-		VSLb(req->vsl, SLT_Debug, "VDP_push(%s)", vdp->name);
+		VSLb(vdc->vsl, SLT_Debug, "VDP_push(%s)", vdp->name);
 
-	vdpe = WS_Alloc(req->ws, sizeof *vdpe);
+	vdpe = WS_Alloc(ws, sizeof *vdpe);
 	if (vdpe == NULL) {
 		AZ(vdc->retval);
 		vdc->retval = -1;
@@ -121,8 +167,12 @@ VDP_Push(struct req *req, const struct vdp *vdp, void *priv)
 	vdc->nxt = VTAILQ_FIRST(&vdc->vdp);
 
 	AZ(vdc->retval);
-	if (vdpe->vdp->init != NULL)
-		vdc->retval = vdpe->vdp->init(req, &vdpe->priv);
+	if (vdpe->vdp->init == NULL)
+		return (vdc->retval);
+
+	vdc->retval = vdpe->vdp->init(ctx, vdc, &vdpe->priv,
+	    vdpe == vdc->nxt ? vdc->req->objcore : NULL);
+
 	if (vdc->retval > 0) {
 		VTAILQ_REMOVE(&vdc->vdp, vdpe, list);
 		vdc->nxt = VTAILQ_FIRST(&vdc->vdp);
@@ -131,27 +181,35 @@ VDP_Push(struct req *req, const struct vdp *vdp, void *priv)
 	return (vdc->retval);
 }
 
-void
-VDP_close(struct req *req)
+uint64_t
+VDP_Close(struct vdp_ctx *vdc)
 {
 	struct vdp_entry *vdpe;
-	struct vdp_ctx *vdc;
+	uint64_t rv = 0;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	vdc = req->vdc;
+	CHECK_OBJ_NOTNULL(vdc, VDP_CTX_MAGIC);
 	while (!VTAILQ_EMPTY(&vdc->vdp)) {
 		vdpe = VTAILQ_FIRST(&vdc->vdp);
+		rv = vdpe->bytes_in;
+		VSLb(vdc->vsl, SLT_VdpAcct, "%s %ju %ju", vdpe->vdp->name,
+		    (uintmax_t)vdpe->calls, (uintmax_t)rv);
 		if (vdc->retval >= 0)
 			AN(vdpe);
 		if (vdpe != NULL) {
 			CHECK_OBJ(vdpe, VDP_ENTRY_MAGIC);
 			if (vdpe->vdp->fini != NULL)
-				AZ(vdpe->vdp->fini(req, &vdpe->priv));
+				AZ(vdpe->vdp->fini(vdc, &vdpe->priv));
 			AZ(vdpe->priv);
 			VTAILQ_REMOVE(&vdc->vdp, vdpe, list);
 		}
 		vdc->nxt = VTAILQ_FIRST(&vdc->vdp);
+#ifdef VDP_PEDANTIC_ARMED
+		// enable when we are confident to get VDP_END right
+		if (vdc->nxt == NULL && vdc->retval >= 0)
+			assert(vdpe->end == VDP_END);
+#endif
 	}
+	return (rv);
 }
 
 /*--------------------------------------------------------------------*/
@@ -159,22 +217,31 @@ VDP_close(struct req *req)
 static int v_matchproto_(objiterate_f)
 vdp_objiterator(void *priv, unsigned flush, const void *ptr, ssize_t len)
 {
+	enum vdp_action act;
 
-	return (VDP_bytes(priv, flush ? VDP_FLUSH : VDP_NULL, ptr, len));
+	if (flush == 0)
+		act = VDP_NULL;
+	else if ((flush & OBJ_ITER_END) != 0)
+		act = VDP_END;
+	else
+		act = VDP_FLUSH;
+
+	return (VDP_bytes(priv, act, ptr, len));
 }
 
 
 int
-VDP_DeliverObj(struct req *req)
+VDP_DeliverObj(struct vdp_ctx *vdc, struct objcore *oc)
 {
 	int r, final;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	final = req->objcore->flags & (OC_F_PRIVATE | OC_F_HFM | OC_F_HFP)
-	    ? 1 : 0;
-	r = ObjIterate(req->wrk, req->objcore, req, vdp_objiterator, final);
-	if (r == 0)
-		r = VDP_bytes(req, VDP_END, NULL, 0);
+	CHECK_OBJ_NOTNULL(vdc, VDP_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+	CHECK_OBJ_NOTNULL(vdc->wrk, WORKER_MAGIC);
+	AN(vdc->vsl);
+	vdc->req = NULL;
+	final = oc->flags & (OC_F_PRIVATE | OC_F_HFM | OC_F_HFP) ? 1 : 0;
+	r = ObjIterate(vdc->wrk, oc, vdc, vdp_objiterator, final);
 	if (r < 0)
 		return (r);
 	return (0);
