@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -34,6 +36,10 @@
 struct vcldir {
 	unsigned			magic;
 #define VCLDIR_MAGIC			0xbf726c7d
+	int				refcnt;
+	unsigned			flags;
+#define VDIR_FLG_NOREFCNT		1
+	struct lock			dlck;
 	struct director			*dir;
 	struct vcl			*vcl;
 	const struct vdi_methods	*methods;
