@@ -60,7 +60,7 @@ struct fetch_step {
 
 #define FETCH_STEP(l, U) \
     static vbf_state_f vbf_stp_##l; \
-    static const struct fetch_step F_STP_##U[1] = {{ .name = "Fetch Step" #l, .func = vbf_stp_##l, }};
+    static const struct fetch_step F_STP_##U[1] = {{ .name = "Fetch Step " #l, .func = vbf_stp_##l, }};
 FETCH_STEPS
 #undef FETCH_STEP
 
@@ -110,8 +110,7 @@ vbf_allocobj(struct busyobj *bo, unsigned l)
 	 * on Transient storage.
 	 */
 
-	if (oc->ttl > cache_param->shortlived)
-		oc->ttl = cache_param->shortlived;
+	oc->ttl = vmin_t(float, oc->ttl, cache_param->shortlived);
 	oc->grace = 0.0;
 	oc->keep = 0.0;
 	return (STV_NewObject(bo->wrk, oc, stv_transient, l));
@@ -153,7 +152,7 @@ Bereq_Rollback(VRT_CTX)
 	HTTP_Clone(bo->bereq, bo->bereq0);
 	bo->vfp_filter_list = NULL;
 	bo->err_reason = NULL;
-	WS_Rollback(bo->bereq->ws, bo->ws_bo);
+	AN(bo->ws_bo);
 	WS_Rollback(bo->ws, bo->ws_bo);
 }
 
@@ -259,9 +258,9 @@ vbf_stp_mkbereq(struct worker *wrk, struct busyobj *bo)
 
 	HTTP_Setup(bo->bereq0, bo->ws, bo->vsl, SLT_BereqMethod);
 	http_FilterReq(bo->bereq0, bo->req->http,
-	    bo->do_pass ? HTTPH_R_PASS : HTTPH_R_FETCH);
+	    bo->uncacheable ? HTTPH_R_PASS : HTTPH_R_FETCH);
 
-	if (bo->do_pass)
+	if (bo->uncacheable)
 		AZ(bo->stale_oc);
 	else {
 		http_ForceField(bo->bereq0, HTTP_HDR_METHOD, "GET");
@@ -333,6 +332,8 @@ vbf_stp_retry(struct worker *wrk, struct busyobj *bo)
 	bo->was_304 = 0;
 	bo->err_code = 0;
 	bo->err_reason = NULL;
+	if (bo->htc != NULL)
+		bo->htc->doclose = SC_NULL;
 
 	// XXX: BereqEnd + BereqAcct ?
 	VSL_ChgId(bo->vsl, "bereq", "retry", VXID_Get(wrk, VSL_BACKENDMARKER));
@@ -366,7 +367,7 @@ vbf_304_logic(struct busyobj *bo)
 		HTTP_Merge(bo->wrk, bo->stale_oc, bo->beresp);
 		assert(http_IsStatus(bo->beresp, 200));
 		bo->was_304 = 1;
-	} else if (!bo->do_pass) {
+	} else if (!bo->uncacheable) {
 		/*
 		 * Backend sent unallowed 304
 		 */
@@ -397,7 +398,7 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 
 	AZ(bo->storage);
-	bo->storage = bo->do_pass ? stv_transient : STV_next();
+	bo->storage = bo->uncacheable ? stv_transient : STV_next();
 
 	if (bo->retries > 0)
 		http_Unset(bo->bereq, "\012X-Varnish:");
@@ -406,7 +407,6 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 
 	VCL_backend_fetch_method(bo->vcl, wrk, NULL, bo, NULL);
 
-	bo->uncacheable = bo->do_pass;
 	if (wrk->handling == VCL_RET_ABANDON || wrk->handling == VCL_RET_FAIL)
 		return (F_STP_FAIL);
 
@@ -428,8 +428,10 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 
 	VSLb_ts_busyobj(bo, "Fetch", W_TIM_real(wrk));
 	i = VDI_GetHdr(bo);
+	if (bo->htc != NULL)
+		CHECK_OBJ_NOTNULL(bo->htc->doclose, STREAM_CLOSE_MAGIC);
 
-	now = W_TIM_real(wrk);
+	bo->t_resp = now = W_TIM_real(wrk);
 	VSLb_ts_busyobj(bo, "Beresp", now);
 
 	if (i) {
@@ -493,6 +495,11 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 	    http_GetHdrField(bo->beresp, H_Connection, "close", NULL))
 		bo->htc->doclose = SC_RESP_CLOSE;
 
+	if (VRG_CheckBo(bo) < 0) {
+		VDI_Finish(bo);
+		return (F_STP_ERROR);
+	}
+
 	if (wrk->handling == VCL_RET_ABANDON || wrk->handling == VCL_RET_FAIL ||
 	    wrk->handling == VCL_RET_ERROR) {
 		/* do not count deliberately ending the backend connection as
@@ -538,7 +545,7 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 		bo->uncacheable = 1;
 		wrk->handling = VCL_RET_DELIVER;
 	}
-	if (bo->do_pass || bo->uncacheable)
+	if (bo->uncacheable)
 		oc->flags |= OC_F_HFM;
 
 	assert(wrk->handling == VCL_RET_DELIVER);
@@ -572,7 +579,7 @@ vbf_stp_fetchbody(struct worker *wrk, struct busyobj *bo)
 		est = 0;
 
 	do {
-		if (oc->flags & OC_F_ABANDON) {
+		if (oc->flags & OC_F_CANCEL) {
 			/*
 			 * A pass object and delivery was terminated
 			 * We don't fail the fetch, in order for HitMiss
@@ -623,6 +630,7 @@ vbf_stp_fetchbody(struct worker *wrk, struct busyobj *bo)
 static const struct fetch_step * v_matchproto_(vbf_state_f)
 vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
 {
+	struct vrt_ctx ctx[1];
 	struct objcore *oc;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
@@ -660,7 +668,10 @@ vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
 
 	oc->boc->len_so_far = 0;
 
-	if (VFP_Open(bo->vfc)) {
+	INIT_OBJ(ctx, VRT_CTX_MAGIC);
+	VCL_Bo2Ctx(ctx, bo);
+
+	if (VFP_Open(ctx, bo->vfc)) {
 		(void)VFP_Error(bo->vfc, "Fetch pipeline failed to open");
 		bo->htc->doclose = SC_RX_BODY;
 		vbf_cleanup(bo);
@@ -758,8 +769,7 @@ vbf_objiterator(void *priv, unsigned flush, const void *ptr, ssize_t len)
 		l = len;
 		if (VFP_GetStorage(bo->vfc, &l, &pd) != VFP_OK)
 			return (1);
-		if (len < l)
-			l = len;
+		l = vmin(l, len);
 		memcpy(pd, ps, l);
 		VFP_Extend(bo->vfc, l, l == len ? VFP_END : VFP_OK);
 		ps += l;
@@ -800,14 +810,18 @@ vbf_stp_condfetch(struct worker *wrk, struct busyobj *bo)
 		HSH_DerefBoc(bo->wrk, stale_oc);
 		stale_boc = NULL;
 		if (stale_state != BOS_FINISHED) {
-			(void)VFP_Error(bo->vfc, "Template object failed");
-			vbf_cleanup(bo);
-			wrk->stats->fetch_failed++;
-			return (F_STP_FAIL);
+			assert(stale_state == BOS_FAILED);
+			AN(stale_oc->flags & OC_F_FAILED);
 		}
 	}
+
 	AZ(stale_boc);
-	AZ(stale_oc->flags & OC_F_FAILED);
+	if (stale_oc->flags & OC_F_FAILED) {
+		(void)VFP_Error(bo->vfc, "Template object failed");
+		vbf_cleanup(bo);
+		wrk->stats->fetch_failed++;
+		return (F_STP_FAIL);
+	}
 
 	if (vbf_beresp2obj(bo)) {
 		vbf_cleanup(bo);
@@ -958,8 +972,7 @@ vbf_stp_error(struct worker *wrk, struct busyobj *bo)
 		l = ll;
 		if (VFP_GetStorage(bo->vfc, &l, &ptr) != VFP_OK)
 			break;
-		if (l > ll)
-			l = ll;
+		l = vmin(l, ll);
 		memcpy(ptr, VSB_data(synth_body) + o, l);
 		VFP_Extend(bo->vfc, l, l == ll ? VFP_END : VFP_OK);
 		ll -= l;
@@ -1110,7 +1123,7 @@ VBF_Fetch(struct worker *wrk, struct req *req, struct objcore *oc,
 	switch (mode) {
 	case VBF_PASS:
 		how = "pass";
-		bo->do_pass = 1;
+		bo->uncacheable = 1;
 		break;
 	case VBF_NORMAL:
 		how = "fetch";
