@@ -95,11 +95,11 @@ VFP_GetStorage(struct vfp_ctx *vc, ssize_t *sz, uint8_t **ptr)
 }
 
 void
-VFP_Extend(const struct vfp_ctx *vc, ssize_t sz)
+VFP_Extend(const struct vfp_ctx *vc, ssize_t sz, enum vfp_status flg)
 {
 	CHECK_OBJ_NOTNULL(vc, VFP_CTX_MAGIC);
 
-	ObjExtend(vc->wrk, vc->oc, sz);
+	ObjExtend(vc->wrk, vc->oc, sz, flg == VFP_END ? 1 : 0);
 }
 
 /**********************************************************************
@@ -115,27 +115,32 @@ VFP_Setup(struct vfp_ctx *vc, struct worker *wrk)
 }
 
 /**********************************************************************
+ * Returns the number of bytes processed by the lowest VFP in the stack
  */
 
-void
+uint64_t
 VFP_Close(struct vfp_ctx *vc)
 {
 	struct vfp_entry *vfe, *tmp;
+	uint64_t rv = 0;
 
 	VTAILQ_FOREACH_SAFE(vfe, &vc->vfp, list, tmp) {
 		if (vfe->vfp->fini != NULL)
 			vfe->vfp->fini(vc, vfe);
+		rv = vfe->bytes_out;
 		VSLb(vc->wrk->vsl, SLT_VfpAcct, "%s %ju %ju", vfe->vfp->name,
-		    (uintmax_t)vfe->calls, (uintmax_t)vfe->bytes_out);
+		    (uintmax_t)vfe->calls, (uintmax_t)rv);
 		VTAILQ_REMOVE(&vc->vfp, vfe, list);
 	}
+	return (rv);
 }
 
 int
-VFP_Open(struct vfp_ctx *vc)
+VFP_Open(VRT_CTX, struct vfp_ctx *vc)
 {
 	struct vfp_entry *vfe;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(vc, VFP_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(vc->resp, HTTP_MAGIC);
 	CHECK_OBJ_NOTNULL(vc->wrk, WORKER_MAGIC);
@@ -147,11 +152,11 @@ VFP_Open(struct vfp_ctx *vc)
 		if (DO_DEBUG(DBG_PROCESSORS))
 			VSLb(vc->wrk->vsl, SLT_Debug, "VFP_Open(%s)",
 			     vfe->vfp->name);
-		vfe->closed = vfe->vfp->init(vc, vfe);
+		vfe->closed = vfe->vfp->init(ctx, vc, vfe);
 		if (vfe->closed != VFP_OK && vfe->closed != VFP_NULL) {
 			(void)VFP_Error(vc, "Fetch filter %s failed to open",
 			    vfe->vfp->name);
-			VFP_Close(vc);
+			(void)VFP_Close(vc);
 			return (-1);
 		}
 	}
