@@ -36,8 +36,9 @@
 #include "cache_varnishd.h"
 #include <stdio.h>
 
-#include "vend.h"
 #include "vct.h"
+#include "vend.h"
+#include "vnum.h"
 #include "vtim.h"
 
 #define BODYSTATUS(U, l, n, a, k)				\
@@ -57,6 +58,131 @@ const char H__Status[]	= "\010:status:";
 const char H__Proto[]	= "\007:proto:";
 const char H__Reason[]	= "\010:reason:";
 
+/*--------------------------------------------------------------------
+ * Perfect hash to rapidly recognize headers from tbl/http_headers.h
+ * which have non-zero flags.
+ *
+ * A suitable algorithm can be found with `gperf`:
+ *
+ *	tr '" ,' '   ' < include/tbl/http_headers.h |
+ *		awk '$1 == "H(" && $4 != "0" {print$2}' |
+ *		gperf --ignore-case
+ *
+ */
+
+static const unsigned char http_asso_values[256] = {
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 25, 39,  0, 20,  5, 39, 39, 39, 15,  0, 39,
+	10, 39,  0, 39, 15, 10, 39, 39,  0, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 25, 39,  0, 20,  5, 39, 39, 39, 15,  0, 39,
+	10, 39,  0, 39, 15, 10, 39, 39,  0, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
+	39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39
+};
+
+static struct http_hdrflg {
+	char		*hdr;
+	unsigned	flag;
+} http_hdrflg[38 + 1] = {			// MAX_HASH_VALUE
+	{ NULL },
+	{ NULL },
+	{ H_TE },
+	{ H_Age },
+	{ NULL },
+	{ H_Range },
+	{ NULL },
+	{ H_Upgrade },
+	{ H_If_Range },
+	{ NULL },
+	{ H_Connection },
+	{ NULL },
+	{ H_Trailer },
+	{ H_If_None_Match },
+	{ NULL },
+	{ NULL },
+	{ NULL },
+	{ H_Transfer_Encoding },
+	{ H_Proxy_Authenticate },
+	{ H_Proxy_Authorization },
+	{ H_Keep_Alive },
+	{ NULL },
+	{ NULL },
+	{ H_If_Match },
+	{ H_HTTP2_Settings },
+	{ NULL },
+	{ NULL },
+	{ NULL },
+	{ H_Content_Range },
+	{ H_If_Unmodified_Since },
+	{ NULL },
+	{ NULL },
+	{ H_If_Modified_Since },
+	{ H_Cache_Control },
+	{ NULL },
+	{ NULL },
+	{ NULL },
+	{ NULL },
+	{ H_Accept_Ranges }
+};
+
+static struct http_hdrflg *
+http_hdr_flags(const char *b, const char *e)
+{
+	unsigned u;
+	struct http_hdrflg *retval;
+
+	if (b == NULL || e == NULL)
+		return (NULL);
+	assert(b <= e);
+	u = (unsigned)(e - b);
+	assert(b + u == e);
+	if (u < 2 || u > 19)		// MIN_WORD_LENGTH & MAX_WORD_LENGTH
+		return(NULL);
+	if (u > 3)
+		u += http_asso_values[((const uint8_t*)b)[3]];
+	if (u > 38)			// MAX_HASH_VALUE
+		return(NULL);
+	retval = &http_hdrflg[u];
+	if (retval->hdr == NULL)
+		return(NULL);
+	if (!http_hdr_at(retval->hdr + 1, b, e - b))
+		return(NULL);
+	return(retval);
+}
+
+/*--------------------------------------------------------------------*/
+
+static void
+http_init_hdr(char *hdr, int flg)
+{
+	struct http_hdrflg *f;
+
+	hdr[0] = strlen(hdr + 1);
+	f = http_hdr_flags(hdr + 1, hdr + hdr[0]);
+	if (flg) {
+		AN(f);
+		assert(f->hdr == hdr);
+		f->flag = flg;
+	}
+}
+
+void
+HTTP_Init(void)
+{
+
+#define HTTPH(a, b, c) http_init_hdr(b, c);
+#include "tbl/http_headers.h"
+}
+
 /*--------------------------------------------------------------------
  * These two functions are in an incestuous relationship with the
  * order of macros in include/tbl/vsl_tags_http.h
@@ -265,12 +391,12 @@ http_Proto(struct http *to)
 /*--------------------------------------------------------------------*/
 
 void
-http_SetH(struct http *to, unsigned n, const char *fm)
+http_SetH(struct http *to, unsigned n, const char *header)
 {
 
 	assert(n < to->nhd);
-	AN(fm);
-	to->hd[n].b = TRUST_ME(fm);
+	AN(header);
+	to->hd[n].b = TRUST_ME(header);
 	to->hd[n].e = strchr(to->hd[n].b, '\0');
 	to->hdf[n] = 0;
 	http_VSLH(to, n);
@@ -297,8 +423,8 @@ http_PutField(struct http *to, int field, const char *string)
 
 /*--------------------------------------------------------------------*/
 
-static int
-http_IsHdr(const txt *hh, const char *hdr)
+int
+http_IsHdr(const txt *hh, hdr_t hdr)
 {
 	unsigned l;
 
@@ -308,7 +434,7 @@ http_IsHdr(const txt *hh, const char *hdr)
 	assert(l == strlen(hdr + 1));
 	assert(hdr[l] == ':');
 	hdr++;
-	return (!strncasecmp(hdr, hh->b, l));
+	return (http_hdr_at(hdr, hh->b, l));
 }
 
 /*--------------------------------------------------------------------*/
@@ -324,7 +450,7 @@ http_findhdr(const struct http *hp, unsigned l, const char *hdr)
 			continue;
 		if (hp->hd[u].b[l] != ':')
 			continue;
-		if (strncasecmp(hdr, hp->hd[u].b, l))
+		if (!http_hdr_at(hdr, hp->hd[u].b, l))
 			continue;
 		return (u);
 	}
@@ -336,7 +462,7 @@ http_findhdr(const struct http *hp, unsigned l, const char *hdr)
  */
 
 unsigned
-http_CountHdr(const struct http *hp, const char *hdr)
+http_CountHdr(const struct http *hp, hdr_t hdr)
 {
 	unsigned retval = 0;
 	unsigned u;
@@ -357,7 +483,7 @@ http_CountHdr(const struct http *hp, const char *hdr)
  */
 
 void
-http_CollectHdr(struct http *hp, const char *hdr)
+http_CollectHdr(struct http *hp, hdr_t hdr)
 {
 
 	http_CollectHdrSep(hp, hdr, NULL);
@@ -370,7 +496,7 @@ http_CollectHdr(struct http *hp, const char *hdr)
  */
 
 void
-http_CollectHdrSep(struct http *hp, const char *hdr, const char *sep)
+http_CollectHdrSep(struct http *hp, hdr_t hdr, const char *sep)
 {
 	unsigned u, l, lsep, ml, f, x, d;
 	char *b = NULL, *e = NULL;
@@ -453,7 +579,7 @@ http_CollectHdrSep(struct http *hp, const char *hdr, const char *sep)
 /*--------------------------------------------------------------------*/
 
 int
-http_GetHdr(const struct http *hp, const char *hdr, const char **ptr)
+http_GetHdr(const struct http *hp, hdr_t hdr, const char **ptr)
 {
 	unsigned u, l;
 	const char *p;
@@ -529,7 +655,7 @@ http_split(const char **src, const char *stop, const char *sep,
  * Comparison rule for tokens:
  *	if target string starts with '"', we use memcmp() and expect closing
  *	double quote as well
- *	otherwise we use strncasecmp()
+ *	otherwise we use http_tok_at()
  *
  * On match we increment *bp past the token name.
  */
@@ -551,7 +677,7 @@ http_istoken(const char **bp, const char *e, const char *token)
 		*bp += fl + 2;
 		return (1);
 	}
-	if (b + fl <= e && !strncasecmp(b, token, fl) &&
+	if (b + fl <= e && http_tok_at(b, token, fl) &&
 	    (b + fl == e || !vct_istchar(b[fl]))) {
 		*bp += fl;
 		return (1);
@@ -580,7 +706,7 @@ http_istoken(const char **bp, const char *e, const char *token)
  */
 
 int
-http_GetHdrToken(const struct http *hp, const char *hdr,
+http_GetHdrToken(const struct http *hp, hdr_t hdr,
     const char *token, const char **pb, const char **pe)
 {
 	const char *h, *b, *e;
@@ -617,7 +743,7 @@ http_GetHdrToken(const struct http *hp, const char *hdr,
  */
 
 double
-http_GetHdrQ(const struct http *hp, const char *hdr, const char *field)
+http_GetHdrQ(const struct http *hp, hdr_t hdr, const char *field)
 {
 	const char *hb, *he, *b, *e;
 	int i;
@@ -666,7 +792,7 @@ http_GetHdrQ(const struct http *hp, const char *hdr, const char *field)
  */
 
 int
-http_GetHdrField(const struct http *hp, const char *hdr,
+http_GetHdrField(const struct http *hp, hdr_t hdr,
     const char *field, const char **ptr)
 {
 	const char *h;
@@ -701,24 +827,14 @@ http_GetContentLength(const struct http *hp)
 {
 	ssize_t cl;
 	const char *b;
-	unsigned n;
 
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
 
 	if (!http_GetHdr(hp, H_Content_Length, &b))
 		return (-1);
-	cl = 0;
-	if (!vct_isdigit(*b))
+	cl = VNUM_uint(b, NULL, &b);
+	if (cl < 0)
 		return (-2);
-	for (; vct_isdigit(*b); b++) {
-		if (cl > (SSIZE_MAX / 10))
-			return (-2);
-		cl *= 10;
-		n = *b - '0';
-		if (cl > (SSIZE_MAX - n))
-			return (-2);
-		cl += n;
-	}
 	while (vct_islws(*b))
 		b++;
 	if (*b != '\0')
@@ -726,15 +842,127 @@ http_GetContentLength(const struct http *hp)
 	return (cl);
 }
 
+ssize_t
+http_GetContentRange(const struct http *hp, ssize_t *lo, ssize_t *hi)
+{
+	ssize_t tmp, cl;
+	const char *b, *t;
+
+	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
+
+	if (lo == NULL)
+		lo = &tmp;
+	if (hi == NULL)
+		hi = &tmp;
+
+	*lo = *hi = -1;
+
+	if (!http_GetHdr(hp, H_Content_Range, &b))
+		return (-1);
+
+	t = strchr(b, ' ');
+	if (t == NULL)
+		return (-2);		// Missing space after range unit
+
+	if (!http_range_at(b, bytes, t - b))
+		return (-1);		// Unknown range unit, ignore
+	b = t + 1;
+
+	if (*b == '*') {		// Content-Range: bytes */123
+		*lo = *hi = -1;
+		b++;
+	} else {			// Content-Range: bytes 1-2/3
+		*lo = VNUM_uint(b, NULL, &b);
+		if (*lo < 0)
+			return (-2);
+		if (*b != '-')
+			return (-2);
+		*hi = VNUM_uint(b + 1, NULL, &b);
+		if (*hi < 0)
+			return (-2);
+	}
+	if (*b != '/')
+		return (-2);
+	if (b[1] == '*') {		// Content-Range: bytes 1-2/*
+		cl = -1;
+		b += 2;
+	} else {
+		cl = VNUM_uint(b + 1, NULL, &b);
+		if (cl <= 0)
+			return (-2);
+	}
+	while (vct_islws(*b))
+		b++;
+	if (*b != '\0')
+		return (-2);
+	if (*lo > *hi)
+		return (-2);
+	assert(cl >= -1);
+	if (*lo >= cl || *hi >= cl)
+		return (-2);
+	AN(cl);
+	return (cl);
+}
+
+const char *
+http_GetRange(const struct http *hp, ssize_t *lo, ssize_t *hi)
+{
+	ssize_t tmp;
+	const char *b, *t;
+
+	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
+
+	if (lo == NULL)
+		lo = &tmp;
+	if (hi == NULL)
+		hi = &tmp;
+
+	*lo = *hi = -1;
+
+	if (!http_GetHdr(hp, H_Range, &b))
+		return (NULL);
+
+	t = strchr(b, '=');
+	if (t == NULL)
+		return ("Missing '='");
+
+	if (!http_range_at(b, bytes, t - b))
+		return ("Not Bytes");
+	b = t + 1;
+
+	*lo = VNUM_uint(b, NULL, &b);
+	if (*lo == -2)
+		return ("Low number too big");
+	if (*b++ != '-')
+		return ("Missing hyphen");
+
+	*hi = VNUM_uint(b, NULL, &b);
+	if (*hi == -2)
+		return ("High number too big");
+	if (*lo == -1 && *hi == -1)
+		return ("Neither high nor low");
+	if (*lo == -1 && *hi == 0)
+		return ("No low, high is zero");
+	if (*hi >= 0 && *hi < *lo)
+		return ("high smaller than low");
+
+	while (vct_islws(*b))
+		b++;
+	if (*b != '\0')
+		return ("Trailing stuff");
+	return (NULL);
+}
+
 /*--------------------------------------------------------------------
  */
 
-enum sess_close
-http_DoConnection(struct http *hp, enum sess_close sc_close)
+stream_close_t
+http_DoConnection(struct http *hp, stream_close_t sc_close)
 {
 	const char *h, *b, *e;
-	enum sess_close retval;
+	stream_close_t retval;
 	unsigned u, v;
+	struct http_hdrflg *f;
 
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
 	assert(sc_close == SC_REQ_CLOSE || sc_close == SC_RESP_CLOSE);
@@ -750,19 +978,15 @@ http_DoConnection(struct http *hp, enum sess_close sc_close)
 	AN(h);
 	while (http_split(&h, NULL, ",", &b, &e)) {
 		u = pdiff(b, e);
-		if (u == 5 && !strncasecmp(b, "close", u))
+		if (u == 5 && http_hdr_at(b, "close", u))
 			retval = sc_close;
-		if (u == 10 && !strncasecmp(b, "keep-alive", u))
+		if (u == 10 && http_hdr_at(b, "keep-alive", u))
 			retval = SC_NULL;
 
 		/* Refuse removal of well-known-headers if they would pass. */
-/*lint -save -e506 [constant value boolean] */
-#define HTTPH(a, x, c)						\
-		if (!((c) & HTTPH_R_PASS) &&			\
-		    strlen(a) == u && !strncasecmp(a, b, u))	\
+		f = http_hdr_flags(b, e);
+		if (f != NULL && !(f->flag & HTTPH_R_PASS))
 			return (SC_RX_BAD);
-#include "tbl/http_headers.h"
-/*lint -restore */
 
 		for (v = HTTP_HDR_FIRST; v < hp->nhd; v++) {
 			Tcheck(hp->hd[v]);
@@ -770,25 +994,26 @@ http_DoConnection(struct http *hp, enum sess_close sc_close)
 				continue;
 			if (hp->hd[v].b[u] != ':')
 				continue;
-			if (strncasecmp(b, hp->hd[v].b, u))
+			if (!http_hdr_at(b, hp->hd[v].b, u))
 				continue;
 			hp->hdf[v] |= HDF_FILTER;
 		}
 	}
+	CHECK_OBJ_NOTNULL(retval, STREAM_CLOSE_MAGIC);
 	return (retval);
 }
 
 /*--------------------------------------------------------------------*/
 
 int
-http_HdrIs(const struct http *hp, const char *hdr, const char *val)
+http_HdrIs(const struct http *hp, hdr_t hdr, const char *val)
 {
 	const char *p;
 
 	if (!http_GetHdr(hp, hdr, &p))
 		return (0);
 	AN(p);
-	if (!strcasecmp(p, val))
+	if (http_tok_eq(p, val))
 		return (1);
 	return (0);
 }
@@ -867,10 +1092,14 @@ http_ForceField(struct http *to, unsigned n, const char *t)
 
 	CHECK_OBJ_NOTNULL(to, HTTP_MAGIC);
 	assert(n < HTTP_HDR_FIRST);
+	assert(n == HTTP_HDR_METHOD || n == HTTP_HDR_PROTO);
 	AN(t);
+
+	/* NB: method names and protocol versions are case-sensitive. */
 	if (to->hd[n].b == NULL || strcmp(to->hd[n].b, t)) {
 		i = (HTTP_HDR_UNSET - HTTP_HDR_METHOD);
 		i += to->logtag;
+		/* XXX: this is a dead branch */
 		if (n >= HTTP_HDR_FIRST)
 			VSLbt(to->vsl, (enum VSL_tag_e)i, to->hd[n]);
 		http_SetH(to, n, t);
@@ -898,13 +1127,18 @@ http_PutResponse(struct http *to, const char *proto, uint16_t status,
 static inline int
 http_isfiltered(const struct http *fm, unsigned u, unsigned how)
 {
+	const char *e;
+	const struct http_hdrflg *f;
+
 	if (fm->hdf[u] & HDF_FILTER)
 		return (1);
-#define HTTPH(a, b, c) \
-	if (((c) & how) && http_IsHdr(&fm->hd[u], (b))) \
-		return (1);
-#include "tbl/http_headers.h"
-	return (0);
+	if (u < HTTP_HDR_FIRST)
+		return (0);
+	e = strchr(fm->hd[u].b, ':');
+	if (e == NULL)
+		return (0);
+	f = http_hdr_flags(fm->hd[u].b, e);
+	return (f != NULL && f->flag & how);
 }
 
 int
@@ -1058,7 +1292,7 @@ HTTP_IterHdrPack(struct worker *wrk, struct objcore *oc, const char **p)
 }
 
 const char *
-HTTP_GetHdrPack(struct worker *wrk, struct objcore *oc, const char *hdr)
+HTTP_GetHdrPack(struct worker *wrk, struct objcore *oc, hdr_t hdr)
 {
 	const char *ptr;
 	unsigned l;
@@ -1079,6 +1313,7 @@ HTTP_GetHdrPack(struct worker *wrk, struct objcore *oc, const char *hdr)
 		AN(ptr);
 		ptr += 4;	/* Skip nhd and status */
 
+		/* XXX: should we also have h2_hdr_eq() ? */
 		if (!strcmp(hdr, ":proto:"))
 			return (ptr);
 		ptr = strchr(ptr, '\0') + 1;
@@ -1091,7 +1326,7 @@ HTTP_GetHdrPack(struct worker *wrk, struct objcore *oc, const char *hdr)
 	}
 
 	HTTP_FOREACH_PACK(wrk, oc, ptr) {
-		if (!strncasecmp(ptr, hdr, l)) {
+		if (http_hdr_at(ptr, hdr, l)) {
 			ptr += l;
 			while (vct_islws(*ptr))
 				ptr++;
@@ -1200,10 +1435,11 @@ http_CopyHome(const struct http *hp)
 			assert(u < HTTP_HDR_FIRST);
 			continue;
 		}
-		if (WS_Inside(hp->ws, hp->hd[u].b, hp->hd[u].e))
-			continue;
 
 		l = Tlen(hp->hd[u]);
+		if (WS_Allocated(hp->ws, hp->hd[u].b, l))
+			continue;
+
 		p = WS_Copy(hp->ws, hp->hd[u].b, l + 1L);
 		if (p == NULL) {
 			http_fail(hp);
@@ -1218,22 +1454,22 @@ http_CopyHome(const struct http *hp)
 /*--------------------------------------------------------------------*/
 
 void
-http_SetHeader(struct http *to, const char *hdr)
+http_SetHeader(struct http *to, const char *header)
 {
 
 	CHECK_OBJ_NOTNULL(to, HTTP_MAGIC);
 	if (to->nhd >= to->shd) {
-		VSLb(to->vsl, SLT_LostHeader, "%s", hdr);
+		VSLb(to->vsl, SLT_LostHeader, "%s", header);
 		http_fail(to);
 		return;
 	}
-	http_SetH(to, to->nhd++, hdr);
+	http_SetH(to, to->nhd++, header);
 }
 
 /*--------------------------------------------------------------------*/
 
 void
-http_ForceHeader(struct http *to, const char *hdr, const char *val)
+http_ForceHeader(struct http *to, hdr_t hdr, const char *val)
 {
 
 	CHECK_OBJ_NOTNULL(to, HTTP_MAGIC);
@@ -1295,7 +1531,7 @@ http_TimeHeader(struct http *to, const char *fmt, vtim_real now)
 /*--------------------------------------------------------------------*/
 
 void
-http_Unset(struct http *hp, const char *hdr)
+http_Unset(struct http *hp, hdr_t hdr)
 {
 	uint16_t u, v;
 
@@ -1313,13 +1549,3 @@ http_Unset(struct http *hp, const char *hdr)
 	}
 	hp->nhd = v;
 }
-
-/*--------------------------------------------------------------------*/
-
-void
-HTTP_Init(void)
-{
-
-#define HTTPH(a, b, c) b[0] = (char)strlen(b + 1);
-#include "tbl/http_headers.h"
-}
