@@ -42,9 +42,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#if defined (__APPLE__)
-#    include "vtim.h"
-#endif
+#include "vtim.h"
 
 #include "VSC_lck.h"
 
@@ -58,8 +56,6 @@ struct ilck {
 	struct VSC_lck		*stat;
 };
 
-static pthread_mutexattr_t attr;
-
 /*--------------------------------------------------------------------*/
 
 static void
@@ -115,6 +111,7 @@ Lck__Lock(struct lock *lck, const char *p, int l)
 	struct ilck *ilck;
 	int r = EINVAL;
 
+	AN(lck);
 	CAST_OBJ_NOTNULL(ilck, lck->priv, ILCK_MAGIC);
 	if (DO_DEBUG(DBG_WITNESS))
 		Lck_Witness_Lock(ilck, p, l, "");
@@ -140,6 +137,7 @@ Lck__Unlock(struct lock *lck, const char *p, int l)
 	(void)p;
 	(void)l;
 
+	AN(lck);
 	CAST_OBJ_NOTNULL(ilck, lck->priv, ILCK_MAGIC);
 	assert(pthread_equal(ilck->owner, pthread_self()));
 	AN(ilck->held);
@@ -170,6 +168,7 @@ Lck__Trylock(struct lock *lck, const char *p, int l)
 	struct ilck *ilck;
 	int r;
 
+	AN(lck);
 	CAST_OBJ_NOTNULL(ilck, lck->priv, ILCK_MAGIC);
 	if (DO_DEBUG(DBG_WITNESS))
 		Lck_Witness_Lock(ilck, p, l, "?");
@@ -190,6 +189,7 @@ Lck__Held(const struct lock *lck)
 {
 	struct ilck *ilck;
 
+	AN(lck);
 	CAST_OBJ_NOTNULL(ilck, lck->priv, ILCK_MAGIC);
 	return (ilck->held);
 }
@@ -199,18 +199,65 @@ Lck__Owned(const struct lock *lck)
 {
 	struct ilck *ilck;
 
+	AN(lck);
 	CAST_OBJ_NOTNULL(ilck, lck->priv, ILCK_MAGIC);
 	AN(ilck->held);
 	return (pthread_equal(ilck->owner, pthread_self()));
 }
 
 int v_matchproto_()
-Lck_CondWait(pthread_cond_t *cond, struct lock *lck, vtim_real when)
+Lck_CondWait(pthread_cond_t *cond, struct lock *lck)
+{
+       return (Lck_CondWaitUntil(cond, lck, 0));
+}
+
+int v_matchproto_()
+Lck_CondWaitTimeout(pthread_cond_t *cond, struct lock *lck, vtim_dur timeout)
+{
+	assert(timeout >= 0);
+	assert(timeout < 3600);
+
+	if (timeout == 0)
+		return (Lck_CondWaitUntil(cond, lck, 0));
+	else
+		return (Lck_CondWaitUntil(cond, lck, VTIM_real() + timeout));
+}
+
+int v_matchproto_()
+Lck_CondWaitUntil(pthread_cond_t *cond, struct lock *lck, vtim_real when)
 {
 	struct ilck *ilck;
 	struct timespec ts;
-	vtim_real t;
 
+#if defined (__APPLE__)
+	/*
+	 * I hate woo-doo programming in all it's forms and all it's
+	 * manifestations, but for reasons I utterly fail to isolate
+	 * yielding here is stops OSX from throwing a EINVAL to the
+	 * pthread_cond_wait(3) call.
+	 *
+	 * I have tried very hard to determine if any of the three
+	 * arguments are in fact invalid, and found nothing which
+	 * even hints that it might be the case, and with high probability
+	 * repeating the failed call with the exact same arguments
+	 * will succeed.
+	 *
+	 * If you want to dive into this you can trigger the situation
+	 * approx 30% of the time with:
+	 *
+	 *	cd .../vmods && make -j check
+	 *
+	 * Env:
+	 *	Darwin Kernel Version 20.5.0:
+	 *	Sat May  8 05:10:31 PDT 2021;
+	 *	root:xnu-7195.121.3~9/RELEASE_ARM64_T8101 arm64
+	 *
+	 * 20211027 /phk
+	 */
+	pthread_yield_np();
+#endif
+
+	AN(lck);
 	CAST_OBJ_NOTNULL(ilck, lck->priv, ILCK_MAGIC);
 	AN(ilck->held);
 	assert(pthread_equal(ilck->owner, pthread_self()));
@@ -220,23 +267,9 @@ Lck_CondWait(pthread_cond_t *cond, struct lock *lck, vtim_real when)
 		AZ(errno);
 	} else {
 		assert(when > 1e9);
-		ts.tv_nsec = (long)(modf(when, &t) * 1e9);
-		ts.tv_sec = (long)t;
+		ts = VTIM_timespec(when);
 		assert(ts.tv_nsec >= 0 && ts.tv_nsec <= 999999999);
 		errno = pthread_cond_timedwait(cond, &ilck->mtx, &ts);
-#if defined (__APPLE__)
-		if (errno == EINVAL && when <= VTIM_real()) {
-			/*
-			 * Most kernels treat this as honest error,
-			 * recognizing that a thread has no way to
-			 * prevent being descheduled between a user-
-			 * land check of the timestamp, and getting
-			 * the timestamp into the kernel before it
-			 * expires.  OS/X on the other hand...
-			 */
-			errno = ETIMEDOUT;
-		}
-#endif
 		assert(errno == 0 ||
 		    errno == ETIMEDOUT ||
 		    errno == EINTR);
@@ -254,13 +287,14 @@ Lck__New(struct lock *lck, struct VSC_lck *st, const char *w)
 
 	AN(st);
 	AN(w);
+	AN(lck);
 	AZ(lck->priv);
 	ALLOC_OBJ(ilck, ILCK_MAGIC);
 	AN(ilck);
 	ilck->w = w;
 	ilck->stat = st;
 	ilck->stat->creat++;
-	AZ(pthread_mutex_init(&ilck->mtx, &attr));
+	AZ(pthread_mutex_init(&ilck->mtx, &mtxattr_errorcheck));
 	lck->priv = ilck;
 }
 
@@ -269,9 +303,9 @@ Lck_Delete(struct lock *lck)
 {
 	struct ilck *ilck;
 
-	CAST_OBJ_NOTNULL(ilck, lck->priv, ILCK_MAGIC);
+	AN(lck);
+	TAKE_OBJ_NOTNULL(ilck, &lck->priv, ILCK_MAGIC);
 	ilck->stat->destroy++;
-	lck->priv = NULL;
 	AZ(pthread_mutex_destroy(&ilck->mtx));
 	FREE_OBJ(ilck);
 }
@@ -295,8 +329,6 @@ void
 LCK_Init(void)
 {
 
-	AZ(pthread_mutexattr_init(&attr));
-	AZ(pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK));
 #define LOCK(nam)	lck_##nam = Lck_CreateClass(NULL, #nam);
 #include "tbl/locks.h"
 }
