@@ -65,12 +65,9 @@
  *
  */
 
-#ifdef GCOVING
-    int __gcov_flush(void);
-#endif
-
 static struct vsb pan_vsb_storage, *pan_vsb;
 static pthread_mutex_t panicstr_mtx;
+static pthread_t panicy;
 
 static void pan_sess(struct vsb *, const struct sess *);
 static void pan_req(struct vsb *, const struct req *);
@@ -90,17 +87,14 @@ boc_state_2str(enum boc_state_e e)
 
 /*--------------------------------------------------------------------*/
 
-const char *
-sess_close_2str(enum sess_close sc, int want_desc)
+static void
+pan_stream_close(struct vsb *vsb, stream_close_t sc)
 {
-	switch (sc) {
-	case SC_NULL:		return (want_desc ? "(null)" : "NULL");
-#define SESS_CLOSE(nm, s, err, desc)			\
-	case SC_##nm: return(want_desc ? desc : #nm);
-#include "tbl/sess_close.h"
 
-	default:		return (want_desc ? "(invalid)" : "INVALID");
-	}
+	if (sc != NULL && sc->magic == STREAM_CLOSE_MAGIC)
+		VSB_printf(vsb, "%s(%s)", sc->name, sc->desc);
+	else
+		VSB_printf(vsb, "%p", sc);
 }
 
 /*--------------------------------------------------------------------*/
@@ -110,7 +104,7 @@ static const void *already_list[N_ALREADY];
 static int already_idx;
 
 int
-PAN_dump_struct2(struct vsb *vsb, int block, const void *ptr,
+PAN__DumpStruct(struct vsb *vsb, int block, int track, const void *ptr,
     const char *smagic, unsigned magic, const char *fmt, ...)
 {
 	va_list ap;
@@ -128,17 +122,19 @@ PAN_dump_struct2(struct vsb *vsb, int block, const void *ptr,
 	VSB_printf(vsb, " = %p {", ptr);
 	if (block)
 		VSB_putc(vsb, '\n');
-	for (i = 0; i < already_idx; i++) {
-		if (already_list[i] == ptr) {
-			VSB_cat(vsb, "  [Already dumped, see above]");
-			if (block)
-				VSB_putc(vsb, '\n');
-			VSB_cat(vsb, "},\n");
-			return (-2);
+	if (track) {
+		for (i = 0; i < already_idx; i++) {
+			if (already_list[i] == ptr) {
+				VSB_cat(vsb, "  [Already dumped, see above]");
+				if (block)
+					VSB_putc(vsb, '\n');
+				VSB_cat(vsb, "},\n");
+				return (-2);
+			}
 		}
+		if (already_idx < N_ALREADY)
+			already_list[already_idx++] = ptr;
 	}
-	if (already_idx < N_ALREADY)
-		already_list[already_idx++] = ptr;
 	uptr = ptr;
 	if (*uptr != magic) {
 		VSB_printf(vsb, "  .magic = 0x%08x", *uptr);
@@ -163,7 +159,9 @@ pan_htc(struct vsb *vsb, const struct http_conn *htc)
 		return;
 	if (htc->rfd != NULL)
 		VSB_printf(vsb, "fd = %d (@%p),\n", *htc->rfd, htc->rfd);
-	VSB_printf(vsb, "doclose = %s,\n", sess_close_2str(htc->doclose, 0));
+	VSB_printf(vsb, "doclose = ");
+	pan_stream_close(vsb, htc->doclose);
+	VSB_printf(vsb, "\n");
 	WS_Panic(vsb, htc->ws);
 	VSB_printf(vsb, "{rxbuf_b, rxbuf_e} = {%p, %p},\n",
 	    htc->rxbuf_b, htc->rxbuf_e);
@@ -603,12 +601,14 @@ pan_backtrace(struct vsb *vsb)
 	}
 	while (unw_step(&cursor) > 0) {
 		fname[0] = '\0';
-		ip = sp = 0;
-		unw_get_reg(&cursor, UNW_REG_IP, &ip);
-		unw_get_reg(&cursor, UNW_REG_SP, &sp);
-		unw_get_proc_name(&cursor, fname, sizeof(fname), &offp);
-		VSB_printf(vsb, "ip=0x%lx, sp=0x%lx <%s+0x%lx>\n", (long) ip,
-		    (long) sp, fname[0] ? fname : "<unknown>", (long)offp);
+		if (!unw_get_reg(&cursor, UNW_REG_IP, &ip))
+			VSB_printf(vsb, "ip=0x%lx", (long) ip);
+		if (!unw_get_reg(&cursor, UNW_REG_SP, &sp))
+			VSB_printf(vsb, " sp=0x%lx", (long) sp);
+		if (!unw_get_proc_name(&cursor, fname, sizeof(fname), &offp))
+			VSB_printf(vsb, " <%s+0x%lx>",
+			    fname[0] ? fname : "<unknown>", (long)offp);
+		VSB_putc(vsb, '\n');
 	}
 
 	VSB_indent(vsb, -2);
@@ -616,7 +616,11 @@ pan_backtrace(struct vsb *vsb)
 
 #else /* WITH_UNWIND */
 
-#define BACKTRACE_LEVELS	10
+#if ENABLE_SANITIZER
+#  define BACKTRACE_LEVELS	20
+#else
+#  define BACKTRACE_LEVELS	10
+#endif
 
 static void
 pan_backtrace(struct vsb *vsb)
@@ -712,12 +716,20 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 	const char *q;
 	struct req *req;
 	struct busyobj *bo;
+	struct worker *wrk;
 	struct sigaction sa;
 	int err = errno;
 
-	AZ(pthread_mutex_lock(&panicstr_mtx)); /* Won't be released,
-						  we're going to die
-						  anyway */
+	/* If we already panicing in another thread, do nothing */
+	while (heritage.panic_str[0] && panicy != pthread_self())
+		sleep(1);
+
+	if (pthread_mutex_lock(&panicstr_mtx)) {
+		/* Reentrant panic */
+		VSB_printf(pan_vsb,"\n\nPANIC REENTRANCY\n\n");
+		abort();
+	}
+	panicy = pthread_self();
 
 	/*
 	 * should we trigger a SIGSEGV while handling a panic, our sigsegv
@@ -769,7 +781,7 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 	pan_backtrace(pan_vsb);
 
 	if (err)
-		VSB_printf(pan_vsb, "errno = %d (%s)\n", err, vstrerror(err));
+		VSB_printf(pan_vsb, "errno = %d (%s)\n", err, VAS_errtxt(err));
 
 	pan_argv(pan_vsb);
 
@@ -794,6 +806,9 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 		pan_busyobj(pan_vsb, bo);
 		if (bo != NULL)
 			VSL_Flush(bo->vsl, 0);
+		wrk = THR_GetWorker();
+		VSB_cat(pan_vsb, "thr.");
+		pan_wrk(pan_vsb, wrk);
 		VMOD_Panic(pan_vsb);
 		pan_pool(pan_vsb);
 	} else {
@@ -802,9 +817,8 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 	VSB_cat(pan_vsb, "\n");
 	VSB_putc(pan_vsb, '\0');	/* NUL termination */
 
-#ifdef GCOVING
-	__gcov_flush();
-#endif
+	v_gcov_flush();
+	AZ(pthread_mutex_unlock(&panicstr_mtx));
 	abort();
 }
 
@@ -835,7 +849,7 @@ void
 PAN_Init(void)
 {
 
-	AZ(pthread_mutex_init(&panicstr_mtx, NULL));
+	AZ(pthread_mutex_init(&panicstr_mtx, &mtxattr_errorcheck));
 	VAS_Fail_Func = pan_ic;
 	pan_vsb = &pan_vsb_storage;
 	AN(heritage.panic_str);
