@@ -197,10 +197,10 @@ pool_poolherder(void *priv)
 				continue;
 			}
 		} else if (nwq > cache_param->wthread_pools &&
-				DO_DEBUG(DBG_DROP_POOLS)) {
+				EXPERIMENT(EXPERIMENT_DROP_POOLS)) {
 			Lck_Lock(&pool_mtx);
 			pp = VTAILQ_FIRST(&pools);
-			AN(pp);
+			CHECK_OBJ_NOTNULL(pp, POOL_MAGIC);
 			VTAILQ_REMOVE(&pools, pp, list);
 			VTAILQ_INSERT_TAIL(&pools, pp, list);
 			if (!pp->die)
@@ -218,6 +218,8 @@ pool_poolherder(void *priv)
 		ppx = NULL;
 		Lck_Lock(&pool_mtx);
 		VTAILQ_FOREACH(pp, &pools, list) {
+			CHECK_OBJ_NOTNULL(pp, POOL_MAGIC);
+
 			if (pp->die && pp->nthr == 0)
 				ppx = pp;
 			u += pp->lqueue;
@@ -239,6 +241,27 @@ pool_poolherder(void *priv)
 	NEEDLESS(return (NULL));
 }
 
+/*--------------------------------------------------------------------*/
+void
+pan_pool(struct vsb *vsb)
+{
+	struct pool *pp;
+
+	VSB_cat(vsb, "pools = {\n");
+	VSB_indent(vsb, 2);
+	VTAILQ_FOREACH(pp, &pools, list) {
+		if (PAN_dump_struct(vsb, pp, POOL_MAGIC, "pool"))
+			continue;
+		VSB_printf(vsb, "nidle = %u,\n", pp->nidle);
+		VSB_printf(vsb, "nthr = %u,\n", pp->nthr);
+		VSB_printf(vsb, "lqueue = %u\n", pp->lqueue);
+		VSB_indent(vsb, -2);
+		VSB_cat(vsb, "},\n");
+	}
+	VSB_indent(vsb, -2);
+	VSB_cat(vsb, "},\n");
+}
+
 /*--------------------------------------------------------------------*/
 
 void
