@@ -72,12 +72,39 @@ Req_AcctLogCharge(struct VSC_main_wrk *ds, struct req *req)
 	memset(a, 0, sizeof *a);
 }
 
+void
+Req_LogHit(struct worker *wrk, struct req *req, struct objcore *oc,
+    intmax_t fetch_progress)
+{
+	const char *clen, *sep;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+
+	if (fetch_progress >= 0) {
+		clen = HTTP_GetHdrPack(wrk, oc, H_Content_Length);
+		if (clen == NULL)
+			clen = sep = "";
+		else
+			sep = " ";
+		VSLb(req->vsl, SLT_Hit, "%u %.6f %.6f %.6f %jd%s%s",
+		    ObjGetXID(wrk, oc), EXP_Dttl(req, oc),
+		    oc->grace, oc->keep,
+		    fetch_progress, sep, clen);
+	} else {
+		VSLb(req->vsl, SLT_Hit, "%u %.6f %.6f %.6f",
+		    ObjGetXID(wrk, oc), EXP_Dttl(req, oc),
+		    oc->grace, oc->keep);
+	}
+}
+
 /*--------------------------------------------------------------------
  * Alloc/Free a request
  */
 
 struct req *
-Req_New(const struct worker *wrk, struct sess *sp)
+Req_New(struct sess *sp)
 {
 	struct pool *pp;
 	struct req *req;
@@ -85,7 +112,6 @@ Req_New(const struct worker *wrk, struct sess *sp)
 	unsigned sz, hl;
 	char *p, *e;
 
-	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	pp = sp->pool;
 	CHECK_OBJ_NOTNULL(pp, POOL_MAGIC);
@@ -127,15 +153,13 @@ Req_New(const struct worker *wrk, struct sess *sp)
 	INIT_OBJ(req->vfc, VFP_CTX_MAGIC);
 	p = (void*)PRNDUP(p + sizeof(*req->vfc));
 
-	req->htc = (void*)p;
-	p = (void*)PRNDUP(p + sizeof(*req->htc));
-
 	req->vdc = (void*)p;
 	memset(req->vdc, 0, sizeof *req->vdc);
 	p = (void*)PRNDUP(p + sizeof(*req->vdc));
 
 	req->htc = (void*)p;
 	INIT_OBJ(req->htc, HTTP_CONN_MAGIC);
+	req->htc->doclose = SC_NULL;
 	p = (void*)PRNDUP(p + sizeof(*req->htc));
 
 	req->top = (void*)p;
@@ -150,8 +174,8 @@ Req_New(const struct worker *wrk, struct sess *sp)
 	req->t_first = NAN;
 	req->t_prev = NAN;
 	req->t_req = NAN;
-
 	req->req_step = R_STP_TRANSPORT;
+	req->doclose = SC_NULL;
 
 	return (req);
 }
@@ -172,6 +196,9 @@ Req_Release(struct req *req)
 	AZ(req->vcl);
 	if (req->vsl->wid)
 		VSL_End(req->vsl);
+#ifdef ENABLE_WORKSPACE_EMULATOR
+	WS_Rollback(req->ws, 0);
+#endif
 	TAKE_OBJ_NOTNULL(sp, &req->sp, SESS_MAGIC);
 	pp = sp->pool;
 	CHECK_OBJ_NOTNULL(pp, POOL_MAGIC);
@@ -206,8 +233,10 @@ Req_Rollback(VRT_CTX)
 		VCL_TaskEnter(req->top->privs);
 	HTTP_Clone(req->http, req->http0);
 	req->filter_list = NULL;
+	req->vcf = NULL;
 	if (WS_Overflowed(req->ws))
 		req->wrk->stats->ws_client_overflow++;
+	AN(req->ws_req);
 	WS_Rollback(req->ws, req->ws_req);
 }
 
@@ -226,7 +255,7 @@ Req_Cleanup(struct sess *sp, struct worker *wrk, struct req *req)
 	if (IS_TOPREQ(req))
 		AZ(req->top->vcl0);
 
-	req->director_hint = NULL;
+	AZ(req->director_hint);
 	req->restarts = 0;
 
 	if (req->vcl != NULL)
@@ -253,21 +282,25 @@ Req_Cleanup(struct sess *sp, struct worker *wrk, struct req *req)
 
 	req->hash_always_miss = 0;
 	req->hash_ignore_busy = 0;
+	req->hash_ignore_vary = 0;
 	req->esi_level = 0;
 	req->is_hit = 0;
 	req->req_step = R_STP_TRANSPORT;
+	req->vcf = NULL;
+	req->doclose = SC_NULL;
+	req->htc->doclose = SC_NULL;
 
 	if (WS_Overflowed(req->ws))
 		wrk->stats->ws_client_overflow++;
 
-	WS_Rollback(req->ws, 0);
+	wrk->seen_methods = 0;
 }
 
 /*----------------------------------------------------------------------
  */
 
 void v_matchproto_(vtr_req_fail_f)
-Req_Fail(struct req *req, enum sess_close reason)
+Req_Fail(struct req *req, stream_close_t reason)
 {
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 
