@@ -50,14 +50,39 @@
 #include "storage/storage.h"
 #include "common/heritage.h"
 #include "vcl.h"
+#include "vct.h"
 #include "vsha256.h"
 #include "vtim.h"
 
+#define REQ_STEPS \
+  REQ_STEP(transport,           TRANSPORT,	) \
+  REQ_STEP(restart,             RESTART,	static) \
+  REQ_STEP(recv,                RECV,		) \
+  REQ_STEP(pipe,                PIPE,		static) \
+  REQ_STEP(pass,                PASS,		static) \
+  REQ_STEP(lookup,              LOOKUP,		static) \
+  REQ_STEP(purge,               PURGE,		static) \
+  REQ_STEP(miss,                MISS,		static) \
+  REQ_STEP(fetch,               FETCH,		static) \
+  REQ_STEP(deliver,             DELIVER,	static) \
+  REQ_STEP(vclfail,             VCLFAIL,	static) \
+  REQ_STEP(synth,               SYNTH,		static) \
+  REQ_STEP(transmit,            TRANSMIT,	static)
+
+#define REQ_STEP(l, U, priv) \
+    static req_state_f cnt_##l; \
+    priv const struct req_step R_STP_##U[1] = {{ \
+	.name = "Req Step " #l, \
+	.func = cnt_##l, \
+    }};
+REQ_STEPS
+#undef REQ_STEP
+
 /*--------------------------------------------------------------------
  * Handle "Expect:" and "Connection:" on incoming request
  */
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_transport(struct worker *wrk, struct req *req)
 {
 	const char *p;
@@ -69,7 +94,7 @@ cnt_transport(struct worker *wrk, struct req *req)
 	AN(req->req_body_status);
 
 	if (http_GetHdr(req->http, H_Expect, &p)) {
-		if (strcasecmp(p, "100-continue")) {
+		if (!http_expect_eq(p, 100-continue)) {
 			req->doclose = SC_RX_JUNK;
 			(void)req->transport->minimal_response(req, 417);
 			wrk->stats->client_req_417++;
@@ -83,8 +108,9 @@ cnt_transport(struct worker *wrk, struct req *req)
 
 	AZ(req->err_code);
 
-	req->doclose = http_DoConnection(req->http);
+	req->doclose = http_DoConnection(req->http, SC_REQ_CLOSE);
 	if (req->doclose == SC_RX_BAD) {
+		wrk->stats->client_req_400++;
 		(void)req->transport->minimal_response(req, 400);
 		return (REQ_FSM_DONE);
 	}
@@ -141,13 +167,13 @@ Resp_Setup_Deliver(struct req *req)
 	http_PrintfHeader(h, "Age: %.0f",
 	    floor(fmax(0., req->t_prev - oc->t_origin)));
 
-	http_SetHeader(h, "Via: 1.1 varnish (Varnish/6.5)");
+	http_SetHeader(h, "Via: 1.1 varnish (Varnish/7.1)");
 
 	if (cache_param->http_gzip_support &&
 	    ObjCheckFlag(req->wrk, oc, OF_GZIPED) &&
 	    !RFC2616_Req_Gzip(req->http))
 		RFC2616_Weaken_Etag(h);
-	return(0);
+	return (0);
 }
 
 void
@@ -177,9 +203,10 @@ Resp_Setup_Synth(struct req *req)
 		http_SetHeader(h, "Connection: close");
 }
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_deliver(struct worker *wrk, struct req *req)
 {
+	unsigned status;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
@@ -199,7 +226,14 @@ cnt_deliver(struct worker *wrk, struct req *req)
 		return (REQ_FSM_MORE);
 	}
 
+	status = http_GetStatus(req->resp);
+	if (cache_param->http_range_support && status == 200 &&
+	    !(req->objcore->flags & OC_F_PRIVATE))
+		http_ForceHeader(req->resp, H_Accept_Ranges, "bytes");
+
+	req->t_resp = W_TIM_real(wrk);
 	VCL_deliver_method(req->vcl, wrk, req, NULL, NULL);
+
 	VSLb_ts_req(req, "Process", W_TIM_real(wrk));
 
 	assert(req->restarts <= cache_param->max_restarts);
@@ -239,9 +273,10 @@ cnt_deliver(struct worker *wrk, struct req *req)
  * VCL failed, die horribly
  */
 
-static enum req_fsm_nxt
-cnt_vclfail(const struct worker *wrk, struct req *req)
+static enum req_fsm_nxt v_matchproto_(req_state_f)
+cnt_vclfail(struct worker *wrk, struct req *req)
 {
+	struct vrt_ctx ctx[1];
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
@@ -249,7 +284,10 @@ cnt_vclfail(const struct worker *wrk, struct req *req)
 	AZ(req->objcore);
 	AZ(req->stale_oc);
 
-	Req_Rollback(req);
+	INIT_OBJ(ctx, VRT_CTX_MAGIC);
+	VCL_Req2Ctx(ctx, req);
+
+	Req_Rollback(ctx);
 
 	req->err_code = 503;
 	req->err_reason = "VCL failed";
@@ -263,12 +301,13 @@ cnt_vclfail(const struct worker *wrk, struct req *req)
  * Emit a synthetic response
  */
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_synth(struct worker *wrk, struct req *req)
 {
 	struct vsb *synth_body;
 	ssize_t sz, szl;
 	uint8_t *ptr;
+	const char *body;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
@@ -287,6 +326,7 @@ cnt_synth(struct worker *wrk, struct req *req)
 	synth_body = VSB_new_auto();
 	AN(synth_body);
 
+	req->t_resp = W_TIM_real(wrk);
 	VCL_synth_method(req->vcl, wrk, req, NULL, synth_body);
 
 	AZ(VSB_finish(synth_body));
@@ -295,7 +335,9 @@ cnt_synth(struct worker *wrk, struct req *req)
 
 	if (wrk->handling == VCL_RET_FAIL) {
 		VSB_destroy(&synth_body);
-		req->doclose = SC_VCL_FAILURE;
+		(void)VRB_Ignore(req);
+		(void)req->transport->minimal_response(req, 500);
+		req->doclose = SC_VCL_FAILURE; // XXX: Not necessary any more ?
 		VSLb_ts_req(req, "Resp", W_TIM_real(wrk));
 		http_Teardown(req->resp);
 		return (REQ_FSM_DONE);
@@ -321,7 +363,8 @@ cnt_synth(struct worker *wrk, struct req *req)
 	http_PrintfHeader(req->resp, "Content-Length: %zd",
 	    VSB_len(synth_body));
 
-	if (!req->doclose && http_HdrIs(req->resp, H_Connection, "close"))
+	if (req->doclose == SC_NULL &&
+	    http_HdrIs(req->resp, H_Connection, "close"))
 		req->doclose = SC_RESP_CLOSE;
 
 	/* Discard any lingering request body before delivery */
@@ -331,20 +374,26 @@ cnt_synth(struct worker *wrk, struct req *req)
 	CHECK_OBJ_NOTNULL(req->objcore, OBJCORE_MAGIC);
 	szl = -1;
 	if (STV_NewObject(wrk, req->objcore, stv_transient, 1024)) {
+		body = VSB_data(synth_body);
 		szl = VSB_len(synth_body);
 		assert(szl >= 0);
-		sz = szl;
-		if (sz > 0 &&
-		    ObjGetSpace(wrk, req->objcore, &sz, &ptr) && sz >= szl) {
-			memcpy(ptr, VSB_data(synth_body), szl);
-			ObjExtend(wrk, req->objcore, szl);
-		} else if (sz > 0) {
-			szl = -1;
+		while (szl > 0) {
+			sz = szl;
+			if (! ObjGetSpace(wrk, req->objcore, &sz, &ptr)) {
+				szl = -1;
+				break;
+			}
+			if (sz > szl)
+				sz = szl;
+			szl -= sz;
+			memcpy(ptr, body, sz);
+			ObjExtend(wrk, req->objcore, sz, szl == 0 ? 1 : 0);
+			body += sz;
 		}
 	}
 
 	if (szl >= 0)
-		AZ(ObjSetU64(wrk, req->objcore, OA_LEN, szl));
+		AZ(ObjSetU64(wrk, req->objcore, OA_LEN, VSB_len(synth_body)));
 	HSH_DerefBoc(wrk, req->objcore);
 	VSB_destroy(&synth_body);
 
@@ -365,7 +414,7 @@ cnt_synth(struct worker *wrk, struct req *req)
  * The mechanics of sending a response (from deliver or synth)
  */
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_transmit(struct worker *wrk, struct req *req)
 {
 	struct boc *boc;
@@ -387,9 +436,9 @@ cnt_transmit(struct worker *wrk, struct req *req)
 	clval = http_GetContentLength(req->resp);
 	/* RFC 7230, 3.3.3 */
 	status = http_GetStatus(req->resp);
-	head = !strcmp(req->http0->hd[HTTP_HDR_METHOD].b, "HEAD");
+	head = http_method_eq(req->http0->hd[HTTP_HDR_METHOD].b, HEAD);
 
-	if (boc != NULL)
+	if (boc != NULL || (req->objcore->flags & (OC_F_FAILED)))
 		req->resp_len = clval;
 	else
 		req->resp_len = ObjGetLen(req->wrk, req->objcore);
@@ -401,6 +450,7 @@ cnt_transmit(struct worker *wrk, struct req *req)
 		sendbody = 1;
 	}
 
+	VDP_Init(req->vdc, req->wrk, req->vsl, req);
 	if (req->filter_list == NULL)
 		req->filter_list = resp_Get_Filter_List(req);
 	if (req->filter_list == NULL ||
@@ -408,9 +458,6 @@ cnt_transmit(struct worker *wrk, struct req *req)
 		VSLb(req->vsl, SLT_Error, "Failure to push processors");
 		req->doclose = SC_OVERLOAD;
 	} else {
-		if (cache_param->http_range_support && status == 200)
-			http_ForceHeader(req->resp, H_Accept_Ranges, "bytes");
-
 		if (status < 200 || status == 204) {
 			// rfc7230,l,1691,1695
 			http_Unset(req->resp, H_Content_Length);
@@ -436,13 +483,19 @@ cnt_transmit(struct worker *wrk, struct req *req)
 		}
 		if (req->resp_len == 0)
 			sendbody = 0;
+		req->transport->deliver(req, boc, sendbody);
 	}
-	req->transport->deliver(req, boc, sendbody);
 
 	VSLb_ts_req(req, "Resp", W_TIM_real(wrk));
 
 	HSH_Cancel(wrk, req->objcore, boc);
 
+	if (req->doclose == SC_NULL && (req->objcore->flags & OC_F_FAILED)) {
+		/* The object we delivered failed due to a streaming error.
+		 * Fail the request. */
+		req->doclose = SC_TX_ERROR;
+	}
+
 	if (boc != NULL)
 		HSH_DerefBoc(wrk, req->objcore);
 
@@ -458,7 +511,7 @@ cnt_transmit(struct worker *wrk, struct req *req)
  * Initiated a fetch (pass/miss) which we intend to deliver
  */
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_fetch(struct worker *wrk, struct req *req)
 {
 
@@ -487,7 +540,7 @@ cnt_fetch(struct worker *wrk, struct req *req)
  * this state if we get suspended on a busy objhdr.
  */
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_lookup(struct worker *wrk, struct req *req)
 {
 	struct objcore *oc, *busy;
@@ -506,6 +559,7 @@ cnt_lookup(struct worker *wrk, struct req *req)
 	AZ(req->objcore);
 	if (req->hash_objhead)
 		had_objhead = 1;
+	wrk->strangelove = 0;
 	lr = HSH_Lookup(req, &oc, &busy);
 	if (lr == HSH_BUSY) {
 		/*
@@ -516,6 +570,10 @@ cnt_lookup(struct worker *wrk, struct req *req)
 		 */
 		return (REQ_FSM_DISEMBARK);
 	}
+	assert(wrk->strangelove >= 0);
+	if ((unsigned)wrk->strangelove >= cache_param->vary_notice)
+		VSLb(req->vsl, SLT_Notice, "vsl: High number of variants (%d)",
+		    wrk->strangelove);
 	if (had_objhead)
 		VSLb_ts_req(req, "Waitinglist", W_TIM_real(wrk));
 
@@ -557,12 +615,6 @@ cnt_lookup(struct worker *wrk, struct req *req)
 	req->objcore = oc;
 	AZ(oc->flags & OC_F_HFM);
 
-	VSLb(req->vsl, SLT_Hit, "%u %.6f %.6f %.6f",
-	    ObjGetXID(wrk, req->objcore),
-	    EXP_Dttl(req, req->objcore),
-	    req->objcore->grace,
-	    req->objcore->keep);
-
 	VCL_hit_method(req->vcl, wrk, req, NULL, NULL);
 
 	switch (wrk->handling) {
@@ -572,6 +624,8 @@ cnt_lookup(struct worker *wrk, struct req *req)
 			CHECK_OBJ_NOTNULL(busy->boc, BOC_MAGIC);
 			// XXX: shouldn't we go to miss?
 			VBF_Fetch(wrk, req, busy, oc, VBF_BACKGROUND);
+			wrk->stats->s_fetch++;
+			wrk->stats->s_bgfetch++;
 		} else {
 			(void)VRB_Ignore(req);// XXX: handle err
 		}
@@ -614,7 +668,7 @@ cnt_lookup(struct worker *wrk, struct req *req)
  * Cache miss.
  */
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_miss(struct worker *wrk, struct req *req)
 {
 
@@ -659,7 +713,7 @@ cnt_miss(struct worker *wrk, struct req *req)
  * Pass processing
  */
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_pass(struct worker *wrk, struct req *req)
 {
 
@@ -697,7 +751,7 @@ cnt_pass(struct worker *wrk, struct req *req)
  * Pipe mode
  */
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_pipe(struct worker *wrk, struct req *req)
 {
 	struct busyobj *bo;
@@ -714,11 +768,12 @@ cnt_pipe(struct worker *wrk, struct req *req)
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 	VSLb(bo->vsl, SLT_Begin, "bereq %u pipe", VXID(req->vsl->wid));
 	VSLb(req->vsl, SLT_Link, "bereq %u pipe", VXID(bo->vsl->wid));
+	VSLb_ts_busyobj(bo, "Start", W_TIM_real(wrk));
 	THR_SetBusyobj(bo);
 	bo->sp = req->sp;
 	SES_Ref(bo->sp);
 
-	HTTP_Setup(bo->bereq, bo->ws, bo->vsl, SLT_BereqMethod);
+	HTTP_Setup(bo->bereq, req->ws, bo->vsl, SLT_BereqMethod);
 	http_FilterReq(bo->bereq, req->http, 0);	// XXX: 0 ?
 	http_PrintfHeader(bo->bereq, "X-Varnish: %u", VXID(req->vsl->wid));
 	http_ForceHeader(bo->bereq, H_Connection, "close");
@@ -729,7 +784,10 @@ cnt_pipe(struct worker *wrk, struct req *req)
 	}
 
 	bo->wrk = wrk;
-	VCL_pipe_method(req->vcl, wrk, req, bo, NULL);
+	if (WS_Overflowed(req->ws))
+		wrk->handling = VCL_RET_FAIL;
+	else
+		VCL_pipe_method(req->vcl, wrk, req, bo, NULL);
 
 	switch (wrk->handling) {
 	case VCL_RET_SYNTH:
@@ -737,6 +795,8 @@ cnt_pipe(struct worker *wrk, struct req *req)
 		nxt = REQ_FSM_MORE;
 		break;
 	case VCL_RET_PIPE:
+		VSLb_ts_req(req, "Process", W_TIM_real(wrk));
+		VSLb_ts_busyobj(bo, "Process", wrk->lastused);
 		if (V1P_Enter() == 0) {
 			AZ(bo->req);
 			bo->req = req;
@@ -769,7 +829,7 @@ cnt_pipe(struct worker *wrk, struct req *req)
  * Handle restart events
  */
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_restart(struct worker *wrk, struct req *req)
 {
 
@@ -803,7 +863,7 @@ cnt_restart(struct worker *wrk, struct req *req)
  * - remove duplicatation with Req_Cleanup()
  */
 
-static void
+static void v_matchproto_(req_state_f)
 cnt_recv_prep(struct req *req, const char *ci)
 {
 	const char *xff;
@@ -824,18 +884,19 @@ cnt_recv_prep(struct req *req, const char *ci)
 		http_CollectHdr(req->http, H_Cache_Control);
 
 		/* By default we use the first backend */
-		req->director_hint = VCL_DefaultDirector(req->vcl);
+		VRT_Assign_Backend(&req->director_hint,
+		    VCL_DefaultDirector(req->vcl));
 
 		req->d_ttl = -1;
 		req->d_grace = -1;
 		req->disable_esi = 0;
 		req->hash_always_miss = 0;
 		req->hash_ignore_busy = 0;
+		req->hash_ignore_vary = 0;
 		req->client_identity = NULL;
 		req->storage = NULL;
 	}
 
-	req->vdc->retval = 0;
 	req->is_hit = 0;
 	req->is_hitmiss = 0;
 	req->is_hitpass = 0;
@@ -849,7 +910,7 @@ cnt_recv_prep(struct req *req, const char *ci)
  * a interior request during ESI delivery.
  */
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_recv(struct worker *wrk, struct req *req)
 {
 	unsigned recv_handling;
@@ -876,6 +937,20 @@ cnt_recv(struct worker *wrk, struct req *req)
 
 	http_VSL_log(req->http);
 
+	if (http_CountHdr(req->http0, H_Host) > 1) {
+		VSLb(req->vsl, SLT_BogoHeader, "Multiple Host: headers");
+		wrk->stats->client_req_400++;
+		(void)req->transport->minimal_response(req, 400);
+		return (REQ_FSM_DONE);
+	}
+
+	if (http_CountHdr(req->http0, H_Content_Length) > 1) {
+		VSLb(req->vsl, SLT_BogoHeader, "Multiple Content-Length: headers");
+		wrk->stats->client_req_400++;
+		(void)req->transport->minimal_response(req, 400);
+		return (REQ_FSM_DONE);
+	}
+
 	cnt_recv_prep(req, ci);
 
 	if (req->req_body_status == BS_ERROR) {
@@ -986,7 +1061,7 @@ cnt_recv(struct worker *wrk, struct req *req)
  * In VCL, a restart is necessary to get a new object
  */
 
-static enum req_fsm_nxt
+static enum req_fsm_nxt v_matchproto_(req_state_f)
 cnt_purge(struct worker *wrk, struct req *req)
 {
 	struct objcore *oc, *boc;
@@ -1037,7 +1112,7 @@ cnt_purge(struct worker *wrk, struct req *req)
  *
  */
 
-static void
+static void v_matchproto_(req_state_f)
 cnt_diag(struct req *req, const char *state)
 {
 
@@ -1059,9 +1134,9 @@ CNT_Embark(struct worker *wrk, struct req *req)
 	req->vfc->wrk = req->wrk = wrk;
 	wrk->vsl = req->vsl;
 	if (req->req_step == R_STP_TRANSPORT && req->vcl == NULL) {
-		VCL_Refresh(&wrk->vcl);
-		req->vcl = wrk->vcl;
-		wrk->vcl = NULL;
+		VCL_Refresh(&wrk->wpriv->vcl);
+		req->vcl = wrk->wpriv->vcl;
+		wrk->wpriv->vcl = NULL;
 		VSLb(req->vsl, SLT_VCL_use, "%s", VCL_Name(req->vcl));
 	}
 
@@ -1071,6 +1146,7 @@ CNT_Embark(struct worker *wrk, struct req *req)
 enum req_fsm_nxt
 CNT_Request(struct req *req)
 {
+	struct vrt_ctx ctx[1];
 	struct worker *wrk;
 	enum req_fsm_nxt nxt;
 
@@ -1099,32 +1175,32 @@ CNT_Request(struct req *req)
 		 * pointers still pointing to the things we expect.
 		 */
 		CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
-		CHECK_OBJ_ORNULL(wrk->nobjhead, OBJHEAD_MAGIC);
+		CHECK_OBJ_NOTNULL(wrk->wpriv, WORKER_PRIV_MAGIC);
+		CHECK_OBJ_ORNULL(wrk->wpriv->nobjhead, OBJHEAD_MAGIC);
 		CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+		CHECK_OBJ_NOTNULL(req->doclose, STREAM_CLOSE_MAGIC);
 
-		switch (req->req_step) {
-#define REQ_STEP(l,u,arg) \
-		    case R_STP_##u: \
-			if (DO_DEBUG(DBG_REQ_STATE)) \
-				cnt_diag(req, #u); \
-			nxt = cnt_##l arg; \
-			break;
-#include "tbl/steps.h"
-		default:
-			WRONG("State engine misfire");
-		}
-		CHECK_OBJ_ORNULL(wrk->nobjhead, OBJHEAD_MAGIC);
+		AN(req->req_step);
+		AN(req->req_step->name);
+		AN(req->req_step->func);
+		if (DO_DEBUG(DBG_REQ_STATE))
+			cnt_diag(req, req->req_step->name);
+		nxt = req->req_step->func(wrk, req);
+		CHECK_OBJ_ORNULL(wrk->wpriv->nobjhead, OBJHEAD_MAGIC);
 	}
 	wrk->vsl = NULL;
 	if (nxt == REQ_FSM_DONE) {
+		INIT_OBJ(ctx, VRT_CTX_MAGIC);
+		VCL_Req2Ctx(ctx, req);
 		if (IS_TOPREQ(req)) {
-			VCL_TaskLeave(req->top->privs);
+			VCL_TaskLeave(ctx, req->top->privs);
 			if (req->top->vcl0 != NULL)
-				VCL_Rel(&req->top->vcl0);
+				VCL_Recache(wrk, &req->top->vcl0);
 		}
-		VCL_TaskLeave(req->privs);
+		VCL_TaskLeave(ctx, req->privs);
 		AN(req->vsl->wid);
 		VRB_Free(req);
+		VRT_Assign_Backend(&req->director_hint, NULL);
 		req->wrk = NULL;
 	}
 	assert(nxt == REQ_FSM_DISEMBARK || !WS_IsReserved(req->ws));
