@@ -80,9 +80,7 @@ strands_cat(char *buf, unsigned bufl, const struct strands *s)
 	for (i = 0; i < s->n && bufl > 0; i++) {
 		if (s->p[i] == NULL || *s->p[i] == '\0')
 			continue;
-		ll = strlen(s->p[i]);
-		if (ll > bufl)
-			ll = bufl;
+		ll = vmin_t(unsigned, strlen(s->p[i]), bufl);
 		memcpy(buf, s->p[i], ll);
 		l += ll;
 		buf += ll;
@@ -272,8 +270,7 @@ VSLv(enum VSL_tag_e tag, uint32_t vxid, const char *fmt, va_list ap)
 		vslr(tag, vxid, fmt, strlen(fmt) + 1);
 	} else {
 		n = vsnprintf(buf, mlen, fmt, ap);
-		if (n > mlen - 1)
-			n = mlen - 1;
+		n = vmin_t(unsigned, n, mlen - 1);
 		buf[n++] = '\0'; /* NUL-terminated */
 		vslr(tag, vxid, buf, n);
 	}
@@ -350,17 +347,17 @@ VSLbt(struct vsl_log *vsl, enum VSL_tag_e tag, txt t)
 	if (l > mlen - 1)
 		l = mlen - 1;
 
-	assert(vsl->wlp < vsl->wle);
+	assert(vsl->wlp <= vsl->wle);
 
 	/* Flush if necessary */
-	if (VSL_END(vsl->wlp, l + 1) >= vsl->wle)
+	if (VSL_END(vsl->wlp, l + 1) > vsl->wle)
 		VSL_Flush(vsl, 1);
-	assert(VSL_END(vsl->wlp, l + 1) < vsl->wle);
+	assert(VSL_END(vsl->wlp, l + 1) <= vsl->wle);
 	p = VSL_DATA(vsl->wlp);
 	memcpy(p, t.b, l);
 	p[l++] = '\0';		/* NUL-terminated */
 	vsl->wlp = vsl_hdr(tag, vsl->wlp, l, vsl->wid);
-	assert(vsl->wlp < vsl->wle);
+	assert(vsl->wlp <= vsl->wle);
 	vsl->wlr++;
 
 	if (DO_DEBUG(DBG_SYNCVSL))
@@ -381,9 +378,7 @@ VSLbs(struct vsl_log *vsl, enum VSL_tag_e tag, const struct strands *s)
 	mlen = cache_param->vsl_reclen;
 
 	/* including NUL */
-	l = strands_len(s) + 1;
-	if (l > mlen)
-		l = mlen;
+	l = vmin_t(unsigned, strands_len(s) + 1, mlen);
 
 	assert(vsl->wlp < vsl->wle);
 
@@ -414,6 +409,8 @@ VSLbv(struct vsl_log *vsl, enum VSL_tag_e tag, const char *fmt, va_list ap)
 	const char *u, *f;
 	unsigned n, mlen;
 	txt t;
+	va_list ap2;
+	int first;
 
 	vsl_sanity(vsl);
 	AN(fmt);
@@ -442,19 +439,43 @@ VSLbv(struct vsl_log *vsl, enum VSL_tag_e tag, const char *fmt, va_list ap)
 		return;
 	}
 
-	mlen = cache_param->vsl_reclen;
+	assert(vsl->wlp <= vsl->wle);
 
-	/* Flush if we cannot fit a full size record */
-	if (VSL_END(vsl->wlp, mlen + 1) >= vsl->wle)
+	/* Flush if we can't fit any bytes */
+	if (vsl->wle - vsl->wlp <= VSL_OVERHEAD)
 		VSL_Flush(vsl, 1);
 
-	p = VSL_DATA(vsl->wlp);
-	n = vsnprintf(p, mlen, fmt, ap);
+	/* Do the vsnprintf formatting in one or two stages. If the first
+	   stage shows that we overflowed, and the available space to work
+	   with was less than vsl_reclen, flush and do the formatting
+	   again. */
+	first = 1;
+	while (1) {
+		assert(vsl->wle - vsl->wlp > VSL_OVERHEAD);
+		mlen = VSL_BYTES((vsl->wle - vsl->wlp) - VSL_OVERHEAD);
+		if (mlen > cache_param->vsl_reclen)
+			mlen = cache_param->vsl_reclen;
+		assert(mlen > 0);
+		assert(VSL_END(vsl->wlp, mlen) <= vsl->wle);
+		p = VSL_DATA(vsl->wlp);
+		va_copy(ap2, ap);
+		n = vsnprintf(p, mlen, fmt, ap2);
+		va_end(ap2);
+
+		if (first && n >= mlen && mlen < cache_param->vsl_reclen) {
+			first = 0;
+			VSL_Flush(vsl, 1);
+			continue;
+		}
+
+		break;
+	}
+
 	if (n > mlen - 1)
 		n = mlen - 1;	/* we truncate long fields */
 	p[n++] = '\0';		/* NUL-terminated */
 	vsl->wlp = vsl_hdr(tag, vsl->wlp, n, vsl->wid);
-	assert(vsl->wlp < vsl->wle);
+	assert(vsl->wlp <= vsl->wle);
 	vsl->wlr++;
 
 	if (DO_DEBUG(DBG_SYNCVSL))
@@ -508,19 +529,18 @@ VSLb_bin(struct vsl_log *vsl, enum VSL_tag_e tag, ssize_t len, const void *ptr)
 	mlen = cache_param->vsl_reclen;
 
 	/* Truncate */
-	if (len > mlen)
-		len = mlen;
+	len = vmin_t(ssize_t, len, mlen);
 
-	assert(vsl->wlp < vsl->wle);
+	assert(vsl->wlp <= vsl->wle);
 
 	/* Flush if necessary */
-	if (VSL_END(vsl->wlp, len) >= vsl->wle)
+	if (VSL_END(vsl->wlp, len) > vsl->wle)
 		VSL_Flush(vsl, 1);
-	assert(VSL_END(vsl->wlp, len) < vsl->wle);
+	assert(VSL_END(vsl->wlp, len) <= vsl->wle);
 	p = VSL_DATA(vsl->wlp);
 	memcpy(p, ptr, len);
 	vsl->wlp = vsl_hdr(tag, vsl->wlp, len, vsl->wid);
-	assert(vsl->wlp < vsl->wle);
+	assert(vsl->wlp <= vsl->wle);
 	vsl->wlr++;
 
 	if (DO_DEBUG(DBG_SYNCVSL))
@@ -614,9 +634,9 @@ VSM_Init(void)
 
 	assert(UINT_MAX % VSL_SEGMENTS == VSL_SEGMENTS - 1);
 
-	AZ(pthread_mutex_init(&vsl_mtx, NULL));
-	AZ(pthread_mutex_init(&vsc_mtx, NULL));
-	AZ(pthread_mutex_init(&vsm_mtx, NULL));
+	AZ(pthread_mutex_init(&vsl_mtx, &mtxattr_errorcheck));
+	AZ(pthread_mutex_init(&vsc_mtx, &mtxattr_errorcheck));
+	AZ(pthread_mutex_init(&vsm_mtx, &mtxattr_errorcheck));
 
 	vsc_lock = vsm_vsc_lock;
 	vsc_unlock = vsm_vsc_unlock;
