@@ -54,8 +54,16 @@
 #include "common/vsmw.h"
 #include "proxy/cache_proxy.h"
 
-const void * const vrt_magic_string_end = &vrt_magic_string_end;
-const void * const vrt_magic_string_unset = &vrt_magic_string_unset;
+// NOT using TOSTRANDS() to create a NULL pointer element despite n == 0
+const struct strands *vrt_null_strands = &(struct strands){
+	.n = 0,
+	.p = (const char *[1]){NULL}
+};
+const struct vrt_blob *vrt_null_blob = &(struct vrt_blob){
+	.type = VRT_NULL_BLOB_TYPE,
+	.len = 0,
+	.blob = "\0"
+};
 
 /*--------------------------------------------------------------------*/
 
@@ -85,7 +93,7 @@ VRT_synth(VRT_CTX, VCL_INT code, VCL_STRING reason)
 		return;
 	}
 
-	if (reason && !WS_Inside(ctx->ws, reason, NULL)) {
+	if (reason && !WS_Allocated(ctx->ws, reason, -1)) {
 		reason = WS_Copy(ctx->ws, reason, -1);
 		if (!reason) {
 			VRT_fail(ctx, "Workspace overflow");
@@ -125,15 +133,84 @@ VRT_acl_match(VRT_CTX, VCL_ACL acl, VCL_IP ip)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(acl, VRT_ACL_MAGIC);
-	if (ip == NULL) {
-		VRT_fail(ctx, "Cannot match a null IP address");
+	if (acl == NULL || ip == NULL) {
+		VRT_fail(ctx, "Cannot match a null %s",
+			 acl == NULL ? "ACL" : "IP address");
 		return (0);
 	}
+	CHECK_OBJ(acl, VRT_ACL_MAGIC);
 	assert(VSA_Sane(ip));
 	return (acl->match(ctx, ip));
 }
 
+static int
+acl_tbl_cmp(int fam, const uint8_t *key, const uint8_t *b)
+{
+	int rv;
+
+	rv = fam - (int)b[3];
+	if (rv == 0 && b[1] > 0)
+		rv = memcmp(key, b + 4, b[1]);
+	if (rv == 0 && b[2])
+		rv = (int)(key[b[1]] & b[2]) - (int)b[4 + b[1]];
+	return (rv);
+}
+
+static const uint8_t *
+bbsearch(int fam, const uint8_t *key, const uint8_t *base0,
+    size_t nmemb, size_t size)
+{
+	const uint8_t *base = base0;
+	size_t lim;
+	int cmp;
+	const uint8_t *p;
+
+	for (lim = nmemb; lim != 0; lim >>= 1) {
+		p = base + (lim >> 1) * size;
+		cmp = acl_tbl_cmp(fam, key, p);
+		if (cmp == 0)
+			return (p);
+		if (cmp > 0) {
+			/* key > p: move right */
+			base = p + size;
+			lim--;
+		} /* else move left */
+	}
+	return (NULL);
+}
+
+int
+VPI_acl_table(VRT_CTX, VCL_IP p, unsigned n, unsigned m, const uint8_t *tbl,
+    const char * const *str, const char *fin)
+{
+	int fam;
+	const uint8_t *key;
+	const uint8_t *ptr;
+	size_t sz;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(p);
+	AN(n);
+	assert(m == 20);
+	AN(tbl);
+	AN(fin);
+
+	fam = VRT_VSA_GetPtr(ctx, p, &key);
+	ptr = bbsearch(fam, key, tbl, n, m);
+
+	if (ptr != NULL) {
+		sz = ptr - tbl;
+		AZ(sz % m);
+		sz /= m;
+		if (str != NULL)
+			VPI_acl_log(ctx, str[sz]);
+		return (*ptr);
+	}
+	if (str != NULL)
+		VPI_acl_log(ctx, fin);
+	return (0);
+}
+
 /*--------------------------------------------------------------------*/
 
 VCL_VOID
@@ -180,6 +257,9 @@ VRT_selecthttp(VRT_CTX, enum gethdr_e where)
 	case HDR_RESP:
 		hp = ctx->http_resp;
 		break;
+	case HDR_OBJ:
+		hp = NULL;
+		break;
 	default:
 		WRONG("VRT_selecthttp 'where' invalid");
 	}
@@ -315,126 +395,6 @@ VRT_HashStrands32(VCL_STRANDS s)
 	return (vle32dec(sha256 + VSHA256_LEN - 4));
 }
 
-/*--------------------------------------------------------------------
- * Collapse a STRING_LIST in the space provided, or return NULL
- */
-
-char *
-VRT_StringList(char *d, unsigned dl, const char *p, va_list ap)
-{
-	char *b, *e;
-	unsigned x;
-
-	b = d;
-	e = b + dl;
-	while (p != vrt_magic_string_end && b < e) {
-		if (p != NULL && *p != '\0') {
-			x = strlen(p);
-			if (b + x < e)
-				memcpy(b, p, x);
-			b += x;
-		}
-		p = va_arg(ap, const char *);
-	}
-	if (b >= e)
-		return (NULL);
-	*b++ = '\0';
-	return (b);
-}
-
-/*--------------------------------------------------------------------
- * Copy and merge a STRING_LIST into a workspace.
- */
-
-const char *
-VRT_String(struct ws *ws, const char *h, const char *p, va_list ap)
-{
-	char *b, *e;
-	const char *q;
-	unsigned u, x;
-	va_list aq;
-
-	u = WS_ReserveAll(ws);
-	e = b = WS_Reservation(ws);
-	e += u;
-
-	va_copy(aq, ap);
-	do
-		q = va_arg(aq, const char *);
-	while (q == NULL || (q != vrt_magic_string_end && *q == '\0'));
-
-	if (h != NULL && p == NULL && q == vrt_magic_string_end &&
-	    WS_Inside(ws, h, NULL)) {
-		va_end(aq);
-		WS_Release(ws, 0);
-		return (h);
-	}
-
-	if (h == NULL && p != NULL && q == vrt_magic_string_end &&
-	    WS_Inside(ws, p, NULL)) {
-		va_end(aq);
-		WS_Release(ws, 0);
-		return (p);
-	}
-
-	if (h == NULL && p == NULL) {
-		if (q == vrt_magic_string_end) {
-			va_end(aq);
-			WS_Release(ws, 0);
-			return ("");
-		}
-		do
-			p = va_arg(aq, const char *);
-		while (p == NULL || (p != vrt_magic_string_end && *p == '\0'));
-		if (p == vrt_magic_string_end && WS_Inside(ws, q, NULL)) {
-			va_end(aq);
-			WS_Release(ws, 0);
-			return (q);
-		}
-		p = NULL;
-		va_end(aq);
-	}
-
-	if (h != NULL) {
-		x = strlen(h);
-		if (b + x < e)
-			memcpy(b, h, x);
-		b += x;
-		if (b < e)
-			*b = ' ';
-		b++;
-	}
-	b = VRT_StringList(b, e > b ? e - b : 0, p, ap);
-	if (b == NULL || b == e) {
-		WS_MarkOverflow(ws);
-		WS_Release(ws, 0);
-		return (NULL);
-	}
-	e = b;
-	b = WS_Reservation(ws);
-	WS_Release(ws, e - b);
-	return (b);
-}
-
-/*--------------------------------------------------------------------
- * Copy and merge a STRING_LIST on the current workspace
- */
-
-VCL_STRING
-VRT_CollectString(VRT_CTX, const char *p, ...)
-{
-	va_list ap;
-	const char *b;
-
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->ws, WS_MAGIC);
-	va_start(ap, p);
-	b = VRT_String(ctx->ws, NULL, p, ap);
-	va_end(ap);
-	if (b == NULL)
-		VRT_fail(ctx, "Workspace overflow");
-	return (b);
-}
 
 /*--------------------------------------------------------------------
  * Collapse STRANDS into the space provided, or return NULL
@@ -488,9 +448,9 @@ VRT_StrandsWS(struct ws *ws, const char *h, VCL_STRANDS s)
 	if (q == NULL) {
 		if (h == NULL)
 			return ("");
-		if (WS_Inside(ws, h, NULL))
+		if (WS_Allocated(ws, h, -1))
 			return (h);
-	} else if (h == NULL && WS_Inside(ws, q, NULL)) {
+	} else if (h == NULL && WS_Allocated(ws, q, -1)) {
 		for (i++; i < s->n; i++)
 			if (s->p[i] != NULL && *s->p[i] != '\0')
 				break;
@@ -513,7 +473,7 @@ VRT_StrandsWS(struct ws *ws, const char *h, VCL_STRANDS s)
  */
 
 VCL_STRING
-VRT_CollectStrands(VRT_CTX, VCL_STRANDS s)
+VRT_STRANDS_string(VRT_CTX, VCL_STRANDS s)
 {
 	const char *b;
 
@@ -611,34 +571,67 @@ VRT_ValidHdr(VRT_CTX, VCL_STRANDS s)
 /*--------------------------------------------------------------------*/
 
 VCL_VOID
-VRT_SetHdr(VRT_CTX , VCL_HEADER hs, const char *p, ...)
+VRT_UnsetHdr(VRT_CTX, VCL_HEADER hs)
 {
 	VCL_HTTP hp;
-	va_list ap;
-	const char *b;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	AN(hs);
 	AN(hs->what);
 	hp = VRT_selecthttp(ctx, hs->where);
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
-	if (p == vrt_magic_string_unset) {
-		http_Unset(hp, hs->what);
-	} else {
-		va_start(ap, p);
-		b = VRT_String(hp->ws, hs->what + 1, p, ap);
-		va_end(ap);
-		if (b == NULL) {
+	http_Unset(hp, hs->what);
+}
+
+VCL_VOID
+VRT_SetHdr(VRT_CTX, VCL_HEADER hs, const char *pfx, VCL_STRANDS s)
+{
+	VCL_HTTP hp;
+	unsigned u, l, pl;
+	char *p, *b;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(hs);
+	AN(hs->what);
+	hp = VRT_selecthttp(ctx, hs->where);
+	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
+
+	u = WS_ReserveAll(hp->ws);
+	pl = (pfx == NULL) ? 0 : strlen(pfx);
+	l = hs->what[0] + 1 + pl;
+	if (u <= l) {
+		WS_Release(hp->ws, 0);
+		WS_MarkOverflow(hp->ws);
+		VSLb(ctx->vsl, SLT_LostHeader, "%s", hs->what + 1);
+		return;
+	}
+	b = WS_Reservation(hp->ws);
+	if (s != NULL) {
+		p = VRT_Strands(b + l, u - l, s);
+		if (p == NULL) {
+			WS_Release(hp->ws, 0);
+			WS_MarkOverflow(hp->ws);
 			VSLb(ctx->vsl, SLT_LostHeader, "%s", hs->what + 1);
 			return;
 		}
-		if (FEATURE(FEATURE_VALIDATE_HEADERS) && ! validhdr(b)) {
-			VRT_fail(ctx, "Bad header %s", b);
-			return;
-		}
-		http_Unset(hp, hs->what);
-		http_SetHeader(hp, b);
+	} else {
+		b[l] = '\0';
 	}
+	p = b;
+	memcpy(p, hs->what + 1, hs->what[0]);
+	p += hs->what[0];
+	*p++ = ' ';
+	if (pfx != NULL)
+		memcpy(p, pfx, pl);
+	p += pl;
+	if (FEATURE(FEATURE_VALIDATE_HEADERS) && !validhdr(b)) {
+		VRT_fail(ctx, "Bad header %s", b);
+		WS_Release(hp->ws, 0);
+		return;
+	}
+	WS_ReleaseP(hp->ws, strchr(p, '\0') + 1);
+	http_Unset(hp, hs->what);
+	http_SetHeader(hp, b);
 }
 
 /*--------------------------------------------------------------------*/
@@ -671,6 +664,7 @@ VRT_fail(VRT_CTX, const char *fmt, ...)
 {
 	va_list ap;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	assert(ctx->vsl != NULL || ctx->msg != NULL);
 	AN(ctx->handling);
 	if (*ctx->handling == VCL_RET_FAIL)
@@ -726,38 +720,51 @@ VRT_r_now(VRT_CTX)
 VCL_STRING v_matchproto_()
 VRT_IP_string(VRT_CTX, VCL_IP ip)
 {
-	char *p;
-	unsigned len;
+	char buf[VTCP_ADDRBUFSIZE];
+	struct vsb vsb[1];
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	if (ip == NULL) {
 		VRT_fail(ctx, "%s: Illegal IP", __func__);
 		return (NULL);
 	}
-	len = WS_ReserveAll(ctx->ws);
-	if (len == 0) {
-		WS_Release(ctx->ws, 0);
-		return (NULL);
-	}
-	p = WS_Reservation(ctx->ws);
-	VTCP_name(ip, p, len, NULL, 0);
-	WS_Release(ctx->ws, strlen(p) + 1);
-	return (p);
+	VTCP_name(ip, buf, sizeof buf, NULL, 0);
+	WS_VSB_new(vsb, ctx->ws);
+	VSB_cat(vsb, buf);
+	return (WS_VSB_finish(vsb, ctx->ws, NULL));
 }
 
+int
+VRT_INT_is_valid(VCL_INT arg)
+{
+	return (arg >= VRT_INTEGER_MIN && arg <= VRT_INTEGER_MAX);
+}
+
+
 VCL_STRING v_matchproto_()
 VRT_INT_string(VRT_CTX, VCL_INT num)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	if (!VRT_INT_is_valid(num))
+		VRT_fail(ctx, "INT overflow converting to string (%jX)",
+		    (intmax_t)num);
 	return (WS_Printf(ctx->ws, "%jd", (intmax_t)num));
 }
 
+int
+VRT_REAL_is_valid(VCL_REAL arg)
+{
+	return (!isnan(arg) && arg >= VRT_DECIMAL_MIN && arg <= VRT_DECIMAL_MAX);
+}
+
 VCL_STRING v_matchproto_()
 VRT_REAL_string(VRT_CTX, VCL_REAL num)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	if (!VRT_REAL_is_valid(num))
+		VRT_fail(ctx, "REAL overflow converting to string (%e)", num);
 	return (WS_Printf(ctx->ws, "%.3f", num));
 }
 
@@ -768,8 +775,15 @@ VRT_TIME_string(VRT_CTX, VCL_TIME t)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	p = WS_Alloc(ctx->ws, VTIM_FORMAT_SIZE);
-	if (p != NULL)
-		VTIM_format(t, p);
+	if (p == NULL) {
+		VRT_fail(ctx, "Workspace overflow");
+		return (NULL);
+	}
+	VTIM_format(t, p);
+	if (*p == '\0') {
+		VRT_fail(ctx, "Unformatable VCL_TIME");
+		return (NULL);
+	}
 	return (p);
 }
 
@@ -1061,6 +1075,9 @@ VRT_blob(VRT_CTX, const char *err, const void *src, size_t len, unsigned type)
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->ws, WS_MAGIC);
 
+	if (src == NULL || len == 0)
+		return (vrt_null_blob);
+
 	p = (void *)WS_Alloc(ctx->ws, sizeof *p);
 	if (p == NULL) {
 		VRT_fail(ctx, "Workspace overflow (%s)", err);
