@@ -35,6 +35,7 @@
 #include <stdio.h>
 
 #include "cache_varnishd.h"
+#include "cache_transport.h"
 #include "common/heritage.h"
 
 #include "vcl.h"
@@ -50,14 +51,14 @@ static char vrt_hostname[255] = "";
  */
 
 static void
-vrt_do_string(VRT_CTX, struct http *hp, int fld,
-    const char *err, const char *p, va_list ap)
+vrt_do_strands(VRT_CTX, struct http *hp, int fld,
+    const char *err, const char *str, VCL_STRANDS s)
 {
 	const char *b;
 
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
 
-	b = VRT_String(hp->ws, NULL, p, ap);
+	b = VRT_StrandsWS(hp->ws, str, s);
 	if (b == NULL) {
 		VRT_fail(ctx, "Workspace overflow (%s)", err);
 		WS_MarkOverflow(hp->ws);
@@ -72,14 +73,11 @@ vrt_do_string(VRT_CTX, struct http *hp, int fld,
 
 #define VRT_HDR_L(obj, hdr, fld)					\
 VCL_VOID								\
-VRT_l_##obj##_##hdr(VRT_CTX, const char *p, ...)			\
+VRT_l_##obj##_##hdr(VRT_CTX, const char *str, VCL_STRANDS s)		\
 {									\
-	va_list ap;							\
 									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
-	va_start(ap, p);						\
-	vrt_do_string(ctx, ctx->http_##obj, fld, #obj "." #hdr, p, ap);	\
-	va_end(ap);							\
+	vrt_do_strands(ctx, ctx->http_##obj, fld, #obj "." #hdr, str, s);	\
 }
 
 #define VRT_HDR_R(obj, hdr, fld)					\
@@ -262,7 +260,7 @@ VRT_r_bereq_uncacheable(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-	return (ctx->bo->do_pass);
+	return (ctx->bo->uncacheable);
 }
 
 VCL_VOID
@@ -316,16 +314,13 @@ VRT_r_client_identity(VRT_CTX)
 }
 
 VCL_VOID
-VRT_l_client_identity(VRT_CTX, const char *str, ...)
+VRT_l_client_identity(VRT_CTX, const char *str, VCL_STRANDS s)
 {
-	va_list ap;
 	const char *b;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
-	va_start(ap, str);
-	b = VRT_String(ctx->req->http->ws, NULL, str, ap);
-	va_end(ap);
+	b = VRT_StrandsWS(ctx->req->http->ws, str, s);
 	if (b == NULL) {
 		VSLb(ctx->vsl, SLT_LostHeader, "client.identity");
 		WS_MarkOverflow(ctx->req->http->ws);
@@ -441,24 +436,18 @@ VRT_r_beresp_storage_hint(VRT_CTX)
 }
 
 VCL_VOID
-VRT_l_beresp_storage_hint(VRT_CTX, const char *str, ...)
+VRT_l_beresp_storage_hint(VRT_CTX, const char *str, VCL_STRANDS s)
 {
 	const char *p;
-	va_list ap;
-	uintptr_t sn;
 	VCL_STEVEDORE stv;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
 
-	sn = WS_Snapshot(ctx->ws);
-	va_start(ap, str);
-	p = VRT_String(ctx->ws, NULL, str, ap);
-	va_end(ap);
+	p = VRT_StrandsWS(ctx->ws, str, s);
 
 	if (p == NULL) {
 		VSLb(ctx->vsl, SLT_LostHeader, "storage_hint");
-		WS_Reset(ctx->ws, sn);
 		WS_MarkOverflow(ctx->ws);
 		return;
 	}
@@ -466,8 +455,6 @@ VRT_l_beresp_storage_hint(VRT_CTX, const char *str, ...)
 	stv = VRT_stevedore(p);
 	if (stv != NULL)
 		ctx->bo->storage = stv;
-
-	WS_Reset(ctx->ws, sn);
 }
 
 /*--------------------------------------------------------------------*/
@@ -518,13 +505,22 @@ VRT_r_req_##nm(VRT_CTX)							\
 	return (ctx->req->elem);					\
 }
 
-REQ_VAR_L(backend_hint, director_hint, VCL_BACKEND,)
 REQ_VAR_R(backend_hint, director_hint, VCL_BACKEND)
+
 REQ_VAR_L(ttl, d_ttl, VCL_DURATION, if (!(arg>0.0)) arg = 0;)
 REQ_VAR_R(ttl, d_ttl, VCL_DURATION)
 REQ_VAR_L(grace, d_grace, VCL_DURATION, if (!(arg>0.0)) arg = 0;)
 REQ_VAR_R(grace, d_grace, VCL_DURATION)
 
+VCL_VOID
+VRT_l_req_backend_hint(VRT_CTX, VCL_BACKEND be)
+{
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	VRT_Assign_Backend(&ctx->req->director_hint, be);
+}
+
 /*--------------------------------------------------------------------*/
 
 VCL_VOID
@@ -533,7 +529,7 @@ VRT_l_bereq_backend(VRT_CTX, VCL_BACKEND be)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-	ctx->bo->director_req = be;
+	VRT_Assign_Backend(&ctx->bo->director_req, be);
 }
 
 VCL_BACKEND
@@ -639,6 +635,18 @@ VRT_r_bereq_retries(VRT_CTX)
 	return (ctx->bo->retries);
 }
 
+/*--------------------------------------------------------------------*/
+
+VCL_STRING
+VRT_r_req_transport(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->req->transport, TRANSPORT_MAGIC);
+	return (ctx->req->transport->name);
+}
+
 /*--------------------------------------------------------------------
  * In exp.*:
  *	t_origin is absolute
@@ -712,6 +720,25 @@ VRT_DO_EXP_R(beresp, ctx->bo->fetch_objcore, keep, 0)
 
 /*lint -restore */
 
+// XXX more assertions?
+#define VRT_DO_TIME_R(which, where, field)				\
+									\
+VCL_TIME								\
+VRT_r_##which##_time(VRT_CTX)						\
+{									\
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
+	AN((ctx)->where);						\
+									\
+	return ((ctx)->where->field);					\
+}
+
+VRT_DO_TIME_R(req, req, t_req)
+VRT_DO_TIME_R(req_top, req->top->topreq, t_req)
+VRT_DO_TIME_R(resp, req, t_resp)
+VRT_DO_TIME_R(bereq, bo, t_first)
+VRT_DO_TIME_R(beresp, bo, t_resp)
+VRT_DO_TIME_R(obj, req->objcore, t_origin)
+
 /*--------------------------------------------------------------------
  */
 
@@ -951,25 +978,32 @@ VRT_r_resp_do_esi(VRT_CTX)
 #define VRT_BODY_L(which)					\
 VCL_VOID							\
 VRT_l_##which##_body(VRT_CTX, enum lbody_e type,		\
-    const char *str, ...)					\
+    const char *str, VCL_BODY body)				\
 {								\
-	va_list ap;						\
-	const char *p;						\
+	int n;							\
 	struct vsb *vsb;					\
+	VCL_STRANDS s;						\
+	VCL_BLOB b;						\
 								\
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
+	AN(body);						\
 	CAST_OBJ_NOTNULL(vsb, ctx->specific, VSB_MAGIC);	\
-	assert(type == LBODY_SET || type == LBODY_ADD);		\
-	if (type == LBODY_SET)					\
+	if (type == LBODY_SET_STRING || type == LBODY_SET_BLOB)	\
 		VSB_clear(vsb);					\
-	va_start(ap, str);					\
-	p = str;						\
-	while (p != vrt_magic_string_end) {			\
-		if (p == NULL)					\
-			p = "(null)";				\
-		VSB_cat(vsb, p);				\
-		p = va_arg(ap, const char *);			\
+	if (type == LBODY_SET_BLOB || type == LBODY_ADD_BLOB) {	\
+		AZ(str);					\
+		b = body;					\
+		VSB_bcat(vsb, b->blob, b->len);			\
+		return;						\
 	}							\
-	va_end(ap);						\
+	if (str != NULL)					\
+		VSB_cat(vsb, str);				\
+	assert(type == LBODY_SET_STRING ||			\
+	    type == LBODY_ADD_STRING);				\
+	s = body;						\
+	for (n = 0; s != NULL && n < s->n; n++)			\
+		if (s->p[n] != NULL)				\
+			VSB_cat(vsb, s->p[n]);			\
 }
 
 VRT_BODY_L(beresp)
@@ -1019,14 +1053,9 @@ HTTP_VAR(beresp)
 static inline void
 set_idle_send_timeout(const struct sess *sp, VCL_DURATION d)
 {
-#ifdef SO_SNDTIMEO_WORKS
 	struct timeval tv = VTIM_timeval(d);
 	VTCP_Assert(setsockopt(sp->fd, SOL_SOCKET, SO_SNDTIMEO,
 	    &tv, sizeof tv));
-#else
-	(void)sp;
-	(void)d;
-#endif
 }
 
 #define SESS_VAR_DUR(x, setter)				\
@@ -1035,8 +1064,7 @@ VRT_l_sess_##x(VRT_CTX, VCL_DURATION d)		\
 {							\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);		\
 	CHECK_OBJ_NOTNULL(ctx->sp, SESS_MAGIC);		\
-	if (d < 0.0)					\
-		d = 0.0;				\
+	d = vmax(d, 0.0);				\
 	setter;						\
 	ctx->sp->x = d;					\
 }							\
