@@ -42,7 +42,7 @@
  *
  * In our case, any worker thread which we wake up comes from the idle queue,
  * where it put itself under the mutex, releasing that mutex implicitly via
- * Lck_CondWait() (which calls some variant of pthread_cond_wait). So we avoid
+ * Lck_CondWaitUntil() (which calls some variant of pthread_cond_wait). So we avoid
  * additional mutex contention knowing that any worker thread on the idle queue
  * is blocking on the cv.
  *
@@ -87,6 +87,7 @@ wrk_bgthread(void *arg)
 {
 	struct bgthread *bt;
 	struct worker wrk;
+	struct worker_priv wpriv[1];
 	struct VSC_main_wrk ds;
 	void *r;
 
@@ -94,6 +95,8 @@ wrk_bgthread(void *arg)
 	THR_SetName(bt->name);
 	THR_Init();
 	INIT_OBJ(&wrk, WORKER_MAGIC);
+	INIT_OBJ(wpriv, WORKER_PRIV_MAGIC);
+	wrk.wpriv = wpriv;
 	memset(&ds, 0, sizeof ds);
 	wrk.stats = &ds;
 
@@ -128,6 +131,7 @@ WRK_Thread(struct pool *qp, size_t stacksize, unsigned thread_workspace)
 	struct worker *w, ww;
 	struct VSC_main_wrk ds;
 	unsigned char ws[thread_workspace];
+	struct worker_priv wpriv[1];
 
 	AN(qp);
 	AN(stacksize);
@@ -136,9 +140,12 @@ WRK_Thread(struct pool *qp, size_t stacksize, unsigned thread_workspace)
 	THR_SetName("cache-worker");
 	w = &ww;
 	INIT_OBJ(w, WORKER_MAGIC);
+	INIT_OBJ(wpriv, WORKER_PRIV_MAGIC);
+	w->wpriv = wpriv;
 	w->lastused = NAN;
 	memset(&ds, 0, sizeof ds);
 	w->stats = &ds;
+	THR_SetWorker(w);
 	AZ(pthread_cond_init(&w->cond, NULL));
 
 	WS_Init(w->aws, "wrk", ws, thread_workspace);
@@ -149,8 +156,8 @@ WRK_Thread(struct pool *qp, size_t stacksize, unsigned thread_workspace)
 	AZ(w->pool);
 
 	VSL(SLT_WorkThread, 0, "%p end", w);
-	if (w->vcl != NULL)
-		VCL_Rel(&w->vcl);
+	if (w->wpriv->vcl != NULL)
+		VCL_Rel(&w->wpriv->vcl);
 	AZ(pthread_cond_destroy(&w->cond));
 	HSH_Cleanup(w);
 	Pool_Sumstat(w);
@@ -369,7 +376,7 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 {
 	struct pool_task *tp;
 	struct pool_task tpx, tps;
-	vtim_real tmo;
+	vtim_real tmo, now;
 	unsigned i, reserve;
 
 	CHECK_OBJ_NOTNULL(pp, POOL_MAGIC);
@@ -414,6 +421,7 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 			wrk->task->priv = wrk;
 			VTAILQ_INSERT_HEAD(&pp->idle_queue, wrk->task, list);
 			pp->nidle++;
+			now = wrk->lastused;
 			do {
 				// see signaling_note at the top for explanation
 				if (DO_DEBUG(DBG_VCLREL) &&
@@ -425,16 +433,15 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 					 * stats. Set a 1 second timeout
 					 * so that we'll wake up and get a
 					 * chance to push stats. */
-					tmo = wrk->lastused + 1.;
-				else if (wrk->vcl == NULL)
+					tmo = now + 1.;
+				else if (wrk->wpriv->vcl == NULL)
 					tmo = 0;
 				else if (DO_DEBUG(DBG_VTC_MODE))
-					tmo =  wrk->lastused+1.;
+					tmo = now + 1.;
 				else
-					tmo =  wrk->lastused+60.;
-				i = Lck_CondWait(&wrk->cond, &pp->mtx, tmo);
-				if (i == ETIMEDOUT)
-					VCL_Rel(&wrk->vcl);
+					tmo = now + 60.;
+				(void)Lck_CondWaitUntil(
+				    &wrk->cond, &pp->mtx, tmo);
 				if (wrk->task->func != NULL) {
 					/* We have been handed a new task */
 					tpx = *wrk->task;
@@ -459,6 +466,13 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 					pp->a_stat = pp->b_stat;
 					pp->b_stat = NULL;
 					tp = &tps;
+				} else {
+					// Presumably ETIMEDOUT but we do not
+					// assert this because pthread condvars
+					// are not airtight.
+					if (wrk->wpriv->vcl)
+						VCL_Rel(&wrk->wpriv->vcl);
+					now = VTIM_real();
 				}
 			} while (tp == NULL);
 		}
@@ -470,9 +484,10 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 		do {
 			memset(wrk->task, 0, sizeof wrk->task);
 			assert(wrk->pool == pp);
+			AN(tp->func);
 			tp->func(wrk, tp->priv);
-			if (DO_DEBUG(DBG_VCLREL) && wrk->vcl != NULL)
-				VCL_Rel(&wrk->vcl);
+			if (DO_DEBUG(DBG_VCLREL) && wrk->wpriv->vcl != NULL)
+				VCL_Rel(&wrk->wpriv->vcl);
 			tpx = *wrk->task;
 			tp = &tpx;
 		} while (tp->func != NULL);
@@ -532,7 +547,7 @@ pool_breed(struct pool *qp)
 	if (errno) {
 		FREE_OBJ(pi);
 		VSL(SLT_Debug, 0, "Create worker thread failed %d %s",
-		    errno, vstrerror(errno));
+		    errno, VAS_errtxt(errno));
 		Lck_Lock(&pool_mtx);
 		VSC_C_main->threads_failed++;
 		Lck_Unlock(&pool_mtx);
@@ -667,13 +682,14 @@ pool_herder(void *priv)
 				VSC_C_main->threads_destroyed++;
 				Lck_Unlock(&pool_mtx);
 				delay = cache_param->wthread_destroy_delay;
-			} else if (delay < cache_param->wthread_destroy_delay)
-				delay = cache_param->wthread_destroy_delay;
+			} else
+				delay = vmax(delay,
+				    cache_param->wthread_destroy_delay);
 		}
 
 		if (pp->die) {
 			if (delay < 2)
-				delay = 10e-3;
+				delay = .01;
 			else
 				delay = 1;
 			VTIM_sleep(delay);
@@ -683,14 +699,14 @@ pool_herder(void *priv)
 		if (pp->lqueue == 0) {
 			if (DO_DEBUG(DBG_VTC_MODE))
 				delay = 0.5;
-			r = Lck_CondWait(&pp->herder_cond, &pp->mtx,
-			    VTIM_real() + delay);
+			r = Lck_CondWaitTimeout(
+			    &pp->herder_cond, &pp->mtx, delay);
 		} else if (pp->nthr >= cache_param->wthread_max) {
 			/* XXX: unsafe counters */
 			if (r != ETIMEDOUT)
 				VSC_C_main->threads_limited++;
-			r = Lck_CondWait(&pp->herder_cond, &pp->mtx,
-			    VTIM_real() + 1.0);
+			r = Lck_CondWaitTimeout(
+			    &pp->herder_cond, &pp->mtx, 1.0);
 		}
 		Lck_Unlock(&pp->mtx);
 	}
@@ -722,6 +738,25 @@ static struct cli_proto debug_cmds[] = {
 	{ NULL }
 };
 
+void
+WRK_Log(enum VSL_tag_e tag, const char *fmt, ...)
+{
+	struct worker *wrk;
+	va_list ap;
+
+	AN(fmt);
+
+	wrk = THR_GetWorker();
+	CHECK_OBJ_ORNULL(wrk, WORKER_MAGIC);
+
+	va_start(ap, fmt);
+	if (wrk != NULL && wrk->vsl != NULL)
+		VSLbv(wrk->vsl, tag, fmt, ap);
+	else
+		VSLv(tag, 0, fmt, ap);
+	va_end(ap);
+}
+
 /*--------------------------------------------------------------------
  *
  */
