@@ -52,6 +52,18 @@ COM_DO_DEBUG(const volatile uint8_t *p, enum debug_bits x)
 	return (p[(unsigned)x>>3] & (0x80U >> ((unsigned)x & 7)));
 }
 
+enum experimental_bits {
+#define EXPERIMENTAL_BIT(U, l, d) EXPERIMENT_##U,
+#include "tbl/experimental_bits.h"
+       EXPERIMENT_Reserved
+};
+
+static inline int
+COM_EXPERIMENT(const volatile uint8_t *p, enum experimental_bits x)
+{
+	return (p[(unsigned)x>>3] & (0x80U >> ((unsigned)x & 7)));
+}
+
 enum feature_bits {
 #define FEATURE_BIT(U, l, d) FEATURE_##U,
 #include "tbl/feature_bits.h"
@@ -64,17 +76,19 @@ COM_FEATURE(const volatile uint8_t *p, enum feature_bits x)
 	return (p[(unsigned)x>>3] & (0x80U >> ((unsigned)x & 7)));
 }
 
-
 struct poolparam {
 	unsigned		min_pool;
 	unsigned		max_pool;
-	double			max_age;
+	vtim_dur		max_age;
 };
 
+#define PARAM_BITMAP(name, len) typedef uint8_t name[(len + 7)>>3]
 
-typedef uint8_t vsl_mask_t[256>>3];
-typedef uint8_t debug_t[(DBG_Reserved+7)>>3];
-typedef uint8_t feature_t[(FEATURE_Reserved+7)>>3];
+PARAM_BITMAP(vsl_mask_t,	256);
+PARAM_BITMAP(debug_t,		DBG_Reserved);
+PARAM_BITMAP(experimental_t,	EXPERIMENT_Reserved);
+PARAM_BITMAP(feature_t,		FEATURE_Reserved);
+#undef PARAM_BITMAP
 
 struct params {
 
@@ -85,7 +99,7 @@ struct params {
 #define ptyp_poolparam		struct poolparam
 #define ptyp_thread_pool_max	unsigned
 #define ptyp_thread_pool_min	unsigned
-#define ptyp_timeout		double
+#define ptyp_timeout		vtim_dur
 #define ptyp_uint		unsigned
 #define ptyp_vsl_buffer		unsigned
 #define ptyp_vsl_reclen		unsigned
@@ -112,5 +126,6 @@ struct params {
 
 	vsl_mask_t		vsl_mask;
 	debug_t			debug_bits;
+	experimental_t		experimental_bits;
 	feature_t		feature_bits;
 };
