@@ -1,4 +1,3 @@
-
 /*-
  * Copyright (c) 2006 Verdens Gang AS
  * Copyright (c) 2006-2011 Varnish Software AS
@@ -38,6 +37,7 @@ struct listen_sock;
 struct transport;
 struct VCLS;
 struct uds_perms;
+struct conn_heritage;
 
 struct listen_sock {
 	unsigned			magic;
@@ -51,6 +51,8 @@ struct listen_sock {
 	struct suckaddr			*addr;
 	const struct transport		*transport;
 	const struct uds_perms		*perms;
+	unsigned			test_heritage;
+	struct conn_heritage		*conn_heritage;
 };
 
 VTAILQ_HEAD(listen_sock_head, listen_sock);
@@ -89,6 +91,9 @@ struct heritage {
 
 	unsigned			min_vcl_version;
 	unsigned			max_vcl_version;
+
+	int				argc;
+	char * const *			argv;
 };
 
 extern struct heritage heritage;
