@@ -119,7 +119,9 @@ http1_new_session(struct worker *wrk, void *arg)
 		/* Out of session workspace. Free the req, close the sess,
 		 * and do not set a new task func, which will exit the
 		 * worker thread. */
-		VSL(SLT_Error, req->sp->vxid, "insufficient workspace");
+		VSL(SLT_Error, req->sp->vxid,
+		    "insufficient workspace (proto_priv)");
+		WS_Release(req->ws, 0);
 		Req_Release(req);
 		SES_Delete(sp, SC_RX_JUNK, NAN);
 		return;
@@ -139,7 +141,7 @@ http1_unwait(struct worker *wrk, void *arg)
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CAST_OBJ_NOTNULL(sp, arg, SESS_MAGIC);
 	WS_Release(sp->ws, 0);
-	req = Req_New(wrk, sp);
+	req = Req_New(sp);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	req->htc->rfd = &sp->fd;
 	HTC_RxInit(req->htc, req->ws);
@@ -172,9 +174,9 @@ http1_req_panic(struct vsb *vsb, const struct req *req)
 }
 
 static void v_matchproto_(vtr_req_fail_f)
-http1_req_fail(struct req *req, enum sess_close reason)
+http1_req_fail(struct req *req, stream_close_t reason)
 {
-	assert(reason > 0);
+	assert(reason != SC_NULL);
 	assert(req->sp->fd != 0);
 	if (req->sp->fd > 0)
 		SES_Close(req->sp, reason);
@@ -208,7 +210,7 @@ http1_minimal_response(struct req *req, uint16_t status)
 	if (wl != l) {
 		if (wl < 0)
 			VTCP_Assert(1);
-		if (!req->doclose)
+		if (req->doclose == SC_NULL)
 			req->doclose = SC_REM_CLOSE;
 		return (-1);
 	}
@@ -235,7 +237,7 @@ struct transport HTTP1_transport = {
 static inline void
 http1_abort(struct req *req, uint16_t status)
 {
-	AN(req->doclose);
+	assert(req->doclose != SC_NULL);
 	assert(status >= 400);
 	(void)http1_minimal_response(req, status);
 }
@@ -356,7 +358,7 @@ HTTP1_Session(struct worker *wrk, struct req *req)
 			req->acct.req_hdrbytes +=
 			    req->htc->rxbuf_e - req->htc->rxbuf_b;
 			if (i) {
-				assert(req->doclose > 0);
+				assert(req->doclose != SC_NULL);
 				SES_Close(req->sp, req->doclose);
 				assert(!WS_IsReserved(req->ws));
 				assert(!WS_IsReserved(wrk->aws));
