@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -48,6 +50,7 @@
 
 #include "common/common_param.h"
 
+struct vjsn;
 struct vsc_seg;
 struct vsmw_cluster;
 #include "VSC_mgt.h"
@@ -99,21 +102,31 @@ void mgt_cli_init_cls(void);
 
 /* mgt_jail.c */
 
-enum jail_subproc_e {
-	JAIL_SUBPROC_VCC,
-	JAIL_SUBPROC_CC,
-	JAIL_SUBPROC_VCLLOAD,
-	JAIL_SUBPROC_WORKER,
-};
-
 enum jail_master_e {
-	JAIL_MASTER_LOW,
+	JAIL_MASTER_LOW = 0,
+	JAIL_MASTER_SYSTEM,
 	JAIL_MASTER_FILE,
 	JAIL_MASTER_STORAGE,
 	JAIL_MASTER_PRIVPORT,
 	JAIL_MASTER_KILL,
+#define JAIL_SUBPROC (JAIL_MASTER_KILL + 1)
 };
 
+enum jail_subproc_e {
+	JAIL_SUBPROC_VCC = JAIL_SUBPROC,
+	JAIL_SUBPROC_CC,
+	JAIL_SUBPROC_VCLLOAD,
+	JAIL_SUBPROC_WORKER,
+#define JAIL_LIMIT (JAIL_SUBPROC_WORKER + 1)
+};
+
+#define ASSERT_JAIL_MASTER(x) assert((x) < JAIL_SUBPROC)
+
+#define ASSERT_JAIL_SUBPROC(x) do {		\
+		assert((x) >= JAIL_SUBPROC);	\
+		assert((x) < JAIL_LIMIT);	\
+	} while (0)
+
 enum jail_fixfd_e {
 	JAIL_FIXFD_FILE,
 	JAIL_FIXFD_VSMMGT,
@@ -144,6 +157,8 @@ void VJ_subproc(enum jail_subproc_e);
 int VJ_make_workdir(const char *);
 int VJ_make_subdir(const char *, const char *, struct vsb *);
 void VJ_fix_fd(int, enum jail_fixfd_e);
+void VJ_unlink(const char *);
+void VJ_rmdir(const char *);
 
 extern const struct jail_tech jail_tech_unix;
 extern const struct jail_tech jail_tech_solaris;
@@ -161,6 +176,7 @@ extern const char C_INFO[];	// Normal stuff, keep a record for later
 extern const char C_DEBUG[];	// More detail than you'd normally want
 extern const char C_SECURITY[];	// Security issues
 extern const char C_CLI[];	// CLI traffic between master and child
+extern int complain_to_stderr;
 
 /* mgt_param.c */
 void MCF_InitParams(struct cli *);
@@ -202,6 +218,7 @@ char **MGT_NamedArg(const char *, const char **, const char *);
 
 
 /* stevedore_mgt.c */
+extern const char *mgt_stv_h2_rxbuf;
 void STV_Config(const char *spec);
 void STV_Config_Transient(void);
 
@@ -214,20 +231,19 @@ void mgt_vcl_init(void);
 void mgt_vcl_startup(struct cli *, const char *vclsrc, const char *origin,
     const char *vclname, int Cflag);
 int mgt_push_vcls(struct cli *, unsigned *status, char **p);
-void mgt_vcl_export_labels(struct vcc *);
-int mgt_has_vcl(void);
-void mgt_vcl_depends(struct vclprog *vp1, const char *name);
-void mgt_vcl_vmod(struct vclprog *, const char *src, const char *dst);
+const char *mgt_has_vcl(void);
 extern char *mgt_cc_cmd;
+extern char *mgt_cc_cmd_def;
+extern char *mgt_cc_warn;
 extern const char *mgt_vcl_path;
 extern const char *mgt_vmod_path;
-extern unsigned mgt_vcc_err_unref;
-extern unsigned mgt_vcc_allow_inline_c;
-extern unsigned mgt_vcc_unsafe_path;
+#define MGT_VCC(t, n, cc) extern t mgt_vcc_ ## n;
+#include <tbl/mgt_vcc.h>
 
 #if defined(PTHREAD_CANCELED) || defined(PTHREAD_MUTEX_DEFAULT)
 #error "Keep pthreads out of in manager process"
 #endif
 
-#define MGT_FEATURE(x)	COM_FEATURE(mgt_param.feature_bits, x)
-#define MGT_DO_DEBUG(x)	COM_DO_DEBUG(mgt_param.debug_bits, x)
+#define MGT_FEATURE(x)		COM_FEATURE(mgt_param.feature_bits, x)
+#define MGT_EXPERIMENT(x)	COM_EXPERIMENT(mgt_param.experimental_bits, x)
+#define MGT_DO_DEBUG(x)		COM_DO_DEBUG(mgt_param.debug_bits, x)
