@@ -130,11 +130,10 @@ MAC_reopen_sockets(void)
 		VJ_master(JAIL_MASTER_LOW);
 		if (err == 0)
 			continue;
-		if (err > fail)
-			fail = err;
+		fail = vmax(fail, err);
 		MGT_Complain(C_ERR,
 		    "Could not reopen listen socket %s: %s",
-		    ls->endpoint, vstrerror(err));
+		    ls->endpoint, VAS_errtxt(err));
 	}
 	return (fail);
 }
@@ -152,8 +151,7 @@ mk_listen_sock(const struct listen_arg *la, const struct suckaddr *sa)
 	ls->sock = -1;
 	ls->addr = VSA_Clone(sa);
 	AN(ls->addr);
-	ls->endpoint = strdup(la->endpoint);
-	AN(ls->endpoint);
+	REPLACE(ls->endpoint, la->endpoint);
 	ls->name = la->name;
 	ls->transport = la->transport;
 	ls->perms = la->perms;
@@ -168,7 +166,7 @@ mk_listen_sock(const struct listen_arg *la, const struct suckaddr *sa)
 		FREE_OBJ(ls);
 		if (fail != EAFNOSUPPORT)
 			ARGV_ERR("Could not get socket %s: %s\n",
-			    la->endpoint, vstrerror(fail));
+			    la->endpoint, VAS_errtxt(fail));
 		return (NULL);
 	}
 	return (ls);
@@ -203,7 +201,10 @@ mac_tcp(void *priv, const struct suckaddr *sa)
 		ls->addr = VTCP_my_suckaddr(ls->sock);
 		VTCP_myname(ls->sock, abuf, sizeof abuf,
 		    pbuf, sizeof pbuf);
-		bprintf(nbuf, "%s:%s", abuf, pbuf);
+		if (VSA_Get_Proto(sa) == AF_INET6)
+			bprintf(nbuf, "[%s]:%s", abuf, pbuf);
+		else
+			bprintf(nbuf, "%s:%s", abuf, pbuf);
 		REPLACE(ls->endpoint, nbuf);
 	}
 	VTAILQ_INSERT_TAIL(&la->socks, ls, arglist);
@@ -331,7 +332,7 @@ MAC_Arg(const char *spec)
 					 val);
 			if (errno)
 				ARGV_ERR("Cannot parse mode sub-arg %s in -a: "
-					 "%s\n", val, vstrerror(errno));
+					 "%s\n", val, VAS_errtxt(errno));
 			if (m <= 0 || m > 0777)
 				ARGV_ERR("Mode sub-arg %s out of range in -a\n",
 					 val);
