@@ -204,8 +204,7 @@ MCH_TrackHighFd(int fd)
 	 * in the master process.
 	 */
 	assert(fd > 0);
-	if (fd > mgt_max_fd)
-		mgt_max_fd = fd;
+	mgt_max_fd = vmax(mgt_max_fd, fd);
 }
 
 /*--------------------------------------------------------------------
@@ -293,7 +292,7 @@ child_poker(const struct vev *e, int what)
 static void
 mgt_launch_child(struct cli *cli)
 {
-	pid_t pid;
+	pid_t pid, pidr;
 	unsigned u;
 	char *p;
 	struct vev *e;
@@ -415,6 +414,18 @@ mgt_launch_child(struct cli *cli)
 	MCH_Fd_Inherit(heritage.cli_out, NULL);
 	closefd(&heritage.cli_out);
 
+	/* Wait for cache/cache_cli.c::CLI_Run() to check in */
+	if (VCLI_ReadResult(child_cli_in, &u, NULL, mgt_param.cli_timeout)) {
+		assert(u == CLIS_COMMS);
+		pidr = waitpid(pid, &i, 0);
+		assert(pidr == pid);
+		perror("Child failed on launch");
+		exit(1);		// XXX Harsh ?
+	} else {
+		assert(u == CLIS_OK);
+		fprintf(stderr, "Child launched OK\n");
+	}
+
 	child_std_vlu = VLU_New(child_line, NULL, 0);
 	AN(child_std_vlu);
 
@@ -519,11 +530,11 @@ mgt_reap_child(void)
 	XXXAN(vsb);
 
 	/* Wait for child to die */
-	for (i = 0; i < mgt_param.cli_timeout; i++) {
+	for (i = 0; i < mgt_param.cli_timeout * 10; i++) {
 		r = waitpid(child_pid, &status, WNOHANG);
 		if (r == child_pid)
 			break;
-		(void)sleep(1);
+		(void)usleep(100000);
 	}
 	if (r == 0) {
 		VSB_printf(vsb, "Child (%jd) not dying, killing", (intmax_t)r);
@@ -629,7 +640,7 @@ MCH_Cli_Fail(void)
 		    " killed it.", (intmax_t)child_pid);
 	else
 		MGT_Complain(C_ERR, "Failed to kill child with PID %jd: %s",
-		    (intmax_t)child_pid, vstrerror(errno));
+		    (intmax_t)child_pid, VAS_errtxt(errno));
 }
 
 /*=====================================================================
@@ -707,15 +718,17 @@ mch_pid_json(struct cli *cli, const char * const *av, void *priv)
 static void v_matchproto_(cli_func_t)
 mch_cli_server_start(struct cli *cli, const char * const *av, void *priv)
 {
+	const char *err;
 
 	(void)av;
 	(void)priv;
 	if (child_state == CH_STOPPED) {
-		if (mgt_has_vcl()) {
+		err = mgt_has_vcl();
+		if (err == NULL) {
 			mgt_launch_child(cli);
 		} else {
 			VCLI_SetResult(cli, CLIS_CANT);
-			VCLI_Out(cli, "No VCL available");
+			VCLI_Out(cli, "%s", err);
 		}
 	} else {
 		VCLI_SetResult(cli, CLIS_CANT);
