@@ -45,10 +45,6 @@
 #include "vav.h"
 #include "vcli_serve.h"
 
-#if !defined(__has_feature)
-#define __has_feature(x)	0
-#endif
-
 struct plist {
 	unsigned			magic;
 #define PLIST_MAGIC			0xbfc3ea16
@@ -105,13 +101,20 @@ static const char ONLY_ROOT_TEXT[] =
 	"NB: This parameter only works if varnishd is run as root.";
 
 static const char NOT_IMPLEMENTED_TEXT[] =
-	"This parameter depends on a feature which is not available"
+	"\n\n"
+	"NB: This parameter depends on a feature which is not available"
 	" on this platform.";
 
 static const char PLATFORM_DEPENDENT_TEXT[] =
+	"\n\n"
 	"NB: This parameter depends on a feature which is not available"
 	" on all platforms.";
 
+static const char BUILD_OPTIONS_TEXT[] =
+	"\n\n"
+	"NB: The actual default value for this parameter depends on the"
+	" Varnish build environment and options.";
+
 /*--------------------------------------------------------------------*/
 
 static struct parspec *
@@ -244,25 +247,42 @@ mcf_wrap(struct cli *cli, const char *text)
 static void v_matchproto_(cli_func_t)
 mcf_param_show(struct cli *cli, const char * const *av, void *priv)
 {
-	int n;
 	struct plist *pl;
-	const struct parspec *pp;
-	int lfmt = 0, chg = 0;
+	const struct parspec *pp, *pa;
+	int n, lfmt = 0, chg = 0;
 	struct vsb *vsb;
+	const char *show = NULL;
+
+	(void)priv;
+
+	for (n = 2; av[n] != NULL; n++) {
+		if (strcmp(av[n], "-l") == 0) {
+			lfmt = 1;
+			continue;
+		}
+		if (strcmp(av[n], "changed") == 0) {
+			chg = 1;
+			continue;
+		}
+		if (show != NULL) {
+			VCLI_SetResult(cli, CLIS_TOOMANY);
+			VCLI_Out(cli, "Too many parameters");
+			return;
+		}
+		show = av[n];
+		lfmt = 1;
+	}
 
 	vsb = VSB_new_auto();
 	AN(vsb);
-	(void)priv;
-
-	if (av[2] != NULL && !strcmp(av[2], "changed"))
-		chg = 1;
-	else if (av[2] != NULL)
-		lfmt = 1;
 
 	n = 0;
 	VTAILQ_FOREACH(pl, &phead, list) {
 		pp = pl->spec;
-		if (lfmt && strcmp(pp->name, av[2]) && strcmp("-l", av[2]))
+		if (lfmt && show != NULL && strcmp(pp->name, show))
+			continue;
+		if (pp->func == tweak_alias &&
+		    (show == NULL || strcmp(pp->name, show)))
 			continue;
 		n++;
 
@@ -296,6 +316,11 @@ mcf_param_show(struct cli *cli, const char * const *av, void *priv)
 		}
 		VCLI_Out(cli, "\n");
 
+		if (lfmt && pp->func == tweak_alias) {
+			pa = TRUST_ME(pp->priv);
+			VCLI_Out(cli, "%-*sAlias of: %s\n",
+			    margin1, " ", pa->name);
+		}
 		if (lfmt && pp->flags & NOT_IMPLEMENTED) {
 			VCLI_Out(cli, "\n");
 			mcf_wrap(cli, NOT_IMPLEMENTED_TEXT);
@@ -328,12 +353,14 @@ mcf_param_show(struct cli *cli, const char * const *av, void *priv)
 				mcf_wrap(cli, PROTECTED_TEXT);
 			if (pp->flags & ONLY_ROOT)
 				mcf_wrap(cli, ONLY_ROOT_TEXT);
+			if (pp->flags & BUILD_OPTIONS)
+				mcf_wrap(cli, BUILD_OPTIONS_TEXT);
 			VCLI_Out(cli, "\n\n");
 		}
 	}
-	if (av[2] != NULL && lfmt && strcmp(av[2], "-l") && n == 0) {
+	if (show != NULL && n == 0) {
 		VCLI_SetResult(cli, CLIS_PARAM);
-		VCLI_Out(cli, "Unknown parameter \"%s\".", av[2]);
+		VCLI_Out(cli, "Unknown parameter \"%s\".", show);
 	}
 	VSB_destroy(&vsb);
 }
@@ -349,12 +376,11 @@ mcf_json_key_valstr(struct cli *cli, const char *key, const char *val)
 static void v_matchproto_(cli_func_t)
 mcf_param_show_json(struct cli *cli, const char * const *av, void *priv)
 {
-	int n, comma = 0;
+	int n, comma = 0, chg = 0;
 	struct plist *pl;
-	const struct parspec *pp;
-	int chg = 0, flags;
+	const struct parspec *pp, *pa;
 	struct vsb *vsb, *def;
-	const char *show = NULL;
+	const char *show = NULL, *sep;
 
 	(void)priv;
 
@@ -370,6 +396,11 @@ mcf_param_show_json(struct cli *cli, const char * const *av, void *priv)
 		}
 		if (strcmp(av[i], "-j") == 0)
 			continue;
+		if (show != NULL) {
+			VCLI_SetResult(cli, CLIS_TOOMANY);
+			VCLI_Out(cli, "Too many parameters");
+			return;
+		}
 		show = av[i];
 	}
 
@@ -385,6 +416,9 @@ mcf_param_show_json(struct cli *cli, const char * const *av, void *priv)
 		pp = pl->spec;
 		if (show != NULL && strcmp(pp->name, show) != 0)
 			continue;
+		if (pp->func == tweak_alias &&
+		    (show == NULL || strcmp(pp->name, show)))
+			continue;
 		n++;
 
 		VSB_clear(vsb);
@@ -403,6 +437,10 @@ mcf_param_show_json(struct cli *cli, const char * const *av, void *priv)
 		VCLI_Out(cli, "{\n");
 		VSB_indent(cli->sb, 2);
 		mcf_json_key_valstr(cli, "name", pp->name);
+		if (pp->func == tweak_alias) {
+			pa = TRUST_ME(pp->priv);
+			mcf_json_key_valstr(cli, "alias", pa->name);
+		}
 		if (pp->flags & NOT_IMPLEMENTED) {
 			VCLI_Out(cli, "\"implemented\": false\n");
 			VSB_indent(cli->sb, -2);
@@ -422,16 +460,15 @@ mcf_param_show_json(struct cli *cli, const char * const *av, void *priv)
 			mcf_json_key_valstr(cli, "maximum", pp->max);
 		mcf_json_key_valstr(cli, "description", pp->descr);
 
-		flags = 0;
-		VCLI_Out(cli, "\"flags\": [\n");
+		VCLI_Out(cli, "\"flags\": [");
 		VSB_indent(cli->sb, 2);
+		sep = "";
 
 #define flag_out(flag, string) do {					\
 			if (pp->flags & flag) {				\
-				if (flags)				\
-					VCLI_Out(cli, ",\n");		\
+				VCLI_Out(cli, "%s\n", sep);		\
 				VCLI_Out(cli, "\"%s\"", #string);	\
-				flags++;				\
+				sep = ",";				\
 			}						\
 		} while(0)
 
@@ -443,13 +480,16 @@ mcf_param_show_json(struct cli *cli, const char * const *av, void *priv)
 		flag_out(WIZARD, wizard);
 		flag_out(PROTECTED, protected);
 		flag_out(ONLY_ROOT, only_root);
+		flag_out(BUILD_OPTIONS, build_options);
 
 #undef flag_out
 
+		if (pp->flags)
+			VCLI_Out(cli, "\n");
 		VSB_indent(cli->sb, -2);
-		VCLI_Out(cli, "\n]");
+		VCLI_Out(cli, "]\n");
 		VSB_indent(cli->sb, -2);
-		VCLI_Out(cli, "\n}");
+		VCLI_Out(cli, "}");
 	}
 	VCLI_JSON_end(cli);
 	if (show != NULL && n == 0) {
@@ -505,6 +545,14 @@ MCF_ParamSet(struct cli *cli, const char *param, const char *val)
 		VCLI_Out(cli, "Unknown parameter \"%s\".", param);
 		return;
 	}
+	if (pp->flags & NOT_IMPLEMENTED) {
+		VCLI_SetResult(cli, CLIS_CANT);
+		VCLI_Out(cli,
+		    "parameter \"%s\" is not available on this platform.",
+		    param
+		);
+		return;
+	}
 	if (pp->flags & PROTECTED) {
 		VCLI_SetResult(cli, CLIS_AUTH);
 		VCLI_Out(cli, "parameter \"%s\" is protected.", param);
@@ -541,6 +589,16 @@ mcf_param_set(struct cli *cli, const char * const *av, void *priv)
 	MCF_ParamSet(cli, av[2], av[3]);
 }
 
+static void v_matchproto_(cli_func_t)
+mcf_param_set_json(struct cli *cli, const char * const *av, void *priv)
+{
+	const char *const avs[] = { av[0], av[1], av[2], av[3], NULL };
+
+	MCF_ParamSet(cli, av[3], av[4]);
+	if (cli->result == CLIS_OK)
+		mcf_param_show_json(cli, avs, priv);
+}
+
 /*--------------------------------------------------------------------*/
 
 static void v_matchproto_(cli_func_t)
@@ -570,8 +628,7 @@ MCF_AddParams(struct parspec *ps)
 			exit(4);
 		}
 		mcf_addpar(pp);
-		if ((int)strlen(pp->name) + 1 > margin2)
-			margin2 = (int)strlen(pp->name) + 1;
+		margin2 = vmax_t(int, margin2, strlen(pp->name) + 1);
 	}
 }
 
@@ -617,6 +674,13 @@ mcf_wash_param(struct cli *cli, struct parspec *pp, enum mcf_which_e which,
 	}
 	AN(val);
 
+	if (pp->func == tweak_alias) {
+		assert(which == MCF_DEFAULT);
+		pp->priv = mcf_findpar(pp->def);
+		pp->def = NULL;
+		return;
+	}
+
 	VSB_clear(vsb);
 	VSB_printf(vsb, "FAILED to set %s for param %s: %s\n",
 	    name, pp->name, val);
@@ -638,10 +702,9 @@ mcf_wash_param(struct cli *cli, struct parspec *pp, enum mcf_which_e which,
 /*--------------------------------------------------------------------*/
 
 static struct cli_proto cli_params[] = {
-	{ CLICMD_PARAM_SHOW,		"", mcf_param_show,
-	  mcf_param_show_json },
-	{ CLICMD_PARAM_SET,		"", mcf_param_set },
-	{ CLICMD_PARAM_RESET,		"", mcf_param_reset },
+	{ CLICMD_PARAM_SHOW,	"", mcf_param_show, mcf_param_show_json },
+	{ CLICMD_PARAM_SET,	"", mcf_param_set, mcf_param_set_json },
+	{ CLICMD_PARAM_RESET,	"", mcf_param_reset },
 	{ NULL }
 };
 
@@ -662,7 +725,7 @@ MCF_InitParams(struct cli *cli)
 
 	MCF_TcpParams();
 
-	def = 56 * 1024;
+	def = 80 * 1024;
 
 	if (sizeof(void *) < 8) {		/*lint !e506 !e774  */
 		/*
@@ -676,15 +739,16 @@ MCF_InitParams(struct cli *cli)
 		MCF_ParamConf(MCF_DEFAULT, "http_resp_size", "8k");
 		MCF_ParamConf(MCF_DEFAULT, "http_req_size", "12k");
 		MCF_ParamConf(MCF_DEFAULT, "gzip_buffer", "4k");
+		MCF_ParamConf(MCF_DEFAULT, "vsl_buffer", "4k");
 		MCF_ParamConf(MCF_MAXIMUM, "vsl_space", "1G");
-		def = 52 * 1024;
+		def = 64 * 1024;
 	}
 
 	low = sysconf(_SC_THREAD_STACK_MIN);
 	MCF_ParamConf(MCF_MINIMUM, "thread_pool_stack", "%jdb", (intmax_t)low);
 
-#if defined(__SANITIZER) || __has_feature(address_sanitizer) || defined(GCOVING)
-	def = 92 * 1024;
+#if defined(ENABLE_SANITIZER) || defined(ENABLE_COVERAGE)
+	def = 192 * 1024;
 #endif
 
 	if (def < low)
@@ -719,6 +783,9 @@ MCF_InitParams(struct cli *cli)
 		mcf_wash_param(cli, pp, MCF_DEFAULT, "default", vsb);
 	}
 	VSB_destroy(&vsb);
+
+	AN(mgt_cc_cmd);
+	REPLACE(mgt_cc_cmd_def, mgt_cc_cmd);
 }
 
 /*--------------------------------------------------------------------*/
@@ -757,6 +824,8 @@ MCF_DumpRstParam(void)
 	    "output from varnishd -x parameter\n\n");
 	VTAILQ_FOREACH(pl, &phead, list) {
 		pp = pl->spec;
+		if (!strcmp("deprecated_dummy", pp->name))
+		    continue;
 		printf(".. _ref_param_%s:\n\n", pp->name);
 		printf("%s\n", pp->name);
 		for (z = 0; z < strlen(pp->name); z++)
@@ -766,6 +835,9 @@ MCF_DumpRstParam(void)
 		if (pp->flags && pp->flags & PLATFORM_DEPENDENT)
 			printf("\n%s\n\n", PLATFORM_DEPENDENT_TEXT);
 
+		if (pp->flags && pp->flags & BUILD_OPTIONS)
+			printf("\n%s\n\n", BUILD_OPTIONS_TEXT);
+
 		if (pp->units != NULL && *pp->units != '\0')
 			printf("\t* Units: %s\n", pp->units);
 #define MCF_DYN_REASON(lbl, nm)					\
@@ -783,7 +855,7 @@ MCF_DumpRstParam(void)
 		 * XXX: that say if ->min/->max are valid, so we
 		 * XXX: can emit those also in help texts.
 		 */
-		if (pp->flags & ~(NOT_IMPLEMENTED|PLATFORM_DEPENDENT)) {
+		if (pp->flags & ~DOCS_FLAGS) {
 			printf("\t* Flags: ");
 			q = "";
 
