@@ -57,6 +57,9 @@ struct parspec {
 #define ONLY_ROOT		(1<<7)
 #define NOT_IMPLEMENTED		(1<<8)
 #define PLATFORM_DEPENDENT	(1<<9)
+#define BUILD_OPTIONS		(1<<10)
+
+#define DOCS_FLAGS	(NOT_IMPLEMENTED|PLATFORM_DEPENDENT|BUILD_OPTIONS)
 
 	const char	*dyn_min_reason;
 	const char	*dyn_max_reason;
@@ -66,11 +69,13 @@ struct parspec {
 	char		*dyn_def;
 };
 
+tweak_t tweak_alias;
 tweak_t tweak_boolean;
 tweak_t tweak_bytes;
 tweak_t tweak_bytes_u;
 tweak_t tweak_double;
 tweak_t tweak_poolparam;
+tweak_t tweak_storage;
 tweak_t tweak_string;
 tweak_t tweak_thread_pool_min;
 tweak_t tweak_thread_pool_max;
