@@ -54,7 +54,7 @@ mgt_vcl_symtab_val(const struct vjsn_val *vv, const char *val)
 
 	jv = vjsn_child(vv, val);
 	AN(jv);
-	assert(jv->type == VJSN_STRING);
+	assert(vjsn_is_string(jv));
 	AN(jv->value);
 	return (jv->value);
 }
@@ -85,13 +85,13 @@ mgt_vcl_cache_vmod(const char *nm, const char *fm, const char *to)
 		return (0);
 	if (fo < 0) {
 		fprintf(stderr, "While creating copy of vmod %s:\n\t%s: %s\n",
-		    nm, to, vstrerror(errno));
+		    nm, to, VAS_errtxt(errno));
 		return (1);
 	}
 	fi = open(fm, O_RDONLY);
 	if (fi < 0) {
 		fprintf(stderr, "Opening vmod %s from %s: %s\n",
-		    nm, fm, vstrerror(errno));
+		    nm, fm, VAS_errtxt(errno));
 		AZ(unlink(to));
 		closefd(&fo);
 		return (1);
@@ -102,7 +102,7 @@ mgt_vcl_cache_vmod(const char *nm, const char *fm, const char *to)
 			break;
 		if (sz < 0 || sz != write(fo, buf, sz)) {
 			fprintf(stderr, "Copying vmod %s: %s\n",
-			    nm, vstrerror(errno));
+			    nm, VAS_errtxt(errno));
 			AZ(unlink(to));
 			ret = 1;
 			break;
@@ -165,13 +165,13 @@ mgt_vcl_symtab(struct vclprog *vp, const char *input)
 	AZ(err);
 	AN(vj);
 	vp->symtab = vj;
-	assert(vj->value->type == VJSN_ARRAY);
+	assert(vjsn_is_array(vj->value));
 	VTAILQ_FOREACH(v1, &vj->value->children, list) {
-		assert(v1->type == VJSN_OBJECT);
+		assert(vjsn_is_object(v1));
 		v2 = vjsn_child(v1, "dir");
 		if (v2 == NULL)
 			continue;
-		assert(v2->type == VJSN_STRING);
+		assert(vjsn_is_string(v2));
 		if (strcmp(v2->value, "import"))
 			continue;
 		typ = mgt_vcl_symtab_val(v1, "type");
