@@ -33,6 +33,7 @@
 
 #include "config.h"
 
+#include <fnmatch.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -60,8 +61,9 @@ static const struct vclstate VCL_STATE_LABEL[1] = {{ "label" }};
 static unsigned vcl_count;
 
 struct vclproghead vclhead = VTAILQ_HEAD_INITIALIZER(vclhead);
+static struct vclproghead discardhead = VTAILQ_HEAD_INITIALIZER(discardhead);
 struct vmodfilehead vmodhead = VTAILQ_HEAD_INITIALIZER(vmodhead);
-static struct vclprog		*active_vcl;
+static struct vclprog *mgt_vcl_active;
 static struct vev *e_poker;
 
 static int mgt_vcl_setstate(struct cli *, struct vclprog *,
@@ -128,7 +130,7 @@ mcf_find_vcl(struct cli *cli, const char *name)
 	vp = mcf_vcl_byname(name);
 	if (vp == NULL) {
 		VCLI_SetResult(cli, CLIS_PARAM);
-		VCLI_Out(cli, "No VCL named %s known.", name);
+		VCLI_Out(cli, "No VCL named %s known\n", name);
 	}
 	return (vp);
 }
@@ -226,14 +228,17 @@ mgt_vcl_del(struct vclprog *vp)
 	char *p;
 	struct vmoddep *vd;
 	struct vmodfile *vf;
+	struct vcldep *dep;
 
 	CHECK_OBJ_NOTNULL(vp, VCLPROG_MAGIC);
 	assert(VTAILQ_EMPTY(&vp->dto));
 
 	mgt_vcl_symtab_clean(vp);
 
-	while (!VTAILQ_EMPTY(&vp->dfrom))
-		mgt_vcl_dep_del(VTAILQ_FIRST(&vp->dfrom));
+	while ((dep = VTAILQ_FIRST(&vp->dfrom)) != NULL) {
+		assert(dep->from == vp);
+		mgt_vcl_dep_del(dep);
+	}
 
 	VTAILQ_REMOVE(&vclhead, vp, list);
 	if (vp->state != VCL_STATE_LABEL)
@@ -276,11 +281,16 @@ mgt_vcl_del(struct vclprog *vp)
 	FREE_OBJ(vp);
 }
 
-int
+const char *
 mgt_has_vcl(void)
 {
-
-	return (!VTAILQ_EMPTY(&vclhead));
+	if (VTAILQ_EMPTY(&vclhead))
+		return ("No VCL loaded");
+	if (mgt_vcl_active == NULL)
+		return ("No active VCL");
+	CHECK_OBJ_NOTNULL(mgt_vcl_active, VCLPROG_MAGIC);
+	AN(mgt_vcl_active->warm);
+	return (NULL);
 }
 
 /*
@@ -295,7 +305,7 @@ mgt_vcl_set_cooldown(struct vclprog *vp, vtim_mono now)
 {
 	CHECK_OBJ_NOTNULL(vp, VCLPROG_MAGIC);
 
-	if (vp == active_vcl ||
+	if (vp == mgt_vcl_active ||
 	    vp->state != VCL_STATE_AUTO ||
 	    vp->warm == 0 ||
 	    !VTAILQ_EMPTY(&vp->dto) ||
@@ -396,7 +406,7 @@ mgt_vcl_setstate(struct cli *cli, struct vclprog *vp, const struct vclstate *vs)
 	os = vp->state;
 	vp->state = vs;
 
-	if (vp == active_vcl) {
+	if (vp == mgt_vcl_active) {
 		assert (vs == VCL_STATE_WARM || vs == VCL_STATE_AUTO);
 		AN(vp->warm);
 		warm = 1;
@@ -416,7 +426,7 @@ mgt_vcl_setstate(struct cli *cli, struct vclprog *vp, const struct vclstate *vs)
 
 /*--------------------------------------------------------------------*/
 
-static void
+static struct vclprog *
 mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
     const char *vclsrcfile, const char *state, int C_flag)
 {
@@ -432,7 +442,7 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 		VCLI_Out(cli, "Too many (%d) VCLs already loaded\n", vcl_count);
 		VCLI_Out(cli, "(See max_vcl and max_vcl_handling parameters)");
 		VCLI_SetResult(cli, CLIS_CANT);
-		return;
+		return (NULL);
 	}
 
 	if (state == NULL)
@@ -441,21 +451,18 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 		vs = mcf_vcl_parse_state(cli, state);
 
 	if (vs == NULL)
-		return;
+		return (NULL);
 
 	vp = mgt_vcl_add(vclname, vs);
 	lib = mgt_VccCompile(cli, vp, vclname, vclsrc, vclsrcfile, C_flag);
 	if (lib == NULL) {
 		mgt_vcl_del(vp);
-		return;
+		return (NULL);
 	}
 
 	AZ(C_flag);
 	vp->fname = lib;
 
-	if (active_vcl == NULL)
-		active_vcl = vp;
-
 	if ((cli->result == CLIS_OK || cli->result == CLIS_TRUNCATED) &&
 	    vcl_count > mgt_param.max_vcl &&
 	    mgt_param.max_vcl_handling == 1) {
@@ -465,7 +472,7 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 	}
 
 	if (!MCH_Running())
-		return;
+		return (vp);
 
 	if (mgt_cli_askchild(&status, &p, "vcl.load %s %s %d%s\n",
 	    vp->name, vp->fname, vp->warm, vp->state->name)) {
@@ -473,11 +480,12 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 		VCLI_Out(cli, "%s", p);
 		VCLI_SetResult(cli, status);
 		free(p);
-		return;
+		return (NULL);
 	}
 	free(p);
 
 	mgt_vcl_set_cooldown(vp, VTIM_mono());
+	return (vp);
 }
 
 /*--------------------------------------------------------------------*/
@@ -488,6 +496,9 @@ mgt_vcl_startup(struct cli *cli, const char *vclsrc, const char *vclname,
 {
 	char buf[20];
 	static int n = 0;
+	struct vclprog *vp;
+
+	AZ(MCH_Running());
 
 	AN(vclsrc);
 	AN(origin);
@@ -495,8 +506,13 @@ mgt_vcl_startup(struct cli *cli, const char *vclsrc, const char *vclname,
 		bprintf(buf, "boot%d", n++);
 		vclname = buf;
 	}
-	active_vcl = NULL;
-	mgt_new_vcl(cli, vclname, vclsrc, origin, NULL, C_flag);
+	vp = mgt_new_vcl(cli, vclname, vclsrc, origin, NULL, C_flag);
+	if (vp != NULL) {
+		/* Last startup VCL becomes the automatically selected
+		 * active VCL. */
+		AN(vp->warm);
+		mgt_vcl_active = vp;
+	}
 }
 
 /*--------------------------------------------------------------------*/
@@ -508,7 +524,7 @@ mgt_push_vcls(struct cli *cli, unsigned *status, char **p)
 	struct vcldep *vd;
 	int done;
 
-	AN(active_vcl);
+	AN(mgt_vcl_active);
 
 	/* The VCL has not been loaded yet, it cannot fail */
 	(void)cli;
@@ -548,8 +564,10 @@ mgt_push_vcls(struct cli *cli, unsigned *status, char **p)
 		}
 	} while (!done);
 
-	if (mgt_cli_askchild(status, p, "vcl.use \"%s\"\n", active_vcl->name))
+	if (mgt_cli_askchild(status, p, "vcl.use \"%s\"\n",
+	      mgt_vcl_active->name)) {
 		return (1);
+	}
 	free(*p);
 	*p = NULL;
 	return (0);
@@ -560,24 +578,30 @@ mgt_push_vcls(struct cli *cli, unsigned *status, char **p)
 static void v_matchproto_(cli_func_t)
 mcf_vcl_inline(struct cli *cli, const char * const *av, void *priv)
 {
+	struct vclprog *vp;
 
 	(void)priv;
 
 	if (!mcf_find_no_vcl(cli, av[2]))
 		return;
 
-	mgt_new_vcl(cli, av[2], av[3], "<vcl.inline>", av[4], 0);
+	vp = mgt_new_vcl(cli, av[2], av[3], "<vcl.inline>", av[4], 0);
+	if (vp != NULL && !MCH_Running())
+		VCLI_Out(cli, "VCL compiled.\n");
 }
 
 static void v_matchproto_(cli_func_t)
 mcf_vcl_load(struct cli *cli, const char * const *av, void *priv)
 {
+	struct vclprog *vp;
 
 	(void)priv;
 	if (!mcf_find_no_vcl(cli, av[2]))
 		return;
 
-	mgt_new_vcl(cli, av[2], NULL, av[3], av[4], 0);
+	vp = mgt_new_vcl(cli, av[2], NULL, av[3], av[4], 0);
+	if (vp != NULL && !MCH_Running())
+		VCLI_Out(cli, "VCL compiled.\n");
 }
 
 
@@ -609,7 +633,7 @@ mcf_vcl_state(struct cli *cli, const char * const *av, void *priv)
 			VCLI_SetResult(cli, CLIS_CANT);
 			return;
 		}
-		if (vp == active_vcl) {
+		if (vp == mgt_vcl_active) {
 			VCLI_Out(cli, "Cannot set the active VCL cold.");
 			VCLI_SetResult(cli, CLIS_CANT);
 			return;
@@ -631,7 +655,7 @@ mcf_vcl_use(struct cli *cli, const char * const *av, void *priv)
 	vp = mcf_find_vcl(cli, av[2]);
 	if (vp == NULL)
 		return;
-	if (vp == active_vcl)
+	if (vp == mgt_vcl_active)
 		return;
 
 	if (mgt_vcl_requirewarm(cli, vp))
@@ -643,8 +667,8 @@ mcf_vcl_use(struct cli *cli, const char * const *av, void *priv)
 		VCLI_Out(cli, "%s", p);
 	} else {
 		VCLI_Out(cli, "VCL '%s' now active", av[2]);
-		vp2 = active_vcl;
-		active_vcl = vp;
+		vp2 = mgt_vcl_active;
+		mgt_vcl_active = vp;
 		now = VTIM_mono();
 		mgt_vcl_set_cooldown(vp, now);
 		if (vp2 != NULL)
@@ -653,43 +677,19 @@ mcf_vcl_use(struct cli *cli, const char * const *av, void *priv)
 	free(p);
 }
 
-static void v_matchproto_(cli_func_t)
-mcf_vcl_discard(struct cli *cli, const char * const *av, void *priv)
+static void
+mgt_vcl_discard(struct cli *cli, struct vclprog *vp)
 {
-	unsigned status;
 	char *p = NULL;
-	struct vclprog *vp;
-	struct vcldep *vd;
-	int n;
+	unsigned status;
+
+	AN(vp);
+	AN(vp->discard);
+	assert(vp != mgt_vcl_active);
+
+	while (!VTAILQ_EMPTY(&vp->dto))
+		mgt_vcl_discard(cli, VTAILQ_FIRST(&vp->dto)->from);
 
-	(void)priv;
-	vp = mcf_find_vcl(cli, av[2]);
-	if (vp == NULL)
-		return;
-	if (vp == active_vcl) {
-		VCLI_SetResult(cli, CLIS_CANT);
-		VCLI_Out(cli, "Cannot discard active VCL program\n");
-		return;
-	}
-	if (!VTAILQ_EMPTY(&vp->dto)) {
-		VCLI_SetResult(cli, CLIS_CANT);
-		AN(vp->warm);
-		if (!mcf_is_label(vp))
-			VCLI_Out(cli, "Cannot discard labeled VCL program.\n");
-		else
-			VCLI_Out(cli,
-			    "Cannot discard this VCL label, "
-			    "other VCLs depend on it.\n");
-		n = 0;
-		VTAILQ_FOREACH(vd, &vp->dto, lto) {
-			if (n++ == 5) {
-				VCLI_Out(cli, "\t[...]");
-				break;
-			}
-			VCLI_Out(cli, "\t%s\n", vd->from->name);
-		}
-		return;
-	}
 	if (mcf_is_label(vp)) {
 		AN(vp->warm);
 		vp->warm = 0;
@@ -698,13 +698,126 @@ mcf_vcl_discard(struct cli *cli, const char * const *av, void *priv)
 	}
 	if (MCH_Running()) {
 		AZ(vp->warm);
-		if (mgt_cli_askchild(&status, &p, "vcl.discard %s\n", av[2]))
+		if (mgt_cli_askchild(&status, &p, "vcl.discard %s\n", vp->name))
 			assert(status == CLIS_OK || status == CLIS_COMMS);
 		free(p);
 	}
+	VTAILQ_REMOVE(&discardhead, vp, discard_list);
 	mgt_vcl_del(vp);
 }
 
+static int
+mgt_vcl_discard_mark(struct cli *cli, const char *glob)
+{
+	struct vclprog *vp;
+	unsigned marked = 0;
+
+	VTAILQ_FOREACH(vp, &vclhead, list) {
+		if (fnmatch(glob, vp->name, 0))
+			continue;
+		if (vp == mgt_vcl_active) {
+			VCLI_SetResult(cli, CLIS_CANT);
+			VCLI_Out(cli, "Cannot discard active VCL program %s\n",
+			    vp->name);
+			return (-1);
+		}
+		if (!vp->discard)
+			VTAILQ_INSERT_TAIL(&discardhead, vp, discard_list);
+		vp->discard = 1;
+		marked++;
+	}
+
+	if (marked == 0) {
+		VCLI_SetResult(cli, CLIS_PARAM);
+		VCLI_Out(cli, "No VCL name matches %s\n", glob);
+	}
+
+	return (marked);
+}
+
+static void
+mgt_vcl_discard_depfail(struct cli *cli, struct vclprog *vp)
+{
+	struct vcldep *vd;
+	int n;
+
+	AN(cli);
+	AN(vp);
+	assert(!VTAILQ_EMPTY(&vp->dto));
+
+	VCLI_SetResult(cli, CLIS_CANT);
+	AN(vp->warm);
+	if (!mcf_is_label(vp))
+		VCLI_Out(cli, "Cannot discard labeled VCL program %s:\n",
+		    vp->name);
+	else
+		VCLI_Out(cli,
+		    "Cannot discard VCL label %s, other VCLs depend on it:\n",
+		    vp->name);
+	n = 0;
+	VTAILQ_FOREACH(vd, &vp->dto, lto) {
+		if (n++ == 5) {
+			VCLI_Out(cli, "\t[...]");
+			break;
+		}
+		VCLI_Out(cli, "\t%s\n", vd->from->name);
+	}
+}
+
+static int
+mgt_vcl_discard_depcheck(struct cli *cli)
+{
+	struct vclprog *vp;
+	struct vcldep *vd;
+
+	VTAILQ_FOREACH(vp, &discardhead, discard_list) {
+		VTAILQ_FOREACH(vd, &vp->dto, lto)
+			if (!vd->from->discard) {
+				mgt_vcl_discard_depfail(cli, vp);
+				return (-1);
+			}
+	}
+
+	return (0);
+}
+
+static void
+mgt_vcl_discard_clear(void)
+{
+	struct vclprog *vp, *vp2;
+
+	VTAILQ_FOREACH_SAFE(vp, &discardhead, discard_list, vp2) {
+		AN(vp->discard);
+		vp->discard = 0;
+		VTAILQ_REMOVE(&discardhead, vp, discard_list);
+	}
+}
+
+static void v_matchproto_(cli_func_t)
+mcf_vcl_discard(struct cli *cli, const char * const *av, void *priv)
+{
+	const struct vclprog *vp;
+
+	(void)priv;
+
+	assert(VTAILQ_EMPTY(&discardhead));
+	VTAILQ_FOREACH(vp, &vclhead, list)
+		AZ(vp->discard);
+
+	for (av += 2; *av != NULL; av++) {
+		if (mgt_vcl_discard_mark(cli, *av) <= 0) {
+			mgt_vcl_discard_clear();
+			break;
+		}
+	}
+
+	if (mgt_vcl_discard_depcheck(cli) != 0)
+		mgt_vcl_discard_clear();
+
+	while (!VTAILQ_EMPTY(&discardhead))
+		mgt_vcl_discard(cli, VTAILQ_FIRST(&discardhead));
+}
+
 static void v_matchproto_(cli_func_t)
 mcf_vcl_list(struct cli *cli, const char * const *av, void *priv)
 {
@@ -732,7 +845,7 @@ mcf_vcl_list(struct cli *cli, const char * const *av, void *priv)
 
 		VTAILQ_FOREACH(vp, &vclhead, list) {
 			VSB_printf(vsb, "%s",
-			    vp == active_vcl ? "active" : "available");
+			    vp == mgt_vcl_active ? "active" : "available");
 			vs = vp->warm ?  VCL_STATE_WARM : VCL_STATE_COLD;
 			VSB_printf(vsb, "\t%s\t%s", vp->state->name, vs->name);
 			VSB_printf(vsb, "\t%6s\t%s", "-", vp->name);
@@ -747,7 +860,7 @@ mcf_vcl_list(struct cli *cli, const char * const *av, void *priv)
 				VSB_printf(vsb, "\t<-\t(%d label%s)",
 				    vp->nto, vp->nto > 1 ? "s" : "");
 			}
-			VSB_printf(vsb, "\n");
+			VSB_cat(vsb, "\n");
 		}
 		VCLI_VTE(cli, &vsb, 80);
 	}
@@ -773,12 +886,12 @@ mcf_vcl_list_json(struct cli *cli, const char * const *av, void *priv)
 		free(p);
 	} else {
 		VCLI_JSON_begin(cli, 2, av);
-		VCLI_Out(cli, ",\n");
 		VTAILQ_FOREACH(vp, &vclhead, list) {
+			VCLI_Out(cli, ",\n");
 			VCLI_Out(cli, "{\n");
 			VSB_indent(cli->sb, 2);
 			VCLI_Out(cli, "\"status\": \"%s\",\n",
-			    vp == active_vcl ? "active" : "available");
+			    vp == mgt_vcl_active ? "active" : "available");
 			VCLI_Out(cli, "\"state\": \"%s\",\n", vp->state->name);
 			vs = vp->warm ?  VCL_STATE_WARM : VCL_STATE_COLD;
 			VCLI_Out(cli, "\"temperature\": \"%s\",\n", vs->name);
@@ -802,8 +915,6 @@ mcf_vcl_list_json(struct cli *cli, const char * const *av, void *priv)
 			}
 			VSB_indent(cli->sb, -2);
 			VCLI_Out(cli, "\n}");
-			if (VTAILQ_NEXT(vp, list) != NULL)
-				VCLI_Out(cli, ",\n");
 		}
 		VCLI_JSON_end(cli);
 	}
@@ -827,7 +938,7 @@ mcf_vcl_check_dag(struct cli *cli, struct vclprog *tree, struct vclprog *target)
 			    tree->name, vd->to->name);
 		return (1);
 	}
-	return(0);
+	return (0);
 }
 
 static void v_matchproto_(cli_func_t)
@@ -896,6 +1007,66 @@ mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 	free(p);
 }
 
+static void v_matchproto_(cli_func_t)
+mcf_vcl_deps(struct cli *cli, const char * const *av, void *priv)
+{
+	struct vclprog *vp;
+	struct vcldep *vd;
+	struct vsb *vsb;
+
+	(void)av;
+	(void)priv;
+
+	vsb = VSB_new_auto();
+	AN(vsb);
+
+	VTAILQ_FOREACH(vp, &vclhead, list) {
+		if (VTAILQ_EMPTY(&vp->dfrom)) {
+			VSB_printf(vsb, "%s\n", vp->name);
+			continue;
+		}
+		VTAILQ_FOREACH(vd, &vp->dfrom, lfrom)
+			VSB_printf(vsb, "%s\t%s\n", vp->name, vd->to->name);
+	}
+	VCLI_VTE(cli, &vsb, 80);
+}
+
+static void v_matchproto_(cli_func_t)
+mcf_vcl_deps_json(struct cli *cli, const char * const *av, void *priv)
+{
+	struct vclprog *vp;
+	struct vcldep *vd;
+	const char *sepd, *sepa;
+
+	(void)priv;
+
+	VCLI_JSON_begin(cli, 1, av);
+
+	VTAILQ_FOREACH(vp, &vclhead, list) {
+		VCLI_Out(cli, ",\n");
+		VCLI_Out(cli, "{\n");
+		VSB_indent(cli->sb, 2);
+		VCLI_Out(cli, "\"name\": \"%s\",\n", vp->name);
+		VCLI_Out(cli, "\"deps\": [");
+		VSB_indent(cli->sb, 2);
+		sepd = "";
+		sepa = "";
+		VTAILQ_FOREACH(vd, &vp->dfrom, lfrom) {
+			VCLI_Out(cli, "%s\n", sepd);
+			VCLI_Out(cli, "\"%s\"", vd->to->name);
+			sepd = ",";
+			sepa = "\n";
+		}
+		VSB_indent(cli->sb, -2);
+		VCLI_Out(cli, "%s", sepa);
+		VCLI_Out(cli, "]\n");
+		VSB_indent(cli->sb, -2);
+		VCLI_Out(cli, "}");
+	}
+
+	VCLI_JSON_end(cli);
+}
+
 /*--------------------------------------------------------------------*/
 
 static int v_matchproto_(vev_cb_f)
@@ -927,6 +1098,7 @@ static struct cli_proto cli_vcl[] = {
 	{ CLICMD_VCL_STATE,		"", mcf_vcl_state },
 	{ CLICMD_VCL_DISCARD,		"", mcf_vcl_discard },
 	{ CLICMD_VCL_LIST,		"", mcf_vcl_list, mcf_vcl_list_json },
+	{ CLICMD_VCL_DEPS,		"", mcf_vcl_deps, mcf_vcl_deps_json },
 	{ CLICMD_VCL_LABEL,		"", mcf_vcl_label },
 	{ CLICMD_DEBUG_VCL_SYMTAB,	"d", mcf_vcl_symtab },
 	{ NULL }
@@ -941,7 +1113,7 @@ mgt_vcl_atexit(void)
 
 	if (getpid() != heritage.mgt_pid)
 		return;
-	active_vcl = NULL;
+	mgt_vcl_active = NULL;
 	while (!VTAILQ_EMPTY(&vclhead))
 		VTAILQ_FOREACH_SAFE(vp, &vclhead, list, vp2)
 			if (VTAILQ_EMPTY(&vp->dto))
