@@ -137,7 +137,7 @@ smp_metrics(struct smp_sc *sc)
  * Set up persistent storage silo in the master process.
  */
 
-void
+void v_matchproto_(storage_init_f)
 smp_mgt_init(struct stevedore *parent, int ac, char * const *av)
 {
 	struct smp_sc		*sc;
@@ -200,7 +200,7 @@ smp_mgt_init(struct stevedore *parent, int ac, char * const *av)
 	/* Try to determine correct mmap address */
 	i = read(sc->fd, &sgn, sizeof sgn);
 	assert(i == sizeof sgn);
-	if (!strcmp(sgn.ident, "SILO"))
+	if (!memcmp(sgn.ident, "SILO", 5))
 		target = (void*)(uintptr_t)sgn.mapped;
 	else
 		target = NULL;
@@ -210,7 +210,7 @@ smp_mgt_init(struct stevedore *parent, int ac, char * const *av)
 
 	if (sc->base == MAP_FAILED)
 		ARGV_ERR("(-spersistent) failed to mmap (%s)\n",
-		    vstrerror(errno));
+		    VAS_errtxt(errno));
 	if (target != NULL && sc->base != target)
 		fprintf(stderr, "WARNING: Persistent silo lost to ASLR %s\n",
 		    sc->filename);
