@@ -47,21 +47,6 @@ enum baninfo {
 	BI_DROP
 };
 
-/* Storage -----------------------------------------------------------*/
-
-struct storage {
-	unsigned		magic;
-#define STORAGE_MAGIC		0x1a4e51c0
-
-
-	VTAILQ_ENTRY(storage)	list;
-	void			*priv;
-
-	unsigned char		*ptr;
-	unsigned		len;
-	unsigned		space;
-};
-
 /* Prototypes --------------------------------------------------------*/
 
 typedef void storage_init_f(struct stevedore *, int ac, char * const *av);
@@ -75,7 +60,12 @@ typedef void storage_banexport_f(const struct stevedore *, const uint8_t *bans,
     unsigned len);
 typedef void storage_panic_f(struct vsb *vsb, const struct objcore *oc);
 
+typedef void *storage_allocbuf_f(struct worker *, const struct stevedore *,
+    size_t size, uintptr_t *ppriv);
+typedef void storage_freebuf_f(struct worker *, const struct stevedore *,
+    uintptr_t priv);
 
+struct storage;
 typedef struct object *sml_getobj_f(struct worker *, struct objcore *);
 typedef struct storage *sml_alloc_f(const struct stevedore *, size_t size);
 typedef void sml_free_f(struct storage *);
@@ -88,46 +78,48 @@ typedef void sml_free_f(struct storage *);
 /*--------------------------------------------------------------------*/
 
 struct stevedore {
-	unsigned		magic;
-#define STEVEDORE_MAGIC		0x4baf43db
-	const char		*name;
+	unsigned			magic;
+#define STEVEDORE_MAGIC			0x4baf43db
+	const char			*name;
 
 	/* Called in MGT process */
-	storage_init_f		*init;
+	storage_init_f			*init;
 
 	/* Called in cache process
 	 * only allocobj is required, other callbacks are optional
 	 */
-	storage_open_f		*open;
-	storage_close_f		*close;
-	storage_allocobj_f	*allocobj;
-	storage_baninfo_f	*baninfo;
-	storage_banexport_f	*banexport;
-	storage_panic_f		*panic;
+	storage_open_f			*open;
+	storage_close_f			*close;
+	storage_allocobj_f		*allocobj;
+	storage_baninfo_f		*baninfo;
+	storage_banexport_f		*banexport;
+	storage_panic_f			*panic;
+	storage_allocbuf_f		*allocbuf;
+	storage_freebuf_f		*freebuf;
 
 	/* Only if SML is used */
-	sml_alloc_f		*sml_alloc;
-	sml_free_f		*sml_free;
-	sml_getobj_f		*sml_getobj;
+	sml_alloc_f			*sml_alloc;
+	sml_free_f			*sml_free;
+	sml_getobj_f			*sml_getobj;
 
-	const struct obj_methods
-				*methods;
+	const struct obj_methods	*methods;
 
 	/* Only if LRU is used */
-	struct lru		*lru;
+	struct lru			*lru;
 
 #define VRTSTVVAR(nm, vtype, ctype, dval) stv_var_##nm *var_##nm;
 #include "tbl/vrt_stv_var.h"
 
 	/* private fields for the stevedore */
-	void			*priv;
+	void				*priv;
 
 	VTAILQ_ENTRY(stevedore)	list;
-	const char		*ident;
-	const char		*vclname;
+	const char			*ident;
+	const char			*vclname;
 };
 
 extern struct stevedore *stv_transient;
+extern struct stevedore *stv_h2_rxbuf;
 
 /*--------------------------------------------------------------------*/
 
@@ -151,5 +143,6 @@ void LRU_Touch(struct worker *, struct objcore *, vtim_real now);
 /*--------------------------------------------------------------------*/
 extern const struct stevedore smu_stevedore;
 extern const struct stevedore sma_stevedore;
+extern const struct stevedore smd_stevedore;
 extern const struct stevedore smf_stevedore;
 extern const struct stevedore smp_stevedore;
