@@ -46,6 +46,10 @@
 
 /*-------------------------------------------------------------------*/
 
+static struct storage *
+objallocwithnuke(struct worker *, const struct stevedore *, ssize_t size,
+    int flags);
+
 static struct storage *
 sml_stv_alloc(const struct stevedore *stv, ssize_t size, int flags)
 {
@@ -161,6 +165,39 @@ SML_allocobj(struct worker *wrk, const struct stevedore *stv,
 	return (1);
 }
 
+void * v_matchproto_(storage_allocbuf_t)
+SML_AllocBuf(struct worker *wrk, const struct stevedore *stv, size_t size,
+    uintptr_t *ppriv)
+{
+	struct storage *st;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(stv, STEVEDORE_MAGIC);
+	AN(ppriv);
+
+	if (size > UINT_MAX)
+		return (NULL);
+	st = objallocwithnuke(wrk, stv, size, 0);
+	if (st == NULL)
+		return (NULL);
+	assert(st->space >= size);
+	st->len = size;
+	*ppriv = (uintptr_t)st;
+	return (st->ptr);
+}
+
+void v_matchproto_(storage_freebuf_t)
+SML_FreeBuf(struct worker *wrk, const struct stevedore *stv, uintptr_t priv)
+{
+	struct storage *st;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(stv, STEVEDORE_MAGIC);
+
+	CAST_OBJ_NOTNULL(st, (void *)priv, STORAGE_MAGIC);
+	sml_stv_free(stv, st);
+}
+
 /*---------------------------------------------------------------------
  */
 
@@ -196,11 +233,13 @@ sml_slim(struct worker *wrk, struct objcore *oc)
 	o = sml_getobj(wrk, oc);
 	CHECK_OBJ_NOTNULL(o, OBJECT_MAGIC);
 
-#define OBJ_AUXATTR(U, l)						\
-	if (o->aa_##l != NULL) {					\
-		sml_stv_free(stv, o->aa_##l);				\
-		o->aa_##l = NULL;					\
-	}
+#define OBJ_AUXATTR(U, l)					\
+	do {							\
+		if (o->aa_##l != NULL) {			\
+			sml_stv_free(stv, o->aa_##l);		\
+			o->aa_##l = NULL;			\
+		}						\
+	} while (0);
 #include "tbl/obj_attr.h"
 
 	VTAILQ_FOREACH_SAFE(st, &o->list, list, stn) {
@@ -213,25 +252,28 @@ sml_slim(struct worker *wrk, struct objcore *oc)
 static void v_matchproto_(objfree_f)
 sml_objfree(struct worker *wrk, struct objcore *oc)
 {
+	struct storage *st;
 	struct object *o;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	sml_slim(wrk, oc);
 	CAST_OBJ_NOTNULL(o, oc->stobj->priv, OBJECT_MAGIC);
-	o->magic = 0;
+	st = o->objstore;
+	CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
+	FINI_OBJ(o);
 
 	if (oc->boc == NULL && oc->stobj->stevedore->lru != NULL)
 		LRU_Remove(oc);
 
-	sml_stv_free(oc->stobj->stevedore, o->objstore);
+	sml_stv_free(oc->stobj->stevedore, st);
 
 	memset(oc->stobj, 0, sizeof oc->stobj);
 
 	wrk->stats->n_object--;
 }
 
-static int v_matchproto_(objiterate_f)
+static int v_matchproto_(objiterator_f)
 sml_iterator(struct worker *wrk, struct objcore *oc,
     void *priv, objiterate_f *func, int final)
 {
@@ -259,9 +301,11 @@ sml_iterator(struct worker *wrk, struct objcore *oc,
 	boc = HSH_RefBoc(oc);
 
 	if (boc == NULL) {
-		VTAILQ_FOREACH_SAFE(st, &obj->list, list, checkpoint) {
+		VTAILQ_FOREACH_REVERSE_SAFE(
+		    st, &obj->list, storagehead, list, checkpoint) {
+
 			u = 0;
-			if (VTAILQ_NEXT(st, list) == NULL)
+			if (VTAILQ_PREV(st, storagehead, list) == NULL)
 				u |= OBJ_ITER_END;
 			if (final)
 				u |= OBJ_ITER_FLUSH;
@@ -280,6 +324,11 @@ sml_iterator(struct worker *wrk, struct objcore *oc,
 	l = 0;
 
 	u = 0;
+	if (boc->len_so_far == 0) {
+		ret = func(priv, OBJ_ITER_FLUSH, NULL, 0);
+		if (ret)
+			return (ret);
+	}
 	while (1) {
 		ol = len;
 		nl = ObjWaitExtend(wrk, oc, ol);
@@ -295,7 +344,7 @@ sml_iterator(struct worker *wrk, struct objcore *oc,
 		Lck_Lock(&boc->mtx);
 		AZ(VTAILQ_EMPTY(&obj->list));
 		if (checkpoint == NULL) {
-			st = VTAILQ_FIRST(&obj->list);
+			st = VTAILQ_LAST(&obj->list, storagehead);
 			sl = 0;
 		} else {
 			st = checkpoint;
@@ -314,8 +363,8 @@ sml_iterator(struct worker *wrk, struct objcore *oc,
 			nl -= st->len;
 			assert(nl > 0);
 			sl += st->len;
-			st = VTAILQ_NEXT(st, list);
-			if (VTAILQ_NEXT(st, list) != NULL) {
+			st = VTAILQ_PREV(st, storagehead, list);
+			if (VTAILQ_PREV(st, storagehead, list) != NULL) {
 				if (final && checkpoint != NULL) {
 					VTAILQ_REMOVE(&obj->list,
 					    checkpoint, list);
@@ -327,7 +376,7 @@ sml_iterator(struct worker *wrk, struct objcore *oc,
 		}
 		CHECK_OBJ_NOTNULL(obj, OBJECT_MAGIC);
 		CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
-		st = VTAILQ_NEXT(st, list);
+		st = VTAILQ_PREV(st, storagehead, list);
 		if (st != NULL && st->len == 0)
 			st = NULL;
 		state = boc->state;
@@ -402,7 +451,7 @@ sml_getspace(struct worker *wrk, struct objcore *oc, ssize_t *sz,
 	CHECK_OBJ_NOTNULL(o, OBJECT_MAGIC);
 	CHECK_OBJ_NOTNULL(oc->boc, BOC_MAGIC);
 
-	st = VTAILQ_LAST(&o->list, storagehead);
+	st = VTAILQ_FIRST(&o->list);
 	if (st != NULL && st->len < st->space) {
 		*sz = st->space - st->len;
 		*ptr = st->ptr + st->len;
@@ -417,7 +466,7 @@ sml_getspace(struct worker *wrk, struct objcore *oc, ssize_t *sz,
 
 	CHECK_OBJ_NOTNULL(oc->boc, BOC_MAGIC);
 	Lck_Lock(&oc->boc->mtx);
-	VTAILQ_INSERT_TAIL(&o->list, st, list);
+	VTAILQ_INSERT_HEAD(&o->list, st, list);
 	Lck_Unlock(&oc->boc->mtx);
 
 	*sz = st->space - st->len;
@@ -437,7 +486,7 @@ sml_extend(struct worker *wrk, struct objcore *oc, ssize_t l)
 
 	o = sml_getobj(wrk, oc);
 	CHECK_OBJ_NOTNULL(o, OBJECT_MAGIC);
-	st = VTAILQ_LAST(&o->list, storagehead);
+	st = VTAILQ_FIRST(&o->list);
 	CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
 	assert(st->len + l <= st->space);
 	st->len += l;
@@ -462,7 +511,7 @@ sml_trimstore(struct worker *wrk, struct objcore *oc)
 
 	o = sml_getobj(wrk, oc);
 	CHECK_OBJ_NOTNULL(o, OBJECT_MAGIC);
-	st = VTAILQ_LAST(&o->list, storagehead);
+	st = VTAILQ_FIRST(&o->list);
 
 	if (st == NULL)
 		return;
@@ -487,7 +536,7 @@ sml_trimstore(struct worker *wrk, struct objcore *oc)
 	st1->len = st->len;
 	Lck_Lock(&oc->boc->mtx);
 	VTAILQ_REMOVE(&o->list, st, list);
-	VTAILQ_INSERT_TAIL(&o->list, st1, list);
+	VTAILQ_INSERT_HEAD(&o->list, st1, list);
 	Lck_Unlock(&oc->boc->mtx);
 	/* sml_bocdone frees this */
 	AZ(oc->boc->stevedore_priv);
@@ -681,8 +730,10 @@ SML_panic(struct vsb *vsb, const struct objcore *oc)
 	VSB_printf(vsb, "%s = {len=%u, ptr=%p},\n", \
 	    #U, o->va_##l##_len, o->va_##l);
 
-#define OBJ_AUXATTR(U, l) \
-	if (o->aa_##l != NULL) sml_panic_st(vsb, #U, o->aa_##l);
+#define OBJ_AUXATTR(U, l)						\
+	do {								\
+		if (o->aa_##l != NULL) sml_panic_st(vsb, #U, o->aa_##l);\
+	} while(0);
 
 #include "tbl/obj_attr.h"
 
