@@ -104,8 +104,7 @@ vwk_thread(void *priv)
 			Wait_Call(w, wp, WAITER_TIMEOUT, now);
 		}
 		then = vwk->next - now;
-		ts.tv_sec = (time_t)floor(then);
-		ts.tv_nsec = (long)(1e9 * (then - ts.tv_sec));
+		ts = VTIM_timespec(then);
 		Lck_Unlock(&vwk->mtx);
 		n = kevent(vwk->kq, NULL, 0, ke, NKEV, &ts);
 		assert(n >= 0);
@@ -113,11 +112,11 @@ vwk_thread(void *priv)
 		now = VTIM_real();
 		for (kp = ke, j = 0; j < n; j++, kp++) {
 			assert(kp->filter == EVFILT_READ);
-			if (ke[j].udata == vwk) {
+			if ((uintptr_t)ke[j].udata == (uintptr_t)vwk) {
 				assert(read(vwk->pipe[0], &c, 1) == 1);
 				continue;
 			}
-			CAST_OBJ_NOTNULL(wp, ke[j].udata, WAITED_MAGIC);
+			CAST_OBJ_NOTNULL(wp, (void*)ke[j].udata, WAITED_MAGIC);
 			Lck_Lock(&vwk->mtx);
 			AN(Wait_HeapDelete(w, wp));
 			Lck_Unlock(&vwk->mtx);
