@@ -75,7 +75,7 @@ static unsigned hist_buckets;
 static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
 
 static int end_of_file = 0;
-static int ms_delay = 1000;
+static unsigned ms_delay = 1000;
 static unsigned rr_hist[HIST_N];
 static unsigned nhist;
 static unsigned next_hist;
@@ -174,8 +174,7 @@ update(void)
 		assert(l < n);
 		bm[l] += bucket_miss[k];
 		bh[l] += bucket_hit[k];
-		if (bm[l] + bh[l] > max)
-			max = bm[l] + bh[l];
+		max = vmax(max, bm[l] + bh[l]);
 	}
 
 	/* scale,time */
@@ -210,7 +209,6 @@ update(void)
 inline static void
 upd_vsl_ts(const char *p)
 {
-	double t;
 
 	if (timebend == 0)
 		return;
@@ -220,10 +218,7 @@ upd_vsl_ts(const char *p)
 	if (p == NULL)
 		return;
 
-	t = strtod(p + 1, NULL);
-
-	if (t > vsl_ts)
-		vsl_ts = t;
+	vsl_ts = vmax_t(double, vsl_ts, strtod(p + 1, NULL));
 }
 
 static int v_matchproto_ (VSLQ_dispatch_f)
@@ -305,12 +300,9 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 			continue;
 
 		/* select bucket */
-		i = lround(HIST_RES * log(value) / log_ten);
-		if (i < hist_low * HIST_RES)
-			i = hist_low * HIST_RES;
-		if (i >= hist_high * HIST_RES)
-			i = hist_high * HIST_RES - 1;
-		i -= hist_low * HIST_RES;
+		i = vlimit_t(int, lround(HIST_RES * log(value) / log_ten),
+		    hist_low * HIST_RES, hist_high * HIST_RES - 1) -
+			hist_low * HIST_RES;
 		assert(i >= 0);
 		assert((unsigned)i < hist_buckets);
 
@@ -367,9 +359,7 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 
 	if (vsl_ts > vsl_to) {
 		double when = VTIM_real() + vsl_ts - vsl_to;
-		struct timespec ts;
-		ts.tv_nsec = (long)(modf(when, &t) * 1e9);
-		ts.tv_sec = (long)t;
+		struct timespec ts = VTIM_timespec(when);
 		i = pthread_cond_timedwait(&timebend_cv, &mtx, &ts);
 		assert(i == 0 || i == ETIMEDOUT);
 	}
@@ -434,9 +424,7 @@ do_curses(void *arg)
 			ms_delay = 1000U << (ch - '0');
 			break;
 		case '+':
-			ms_delay /= 2;
-			if (ms_delay < 1)
-				ms_delay = 1;
+			ms_delay = vmax(ms_delay >> 1, 1U);
 			break;
 		case '-':
 			ms_delay *= 2;
@@ -499,7 +487,7 @@ main(int argc, char **argv)
 			VUT_Usage(vut, &vopt_spec, 0);
 		case 'p':
 			ms_delay = lround(1e3 * strtod(optarg, NULL));
-			if (ms_delay <= 0)
+			if (ms_delay == 0)
 				VUT_Error(vut, 1, "-p: invalid '%s'", optarg);
 			break;
 		case 'P':
