@@ -140,15 +140,15 @@ VTAILQ_HEAD(vsl_watch_head, vsl_watch);
 static struct ctx {
 	/* Options */
 	int			a_opt;
-	int			b_opt;
-	int			c_opt;
-	int			E_opt;
 	char			*w_arg;
 
 	FILE			*fo;
 	struct vsb		*vsb;
 	unsigned		gen;
 	VTAILQ_HEAD(,format)	format;
+	int			quote_how;
+	char			*missing_string;
+	char			*missing_int;
 
 	/* State */
 	struct watch_head	watch_vcl_log;
@@ -167,7 +167,10 @@ openout(int append)
 {
 
 	AN(CTX.w_arg);
-	CTX.fo = fopen(CTX.w_arg, append ? "a" : "w");
+	if (!strcmp(CTX.w_arg, "-"))
+		CTX.fo = stdout;
+	else
+		CTX.fo = fopen(CTX.w_arg, append ? "a" : "w");
 	if (CTX.fo == NULL)
 		VUT_Error(vut, 1, "Can't open output file (%s)",
 		    strerror(errno));
@@ -202,9 +205,9 @@ vsb_fcat(struct vsb *vsb, const struct fragment *f, const char *dflt)
 {
 	if (f->gen == CTX.gen) {
 		assert(f->b <= f->e);
-		VSB_quote(vsb, f->b, f->e - f->b, VSB_QUOTE_ESCHEX);
+		VSB_quote(vsb, f->b, f->e - f->b, CTX.quote_how);
 	} else if (dflt)
-		VSB_quote(vsb, dflt, -1, VSB_QUOTE_ESCHEX);
+		VSB_quote(vsb, dflt, -1, CTX.quote_how);
 	else
 		return (-1);
 	return (0);
@@ -250,7 +253,7 @@ format_fragment(const struct format *format)
 	if (format->frag->gen != CTX.gen) {
 		if (format->string == NULL)
 			return (-1);
-		VSB_quote(CTX.vsb, format->string, -1, VSB_QUOTE_ESCHEX);
+		VSB_quote(CTX.vsb, format->string, -1, CTX.quote_how);
 		return (0);
 	}
 	AZ(vsb_fcat(CTX.vsb, format->frag, NULL));
@@ -260,10 +263,11 @@ format_fragment(const struct format *format)
 static int v_matchproto_(format_f)
 format_time(const struct format *format)
 {
-	double t_start, t_end;
+	double t_start, t_end, d;
 	char *p;
 	char buf[64];
 	time_t t;
+	intmax_t l;
 	struct tm tm;
 
 	CHECK_OBJ_NOTNULL(format, FORMAT_MAGIC);
@@ -289,25 +293,48 @@ format_time(const struct format *format)
 	} else
 		t_end = t_start;
 
+	AN(format->time_fmt);
+
 	switch (format->time_type) {
-	case 'D':
-		AZ(VSB_printf(CTX.vsb, "%d",
-		    (int)((t_end - t_start) * 1e6)));
-		break;
 	case 't':
-		AN(format->time_fmt);
-		t = (long)floor(t_start);
+		t = (intmax_t)floor(t_start);
 		(void)localtime_r(&t, &tm);
 		AN(strftime(buf, sizeof buf, format->time_fmt, &tm));
 		AZ(VSB_cat(CTX.vsb, buf));
+		return (1);
+	case '3':
+		l = (intmax_t)(modf(t_start, &d) * 1e3);
 		break;
-	case 'T':
-		AZ(VSB_printf(CTX.vsb, "%d", (int)(t_end - t_start)));
+	case '6':
+		l = (intmax_t)(modf(t_start, &d) * 1e6);
+		break;
+	case 'S':
+		l = (intmax_t)t_start;
+		break;
+	case 'M':
+		l = (intmax_t)(t_start * 1e3);
+		break;
+	case 'U':
+		l = (intmax_t)(t_start * 1e6);
+		break;
+	case 's':
+		l = (intmax_t)(t_end - t_start);
+		break;
+	case 'm':
+		l = (intmax_t)((t_end - t_start) * 1e3);
+		break;
+	case 'u':
+		l = (intmax_t)((t_end - t_start) * 1e6);
 		break;
 	default:
 		WRONG("Time format specifier");
 	}
 
+#ifdef __FreeBSD__
+	assert(fmtcheck(format->time_fmt, "%jd") == format->time_fmt);
+#endif
+	AZ(VSB_printf(CTX.vsb, format->time_fmt, l));
+
 	return (1);
 }
 
@@ -342,13 +369,13 @@ format_auth(const struct format *format)
 	    CTX.frag[F_auth].e)) {
 		if (format->string == NULL)
 			return (-1);
-		VSB_quote(CTX.vsb, format->string, -1, VSB_QUOTE_ESCHEX);
+		VSB_quote(CTX.vsb, format->string, -1, CTX.quote_how);
 		return (0);
 	}
 	q = strchr(buf, ':');
 	if (q != NULL)
 		*q = '\0';
-	VSB_quote(CTX.vsb, buf, -1, VSB_QUOTE_ESCHEX);
+	VSB_quote(CTX.vsb, buf, -1, CTX.quote_how);
 	return (1);
 }
 
@@ -440,12 +467,48 @@ addf_time(char type, const char *fmt)
 
 	ALLOC_OBJ(f, FORMAT_MAGIC);
 	AN(f);
+	AN(fmt);
 	f->func = format_time;
 	f->time_type = type;
-	if (fmt != NULL) {
-		f->time_fmt = strdup(fmt);
-		AN(f->time_fmt);
+	f->time_fmt = strdup(fmt);
+
+	if (f->time_type == 'T') {
+		if (!strcmp(fmt, "s"))
+			f->time_type = 's';
+		else if (!strcmp(fmt, "ms"))
+			f->time_type = 'm';
+		else if (!strcmp(fmt, "us"))
+			f->time_type = 'u';
+		else
+			VUT_Error(vut, 1, "Unknown specifier: %%{%s}T",
+			    fmt);
+		REPLACE(f->time_fmt, "%jd");
+	} else if (f->time_type == 't') {
+		if (!strcmp(fmt, "sec")) {
+			f->time_type = 'S';
+			REPLACE(f->time_fmt, "%jd");
+		} else if (!strncmp(fmt, "msec", 4)) {
+			fmt += 4;
+			if (!strcmp(fmt, "_frac")) {
+				f->time_type = '3';
+				REPLACE(f->time_fmt, "%03jd");
+			} else if (*fmt == '\0') {
+				f->time_type = 'M';
+				REPLACE(f->time_fmt, "%jd");
+			}
+		} else if (!strncmp(fmt, "usec", 4)) {
+			fmt += 4;
+			if (!strcmp(fmt, "_frac")) {
+				f->time_type = '6';
+				REPLACE(f->time_fmt, "%06jd");
+			} else if (*fmt == '\0') {
+				f->time_type = 'U';
+				REPLACE(f->time_fmt, "%jd");
+			}
+		}
 	}
+
+	AN(f->time_fmt);
 	VTAILQ_INSERT_TAIL(&CTX.format, f, list);
 }
 
@@ -500,7 +563,7 @@ addf_hdr(struct watch_head *head, const char *key)
 	AN(f);
 	f->func = format_fragment;
 	f->frag = &w->frag;
-	f->string = strdup("-");
+	f->string = strdup(CTX.missing_string);
 	AN(f->string);
 	VTAILQ_INSERT_TAIL(&CTX.format, f, list);
 }
@@ -512,7 +575,7 @@ addf_vsl(enum VSL_tag_e tag, long i, const char *prefix)
 
 	ALLOC_OBJ(w, VSL_WATCH_MAGIC);
 	AN(w);
-	if (VSL_tagflags[tag])
+	if (VSL_tagflags[tag] && CTX.quote_how != VSB_QUOTE_JSON)
 		VUT_Error(vut, 1, "Tag %s can contain control characters",
 		    VSL_tags[tag]);
 	w->tag = tag;
@@ -523,7 +586,7 @@ addf_vsl(enum VSL_tag_e tag, long i, const char *prefix)
 		assert(w->prefixlen > 0);
 	}
 	VTAILQ_INSERT_TAIL(&CTX.watch_vsl, w, list);
-	addf_fragment(&w->frag, "-");
+	addf_fragment(&w->frag, CTX.missing_string);
 }
 
 static void
@@ -547,7 +610,7 @@ parse_x_format(char *buf)
 	int slt;
 
 	if (!strcmp(buf, "Varnish:time_firstbyte")) {
-		addf_fragment(&CTX.frag[F_ttfb], "");
+		addf_fragment(&CTX.frag[F_ttfb], CTX.missing_int);
 		return;
 	}
 	if (!strcmp(buf, "Varnish:hitmiss")) {
@@ -624,6 +687,7 @@ parse_format(const char *format)
 	const char *p, *q;
 	struct vsb *vsb;
 	char buf[256];
+	int b;
 
 	if (format == NULL)
 		format = FORMAT;
@@ -658,28 +722,28 @@ parse_format(const char *format)
 		p++;
 		switch (*p) {
 		case 'b':	/* Body bytes sent */
-			addf_fragment(&CTX.frag[F_b], "-");
+			addf_fragment(&CTX.frag[F_b], CTX.missing_int);
 			break;
 		case 'D':	/* Float request time */
-			addf_time(*p, NULL);
+			addf_time('T', "us");
 			break;
 		case 'h':	/* Client host name / IP Address */
-			addf_fragment(&CTX.frag[F_h], "-");
+			addf_fragment(&CTX.frag[F_h], CTX.missing_string);
 			break;
 		case 'H':	/* Protocol */
 			addf_fragment(&CTX.frag[F_H], "HTTP/1.0");
 			break;
 		case 'I':	/* Bytes received */
-			addf_fragment(&CTX.frag[F_I], "-");
+			addf_fragment(&CTX.frag[F_I], CTX.missing_int);
 			break;
 		case 'l':	/* Client user ID (identd) always '-' */
 			AZ(VSB_putc(vsb, '-'));
 			break;
 		case 'm':	/* Method */
-			addf_fragment(&CTX.frag[F_m], "-");
+			addf_fragment(&CTX.frag[F_m], CTX.missing_string);
 			break;
 		case 'O':	/* Bytes sent */
-			addf_fragment(&CTX.frag[F_O], "-");
+			addf_fragment(&CTX.frag[F_O], CTX.missing_int);
 			break;
 		case 'q':	/* Query string */
 			addf_fragment(&CTX.frag[F_q], "");
@@ -688,26 +752,33 @@ parse_format(const char *format)
 			addf_requestline();
 			break;
 		case 's':	/* Status code */
-			addf_fragment(&CTX.frag[F_s], "-");
+			addf_fragment(&CTX.frag[F_s], CTX.missing_int);
 			break;
 		case 't':	/* strftime */
 			addf_time(*p, TIME_FMT);
 			break;
 		case 'T':	/* Int request time */
-			addf_time(*p, NULL);
+			addf_time(*p, "s");
 			break;
 		case 'u':	/* Remote user from auth */
 			addf_auth();
 			break;
 		case 'U':	/* URL */
-			addf_fragment(&CTX.frag[F_U], "-");
+			addf_fragment(&CTX.frag[F_U], CTX.missing_string);
 			break;
 		case '{':
 			p++;
 			q = p;
-			while (*q && *q != '}')
+			b = 1;
+			while (*q) {
+				if (*q == '{')
+					b++;
+				else if (*q == '}')
+					if (--b == 0)
+						break;
 				q++;
-			if (!*q)
+			}
+			if (b > 0)
 				VUT_Error(vut, 1, "Unmatched bracket at: %s",
 				    p - 2);
 			assert((unsigned)(q - p) < sizeof buf - 1);
@@ -724,6 +795,9 @@ parse_format(const char *format)
 			case 't':
 				addf_time(*q, buf);
 				break;
+			case 'T':
+				addf_time(*q, buf);
+				break;
 			case 'x':
 				parse_x_format(buf);
 				break;
@@ -797,7 +871,8 @@ frag_fields(int force, const char *b, const char *e, ...)
 			n++;
 		}
 		assert(p != NULL && q != NULL);
-		assert(p < e && q > p);
+		if (p >= e || q <= p)
+			continue;
 		if (frag->gen != CTX.gen || force) {
 			/* We only grab the same matching field once */
 			frag->gen = CTX.gen;
@@ -849,7 +924,6 @@ process_vsl(const struct vsl_watch_head *head, enum VSL_tag_e tag,
 {
 	struct vsl_watch *w;
 	const char *p;
-
 	VTAILQ_FOREACH(w, head, list) {
 		CHECK_OBJ_NOTNULL(w, VSL_WATCH_MAGIC);
 		if (tag != w->tag)
@@ -873,46 +947,39 @@ dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 	enum VSL_tag_e tag;
 	const char *b, *e, *p;
 	struct watch *w;
-	int i, skip, be_mark;
+	int i, skip;
 
 	(void)vsl;
 	(void)priv;
 
-#define BACKEND_MARKER (INT_MAX / 2 + 1)
-	assert(BACKEND_MARKER >= VSL_t__MAX);
-
 	for (t = pt[0]; t != NULL; t = *++pt) {
 		CTX.gen++;
 
-		/* Consider client requests only if in client mode.
-		   Consider backend requests only if in backend mode. */
-		if (t->type == VSL_t_req && CTX.c_opt) {
+		if (t->type == VSL_t_req) {
 			CTX.side = "c";
-			be_mark = 0;
-		} else if (t->type == VSL_t_bereq && CTX.b_opt) {
+		} else if (t->type == VSL_t_bereq) {
 			CTX.side = "b";
-			be_mark = BACKEND_MARKER;
 		} else
 			continue;
-		if (t->reason == VSL_r_esi && !CTX.E_opt)
-			/* Skip ESI requests */
-			continue;
+
 		CTX.hitmiss = "-";
 		CTX.handling = "-";
 		CTX.vxid = t->vxid;
 		skip = 0;
 		while (skip == 0 && 1 == VSL_Next(t->c)) {
 			tag = VSL_TAG(t->c->rec.ptr);
-			if (VSL_tagflags[tag])
+			if (VSL_tagflags[tag] &&
+			    CTX.quote_how != VSB_QUOTE_JSON)
 				continue;
 
 			b = VSL_CDATA(t->c->rec.ptr);
 			e = b + VSL_LEN(t->c->rec.ptr);
-			while (e > b && e[-1] == '\0')
-				e--;
+			if (!VSL_tagflags[tag]) {
+				while (e > b && e[-1] == '\0')
+					e--;
+			}
 
-			switch (tag + be_mark) {
-			case SLT_HttpGarbage + BACKEND_MARKER:
+			switch (tag) {
 			case SLT_HttpGarbage:
 				skip = 1;
 				break;
@@ -922,7 +989,7 @@ dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 				    4, &CTX.frag[F_O],
 				    0, NULL);
 				break;
-			case (SLT_BackendOpen + BACKEND_MARKER):
+			case SLT_BackendOpen:
 				frag_fields(1, b, e,
 				    3, &CTX.frag[F_h],
 				    0, NULL);
@@ -932,11 +999,11 @@ dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 				    1, &CTX.frag[F_h],
 				    0, NULL);
 				break;
-			case (SLT_BereqMethod + BACKEND_MARKER):
+			case SLT_BereqMethod:
 			case SLT_ReqMethod:
 				frag_line(0, b, e, &CTX.frag[F_m]);
 				break;
-			case (SLT_BereqURL + BACKEND_MARKER):
+			case SLT_BereqURL:
 			case SLT_ReqURL:
 				p = memchr(b, '?', e - b);
 				if (p == NULL)
@@ -944,15 +1011,15 @@ dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 				frag_line(0, b, p, &CTX.frag[F_U]);
 				frag_line(0, p, e, &CTX.frag[F_q]);
 				break;
-			case (SLT_BereqProtocol + BACKEND_MARKER):
+			case SLT_BereqProtocol:
 			case SLT_ReqProtocol:
 				frag_line(0, b, e, &CTX.frag[F_H]);
 				break;
-			case (SLT_BerespStatus + BACKEND_MARKER):
+			case SLT_BerespStatus:
 			case SLT_RespStatus:
 				frag_line(1, b, e, &CTX.frag[F_s]);
 				break;
-			case (SLT_BereqAcct + BACKEND_MARKER):
+			case SLT_BereqAcct:
 			case SLT_ReqAcct:
 				frag_fields(0, b, e,
 				    3, &CTX.frag[F_I],
@@ -960,7 +1027,6 @@ dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 				    6, &CTX.frag[F_O],
 				    0, NULL);
 				break;
-			case (SLT_Timestamp + BACKEND_MARKER):
 			case SLT_Timestamp:
 #define ISPREFIX(a, b, c, d)	isprefix(a, strlen(a), b, c, d)
 				if (ISPREFIX("Start:", b, e, &p)) {
@@ -980,8 +1046,9 @@ dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 					    &CTX.frag[F_ttfb], 0, NULL);
 				}
 				break;
-			case (SLT_BereqHeader + BACKEND_MARKER):
+			case SLT_BereqHeader:
 			case SLT_ReqHeader:
+				process_hdr(&CTX.watch_reqhdr, b, e);
 				if (ISPREFIX("Authorization:", b, e, &p) &&
 				    ISPREFIX("basic ", p, e, &p))
 					frag_line(0, p, e,
@@ -991,6 +1058,10 @@ dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 					    &CTX.frag[F_host]);
 #undef ISPREFIX
 				break;
+			case SLT_BerespHeader:
+			case SLT_RespHeader:
+				process_hdr(&CTX.watch_resphdr, b, e);
+				break;
 			case SLT_VCL_call:
 				if (!strcasecmp(b, "recv")) {
 					CTX.hitmiss = "-";
@@ -1019,7 +1090,6 @@ dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 				} else if (!strcasecmp(b, "restart"))
 					skip = 1;
 				break;
-			case (SLT_VCL_Log + BACKEND_MARKER):
 			case SLT_VCL_Log:
 				VTAILQ_FOREACH(w, &CTX.watch_vcl_log, list) {
 					CHECK_OBJ_NOTNULL(w, WATCH_MAGIC);
@@ -1034,13 +1104,6 @@ dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 				break;
 			}
 
-			if ((tag == SLT_ReqHeader && CTX.c_opt) ||
-			    (tag == SLT_BereqHeader && CTX.b_opt))
-				process_hdr(&CTX.watch_reqhdr, b, e);
-			else if ((tag == SLT_RespHeader && CTX.c_opt) ||
-			    (tag == SLT_BerespHeader && CTX.b_opt))
-				process_hdr(&CTX.watch_resphdr, b, e);
-
 			process_vsl(&CTX.watch_vsl, tag, b, e);
 		}
 		if (skip)
@@ -1085,6 +1148,7 @@ main(int argc, char * const *argv)
 {
 	signed char opt;
 	char *format = NULL;
+	int mode_opt = 0;
 
 	vut = VUT_InitProg(argc, argv, &vopt_spec);
 	AN(vut);
@@ -1097,6 +1161,9 @@ main(int argc, char * const *argv)
 	CTX.vsb = VSB_new_auto();
 	AN(CTX.vsb);
 	VB64_init();
+	CTX.quote_how = VSB_QUOTE_ESCHEX;
+	REPLACE(CTX.missing_string, "-");
+	REPLACE(CTX.missing_int, "-");
 
 	tzset();		// We use localtime_r(3)
 
@@ -1106,23 +1173,16 @@ main(int argc, char * const *argv)
 			/* Append to file */
 			CTX.a_opt = 1;
 			break;
-		case 'b':
-			/* backend mode */
-			CTX.b_opt = 1;
-			break;
-		case 'c':
-			/* client mode */
-			CTX.c_opt = 1;
-			break;
-		case 'E':
-			/* show ESI */
-			CTX.E_opt = 1;
+		case 'b': /* backend mode */
+		case 'c': /* client mode */
+		case 'E': /* show ESI */
+			AN(VUT_Arg(vut, opt, NULL));
+			mode_opt = 1;
 			break;
 		case 'F':
 			if (format != NULL)
 				VUT_Error(vut, 1, "Format already set");
 			REPLACE(format, optarg);
-			AN(format);
 			break;
 		case 'f':
 			if (format != NULL)
@@ -1135,6 +1195,11 @@ main(int argc, char * const *argv)
 			/* Usage help */
 			VUT_Usage(vut, &vopt_spec, 0);
 			break;
+		case 'j':
+			REPLACE(CTX.missing_string, "");
+			REPLACE(CTX.missing_int, "0");
+			CTX.quote_how = VSB_QUOTE_JSON;
+			break;
 		case 'w':
 			/* Write to file */
 			REPLACE(CTX.w_arg, optarg);
@@ -1144,9 +1209,10 @@ main(int argc, char * const *argv)
 				VUT_Usage(vut, &vopt_spec, 1);
 		}
 	}
+
 	/* default is client mode: */
-	if (!CTX.b_opt)
-		CTX.c_opt = 1;
+	if (!mode_opt)
+		AN(VUT_Arg(vut, 'c', NULL));
 
 	if (optind != argc)
 		VUT_Usage(vut, &vopt_spec, 1);
@@ -1154,6 +1220,9 @@ main(int argc, char * const *argv)
 	if (vut->D_opt && !CTX.w_arg)
 		VUT_Error(vut, 1, "Missing -w option");
 
+	if (vut->D_opt && !strcmp(CTX.w_arg, "-"))
+		VUT_Error(vut, 1, "Daemon cannot write to stdout");
+
 	/* Check for valid grouping mode */
 	assert(vut->g_arg < VSL_g__MAX);
 	if (vut->g_arg != VSL_g_vxid && vut->g_arg != VSL_g_request)
