@@ -34,7 +34,8 @@
 #define NCSA_OPT_a							\
 	VOPT("a", "[-a]", "Append to file",				\
 	    "When writing output to a file, append to it rather than"	\
-	    " overwrite it."						\
+	    " overwrite it. This option has no effect without the -w"	\
+	    " option."							\
 	)
 
 #define NCSA_OPT_F							\
@@ -62,7 +63,9 @@
 	    " unless the -a option was specified. If the application"	\
 	    " receives a SIGHUP in daemon mode the file will be"	\
 	    " reopened allowing the old one to be rotated away. This"	\
-	    " option is required when running in daemon mode."		\
+	    " option is required when running in daemon mode. If the"	\
+	    " filename is -, varnishncsa writes to the standard output"	\
+	    " and cannot work as a daemon."				\
 	)
 #define NCSA_OPT_b							\
 	VOPT("b", "[-b]", "Backend mode",				\
@@ -70,13 +73,22 @@
 	    " backend requests will trigger log lines."			\
 	)
 #define NCSA_OPT_c							\
-	VOPT("c", "[-c]", "Client mode",					\
+	VOPT("c", "[-c]", "Client mode",				\
 	    "Log client requests. This is the default. If -b is"	\
 	    " specified, then -c is needed to also log client requests"	\
 	)
 #define NCSA_OPT_E							\
-	VOPT("E", "[-E]", "Show ESI request",				\
-	    "Show ESI request."	\
+	VOPT("E", "[-E]", "Show ESI requests",				\
+	    "Show ESI requests, implies client mode."			\
+	)
+#define NCSA_OPT_j							\
+	VOPT("j", "[-j]", "Make output JSON-compatible",		\
+	    "Make format-specifier replacements JSON-compatible. When"	\
+	    " escaping characters, use JSON-style \\\\uXXXX escape"	\
+	    " sequences instead of C-style \\\\xXX sequences. Empty"	\
+	    " strings will be replaced with \"\" instead of \"-\", and"	\
+	    " empty integers will be replaced with null. Use -F or -f"	\
+	    " in combination with -j to write JSON logs."		\
 	)
 
 NCSA_OPT_a
@@ -90,6 +102,7 @@ NCSA_OPT_F
 NCSA_OPT_f
 NCSA_OPT_g
 VUT_OPT_h
+NCSA_OPT_j
 VSL_OPT_L
 VUT_OPT_n
 VUT_GLOBAL_OPT_P
