@@ -173,18 +173,11 @@ update_position(void)
 		current = 0;
 		page_start = 0;
 	} else {
-		if (current < 0)
-			current = 0;
-		if (current > n_ptarray - 1)
-			current = n_ptarray - 1;
-		if (current < page_start)
-			page_start = current;
+		current = vlimit_t(int, current, 0, n_ptarray - 1);
+		page_start = vmin(page_start, current);
 		if (current > page_start + (l_points - 1))
 			page_start = current - (l_points - 1);
-		if (page_start < 0)
-			page_start = 0;
-		if (page_start > n_ptarray - 1)
-			page_start = n_ptarray - 1;
+		page_start = vlimit_t(int, page_start, 0, n_ptarray - 1);
 	}
 
 	if (current != old_current || page_start != old_page_start)
@@ -363,8 +356,7 @@ make_windows(void)
 		l_points += l_info;
 		l_info = 0;
 	}
-	if (l_points < LINES_POINTS_MIN)
-		l_points = LINES_POINTS_MIN;
+	l_points = vmax(l_points, LINES_POINTS_MIN);
 
 	y = 0;
 	y_status = y;
@@ -1073,12 +1065,7 @@ handle_help_keypress(enum kb_e kb)
 		WRONG("unhandled key binding");
 	}
 
-	if (help_line > bindings_help_len - l_points)
-		help_line = bindings_help_len - l_points;
-
-	if (help_line < 0)
-		help_line = 0;
-
+	help_line = vlimit_t(int, help_line, 0, bindings_help_len - l_points);
 	redraw = (help_line != hl);
 }
 
