@@ -44,7 +44,6 @@
 
 #include "vav.h"
 #include "vrnd.h"
-#include "vtim.h"
 
 #define		MAX_TOKENS		200
 
@@ -80,6 +79,7 @@ struct macro {
 	VTAILQ_ENTRY(macro)	list;
 	char			*name;
 	char			*val;
+	macro_f			*func;
 };
 
 static VTAILQ_HEAD(,macro) macro_list = VTAILQ_HEAD_INITIALIZER(macro_list);
@@ -99,7 +99,7 @@ static const struct cmds top_cmds[] = {
 /**********************************************************************/
 
 static struct macro *
-macro_def_int(const char *name, const char *fmt, va_list ap)
+macro_def_int(const char *name, macro_f *func, const char *fmt, va_list ap)
 {
 	struct macro *m;
 	char buf[512];
@@ -115,9 +115,15 @@ macro_def_int(const char *name, const char *fmt, va_list ap)
 		VTAILQ_INSERT_TAIL(&macro_list, m, list);
 	}
 	AN(m);
-	vbprintf(buf, fmt, ap);
-	REPLACE(m->val, buf);
-	AN(m->val);
+	if (func != NULL) {
+		AZ(fmt);
+		m->func = func;
+	} else {
+		AN(fmt);
+		vbprintf(buf, fmt, ap);
+		REPLACE(m->val, buf);
+		AN(m->val);
+	}
 	return (m);
 }
 
@@ -128,12 +134,12 @@ macro_def_int(const char *name, const char *fmt, va_list ap)
  */
 
 void
-extmacro_def(const char *name, const char *fmt, ...)
+extmacro_def(const char *name, macro_f *func, const char *fmt, ...)
 {
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)macro_def_int(name, fmt, ap);
+	(void)macro_def_int(name, func, fmt, ap);
 	va_end(ap);
 }
 
@@ -149,8 +155,13 @@ init_macro(void)
 	struct macro *m;
 
 	/* Dump the extmacros for completeness */
-	VTAILQ_FOREACH(m, &macro_list, list)
-		vtc_log(vltop, 4, "extmacro def %s=%s", m->name, m->val);
+	VTAILQ_FOREACH(m, &macro_list, list) {
+		if (m->val != NULL)
+			vtc_log(vltop, 4,
+			    "extmacro def %s=%s", m->name, m->val);
+		else
+			vtc_log(vltop, 4, "extmacro def %s(...)", m->name);
+	}
 
 	AZ(pthread_mutex_init(&macro_mtx, NULL));
 }
@@ -172,7 +183,7 @@ macro_def(struct vtclog *vl, const char *instance, const char *name,
 
 	AZ(pthread_mutex_lock(&macro_mtx));
 	va_start(ap, fmt);
-	m = macro_def_int(name, fmt, ap);
+	m = macro_def_int(name, NULL, fmt, ap);
 	va_end(ap);
 	vtc_log(vl, 4, "macro def %s=%s", name, m->val);
 	AZ(pthread_mutex_unlock(&macro_mtx));
@@ -196,6 +207,7 @@ macro_undef(struct vtclog *vl, const char *instance, const char *name)
 	if (m != NULL) {
 		if (!vtc_stop)
 			vtc_log(vl, 4, "macro undef %s", name);
+		CHECK_OBJ(m, MACRO_MAGIC);
 		VTAILQ_REMOVE(&macro_list, m, list);
 		free(m->name);
 		free(m->val);
@@ -204,37 +216,66 @@ macro_undef(struct vtclog *vl, const char *instance, const char *name)
 	AZ(pthread_mutex_unlock(&macro_mtx));
 }
 
-char *
-macro_get(const char *b, const char *e)
+void
+macro_cat(struct vtclog *vl, struct vsb *vsb, const char *b, const char *e)
 {
 	struct macro *m;
-	int l;
-	char *retval = NULL;
+	char **argv, *retval = NULL;
+	const char *err = NULL;
+	int argc;
 
 	AN(b);
 	if (e == NULL)
 		e = strchr(b, '\0');
 	AN(e);
-	l = e - b;
 
-	if (l == 4 && !memcmp(b, "date", l)) {
-		double t = VTIM_real();
-		retval = malloc(VTIM_FORMAT_SIZE);
-		AN(retval);
-		VTIM_format(t, retval);
-		return (retval);
-	}
+	argv = VAV_ParseTxt(b, e, &argc, ARGV_COMMA);
+	AN(argv);
+
+	if (*argv != NULL)
+		vtc_fatal(vl, "Macro ${%.*s} parsing failed: %s",
+		    (int)(e - b), b, *argv);
+
+	assert(argc >= 2);
 
 	AZ(pthread_mutex_lock(&macro_mtx));
 	VTAILQ_FOREACH(m, &macro_list, list) {
 		CHECK_OBJ_NOTNULL(m, MACRO_MAGIC);
-		if (!strncmp(b, m->name, l) && m->name[l] == '\0')
+		if (!strcmp(argv[1], m->name))
 			break;
 	}
-	if (m != NULL)
-		retval = strdup(m->val);
+	if (m != NULL) {
+		if (m->func != NULL) {
+			AZ(m->val);
+			retval = m->func(argc, argv, &err);
+			if (err == NULL)
+				AN(retval);
+		} else {
+			AN(m->val);
+			if (argc == 2)
+				REPLACE(retval, m->val);
+			else
+				err = "macro does not take arguments";
+		}
+	}
 	AZ(pthread_mutex_unlock(&macro_mtx));
-	return (retval);
+
+	VAV_Free(argv);
+
+	if (err != NULL)
+		vtc_fatal(vl, "Macro ${%.*s} failed: %s",
+		    (int)(e - b), b, err);
+
+	if (retval == NULL) {
+		if (!ign_unknown_macro)
+			vtc_fatal(vl, "Macro ${%.*s} not found",
+			    (int)(e - b), b);
+		VSB_printf(vsb, "${%.*s}", (int)(e - b), b);
+		return;
+	}
+
+	VSB_cat(vsb, retval);
+	free(retval);
 }
 
 struct vsb *
@@ -259,7 +300,6 @@ macro_expand(struct vtclog *vl, const char *text)
 {
 	struct vsb *vsb;
 	const char *p, *q;
-	char *m;
 
 	vsb = VSB_new_auto();
 	AN(vsb);
@@ -279,19 +319,7 @@ macro_expand(struct vtclog *vl, const char *text)
 		assert(p[1] == '{');
 		assert(q[0] == '}');
 		p += 2;
-		m = macro_get(p, q);
-		if (m == NULL) {
-			if (!ign_unknown_macro) {
-				VSB_destroy(&vsb);
-				vtc_fatal(vl, "Macro ${%.*s} not found",
-					  (int)(q - p), p);
-				NEEDLESS(return (NULL));
-			}
-			VSB_printf(vsb, "${%.*s}", (int)(q - p), p);
-		} else {
-			VSB_printf(vsb, "%s", m);
-			free(m);
-		}
+		macro_cat(vl, vsb, p, q);
 		text = q + 1;
 	}
 	AZ(VSB_finish(vsb));
@@ -513,7 +541,6 @@ exec_file(const char *fn, const char *script, const char *tmpdir,
 	FILE *f;
 	struct vsb *vsb;
 	const char *p;
-	char *q;
 
 	(void)signal(SIGPIPE, SIG_IGN);
 
@@ -533,15 +560,12 @@ exec_file(const char *fn, const char *script, const char *tmpdir,
 	init_macro();
 	init_server();
 	init_syslog();
+	init_tunnel();
 
 	vsb = VSB_new_auto();
 	AN(vsb);
-	if (*fn != '/') {
-		q = macro_get("pwd", NULL);
-		AN(q);
-		VSB_cat(vsb, q);
-		free(q);
-	}
+	if (*fn != '/')
+		macro_cat(vltop, vsb, "pwd", NULL);
 	p = strrchr(fn, '/');
 	if (p != NULL) {
 		VSB_putc(vsb, '/');
