@@ -4,6 +4,8 @@
  *
  * Author: Frédéric Lécaille <flecaille@haproxy.com>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -43,10 +45,13 @@
 #include "vpf.h"
 #include "vre.h"
 #include "vtcp.h"
+#include "vsa.h"
 #include "vtim.h"
 
 #define HAPROXY_PROGRAM_ENV_VAR	"HAPROXY_PROGRAM"
+#define HAPROXY_ARGS_ENV_VAR	"HAPROXY_ARGS"
 #define HAPROXY_OPT_WORKER	"-W"
+#define HAPROXY_OPT_MCLI	"-S"
 #define HAPROXY_OPT_DAEMON	"-D"
 #define HAPROXY_SIGNAL		SIGINT
 #define HAPROXY_EXPECT_EXIT	(128 + HAPROXY_SIGNAL)
@@ -68,6 +73,7 @@ struct haproxy {
 	const char		*filename;
 	struct vsb		*args;
 	int			opt_worker;
+	int			opt_mcli;
 	int			opt_daemon;
 	int			opt_check_mode;
 	char			*pid_fn;
@@ -87,8 +93,12 @@ struct haproxy {
 	/* TCP socket CLI. */
 	struct haproxy_cli *cli;
 
+	/* master CLI */
+	struct haproxy_cli *mcli;
+
 	char			*workdir;
 	struct vsb		*msgs;
+	char			closed_sock[256]; /* Closed TCP socket */
 	VTAILQ_HEAD(,envar) envars;
 };
 
@@ -165,8 +175,10 @@ haproxy_cli_tcp_connect(struct vtclog *vl, const char *addr, double tmo,
     const char **errp)
 {
 	int fd;
-	char mabuf[32], mpbuf[32];
+	char mabuf[VTCP_ADDRBUFSIZE], mpbuf[VTCP_PORTBUFSIZE];
 
+	AN(addr);
+	AN(errp);
 	fd = VTCP_open(addr, NULL, tmo, errp);
 	if (fd < 0)
 		return (fd);
@@ -188,9 +200,8 @@ cmd_haproxy_cli_send(CMD_ARGS)
 {
 	struct vsb *vsb;
 	struct haproxy_cli *hc;
-	ssize_t wr;
+	int j;
 
-	(void)cmd;
 	(void)vl;
 	CAST_OBJ_NOTNULL(hc, priv, HAPROXY_CLI_MAGIC);
 	AZ(strcmp(av[0], "send"));
@@ -219,11 +230,17 @@ cmd_haproxy_cli_send(CMD_ARGS)
 	}
 	vtc_dump(hc->vl, 4, "CLI send", VSB_data(vsb), -1);
 
-	wr = write(hc->sock, VSB_data(vsb), VSB_len(vsb));
-	if (wr != VSB_len(vsb))
+	if (VSB_tofile(vsb, hc->sock))
 		vtc_fatal(hc->vl,
 		    "CLI fd %d send error %s", hc->sock, strerror(errno));
 
+	/* a CLI command must be followed by a SHUT_WR if we want HAProxy to
+	 * close after the response */
+	j = shutdown(hc->sock, SHUT_WR);
+	vtc_log(hc->vl, 3, "CLI shutting fd %d", hc->sock);
+	if (!VTCP_Check(j))
+		vtc_fatal(hc->vl, "Shutdown failed: %s", strerror(errno));
+
 	VSB_destroy(&vsb);
 }
 
@@ -281,12 +298,11 @@ static void v_matchproto_(cmd_f)
 cmd_haproxy_cli_expect(CMD_ARGS)
 {
 	struct haproxy_cli *hc;
+	struct vsb vsb[1];
 	vre_t *vre;
-	const char *error;
-	int erroroffset, i, ret;
-	char *cmp, *spec;
+	int error, erroroffset, i, ret;
+	char *cmp, *spec, errbuf[VRE_ERROR_LEN];
 
-	(void)cmd;
 	(void)vl;
 	CAST_OBJ_NOTNULL(hc, priv, HAPROXY_CLI_MAGIC);
 	AZ(strcmp(av[0], "expect"));
@@ -302,12 +318,17 @@ cmd_haproxy_cli_expect(CMD_ARGS)
 
 	haproxy_cli_recv(hc);
 
-	vre = VRE_compile(spec, 0, &error, &erroroffset);
-	if (!vre)
+	vre = VRE_compile(spec, 0, &error, &erroroffset, 1);
+	if (vre == NULL) {
+		AN(VSB_init(vsb, errbuf, sizeof errbuf));
+		AZ(VRE_error(vsb, error));
+		AZ(VSB_finish(vsb));
+		VSB_fini(vsb);
 		vtc_fatal(hc->vl, "CLI regexp error: '%s' (@%d) (%s)",
-		    error, erroroffset, spec);
+		    errbuf, erroroffset, spec);
+	}
 
-	i = VRE_exec(vre, hc->rxbuf, strlen(hc->rxbuf), 0, 0, NULL, 0, 0);
+	i = VRE_match(vre, hc->rxbuf, 0, 0, NULL);
 
 	VRE_free(&vre);
 
@@ -350,7 +371,7 @@ haproxy_cli_thread(void *priv)
 		    "CLI failed to open %s: %s", VSB_data(vsb), err);
 	VTCP_blocking(fd);
 	hc->sock = fd;
-	parse_string(hc->spec, haproxy_cli_cmds, hc, hc->vl);
+	parse_string(hc->vl, hc, hc->spec);
 	vtc_log(hc->vl, 2, "CLI ending");
 	VSB_destroy(&vsb);
 	return (NULL);
@@ -462,6 +483,7 @@ haproxy_cli_new(struct haproxy *h)
 	AN(hc);
 
 	hc->vl = h->vl;
+	vtc_log_set_cmd(hc->vl, haproxy_cli_cmds);
 	hc->sock = -1;
 	bprintf(hc->connect, "${%s_cli_sock}", h->name);
 
@@ -474,6 +496,59 @@ haproxy_cli_new(struct haproxy *h)
 	return (hc);
 }
 
+/* creates a master CLI client (-mcli) */
+static struct haproxy_cli *
+haproxy_mcli_new(struct haproxy *h)
+{
+	struct haproxy_cli *hc;
+
+	ALLOC_OBJ(hc, HAPROXY_CLI_MAGIC);
+	AN(hc);
+
+	hc->vl = h->vl;
+	vtc_log_set_cmd(hc->vl, haproxy_cli_cmds);
+	hc->sock = -1;
+	bprintf(hc->connect, "${%s_mcli_sock}", h->name);
+
+	hc->txbuf_sz = hc->rxbuf_sz = 2048 * 1024;
+	hc->txbuf = malloc(hc->txbuf_sz);
+	AN(hc->txbuf);
+	hc->rxbuf = malloc(hc->rxbuf_sz);
+	AN(hc->rxbuf);
+
+	return (hc);
+}
+
+/* Bind an address/port for the master CLI (-mcli) */
+static int
+haproxy_create_mcli(struct haproxy *h)
+{
+	int sock;
+	const char *err;
+	char buf[128], addr[128], port[128];
+	char vsabuf[vsa_suckaddr_len];
+	struct suckaddr *sua;
+
+	sock = VTCP_listen_on(default_listen_addr, NULL, 100, &err);
+	if (err != NULL)
+		vtc_fatal(h->vl,
+			  "Create listen socket failed: %s", err);
+	assert(sock > 0);
+	sua = VSA_getsockname(sock, vsabuf, sizeof vsabuf);
+	AN(sua);
+
+	VTCP_name(sua, addr, sizeof addr, port, sizeof port);
+	bprintf(buf, "%s_mcli", h->name);
+	if (VSA_Get_Proto(sua) == AF_INET)
+		macro_def(h->vl, buf, "sock", "%s:%s", addr, port);
+	else
+		macro_def(h->vl, buf, "sock", "[%s]:%s", addr, port);
+	macro_def(h->vl, buf, "addr", "%s", addr);
+	macro_def(h->vl, buf, "port", "%s", port);
+
+	return (sock);
+}
+
 static void
 haproxy_cli_delete(struct haproxy_cli *hc)
 {
@@ -494,14 +569,26 @@ haproxy_new(const char *name)
 	struct haproxy *h;
 	struct vsb *vsb;
 	char buf[PATH_MAX];
+	int closed_sock;
+	char addr[128], port[128];
+	const char *err;
+	const char *env_args;
+	char vsabuf[vsa_suckaddr_len];
+	struct suckaddr *sua;
 
 	ALLOC_OBJ(h, HAPROXY_MAGIC);
 	AN(h);
 	REPLACE(h->name, name);
 
 	h->args = VSB_new_auto();
+	env_args = getenv(HAPROXY_ARGS_ENV_VAR);
+	if (env_args) {
+		VSB_cat(h->args, env_args);
+		VSB_cat(h->args, " ");
+	}
 
-	h->vl = vtc_logopen(name);
+	h->vl = vtc_logopen("%s", name);
+	vtc_log_set_cmd(h->vl, haproxy_cli_cmds);
 	AN(h->vl);
 
 	h->filename = getenv(HAPROXY_PROGRAM_ENV_VAR);
@@ -523,9 +610,32 @@ haproxy_new(const char *name)
 	h->cfg_fn = strdup(buf);
 	AN(h->cfg_fn);
 
+	/* Create a new TCP socket to reserve an IP:port and close it asap.
+	 * May be useful to simulate an unreachable server.
+	 */
+	bprintf(h->closed_sock, "%s_closed", h->name);
+	closed_sock = VTCP_listen_on("127.0.0.1:0", NULL, 100, &err);
+	if (err != NULL)
+		vtc_fatal(h->vl,
+			"Create listen socket failed: %s", err);
+	assert(closed_sock > 0);
+	sua = VSA_getsockname(closed_sock, vsabuf, sizeof vsabuf);
+	AN(sua);
+	VTCP_name(sua, addr, sizeof addr, port, sizeof port);
+	if (VSA_Get_Proto(sua) == AF_INET)
+		macro_def(h->vl, h->closed_sock, "sock", "%s:%s", addr, port);
+	else
+		macro_def(h->vl, h->closed_sock, "sock", "[%s]:%s", addr, port);
+	macro_def(h->vl, h->closed_sock, "addr", "%s", addr);
+	macro_def(h->vl, h->closed_sock, "port", "%s", port);
+	VTCP_close(&closed_sock);
+
 	h->cli = haproxy_cli_new(h);
 	AN(h->cli);
 
+	h->mcli = haproxy_mcli_new(h);
+	AN(h->mcli);
+
 	bprintf(buf, "rm -rf \"%s\" ; mkdir -p \"%s\"", h->workdir, h->workdir);
 	AZ(system(buf));
 
@@ -559,6 +669,7 @@ haproxy_delete(struct haproxy *h)
 	free(h->pid_fn);
 	VSB_destroy(&h->args);
 	haproxy_cli_delete(h->cli);
+	haproxy_cli_delete(h->mcli);
 
 	/* XXX: MEMLEAK (?) */
 	FREE_OBJ(h);
@@ -579,6 +690,7 @@ haproxy_thread(void *priv)
 	return (NULL);
 }
 
+
 /**********************************************************************
  * Start a HAProxy instance.
  */
@@ -592,22 +704,28 @@ haproxy_start(struct haproxy *h)
 	vtc_log(h->vl, 2, "%s", __func__);
 
 	AZ(VSB_finish(h->args));
-	vtc_log(h->vl, 4, "opt_worker %d opt_daemon %d opt_check_mode %d",
-	    h->opt_worker, h->opt_daemon, h->opt_check_mode);
+	vtc_log(h->vl, 4, "opt_worker %d opt_daemon %d opt_check_mode %d opt_mcli %d",
+	    h->opt_worker, h->opt_daemon, h->opt_check_mode, h->opt_mcli);
 
 	vsb = VSB_new_auto();
 	AN(vsb);
 
 	VSB_printf(vsb, "exec \"%s\"", h->filename);
 	if (h->opt_check_mode)
-		VSB_printf(vsb, " -c");
+		VSB_cat(vsb, " -c");
 	else if (h->opt_daemon)
-		VSB_printf(vsb, " -D");
+		VSB_cat(vsb, " -D");
 	else
-		VSB_printf(vsb, " -d");
+		VSB_cat(vsb, " -d");
 
-	if (h->opt_worker)
-		VSB_printf(vsb, " -W");
+	if (h->opt_worker) {
+		VSB_cat(vsb, " -W");
+		if (h->opt_mcli) {
+			int sock;
+			sock = haproxy_create_mcli(h);
+			VSB_printf(vsb, " -S \"fd@%d\"", sock);
+		}
+	}
 
 	VSB_printf(vsb, " %s", VSB_data(h->args));
 
@@ -688,6 +806,9 @@ haproxy_wait(struct haproxy *h)
 	if (h->cli->spec)
 		haproxy_cli_run(h->cli);
 
+	if (h->mcli->spec)
+		haproxy_cli_run(h->mcli);
+
 	closefd(&h->fds[1]);
 
 	sig = SIGINT;
@@ -742,6 +863,8 @@ haproxy_build_backends(struct haproxy *h, const char *vsb_data)
 		int sock;
 		char buf[128], addr[128], port[128];
 		const char *err;
+		char vsabuf[vsa_suckaddr_len];
+		struct suckaddr *sua;
 
 		p = strstr(p, HAPROXY_BE_FD_STR);
 		if (!p)
@@ -754,15 +877,20 @@ haproxy_build_backends(struct haproxy *h, const char *vsb_data)
 			break;
 
 		*q++ = '\0';
-		sock = VTCP_listen_on("localhost:0", NULL, 100, &err);
+		sock = VTCP_listen_on("127.0.0.1:0", NULL, 100, &err);
 		if (err != NULL)
 			vtc_fatal(h->vl,
 			    "Create listen socket failed: %s", err);
 		assert(sock > 0);
+		sua = VSA_getsockname(sock, vsabuf, sizeof vsabuf);
+		AN(sua);
 
-		VTCP_myname(sock, addr, sizeof addr, port, sizeof port);
+		VTCP_name(sua, addr, sizeof addr, port, sizeof port);
 		bprintf(buf, "%s_%s", h->name, p);
-		macro_def(h->vl, buf, "sock", "%s %s", addr, port);
+		if (VSA_Get_Proto(sua) == AF_INET)
+			macro_def(h->vl, buf, "sock", "%s:%s", addr, port);
+		else
+			macro_def(h->vl, buf, "sock", "[%s]:%s", addr, port);
 		macro_def(h->vl, buf, "addr", "%s", addr);
 		macro_def(h->vl, buf, "port", "%s", port);
 
@@ -808,7 +936,7 @@ haproxy_store_conf(struct haproxy *h, const char *cfg, int auto_be)
 
 	VSB_printf(vsb, "    global\n\tstats socket \"%s\" "
 		   "level admin mode 600\n", h->cli_fn);
-	VSB_printf(vsb, "    stats socket \"fd@${cli}\" level admin\n");
+	VSB_cat(vsb, "    stats socket \"fd@${cli}\" level admin\n");
 	AZ(VSB_cat(vsb, cfg));
 
 	if (auto_be)
@@ -879,12 +1007,19 @@ haproxy_write_conf(struct haproxy *h)
  * \-W
  *         Enable HAproxy in Worker mode.
  *
+ * \-S
+ *         Enable HAproxy Master CLI in Worker mode
+ *
  * \-arg STRING
  *         Pass an argument to haproxy, for example "-h simple_list".
  *
  * \-cli STRING
  *         Specify the spec to be run by the command line interface (CLI).
  *
+ * \-mcli STRING
+ *         Specify the spec to be run by the command line interface (CLI)
+ *         of the Master process.
+ *
  * \-conf STRING
  *         Specify the configuration to be loaded by this HAProxy instance.
  *
@@ -909,7 +1044,6 @@ cmd_haproxy(CMD_ARGS)
 	struct haproxy *h, *h2;
 
 	(void)priv;
-	(void)cmd;
 
 	if (av == NULL) {
 		/* Reset and free */
@@ -965,7 +1099,10 @@ cmd_haproxy(CMD_ARGS)
 			h->opt_worker = 1;
 			continue;
 		}
-
+		if (!strcmp(*av, HAPROXY_OPT_MCLI)) {
+			h->opt_mcli = 1;
+			continue;
+		}
 		if (!strcmp(*av, "-arg")) {
 			AN(av[1]);
 			AZ(h->pid);
@@ -982,6 +1119,15 @@ cmd_haproxy(CMD_ARGS)
 			av++;
 			continue;
 		}
+
+		if (!strcmp(*av, "-mcli")) {
+			REPLACE(h->mcli->spec, av[1]);
+			if (h->tp)
+				haproxy_cli_run(h->mcli);
+			av++;
+			continue;
+		}
+
 		if (!strcmp(*av, "-conf")) {
 			AN(av[1]);
 			haproxy_store_conf(h, av[1], 0);
