@@ -43,6 +43,9 @@
  *                 [vsl arguments] {
  *                         expect <skip> <vxid> <tag> <regex>
  *                         expect <skip> <vxid> <tag> <regex>
+ *                         fail add <vxid> <tag> <regex>
+ *                         fail clear
+ *                         abort
  *                         ...
  *                 } [-start|-wait]
  *
@@ -73,6 +76,10 @@
  * \-m
  *         Also emit log records for misses (only for debugging)
  *
+ * \-err
+ *         Invert the meaning of success. Usually called once to expect the
+ *         logexpect to fail
+ *
  * \-start
  *         Start the logexpect thread in the background.
  *
@@ -93,9 +100,9 @@
  * \-T <seconds>
  *         Transaction end timeout
  *
- * And the arguments of the specifications lines are:
+ * expect specification:
  *
- * skip: [uint|*]
+ * skip: [uint|*|?]
  *         Max number of record to skip
  *
  * vxid: [uint|*|=]
@@ -108,7 +115,36 @@
  *         regular expression to match against (optional)
  *
  * For skip, vxid and tag, '*' matches anything, '=' expects the value of the
- * previous matched record.
+ * previous matched record. The '?' marker is equivalent to zero, expecting a
+ * match on the next record. The difference is that '?' can be used when the
+ * order of individual consecutive logs is not deterministic. In other words,
+ * lines from a block of alternatives marked by '?' can be matched in any order,
+ * but all need to match eventually.
+ *
+ * fail specification:
+ *
+ * add: Add to the fail list
+ *
+ *      Arguments are equivalent to expect, except for skip missing
+ *
+ * clear: Clear the fail list
+ *
+ * Any number of fail specifications can be active during execution of
+ * a logexpect. All active fail specifications are matched against every
+ * log line and, if any match, the logexpect fails immediately.
+ *
+ * For a logexpect to end successfully, there must be no specs on the fail list,
+ * so logexpects should always end with
+ *
+ *      expect <skip> <vxid> <tag> <termination-condition>
+ *      fail clear
+ *
+ * .. XXX can we come up with a better solution which is still safe?
+ *
+ * abort specification:
+ *
+ * abort(3) varnishtest, intended to help debugging of the VSL client library
+ * itself.
  */
 
 #include "config.h"
@@ -126,13 +162,19 @@
 #include "vtim.h"
 #include "vre.h"
 
-#define LE_ANY  (-1)
-#define LE_LAST (-2)
+#define LE_ANY   (-1)
+#define LE_LAST  (-2)
+#define LE_ALT   (-3)
+#define LE_SEEN  (-4)
+#define LE_FAIL  (-5)
+#define LE_CLEAR (-6)	// clear fail list
+#define LE_ABORT (-7)
 
 struct logexp_test {
 	unsigned			magic;
 #define LOGEXP_TEST_MAGIC		0x6F62B350
 	VTAILQ_ENTRY(logexp_test)	list;
+	VTAILQ_ENTRY(logexp_test)	faillist;
 
 	struct vsb			*str;
 	int				vxid;
@@ -141,6 +183,8 @@ struct logexp_test {
 	int				skip_max;
 };
 
+VTAILQ_HEAD(tests_head,logexp_test);
+
 struct logexp {
 	unsigned			magic;
 #define LOGEXP_MAGIC			0xE81D9F1B
@@ -150,14 +194,17 @@ struct logexp {
 	char				*vname;
 	struct vtclog			*vl;
 	char				run;
-	VTAILQ_HEAD(,logexp_test)	tests;
+	struct tests_head		tests;
 
 	struct logexp_test		*test;
 	int				skip_cnt;
 	int				vxid_last;
 	int				tag_last;
 
+	struct tests_head		fail;
+
 	int				m_arg;
+	int				err_arg;
 	int				d_arg;
 	enum VSL_grouping_e		g_arg;
 	char				*query;
@@ -171,6 +218,17 @@ struct logexp {
 static VTAILQ_HEAD(, logexp)		logexps =
 	VTAILQ_HEAD_INITIALIZER(logexps);
 
+static cmd_f cmd_logexp_expect;
+static cmd_f cmd_logexp_fail;
+static cmd_f cmd_logexp_abort;
+
+static const struct cmds logexp_cmds[] = {
+	{ "expect",		cmd_logexp_expect },
+	{ "fail",		cmd_logexp_fail },
+	{ "abort",		cmd_logexp_abort },
+	{ NULL,			NULL },
+};
+
 static void
 logexp_delete_tests(struct logexp *le)
 {
@@ -214,7 +272,8 @@ logexp_new(const char *name, const char *varg)
 	ALLOC_OBJ(le, LOGEXP_MAGIC);
 	AN(le);
 	REPLACE(le->name, name);
-	le->vl = vtc_logopen(name);
+	le->vl = vtc_logopen("%s", name);
+	vtc_log_set_cmd(le->vl, logexp_cmds);
 	VTAILQ_INIT(&le->tests);
 
 	le->d_arg = 0;
@@ -240,20 +299,194 @@ logexp_new(const char *name, const char *varg)
 	return (le);
 }
 
+static void
+logexp_clean(const struct tests_head *head)
+{
+	struct logexp_test *test;
+
+	VTAILQ_FOREACH(test, head, list)
+		if (test->skip_max == LE_SEEN)
+			test->skip_max = LE_ALT;
+}
+
+static struct logexp_test *
+logexp_alt(struct logexp_test *test)
+{
+	assert(test->skip_max == LE_ALT);
+
+	do
+		test = VTAILQ_NEXT(test, list);
+	while (test != NULL && test->skip_max == LE_SEEN);
+
+	if (test == NULL || test->skip_max != LE_ALT)
+		return (NULL);
+
+	return (test);
+}
+
 static void
 logexp_next(struct logexp *le)
 {
 	CHECK_OBJ_NOTNULL(le, LOGEXP_MAGIC);
 
-	if (le->test) {
+	if (le->test && le->test->skip_max == LE_ALT) {
+		/*
+		 * if an alternative was not seen, continue at this expection
+		 * with the next vsl
+		 */
+		(void)0;
+	} else if (le->test) {
 		CHECK_OBJ_NOTNULL(le->test, LOGEXP_TEST_MAGIC);
 		le->test = VTAILQ_NEXT(le->test, list);
-	} else
+	} else {
+		logexp_clean(&le->tests);
+		VTAILQ_INIT(&le->fail);
 		le->test = VTAILQ_FIRST(&le->tests);
+	}
 
-	CHECK_OBJ_ORNULL(le->test, LOGEXP_TEST_MAGIC);
-	if (le->test)
-		vtc_log(le->vl, 3, "expecting| %s", VSB_data(le->test->str));
+	if (le->test == NULL)
+		return;
+
+	CHECK_OBJ(le->test, LOGEXP_TEST_MAGIC);
+
+	switch (le->test->skip_max) {
+	case LE_SEEN:
+		logexp_next(le);
+		return;
+	case LE_CLEAR:
+		vtc_log(le->vl, 3, "cond | fail clear");
+		VTAILQ_INIT(&le->fail);
+		logexp_next(le);
+		return;
+	case LE_FAIL:
+		vtc_log(le->vl, 3, "cond | %s", VSB_data(le->test->str));
+		VTAILQ_INSERT_TAIL(&le->fail, le->test, faillist);
+		logexp_next(le);
+		return;
+	case LE_ABORT:
+		abort();
+		NEEDLESS(return);
+	default:
+		vtc_log(le->vl, 3, "test | %s", VSB_data(le->test->str));
+	}
+}
+
+enum le_match_e {
+	LEM_OK,
+	LEM_SKIP,
+	LEM_FAIL
+};
+
+static enum le_match_e
+logexp_match(const struct logexp *le, struct logexp_test *test,
+    const char *data, int vxid, int tag, int type, int len)
+{
+	const char *legend;
+	int ok = 1, skip = 0, alt, fail, vxid_ok = 0;
+
+	AN(le);
+	AN(test);
+	assert(test->skip_max != LE_SEEN);
+	assert(test->skip_max != LE_CLEAR);
+
+	if (test->vxid == LE_LAST) {
+		if (le->vxid_last != vxid)
+			ok = 0;
+		vxid_ok = ok;
+	} else if (test->vxid >= 0) {
+		if (test->vxid != vxid)
+			ok = 0;
+		vxid_ok = ok;
+	}
+	if (test->tag == LE_LAST) {
+		if (le->tag_last != tag)
+			ok = 0;
+	} else if (test->tag >= 0) {
+		if (test->tag != tag)
+			ok = 0;
+	}
+	if (test->vre &&
+	    test->tag >= 0 &&
+	    test->tag == tag &&
+	    VRE_ERROR_NOMATCH == VRE_match(test->vre, data, len, 0, NULL))
+		ok = 0;
+
+	alt = (test->skip_max == LE_ALT);
+	fail = (test->skip_max == LE_FAIL);
+
+	if (!ok && !alt && (test->skip_max == LE_ANY ||
+		    test->skip_max > le->skip_cnt))
+		skip = 1;
+
+	if (skip && vxid_ok && tag == SLT_End)
+		fail = 1;
+
+	if (fail) {
+		if (ok) {
+			legend = "fail";
+		} else if (skip) {
+			legend = "end";
+			skip = 0;
+		} else if (le->m_arg) {
+			legend = "fmiss";
+		} else {
+			legend = NULL;
+		}
+	}
+	else if (ok)
+		legend = "match";
+	else if (skip && le->m_arg)
+		legend = "miss";
+	else if (skip || alt)
+		legend = NULL;
+	else
+		legend = "err";
+
+	if (legend != NULL)
+		vtc_log(le->vl, 4, "%-5s| %10u %-15s %c %.*s",
+		    legend, vxid, VSL_tags[tag], type, len,
+		    data);
+
+	if (ok) {
+		if (alt)
+			test->skip_max = LE_SEEN;
+		return (LEM_OK);
+	}
+	if (alt) {
+		test = logexp_alt(test);
+		if (test == NULL)
+			return (LEM_FAIL);
+		vtc_log(le->vl, 3, "alt  | %s", VSB_data(test->str));
+		return (logexp_match(le, test, data, vxid, tag, type, len));
+	}
+	if (skip)
+		return (LEM_SKIP);
+	return (LEM_FAIL);
+}
+
+static enum le_match_e
+logexp_failchk(const struct logexp *le,
+    const char *data, int vxid, int tag, int type, int len)
+{
+	struct logexp_test *test;
+	static enum le_match_e r;
+
+	if (VTAILQ_FIRST(&le->fail) == NULL)
+		return (LEM_SKIP);
+
+	VTAILQ_FOREACH(test, &le->fail, faillist) {
+		r = logexp_match(le, test, data, vxid, tag, type, len);
+		if (r == LEM_OK)
+			return (LEM_FAIL);
+		assert (r == LEM_FAIL);
+	}
+	return (LEM_OK);
+}
+
+static int
+logexp_done(const struct logexp *le)
+{
+	return ((VTAILQ_FIRST(&le->fail) == NULL) && le->test == NULL);
 }
 
 static int v_matchproto_(VSLQ_dispatch_f)
@@ -263,9 +496,9 @@ logexp_dispatch(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 	struct logexp *le;
 	struct VSL_transaction *t;
 	int i;
-	int ok, skip;
+	enum le_match_e r;
 	int vxid, tag, type, len;
-	const char *legend, *data;
+	const char *data;
 
 	CAST_OBJ_NOTNULL(le, priv, LOGEXP_MAGIC);
 
@@ -274,76 +507,45 @@ logexp_dispatch(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 			if (!VSL_Match(vsl, t->c))
 				continue;
 
-			CHECK_OBJ_NOTNULL(le->test, LOGEXP_TEST_MAGIC);
 			AN(t->c->rec.ptr);
-			vxid = VSL_ID(t->c->rec.ptr);
 			tag = VSL_TAG(t->c->rec.ptr);
-			data = VSL_CDATA(t->c->rec.ptr);
-			len = VSL_LEN(t->c->rec.ptr) - 1;
 
 			if (tag == SLT__Batch || tag == SLT_Witness)
 				continue;
 
-			ok = 1;
-			if (le->test->vxid == LE_LAST) {
-				if (le->vxid_last != vxid)
-					ok = 0;
-			} else if (le->test->vxid >= 0) {
-				if (le->test->vxid != vxid)
-					ok = 0;
-			}
-			if (le->test->tag == LE_LAST) {
-				if (le->tag_last != tag)
-					ok = 0;
-			} else if (le->test->tag >= 0) {
-				if (le->test->tag != tag)
-					ok = 0;
-			}
-			if (le->test->vre &&
-			    le->test->tag >= 0 &&
-			    le->test->tag == tag &&
-			    VRE_ERROR_NOMATCH == VRE_exec(le->test->vre, data,
-				len, 0, 0, NULL, 0, NULL))
-				ok = 0;
-
-			skip = 0;
-			if (!ok && (le->test->skip_max == LE_ANY ||
-				le->test->skip_max > le->skip_cnt))
-				skip = 1;
-
-			if (ok)
-				legend = "match";
-			else if (skip && le->m_arg)
-				legend = "miss";
-			else if (skip)
-				legend = NULL;
-			else
-				legend = "err";
+			vxid = VSL_ID(t->c->rec.ptr);
+			data = VSL_CDATA(t->c->rec.ptr);
+			len = VSL_LEN(t->c->rec.ptr) - 1;
 			type = VSL_CLIENT(t->c->rec.ptr) ? 'c' :
 			    VSL_BACKEND(t->c->rec.ptr) ? 'b' : '-';
 
-			if (legend != NULL)
-				vtc_log(le->vl, 4, "%-5s| %10u %-15s %c %.*s",
-				    legend, vxid, VSL_tags[tag], type, len,
-				    data);
-
-			if (ok) {
-				le->vxid_last = vxid;
-				le->tag_last = tag;
-				le->skip_cnt = 0;
-				logexp_next(le);
-				if (le->test == NULL)
-					/* End of test script */
-					return (1);
-			} else if (skip)
-				le->skip_cnt++;
-			else {
-				/* Signal fail */
-				return (2);
+			r = logexp_failchk(le, data, vxid, tag, type, len);
+			if (r == LEM_FAIL)
+				return (r);
+			if (le->test == NULL) {
+				assert (r == LEM_OK);
+				continue;
 			}
+
+			CHECK_OBJ_NOTNULL(le->test, LOGEXP_TEST_MAGIC);
+
+			r = logexp_match(le, le->test,
+			    data, vxid, tag, type, len);
+			if (r == LEM_FAIL)
+				return (r);
+			if (r == LEM_SKIP) {
+				le->skip_cnt++;
+				continue;
+			}
+			assert(r == LEM_OK);
+			le->vxid_last = vxid;
+			le->tag_last = tag;
+			le->skip_cnt = 0;
+			logexp_next(le);
+			if (logexp_done(le))
+				return (1);
 		}
 	}
-
 	return (0);
 }
 
@@ -361,18 +563,24 @@ logexp_thread(void *priv)
 	AZ(le->test);
 	vtc_log(le->vl, 4, "begin|");
 	if (le->query != NULL)
-		vtc_log(le->vl, 4, "qry| %s", le->query);
+		vtc_log(le->vl, 4, "qry  | %s", le->query);
 	logexp_next(le);
-	while (le->test) {
+	while (!logexp_done(le) && !vtc_stop && !vtc_error) {
 		i = VSLQ_Dispatch(le->vslq, logexp_dispatch, le);
+		if (i == 2 && le->err_arg) {
+			vtc_log(le->vl, 4, "done | failed as expected");
+			return (NULL);
+		}
 		if (i == 2)
-			vtc_fatal(le->vl, "bad| expectation failed");
+			vtc_fatal(le->vl, "bad  | expectation failed");
 		else if (i < 0)
-			vtc_fatal(le->vl, "bad| dispatch failed (%d)", i);
-		else if (i == 0 && le->test)
+			vtc_fatal(le->vl, "bad  | dispatch failed (%d)", i);
+		else if (i == 0 && ! logexp_done(le))
 			VTIM_sleep(0.01);
 	}
-	vtc_log(le->vl, 4, "end|");
+	if (!logexp_done(le))
+		vtc_fatal(le->vl, "bad  | outstanding expectations");
+	vtc_log(le->vl, 4, "done |");
 
 	return (NULL);
 }
@@ -431,34 +639,18 @@ logexp_wait(struct logexp *le)
 	le->run = 0;
 }
 
+/* shared by expect and fail: parse from av[2] (vxid) onwards */
+
 static void
-cmd_logexp_expect(CMD_ARGS)
+cmd_logexp_common(struct logexp *le, struct vtclog *vl,
+    int skip_max, char * const *av)
 {
-	struct logexp *le;
-	int skip_max;
-	int vxid;
-	int tag;
 	vre_t *vre;
-	const char *err;
-	int pos;
+	struct vsb vsb[1];
+	int err, pos, tag, vxid;
 	struct logexp_test *test;
-	char *end;
+	char *end, errbuf[VRE_ERROR_LEN];
 
-	(void)cmd;
-	CAST_OBJ_NOTNULL(le, priv, LOGEXP_MAGIC);
-	if (av[1] == NULL || av[2] == NULL || av[3] == NULL)
-		vtc_fatal(vl, "Syntax error");
-
-	if (av[4] != NULL && av[5] != NULL)
-		vtc_fatal(vl, "Syntax error");
-
-	if (!strcmp(av[1], "*"))
-		skip_max = LE_ANY;
-	else {
-		skip_max = (int)strtol(av[1], &end, 10);
-		if (*end != '\0' || skip_max < 0)
-			vtc_fatal(vl, "Not a positive integer: '%s'", av[1]);
-	}
 	if (!strcmp(av[2], "*"))
 		vxid = LE_ANY;
 	else if (!strcmp(av[2], "="))
@@ -479,10 +671,15 @@ cmd_logexp_expect(CMD_ARGS)
 	}
 	vre = NULL;
 	if (av[4]) {
-		vre = VRE_compile(av[4], 0, &err, &pos);
-		if (vre == NULL)
+		vre = VRE_compile(av[4], 0, &err, &pos, 1);
+		if (vre == NULL) {
+			AN(VSB_init(vsb, errbuf, sizeof errbuf));
+			AZ(VRE_error(vsb, err));
+			AZ(VSB_finish(vsb));
+			VSB_fini(vsb);
 			vtc_fatal(vl, "Regex error (%s): '%s' pos %d",
-			    err, av[4], pos);
+			    errbuf, av[4], pos);
+		}
 	}
 
 	ALLOC_OBJ(test, LOGEXP_TEST_MAGIC);
@@ -500,10 +697,77 @@ cmd_logexp_expect(CMD_ARGS)
 	VTAILQ_INSERT_TAIL(&le->tests, test, list);
 }
 
-static const struct cmds logexp_cmds[] = {
-	{ "expect",		cmd_logexp_expect },
-	{ NULL,			NULL },
-};
+static void
+cmd_logexp_expect(CMD_ARGS)
+{
+	struct logexp *le;
+	int skip_max;
+	char *end;
+
+	CAST_OBJ_NOTNULL(le, priv, LOGEXP_MAGIC);
+	if (av[1] == NULL || av[2] == NULL || av[3] == NULL)
+		vtc_fatal(vl, "Syntax error");
+
+	if (av[4] != NULL && av[5] != NULL)
+		vtc_fatal(vl, "Syntax error");
+
+	if (!strcmp(av[1], "*"))
+		skip_max = LE_ANY;
+	else if (!strcmp(av[1], "?"))
+		skip_max = LE_ALT;
+	else {
+		skip_max = (int)strtol(av[1], &end, 10);
+		if (*end != '\0' || skip_max < 0)
+			vtc_fatal(vl, "Not a positive integer: '%s'", av[1]);
+	}
+	cmd_logexp_common(le, vl, skip_max, av);
+}
+
+static void
+cmd_logexp_fail(CMD_ARGS)
+{
+	struct logexp *le;
+	struct logexp_test *test;
+
+	CAST_OBJ_NOTNULL(le, priv, LOGEXP_MAGIC);
+
+	if (av[1] == NULL)
+		vtc_fatal(vl, "Syntax error");
+
+	if (!strcmp(av[1], "clear")) {
+		ALLOC_OBJ(test, LOGEXP_TEST_MAGIC);
+		AN(test);
+		test->skip_max = LE_CLEAR;
+		test->str = VSB_new_auto();
+		AN(test->str);
+		AZ(VSB_printf(test->str, "%s %s",
+		    av[0], av[1]));
+		AZ(VSB_finish(test->str));
+
+		VTAILQ_INSERT_TAIL(&le->tests, test, list);
+		return;
+	}
+
+	if (strcmp(av[1], "add"))
+		vtc_fatal(vl, "Unknown fail argument '%s'", av[1]);
+
+	if (av[2] == NULL || av[3] == NULL)
+		vtc_fatal(vl, "Syntax error");
+
+	cmd_logexp_common(le, vl, LE_FAIL, av);
+}
+
+/* aid vsl debugging */
+static void
+cmd_logexp_abort(CMD_ARGS)
+{
+
+	struct logexp *le;
+
+	CAST_OBJ_NOTNULL(le, priv, LOGEXP_MAGIC);
+
+	cmd_logexp_common(le, vl, LE_ABORT, av);
+}
 
 static void
 logexp_spec(struct logexp *le, const char *spec)
@@ -512,7 +776,7 @@ logexp_spec(struct logexp *le, const char *spec)
 
 	logexp_delete_tests(le);
 
-	parse_string(spec, logexp_cmds, le, le->vl);
+	parse_string(le->vl, le, spec);
 }
 
 void
@@ -522,7 +786,6 @@ cmd_logexpect(CMD_ARGS)
 	int i;
 
 	(void)priv;
-	(void)cmd;
 
 	if (av == NULL) {
 		/* Reset and free */
@@ -608,6 +871,10 @@ cmd_logexpect(CMD_ARGS)
 			le->m_arg = !le->m_arg;
 			continue;
 		}
+		if (!strcmp(*av, "-err")) {
+			le->err_arg = !le->err_arg;
+			continue;
+		}
 		if (!strcmp(*av, "-start")) {
 			logexp_start(le);
 			continue;
