@@ -327,7 +327,6 @@ varnish_new(const char *name)
 	v->cli_fd = -1;
 	VTAILQ_INSERT_TAIL(&varnishes, v, list);
 
-
 	return (v);
 }
 
@@ -384,19 +383,19 @@ static void
 varnish_launch(struct varnish *v)
 {
 	struct vsb *vsb, *vsb1;
-	int i, nfd;
+	int i, nfd, asock;
 	char abuf[128], pbuf[128];
-	struct pollfd fd[2];
+	struct pollfd fd[3];
 	enum VCLI_status_e u;
 	const char *err;
 	char *r = NULL;
 
 	/* Create listener socket */
-	v->cli_fd = VTCP_listen_on(default_listen_addr, NULL, 1, &err);
+	asock = VTCP_listen_on(default_listen_addr, NULL, 1, &err);
 	if (err != NULL)
 		vtc_fatal(v->vl, "Create CLI listen socket failed: %s", err);
-	assert(v->cli_fd > 0);
-	VTCP_myname(v->cli_fd, abuf, sizeof abuf, pbuf, sizeof pbuf);
+	assert(asock > 0);
+	VTCP_myname(asock, abuf, sizeof abuf, pbuf, sizeof pbuf);
 
 	AZ(VSB_finish(v->args));
 	vtc_log(v->vl, 2, "Launch");
@@ -465,26 +464,26 @@ varnish_launch(struct varnish *v)
 
 	/* Wait for the varnish to call home */
 	memset(fd, 0, sizeof fd);
-	fd[0].fd = v->cli_fd;
+	fd[0].fd = asock;
 	fd[0].events = POLLIN;
 	fd[1].fd = v->fds[1];
 	fd[1].events = POLLIN;
+	fd[2].fd = v->fds[2];
+	fd[2].events = POLLIN;
 	i = poll(fd, 2, vtc_maxdur * 1000 / 3);
-	vtc_log(v->vl, 4, "CLIPOLL %d 0x%x 0x%x",
-	    i, fd[0].revents, fd[1].revents);
+	vtc_log(v->vl, 4, "CLIPOLL %d 0x%x 0x%x 0x%x",
+	    i, fd[0].revents, fd[1].revents, fd[2].revents);
 	if (i == 0)
 		vtc_fatal(v->vl, "FAIL timeout waiting for CLI connection");
 	if (fd[1].revents & POLLHUP)
 		vtc_fatal(v->vl, "FAIL debug pipe closed");
 	if (!(fd[0].revents & POLLIN))
 		vtc_fatal(v->vl, "FAIL CLI connection wait failure");
-	nfd = accept(v->cli_fd, NULL, NULL);
-	if (nfd < 0) {
-		closefd(&v->cli_fd);
+	nfd = accept(asock, NULL, NULL);
+	closefd(&asock);
+	if (nfd < 0)
 		vtc_fatal(v->vl, "FAIL no CLI connection accepted");
-	}
 
-	closefd(&v->cli_fd);
 	v->cli_fd = nfd;
 
 	vtc_log(v->vl, 3, "CLI connection fd = %d", v->cli_fd);
@@ -700,7 +699,7 @@ varnish_wait(struct varnish *v)
 	/* Then stop it */
 	varnish_stop(v);
 
-	if (varnish_ask_cli(v, "panic.clear", NULL) != CLIS_CANT)
+	if (varnish_ask_cli(v, "panic.show", NULL) != CLIS_CANT)
 		vtc_fatal(v->vl, "Unexpected panic");
 
 	varnish_cleanup(v);
@@ -740,25 +739,36 @@ static void
 varnish_cli(struct varnish *v, const char *cli, unsigned exp, const char *re)
 {
 	enum VCLI_status_e u;
+	struct vsb vsb[1];
 	vre_t *vre = NULL;
-	char *resp = NULL;
-	const char *errptr;
-	int err;
+	char *resp = NULL, errbuf[VRE_ERROR_LEN];
+	int err, erroff;
 
 	VARNISH_LAUNCH(v);
 	if (re != NULL) {
-		vre = VRE_compile(re, 0, &errptr, &err);
-		if (vre == NULL)
-			vtc_fatal(v->vl, "Illegal regexp");
+		vre = VRE_compile(re, 0, &err, &erroff, 1);
+		if (vre == NULL) {
+			AN(VSB_init(vsb, errbuf, sizeof errbuf));
+			AZ(VRE_error(vsb, err));
+			AZ(VSB_finish(vsb));
+			VSB_fini(vsb);
+			vtc_fatal(v->vl, "Illegal regexp: %s (@%d)",
+			    errbuf, erroff);
+		}
 	}
 	u = varnish_ask_cli(v, cli, &resp);
 	vtc_log(v->vl, 2, "CLI %03u <%s>", u, cli);
 	if (exp != 0 && exp != (unsigned)u)
 		vtc_fatal(v->vl, "FAIL CLI response %u expected %u", u, exp);
 	if (vre != NULL) {
-		err = VRE_exec(vre, resp, strlen(resp), 0, 0, NULL, 0, NULL);
-		if (err < 1)
-			vtc_fatal(v->vl, "Expect failed (%d)", err);
+		err = VRE_match(vre, resp, 0, 0, NULL);
+		if (err < 1) {
+			AN(VSB_init(vsb, errbuf, sizeof errbuf));
+			AZ(VRE_error(vsb, err));
+			AZ(VSB_finish(vsb));
+			VSB_fini(vsb);
+			vtc_fatal(v->vl, "Expect failed (%s)", errbuf);
+		}
 		VRE_free(&vre);
 	}
 	free(resp);
@@ -1066,7 +1076,7 @@ vsl_catchup(struct varnish *v)
  *
  * \-errvcl STRING1 STRING2
  *         Load STRING2 as VCL, expecting it to fail, and Varnish to send an
- *         error string matching STRING2
+ *         error string matching STRING1
  *
  * \-jail STRING
  *         Look at ``man varnishd`` (-j) for more information.
