@@ -38,7 +38,7 @@ import re
 def parse_file(fn, cl, tl, sl):
     p = False
     section = ""
-    resec = re.compile("[ /]\* SECTION: ")
+    resec = re.compile("\s*/\* SECTION: ")
 
     try:
         # Python3
@@ -57,13 +57,13 @@ def parse_file(fn, cl, tl, sl):
             cl[section] = []
             if len(a) > 3:
                 tl[section] = re.sub(
-                    r"^[\t ]*\/?\* SECTION: [^ ]+ +",
+                    r"^\s*/?\* SECTION: [^ ]+ +",
                     "", l)
             else:
                 tl[section] = ""
             p = 1
         elif p:
-            cl[section].append(re.sub(r"^ \* ?", "", l))
+            cl[section].append(re.sub(r"^\s*\* ?", "", l))
     f.close()
 
 if __name__ == "__main__":
@@ -74,6 +74,7 @@ if __name__ == "__main__":
         parse_file(fn, cl, tl, sl)
     sl.sort()
     for section in sl:
+        print(".. _vtc-" + section + ":\n")
         print(tl[section], end="")
         a = section
         c = section.count(".")
