@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -30,17 +32,16 @@
 
 struct vcc;
 
-#define VCC_INFO_PREFIX	"/* VCC_INFO"
-
 struct vcc *VCC_New(void);
-void VCC_Allow_InlineC(struct vcc *, unsigned);
 void VCC_Builtin_VCL(struct vcc *, const char *);
-void VCC_Err_Unref(struct vcc *, unsigned);
-void VCC_Unsafe_Path(struct vcc *, unsigned);
 void VCC_VCL_path(struct vcc *, const char *);
 void VCC_VMOD_path(struct vcc *, const char *);
 void VCC_Predef(struct vcc *, const char *type, const char *name);
 void VCC_VCL_Range(unsigned *, unsigned *);
 
-struct vsb *VCC_Compile(struct vcc *, struct vsb **,
-    const char *vclsrc, const char *vclsrcfile);
+#define MGT_VCC(t, n, cc)					\
+	void VCC_ ## cc (struct vcc *, t);
+#include "tbl/mgt_vcc.h"
+
+int VCC_Compile(struct vcc *, struct vsb **,
+    const char *, const char *, const char *, const char *);
