@@ -34,15 +34,16 @@
 /*
  * filters: whether this flag determines beresp.filters default
  *
- * lower, vcl_r, vcl_w, filters, doc */
-BO_FLAG(do_esi,		1, 1, 1, "")
-BO_FLAG(do_gzip,	1, 1, 1, "")
-BO_FLAG(do_gunzip,	1, 1, 1, "")
-BO_FLAG(do_stream,	1, 1, 0, "")
-BO_FLAG(do_pass,	0, 0, 0, "")
-BO_FLAG(uncacheable,	0, 0, 0, "")
-BO_FLAG(was_304,	1, 0, 0, "")
-BO_FLAG(is_bgfetch,	0, 0, 0, "")
+ * lower, vcl_r, vcl_beresp_r, vcl_beresp_w, filters, doc */
+BO_FLAG(do_esi,		0, 1, 1, 1, "")
+BO_FLAG(do_gzip,	0, 1, 1, 1, "")
+BO_FLAG(do_gunzip,	0, 1, 1, 1, "")
+BO_FLAG(do_stream,	0, 1, 1, 0, "")
+BO_FLAG(uncacheable,	0, 0, 0, 0, "")
+BO_FLAG(was_304,	0, 1, 0, 0, "")
+BO_FLAG(is_bgfetch,	1, 0, 0, 0, "")
+BO_FLAG(is_hitmiss,	1, 0, 0, 0, "")
+BO_FLAG(is_hitpass,	1, 0, 0, 0, "")
 #undef BO_FLAG
 
 /*lint -restore */
