@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -90,17 +92,24 @@ CLI_CMD(VCL_INLINE,
 CLI_CMD(VCL_STATE,
 	"vcl.state",
 	"vcl.state <configname> [auto|cold|warm]",
-	"Force the state of the named configuration.",
+	"  Force the state of the named configuration.",
 	"",
 	2, 2
 )
 
 CLI_CMD(VCL_DISCARD,
 	"vcl.discard",
-	"vcl.discard <configname|label>",
-	"Unload the named configuration (when possible).",
-	"",
-	1, 1
+	"vcl.discard <name_pattern>...",
+	"Unload the named configurations (when possible).",
+	"  Unload the named configurations and labels matching at least"
+	" one name pattern. All matching configurations and labels"
+	" are discarded in the correct order with respect to potential"
+	" dependencies. If one configuration or label could not be"
+	" discarded because one of its dependencies would remain,"
+	" nothing is discarded."
+	" Each individual name pattern must match at least one named"
+	" configuration or label.",
+	1, -1
 )
 
 CLI_CMD(VCL_LIST,
@@ -121,12 +130,26 @@ CLI_CMD(VCL_LIST,
 	0, 0
 )
 
+CLI_CMD(VCL_DEPS,
+	"vcl.deps",
+	"vcl.deps [-j]",
+	"List all loaded configuration and their dependencies.",
+	"  Unless ``-j`` is specified for JSON output, the"
+	" output format is up to two columns of dynamic width"
+	" separated by white space with the fields:\n\n"
+	"  * VCL: a VCL program\n\n"
+	"  * Dependency: another VCL program it depends on\n\n"
+	"  Only direct dependencies are listed, and VCLs with"
+	" multiple dependencies are listed multiple times.",
+	0, 0
+)
+
 CLI_CMD(VCL_SHOW,
 	"vcl.show",
-	"vcl.show [-v] <configname>",
+	"vcl.show [-v] [<configname>]",
 	"Display the source code for the specified configuration.",
 	"",
-	1, 2
+	0, 2
 )
 
 CLI_CMD(VCL_USE,
@@ -141,7 +164,9 @@ CLI_CMD(VCL_LABEL,
 	"vcl.label",
 	"vcl.label <label> <configname>",
 	"Apply label to configuration.",
-	"",
+	"  A VCL label is like a UNIX symbolic link, "
+	" a name without substance, which points to another VCL.\n\n"
+	"  Labels are mandatory whenever one VCL references another.",
 	2, 2
 )
 
@@ -170,9 +195,14 @@ CLI_CMD(PARAM_SHOW,
 
 CLI_CMD(PARAM_SET,
 	"param.set",
-	"param.set <param> <value>",
+	"param.set [-j] <param> <value>",
 	"Set parameter value.",
-	"",
+	"  The JSON output is the same as ``param.show -j <param>`` and"
+	" contains the updated value as it would be represented by a"
+	" subsequent execution of ``param.show``.\n\n"
+	"  This can be useful to later verify that a parameter value didn't"
+	" change and to use the value from the JSON output to reset the"
+	" parameter to the desired value.",
 	2,2
 )
 
@@ -202,7 +232,7 @@ CLI_CMD(PING,
 
 CLI_CMD(HELP,
 	"help",
-	"help [-j] [<command>]",
+	"help [-j|<command>]",
 	"Show command/protocol help.",
 	"  ``-j`` specifies JSON output.",
 	0, 1
@@ -308,8 +338,11 @@ CLI_CMD(BACKEND_LIST,
 CLI_CMD(BACKEND_SET_HEALTH,
 	"backend.set_health",
 	"backend.set_health <backend_pattern> [auto|healthy|sick]",
-	"Set health status on the backends.",
-	"",
+	"Set health status of backend(s) matching <backend_pattern>.",
+	"  * With ``auto``, the health status is determined by a probe\n"
+	"    or some other dynamic mechanism, if any\n"
+	"  * ``healthy`` sets the backend as usable\n"
+	"  * ``sick`` sets the backend as unsable\n",
 	2, 2
 )
 
@@ -326,16 +359,26 @@ CLI_CMD(DEBUG_REQPOOLFAIL,
 	"debug.reqpool.fail",
 	"Schedule req-pool failures.",
 	"The argument is read L-R and 'f' means fail:\n\n"
-	"\tparam.set debug.reqpoolfail F__F\n\n"
-	"Means that the frist and the third attempted allocation will fail",
+	"\tparam.set debug.reqpoolfail F__F_F\n\n"
+	"In the example above the first, fourth and sixth attempted\n"
+	"allocations will fail.",
 	1, 1
 )
+
+CLI_CMD(DEBUG_SHUTDOWN_DELAY,
+	"debug.shutdown.delay",
+	"debug.shutdown.delay",
+	"Add a delay to the child process shutdown.",
+	"",
+	1, 1
+)
+
 CLI_CMD(DEBUG_XID,
 	"debug.xid",
-	"debug.xid",
-	"Examine or set XID.",
+	"debug.xid [<xid> [<cachesize>]]",
+	"Examine or set XID. <cachesize> defaults to 1.",
 	"",
-	0, 1
+	0, 2
 )
 
 CLI_CMD(DEBUG_SRANDOM,
@@ -362,6 +405,14 @@ CLI_CMD(DEBUG_PANIC_MASTER,
 	0, 0
 )
 
+CLI_CMD(DEBUG_VCL_SYMTAB,
+	"vcl.symtab",
+	"vcl.symtab",
+	"Dump the VCL symbol-tables.",
+	"",
+	0, 0
+)
+
 CLI_CMD(DEBUG_VMOD,
 	"debug.vmod",
 	"debug.vmod",
@@ -390,6 +441,14 @@ CLI_CMD(STORAGE_LIST,
 	0, 0
 )
 
+CLI_CMD(PID,
+	"pid",
+	"pid [-j]",
+	"Show the pid of the master process, and the worker if it's running.",
+	"  ``-j`` specifies JSON output.",
+	0, 0
+)
+
 #undef CLI_CMD
 
 /*lint -restore */
