@@ -80,14 +80,15 @@ SLTM(SessOpen, 0, "Client connection opened",
 	"The first record for a client connection, with the socket-endpoints"
 	" of the connection.\n\n"
 	"The format is::\n\n"
-	"\t%s %d %s %s %s %d\n"
-	"\t|  |  |  |  |  |\n"
-	"\t|  |  |  |  |  +- File descriptor number\n"
-	"\t|  |  |  |  +---- Local TCP port / 0 for UDS\n"
-	"\t|  |  |  +------- Local IPv4/6 address / 0.0.0.0 for UDS\n"
-	"\t|  |  +---------- Socket name (from -a argument)\n"
-	"\t|  +------------- Remote TCP port / 0 for UDS\n"
-	"\t+---------------- Remote IPv4/6 address / 0.0.0.0 for UDS\n"
+	"\t%s %d %s %s %s %f %d\n"
+	"\t|  |  |  |  |  |  |\n"
+	"\t|  |  |  |  |  |  +- File descriptor number\n"
+	"\t|  |  |  |  |  +---- Session start time (unix epoch)\n"
+	"\t|  |  |  |  +------- Local TCP port / 0 for UDS\n"
+	"\t|  |  |  +---------- Local IPv4/6 address / 0.0.0.0 for UDS\n"
+	"\t|  |  +------------- Socket name (from -a argument)\n"
+	"\t|  +---------------- Remote TCP port / 0 for UDS\n"
+	"\t+------------------- Remote IPv4/6 address / 0.0.0.0 for UDS\n"
 	"\n"
 )
 
@@ -286,18 +287,17 @@ SLTM(Fetch_Body, 0, "Body fetched from backend",
 )
 
 SLTM(VCL_acl, 0, "VCL ACL check results",
-	"Logs VCL ACL evaluation results.\n\n"
+	"ACLs with the `+log` flag emits this record with the result.\n\n"
 	"The format is::\n\n"
-	"\t%s [%s [%s [fixed: %s]]]\n"
-	"\t|   |   |          |\n"
-	"\t|   |   |          +- Fixed entry (see vcc_acl_pedantic parameter)\n"
-	"\t|   |   +------------ Matching entry (only for MATCH)\n"
-	"\t|   +---------------- Name of the ACL for MATCH or NO_MATCH\n"
-	"\t+-------------------- MATCH, NO_MATCH or NO_FAM\n"
+	"\t%s %s [%s [fixed: %s]]\n"
+	"\t|  |   |          |\n"
+	"\t|  |   |          +- Fixed entry (see vcc_acl_pedantic parameter)\n"
+	"\t|  |   +------------ Matching entry (only for MATCH)\n"
+	"\t|  +---------------- Name of the ACL\n"
+	"\t+-------------------- MATCH or NO_MATCH\n"
 	"\n"
 	"MATCH denotes an ACL match\n"
 	"NO_MATCH denotes that a checked ACL has not matched\n"
-	"NO_FAM denotes a missing address family and should not occur.\n"
 	"\n"
 )
 
@@ -338,12 +338,14 @@ SLTM(ReqStart, 0, "Client request start",
 SLTM(Hit, 0, "Hit object in cache",
 	"Object looked up in cache.\n\n"
 	"The format is::\n\n"
-	"\t%u %f %f %f\n"
-	"\t|  |  |  |\n"
-	"\t|  |  |  +- Keep period\n"
-	"\t|  |  +---- Grace period\n"
-	"\t|  +------- Remaining TTL\n"
-	"\t+---------- VXID of the object\n"
+	"\t%u %f %f %f [%u [%u]]\n"
+	"\t|  |  |  |   |   |\n"
+	"\t|  |  |  |   |   +- Content length\n"
+	"\t|  |  |  |   +----- Fetched so far\n"
+	"\t|  |  |  +--------- Keep period\n"
+	"\t|  |  +------------ Grace period\n"
+	"\t|  +--------------- Remaining TTL\n"
+	"\t+------------------ VXID of the object\n"
 	"\n"
 )
 
@@ -455,14 +457,18 @@ SLTM(Backend_health, 0, "Backend health check",
 	"\n"
 
 	"Probe window bits are::\n\n"
-	"\t" "'4'" ": " "Good IPv4" "\n"
-	"\t" "'6'" ": " "Good IPv6" "\n"
-	"\t" "'U'" ": " "Good UNIX" "\n"
-	"\t" "'x'" ": " "Error Xmit" "\n"
-	"\t" "'X'" ": " "Good Xmit" "\n"
-	"\t" "'r'" ": " "Error Recv" "\n"
-	"\t" "'R'" ": " "Good Recv" "\n"
-	"\t" "'H'" ": " "Happy" "\n"
+	"\t'-': Could not connect\n"
+	"\t'4': Good IPv4\n"
+	"\t'6': Good IPv6\n"
+	"\t'U': Good UNIX\n"
+	"\t'x': Error Xmit\n"
+	"\t'X': Good Xmit\n"
+	"\t'r': Error Recv\n"
+	"\t'R': Good Recv\n"
+	"\t'H': Happy\n"
+	"\n"
+	"When the backend is just created, the window bits for health check\n"
+	"slots that haven't run yet appear as '-' like failures to connect.\n"
 	"\n"
 )
 
@@ -683,7 +689,7 @@ SLTM(VCL_use, 0, "VCL in use",
 )
 
 SLTM(Notice, 0, "Informational messages about request handling",
-	"Informational log messages on events occured during request"
+	"Informational log messages on events occurred during request"
 	" handling.\n\n"
 	"The format is::\n\n"
 	"\t%s: %s\n"
@@ -696,6 +702,19 @@ SLTM(Notice, 0, "Informational messages about request handling",
 	"\n"
 )
 
+SLTM(VdpAcct, 0, "Deliver filter accounting",
+	"Contains name of VDP and statistics.\n\n"
+	"The format is::\n\n"
+	"\t%s %d %d\n"
+	"\t|  |  |\n"
+	"\t|  |  +- Total bytes produced\n"
+	"\t|  +---- Number of calls made\n"
+	"\t+------- Name of filter\n"
+	"\n"
+	NODEF_NOTICE
+)
+
+
 #undef NOSUP_NOTICE
 #undef NODEF_NOTICE
 #undef SLTM
