@@ -69,6 +69,7 @@ struct VSC_point {
 	const char *sdesc;		/* short description		*/
 	const char *ldesc;		/* long description		*/
 	void *priv;			/* return val from VSC_new_f	*/
+	unsigned raw;			/* adjusted or raw value	*/
 };
 
 /*---------------------------------------------------------------------
@@ -118,6 +119,7 @@ int VSC_Arg(struct vsc *, char arg, const char *opt);
 	 *	'X' - field exclusion glob
 	 *	'R' - required field glob
 	 *	'f' - legacy field filter glob (deprecated)
+	 *	'r' - toggle raw gauges
 	 *
 	 * Return:
 	 *	-1 error, VSM_Error() returns diagnostic string
@@ -159,4 +161,20 @@ const struct VSC_level_desc *VSC_ChangeLevel(const struct VSC_level_desc*, int);
 	 * Change a level up or down.
 	 */
 
+unsigned VSC_IsRaw(const struct vsc *);
+	/*
+	 * Returns zero if gauges are adjusted by VSC_Value().
+	 */
+
+static inline uint64_t
+VSC_Value(const struct VSC_point * const pt)
+{
+	uint64_t val;
+
+	val = *pt->ptr;
+	if (!pt->raw && pt->semantics == 'g' && val > INT64_MAX)
+		val = 0;
+	return (val);
+}
+
 #endif /* VAPI_VSC_H_INCLUDED */
