@@ -35,6 +35,7 @@
 struct suckaddr;
 extern const size_t vsa_suckaddr_len;
 extern const struct suckaddr *bogo_ip;
+extern const struct suckaddr *bogo_ip6;
 
 void VSA_Init(void);
 int VSA_Sane(const struct suckaddr *);
@@ -42,6 +43,8 @@ unsigned VSA_Port(const struct suckaddr *);
 int VSA_Compare(const struct suckaddr *, const struct suckaddr *);
 int VSA_Compare_IP(const struct suckaddr *, const struct suckaddr *);
 struct suckaddr *VSA_Clone(const struct suckaddr *sua);
+struct suckaddr *VSA_getsockname(int, void *, size_t);
+struct suckaddr *VSA_getpeername(int, void *, size_t);
 
 const void *VSA_Get_Sockaddr(const struct suckaddr *, socklen_t *sl);
 int VSA_Get_Proto(const struct suckaddr *);
