@@ -6,6 +6,8 @@
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  * Author: Martin Blix Grydeland <martin@varnish-software.com>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -39,9 +41,6 @@
 
 #include "vapi/vsl_int.h"
 
-#define VSL_CLASS		"Log"
-#define VSL_SEGMENTS		8U	// power of two
-
 /*
  * Shared memory log format
  *
