@@ -116,48 +116,53 @@ VCLS_func_ping_json(struct cli *cli, const char * const *av, void *priv)
 
 /*--------------------------------------------------------------------*/
 
+static void
+help_helper(struct cli *cli, struct cli_proto *clp, const char * const *av)
+{
+	AN(clp->desc->syntax);
+	if (av[0] != NULL)
+		VCLI_Out(cli, "%s\n%s\n", clp->desc->syntax, clp->desc->help);
+	else
+		VCLI_Out(cli, "%s\n", clp->desc->syntax);
+}
+
 void v_matchproto_(cli_func_t)
 VCLS_func_help(struct cli *cli, const char * const *av, void *priv)
 {
 	struct cli_proto *clp;
-	unsigned all, debug, d;
+	unsigned filter = 1, d;
 	struct VCLS *cs;
 
 	(void)priv;
 	cs = cli->cls;
 	CHECK_OBJ_NOTNULL(cs, VCLS_MAGIC);
 
-	if (av[2] == NULL) {
-		all = debug = 0;
-	} else if (!strcmp(av[2], "-a")) {
-		all = 1;
-		debug = 0;
-	} else if (!strcmp(av[2], "-d")) {
-		all = 0;
-		debug = 1;
-	} else {
-		VTAILQ_FOREACH(clp, &cs->funcs, list) {
-			if (clp->auth <= cli->auth &&
-			    !strcmp(clp->desc->request, av[2])) {
-				VCLI_Out(cli, "%s\n%s\n",
-				    clp->desc->syntax, clp->desc->help);
-				return;
-			}
+	for (av += 2; av[0] != NULL && av[0][0] == '-'; av++) {
+		if (!strcmp(av[0], "-a")) {
+			filter = 3;
+		} else if (!strcmp(av[0], "-d")) {
+			filter = 2;
+		} else {
+			VCLI_Out(cli, "Unknown flag\n");
+			VCLI_SetResult(cli, CLIS_UNKNOWN);
+			return;
 		}
-		VCLI_Out(cli, "Unknown request.\nType 'help' for more info.\n");
-		VCLI_SetResult(cli, CLIS_UNKNOWN);
-		return;
 	}
 	VTAILQ_FOREACH(clp, &cs->funcs, list) {
 		if (clp->auth > cli->auth)
 			continue;
-		d =  strchr(clp->flags, 'd') != NULL ? 1 : 0;
-		if (d && (!all && !debug))
-			continue;
-		if (debug && !d)
-			continue;
-		if (clp->desc->syntax != NULL)
-			VCLI_Out(cli, "%s\n", clp->desc->syntax);
+		if (av[0] != NULL && !strcmp(clp->desc->request, av[0])) {
+			help_helper(cli, clp, av);
+			return;
+		} else if (av[0] == NULL) {
+			d = strchr(clp->flags, 'd') != NULL ? 2 : 1;
+			if (filter & d)
+				help_helper(cli, clp, av);
+		}
+	}
+	if (av[0] != NULL) {
+		VCLI_Out(cli, "Unknown request.\nType 'help' for more info.\n");
+		VCLI_SetResult(cli, CLIS_UNKNOWN);
 	}
 }
 
@@ -638,16 +643,13 @@ VCLI_Out(struct cli *cli, const char *fmt, ...)
 {
 	va_list ap;
 
+	AN(cli);
 	va_start(ap, fmt);
-	if (cli != NULL) {
-		CHECK_OBJ_NOTNULL(cli, CLI_MAGIC);
-		if (VSB_len(cli->sb) < *cli->limit)
-			(void)VSB_vprintf(cli->sb, fmt, ap);
-		else if (cli->result == CLIS_OK)
-			cli->result = CLIS_TRUNCATED;
-	} else {
-		(void)vfprintf(stdout, fmt, ap);
-	}
+	CHECK_OBJ_NOTNULL(cli, CLI_MAGIC);
+	if (VSB_len(cli->sb) < *cli->limit)
+		(void)VSB_vprintf(cli->sb, fmt, ap);
+	else if (cli->result == CLIS_OK)
+		cli->result = CLIS_TRUNCATED;
 	va_end(ap);
 }
 
