@@ -219,12 +219,17 @@ void
 VEV_Destroy(struct vev_root **evbp)
 {
 	struct vev_root *evb;
+	struct vev *e;
 
 	TAKE_OBJ_NOTNULL(evb, evbp, VEV_BASE_MAGIC);
-	assert(evb->thread == pthread_self());
+	assert(pthread_equal(evb->thread, pthread_self()));
+	while ((e = VBH_root(evb->binheap)) != NULL) {
+		VEV_Stop(evb, e);
+		free(e);
+	}
+	VBH_destroy(&evb->binheap);
 	free(evb->pfd);
 	free(evb->pev);
-	/* XXX: destroy evb->binheap */
 	FREE_OBJ(evb);
 }
 
@@ -255,7 +260,7 @@ VEV_Start(struct vev_root *evb, struct vev *e)
 	assert(e->sig >= 0);
 	assert(e->timeout >= 0.0);
 	assert(e->fd < 0 || e->fd_flags);
-	assert(evb->thread == pthread_self());
+	assert(pthread_equal(evb->thread, pthread_self()));
 	DBG(evb, "ev_add(%p) fd = %d\n", e, e->fd);
 
 	if (vev_get_pfd(evb))
@@ -311,7 +316,7 @@ VEV_Stop(struct vev_root *evb, struct vev *e)
 	CHECK_OBJ_NOTNULL(e, VEV_MAGIC);
 	DBG(evb, "ev_del(%p) fd = %d i=%u L=%d\n", e, e->fd, e->__binheap_idx, evb->lpfd);
 	assert(evb == e->__vevb);
-	assert(evb->thread == pthread_self());
+	assert(pthread_equal(evb->thread, pthread_self()));
 	assert(evb->pev[e->__binheap_idx] == e);
 
 	assert(e->__binheap_idx != VBH_NOIDX);
@@ -332,8 +337,8 @@ VEV_Stop(struct vev_root *evb, struct vev *e)
 		es->happened = 0;
 	}
 
-	e->magic = 0;
 	e->__vevb = NULL;
+	FINI_OBJ(e);
 }
 
 /*--------------------------------------------------------------------*/
@@ -344,7 +349,7 @@ VEV_Loop(struct vev_root *evb)
 	int i;
 
 	CHECK_OBJ_NOTNULL(evb, VEV_BASE_MAGIC);
-	assert(evb->thread == pthread_self());
+	assert(pthread_equal(evb->thread, pthread_self()));
 	do
 		i = VEV_Once(evb);
 	while (i == 1);
@@ -406,7 +411,7 @@ VEV_Once(struct vev_root *evb)
 	unsigned u;
 
 	CHECK_OBJ_NOTNULL(evb, VEV_BASE_MAGIC);
-	assert(evb->thread == pthread_self());
+	assert(pthread_equal(evb->thread, pthread_self()));
 	assert(evb->lpfd < evb->npfd);
 
 	if (evb->psig)
