@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2000-2008 Poul-Henning Kamp
+ * Copyright (c) 2000-2008 Poul-Henning Kamp <phk@FreeBSD.org>
  * Copyright (c) 2000-2008 Dag-Erling Coïdan Smørgrav
  * All rights reserved.
  *
@@ -34,6 +34,7 @@ __FBSDID("$FreeBSD: head/sys/kern/subr_vsb.c 222004 2011-05-17 06:36:32Z phk $")
 #include <ctype.h>
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -46,7 +47,7 @@ __FBSDID("$FreeBSD: head/sys/kern/subr_vsb.c 222004 2011-05-17 06:36:32Z phk $")
 #define	SBMALLOC(size)		malloc(size)
 #define	SBFREE(buf)		free(buf)
 
-#define	roundup2(x, y)	(((x)+((y)-1))&(~((y)-1))) /* if y is powers of two */
+#define	rndup2(x, y)	(((x)+((y)-1))&(~((y)-1))) /* if y is powers of two */
 
 /*
  * Predicates
@@ -126,7 +127,7 @@ VSB_extendsize(ssize_t size)
 		while (newsize < size)
 			newsize *= 2;
 	} else {
-		newsize = roundup2(size, VSB_MAXEXTENDINCR);
+		newsize = rndup2(size, VSB_MAXEXTENDINCR);
 	}
 	KASSERT(newsize >= size, ("%s: %d < %d\n", __func__, newsize, size));
 	return (newsize);
@@ -207,35 +208,6 @@ VSB_newbuf(struct vsb *s, char *buf, int length, int flags)
 	return (s);
 }
 
-/*
- * Initialize an vsb.
- * If buf is non-NULL, it points to a static or already-allocated string
- * big enough to hold at least length characters.
- */
-struct vsb *
-VSB_new(struct vsb *s, char *buf, int length, int flags)
-{
-
-	KASSERT(length >= 0,
-	    ("attempt to create an vsb of negative length (%d)", length));
-	KASSERT((flags & ~VSB_USRFLAGMSK) == 0,
-	    ("%s called with invalid flags", __func__));
-
-	flags &= VSB_USRFLAGMSK;
-	if (s != NULL)
-		return (VSB_newbuf(s, buf, length, flags));
-
-	s = SBMALLOC(sizeof(*s));
-	if (s == NULL)
-		return (NULL);
-	if (VSB_newbuf(s, buf, length, flags) == NULL) {
-		SBFREE(s);
-		return (NULL);
-	}
-	VSB_SETFLAG(s, VSB_DYNSTRUCT);
-	return (s);
-}
-
 struct vsb *
 VSB_init(struct vsb *s, void *buf, ssize_t length)
 {
@@ -338,6 +310,8 @@ VSB_bcat(struct vsb *s, const void *buf, ssize_t len)
 int
 VSB_cat(struct vsb *s, const char *str)
 {
+	const char *nl;
+	size_t l;
 
 	assert_VSB_integrity(s);
 	assert_VSB_state(s, 0);
@@ -347,12 +321,15 @@ VSB_cat(struct vsb *s, const char *str)
 	if (s->s_error != 0)
 		return (-1);
 
-	while (*str != '\0') {
-		VSB_put_byte(s, *str++);
-		if (s->s_error != 0)
+	while (s->s_indent > 0 && (nl = strchr(str, '\n')) != NULL) {
+		l = (nl - str) + 1;
+		if (VSB_bcat(s, str, l) < 0)
 			return (-1);
+		str += l;
 	}
-	return (0);
+
+	l = strlen(str);
+	return (VSB_bcat(s, str, l));
 }
 
 /*
@@ -406,9 +383,7 @@ VSB_vprintf(struct vsb *s, const char *fmt, va_list ap)
 	 * vsnprintf() returns the amount that would have been copied,
 	 * given sufficient space, so don't over-increment s_len.
 	 */
-	if (VSB_FREESPACE(s) < len)
-		len = VSB_FREESPACE(s);
-	s->s_len += len;
+	s->s_len += vmin_t(ssize_t, len, VSB_FREESPACE(s));
 	if (!VSB_HASROOM(s) && !VSB_CANEXTEND(s))
 		s->s_error = ENOMEM;
 
@@ -504,31 +479,13 @@ VSB_len(const struct vsb *s)
 	return (s->s_len);
 }
 
-/*
- * Clear an vsb, free its buffer if necessary.
- */
-void
-VSB_delete(struct vsb *s)
-{
-	int isdyn;
-
-	assert_VSB_integrity(s);
-	/* don't care if it's finished or not */
-
-	if (VSB_ISDYNAMIC(s))
-		SBFREE(s->s_buf);
-	isdyn = VSB_ISDYNSTRUCT(s);
-	memset(s, 0, sizeof(*s));
-	if (isdyn)
-		SBFREE(s);
-}
-
 void
 VSB_fini(struct vsb *s)
 {
 
 	assert_VSB_integrity(s);
 	assert(!VSB_ISDYNAMIC(s));
+	assert(!VSB_ISDYNSTRUCT(s));
 	memset(s, 0, sizeof(*s));
 }
 
@@ -539,6 +496,7 @@ VSB_destroy(struct vsb **s)
 	AN(s);
 	assert_VSB_integrity(*s);
 	assert(VSB_ISDYNAMIC(*s));
+	assert(VSB_ISDYNSTRUCT(*s));
 	SBFREE((*s)->s_buf);
 	memset(*s, 0, sizeof(**s));
 	SBFREE(*s);
@@ -548,73 +506,102 @@ VSB_destroy(struct vsb **s)
 /*
  * Quote a string
  */
+
+static void
+vsb_quote_hex(struct vsb *s, const uint8_t *u, size_t len)
+{
+	const uint8_t *w;
+
+	VSB_cat(s, "0x");
+	for (w = u; w < u + len; w++)
+		if (*w != 0x00)
+			break;
+	if (w == u + len && len > 4) {
+		VSB_cat(s, "0...0");
+	} else {
+		for (w = u; w < u + len; w++)
+			VSB_printf(s, "%02x", *w);
+	}
+}
+
 void
 VSB_quote_pfx(struct vsb *s, const char *pfx, const void *v, int len, int how)
 {
-	const char *p;
-	const char *q;
+	const uint8_t *p = v;
+	const uint8_t *q;
 	int quote = 0;
-	int nl = 0;
-	const unsigned char *u, *w;
+	int nl;
 
-	assert(v != NULL);
+	nl = how &
+	    (VSB_QUOTE_JSON|VSB_QUOTE_HEX|VSB_QUOTE_CSTR|VSB_QUOTE_UNSAFE);
+	AZ(nl & (nl - 1)); // Only one bit can be set
+
+	if (how & VSB_QUOTE_ESCHEX)
+		AZ(how & (VSB_QUOTE_JSON|VSB_QUOTE_HEX));
+
+	if (how & VSB_QUOTE_UNSAFE)
+		how |= VSB_QUOTE_NONL;
+
+	assert(p != NULL);
 	if (len == -1)
 		len = strlen(v);
 
 	if (len == 0 && (how & VSB_QUOTE_CSTR)) {
 		VSB_printf(s, "%s\"\"", pfx);
-		return;
-	} else if (len == 0)
+		if ((how & VSB_QUOTE_NONL))
+			VSB_putc(s, '\n');
+	}
+
+	if (len == 0)
 		return;
 
 	VSB_cat(s, pfx);
 
 	if (how & VSB_QUOTE_HEX) {
-		u = v;
-		for (w = u; w < u + len; w++)
-			if (*w != 0x00)
-				break;
-		VSB_cat(s, "0x");
-		if (w == u + len && len > 4) {
-			VSB_cat(s, "0...0");
-		} else {
-			for (w = u; w < u + len; w++)
-				VSB_printf(s, "%02x", *w);
-		}
-		return;
-	}
-	p = v;
-
-	for (q = p; q < p + len; q++) {
-		if (!isgraph(*q) || *q == '"' || *q == '\\') {
-			quote++;
-			break;
-		}
-	}
-	if (!quote && !(how & (VSB_QUOTE_JSON|VSB_QUOTE_CSTR))) {
-		(void)VSB_bcat(s, p, len);
-		if ((how & (VSB_QUOTE_UNSAFE|VSB_QUOTE_NONL)) &&
-		    p[len-1] != '\n')
-			(void)VSB_putc(s, '\n');
+		vsb_quote_hex(s, v, len);
+		if (how & VSB_QUOTE_NONL)
+			VSB_putc(s, '\n');
 		return;
 	}
 
 	if (how & VSB_QUOTE_CSTR)
-		(void)VSB_putc(s, '"');
+		VSB_putc(s, '"');
 
+	for (q = p; q < p + len; q++) {
+		if (
+		    *q < 0x20 ||
+		    *q == '"' ||
+		    *q == '\\' ||
+		    (*q == '?' && (how & VSB_QUOTE_CSTR)) ||
+		    (*q > 0x7e && !(how & VSB_QUOTE_JSON))
+		) {
+			quote++;
+			break;
+		}
+	}
+
+	if (!quote) {
+		VSB_bcat(s, p, len);
+		if ((how & VSB_QUOTE_NONL) &&
+		    p[len-1] != '\n')
+			(void)VSB_putc(s, '\n');
+		if (how & VSB_QUOTE_CSTR)
+			VSB_putc(s, '"');
+		return;
+	}
+
+	nl = 0;
 	for (q = p; q < p + len; q++) {
 		if (nl)
 			VSB_cat(s, pfx);
 		nl = 0;
 		switch (*q) {
 		case '?':
-			if (how & VSB_QUOTE_CSTR)
+			/* Avoid C Trigraph insanity */
+			if (how & VSB_QUOTE_CSTR && !(how & VSB_QUOTE_JSON))
 				(void)VSB_putc(s, '\\');
 			(void)VSB_putc(s, *q);
 			break;
-		case ' ':
-			(void)VSB_putc(s, *q);
-			break;
 		case '\\':
 		case '"':
 			if (!(how & VSB_QUOTE_UNSAFE))
@@ -623,38 +610,43 @@ VSB_quote_pfx(struct vsb *s, const char *pfx, const void *v, int len, int how)
 			break;
 		case '\n':
 			if (how & VSB_QUOTE_CSTR) {
-				(void)VSB_printf(s, "\\n\"\n%s\"", pfx);
-			} else if (how & (VSB_QUOTE_NONL|VSB_QUOTE_UNSAFE)) {
-				(void)VSB_printf(s, "\n");
+				VSB_printf(s, "\\n\"\n%s\"", pfx);
+			} else if (how & VSB_QUOTE_JSON) {
+				VSB_printf(s, "\\n");
+			} else if (how & VSB_QUOTE_NONL) {
+				VSB_putc(s, *q);
 				nl = 1;
 			} else {
-				(void)VSB_printf(s, "\\n");
+				VSB_printf(s, "\\n");
 			}
 			break;
 		case '\r':
-			(void)VSB_cat(s, "\\r");
+			VSB_cat(s, "\\r");
 			break;
 		case '\t':
-			(void)VSB_cat(s, "\\t");
+			VSB_cat(s, "\\t");
 			break;
 		case '\v':
-			(void)VSB_cat(s, "\\v");
+			VSB_cat(s, "\\v");
 			break;
 		default:
-			/* XXX: Implement VSB_QUOTE_JSON */
-			if (isgraph(*q))
-				(void)VSB_putc(s, *q);
+			if (0x20 <= *q && *q <= 0x7e)
+				VSB_putc(s, *q);
+			else if (*q > 0x7e && (how & VSB_QUOTE_JSON))
+				VSB_putc(s, *q);
+			else if (how & VSB_QUOTE_JSON)
+				VSB_printf(s, "\\u%04x", *q);
 			else if (how & VSB_QUOTE_ESCHEX)
-				(void)VSB_printf(s, "\\x%02x", *q & 0xff);
+				VSB_printf(s, "\\x%02x", *q);
 			else
-				(void)VSB_printf(s, "\\%03o", *q & 0xff);
+				VSB_printf(s, "\\%03o", *q);
 			break;
 		}
 	}
 	if (how & VSB_QUOTE_CSTR)
-		(void)VSB_putc(s, '"');
-	if ((how & (VSB_QUOTE_NONL|VSB_QUOTE_UNSAFE)) && !nl)
-		(void)VSB_putc(s, '\n');
+		VSB_putc(s, '"');
+	if ((how & VSB_QUOTE_NONL) && !nl)
+		VSB_putc(s, '\n');
 }
 
 void
