@@ -76,6 +76,7 @@ static const char * const vsl_r_names[VSL_r__MAX] = {
 };
 
 struct vtx;
+VTAILQ_HEAD(vtxhead, vtx);
 
 struct vslc_raw {
 	unsigned		magic;
@@ -161,12 +162,12 @@ struct vtx {
 	enum VSL_reason_e	reason;
 
 	struct vtx		*parent;
-	VTAILQ_HEAD(,vtx)	child;
+	struct vtxhead		child;
 	unsigned		n_child;
 	unsigned		n_childready;
 	unsigned		n_descend;
 
-	VTAILQ_HEAD(,synth)	synth;
+	struct synthhead	synth;
 
 	struct chunk		shmchunks[VTX_SHMCHUNKS];
 	struct chunkhead	shmchunks_free;
@@ -189,11 +190,11 @@ struct VSLQ {
 
 	/* Structured mode */
 	struct vtx_tree		tree;
-	VTAILQ_HEAD(,vtx)	ready;
-	VTAILQ_HEAD(,vtx)	incomplete;
+	struct vtxhead		ready;
+	struct vtxhead		incomplete;
 	int			n_outstanding;
 	struct chunkhead	shmrefs;
-	VTAILQ_HEAD(,vtx)	cache;
+	struct vtxhead		cache;
 	unsigned		n_cache;
 
 	/* Rate limiting */
@@ -228,8 +229,11 @@ vtx_keycmp(const struct vtx_key *a, const struct vtx_key *b)
 	return (0);
 }
 
-VRBT_PROTOTYPE_STATIC(vtx_tree, vtx_key, entry, vtx_keycmp)
-VRBT_GENERATE_STATIC(vtx_tree, vtx_key, entry, vtx_keycmp)
+VRBT_GENERATE_REMOVE_COLOR(vtx_tree, vtx_key, entry, static)
+VRBT_GENERATE_REMOVE(vtx_tree, vtx_key, entry, static)
+VRBT_GENERATE_INSERT_COLOR(vtx_tree, vtx_key, entry, static)
+VRBT_GENERATE_INSERT(vtx_tree, vtx_key, entry, vtx_keycmp, static)
+VRBT_GENERATE_FIND(vtx_tree, vtx_key, entry, vtx_keycmp, static)
 
 static enum vsl_status v_matchproto_(vslc_next_f)
 vslc_raw_next(const struct VSL_cursor *cursor)
@@ -372,12 +376,12 @@ chunk_newbuf(struct vtx *vtx, const uint32_t *ptr, size_t len)
 static void
 chunk_freebuf(struct chunk **pchunk)
 {
+	struct chunk *chunk;
 
-	CHECK_OBJ_NOTNULL(*pchunk, CHUNK_MAGIC);
-	assert((*pchunk)->type == chunk_t_buf);
-	free((*pchunk)->buf.data);
-	FREE_OBJ(*pchunk);
-	*pchunk = NULL;
+	TAKE_OBJ_NOTNULL(chunk, pchunk, CHUNK_MAGIC);
+	assert(chunk->type == chunk_t_buf);
+	free(chunk->buf.data);
+	FREE_OBJ(chunk);
 }
 
 /* Append a set of records to a chunk */
@@ -559,7 +563,8 @@ vtx_retire(struct VSLQ *vslq, struct vtx **pvtx)
 	AZ(vtx->n_child);
 	AZ(vtx->n_descend);
 	vtx->n_childready = 0;
-	AN(VRBT_REMOVE(vtx_tree, &vslq->tree, &vtx->key));
+	// remove rval is no way to check if element was present
+	(void)VRBT_REMOVE(vtx_tree, &vslq->tree, &vtx->key);
 	vtx->key.vxid = 0;
 	vtx->flags = 0;
 
@@ -590,9 +595,9 @@ vtx_retire(struct VSLQ *vslq, struct vtx **pvtx)
 	if (vslq->n_cache < VTX_CACHE) {
 		VTAILQ_INSERT_HEAD(&vslq->cache, vtx, list_child);
 		vslq->n_cache++;
-	} else {
+	} else
 		FREE_OBJ(vtx);
-	}
+
 }
 
 /* Lookup a vtx by vxid from the managed list */
@@ -930,8 +935,7 @@ vslq_ratelimit(struct VSLQ *vslq)
 	now = VTIM_mono();
 	delta = now - vslq->last_use;
 	vslq->credits += (delta / vslq->vsl->R_opt_p) * vslq->vsl->R_opt_l;
-	if (vslq->credits > vslq->vsl->R_opt_l)
-		vslq->credits = vslq->vsl->R_opt_l;
+	vslq->credits = vmin_t(double, vslq->credits, vslq->vsl->R_opt_l);
 	vslq->last_use = now;
 
 	if (vslq->credits < 1.0)
@@ -1163,6 +1167,7 @@ VSLQ_Delete(struct VSLQ **pvslq)
 	while (!VTAILQ_EMPTY(&vslq->cache)) {
 		AN(vslq->n_cache);
 		vtx = VTAILQ_FIRST(&vslq->cache);
+		CHECK_OBJ_NOTNULL(vtx, VTX_MAGIC);
 		VTAILQ_REMOVE(&vslq->cache, vtx, list_child);
 		vslq->n_cache--;
 		FREE_OBJ(vtx);
