@@ -123,7 +123,6 @@ vslq_test_rec(const struct vex *vex, const struct VSLC_ptr *rec)
 	long long lhs_int = 0;
 	double lhs_float = 0.;
 	const char *b, *e, *q;
-	char *p;
 	int i, dq;
 
 	AN(vex);
@@ -208,15 +207,13 @@ vslq_test_rec(const struct vex *vex, const struct VSLC_ptr *rec)
 			return (0);
 		switch (rhs->type) {
 		case VEX_INT:
-			lhs_int = strtoll(b, &p, 0);
-			if (*p != '\0' && !isspace(*p))
-				return (0); /* Can't parse - no match */
+			lhs_int = (long long)SF_Parse_Number(&b, 0, &q);
+			if (errno)
+				return (0);
 			break;
 		case VEX_FLOAT:
-			lhs_float = VNUMpfx(b, &q);
-			if (isnan(lhs_float))
-				return (0);
-			if (q != NULL && q > b && !isspace(q[-1]))
+			lhs_float = SF_Parse_Decimal(&b, 0, &q);
+			if (errno)
 				return (0);
 			break;
 		default:
@@ -267,13 +264,13 @@ vslq_test_rec(const struct vex *vex, const struct VSLC_ptr *rec)
 		return (0);
 	case '~':		/* ~ */
 		assert(rhs->type == VEX_REGEX && rhs->val_regex != NULL);
-		i = VRE_exec(rhs->val_regex, b, e - b, 0, 0, NULL, 0, NULL);
+		i = VRE_match(rhs->val_regex, b, e - b, 0, NULL);
 		if (i != VRE_ERROR_NOMATCH)
 			return (1);
 		return (0);
 	case T_NOMATCH:		/* !~ */
 		assert(rhs->type == VEX_REGEX && rhs->val_regex != NULL);
-		i = VRE_exec(rhs->val_regex, b, e - b, 0, 0, NULL, 0, NULL);
+		i = VRE_match(rhs->val_regex, b, e - b, 0, NULL);
 		if (i == VRE_ERROR_NOMATCH)
 			return (1);
 		return (0);
