@@ -304,18 +304,22 @@ static void
 vsm_delset(struct vsm_set **p)
 {
 	struct vsm_set *vs;
+	struct vsm_seg *vg;
+
+	TAKE_OBJ_NOTNULL(vs, p, VSM_SET_MAGIC);
 
-	AN(p);
-	vs = *p;
-	*p = NULL;
 	if (vs->fd >= 0)
 		closefd(&vs->fd);
 	if (vs->dfd >= 0)
 		closefd(&vs->dfd);
-	while (!VTAILQ_EMPTY(&vs->stale))
-		vsm_delseg(VTAILQ_FIRST(&vs->stale), 0);
-	while (!VTAILQ_EMPTY(&vs->segs))
-		vsm_delseg(VTAILQ_FIRST(&vs->segs), 0);
+	while ((vg = VTAILQ_FIRST(&vs->stale)) != NULL) {
+		AN(vg->flags & VSM_FLAG_STALE);
+		vsm_delseg(vg, 0);
+	}
+	while ((vg = VTAILQ_FIRST(&vs->segs)) != NULL) {
+		AZ(vg->flags & VSM_FLAG_STALE);
+		vsm_delseg(vg, 0);
+	}
 	assert(VTAILQ_EMPTY(&vs->clusters));
 	VLU_Destroy(&vs->vlu);
 	FREE_OBJ(vs);
@@ -383,12 +387,12 @@ VSM_Arg(struct vsm *vd, char flag, const char *arg)
 		}
 		break;
 	case 'n':
-		if (VIN_n_Arg(arg, &p))
+		if (vd->wdname != NULL)
+			free(vd->wdname);
+		vd->wdname = VIN_n_Arg(arg);
+		if (vd->wdname == NULL)
 			return (vsm_diag(vd, "Invalid instance name: %s",
 			    strerror(errno)));
-		AN(p);
-		REPLACE(vd->wdname, p);
-		free(p);
 		break;
 	default:
 		return (vsm_diag(vd, "Unknown VSM_Arg('%c')", flag));
@@ -511,7 +515,7 @@ vsm_vlu_plus(struct vsm *vd, struct vsm_set *vs, const char *line)
 		(void)(vsm_diag(vd, "vsm_vlu_plus: bad index (%d/%s)",
 		    ac, av[0]));
 		VAV_Free(av);
-		return(-1);
+		return (-1);
 	}
 
 	vg = vs->vg;
@@ -559,7 +563,7 @@ vsm_vlu_minus(struct vsm *vd, const struct vsm_set *vs, const char *line)
 		(void)(vsm_diag(vd, "vsm_vlu_minus: bad index (%d/%s)",
 		    ac, av[0]));
 		VAV_Free(av);
-		return(-1);
+		return (-1);
 	}
 
 	/* Clustered segments cannot come before their cluster */
@@ -678,7 +682,7 @@ vsm_refresh_set(struct vsm *vd, struct vsm_set *vs)
 		vs->vg = VTAILQ_FIRST(&vs->segs);
 		vs->fd = openat(vs->dfd, "_.index", O_RDONLY);
 		if (vs->fd < 0)
-			return (vs->retval|vs->flag_restarted);
+			return (vs->retval | vs->flag_restarted);
 		VLU_Reset(vs->vlu);
 		AZ(fstat(vs->fd, &vs->fst));
 		vsm_readlines(vs);
