@@ -5,6 +5,8 @@
  *
  * Author: Martin Blix Grydeland <martin@varnish-software.com>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -33,6 +35,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h> /* for MUSL */
 
 #include "vdef.h"
 #include "vas.h"
@@ -49,7 +52,7 @@ vxp_ErrToken(const struct vxp *vxp, const struct token *t)
 {
 
 	if (t->tok == EOI)
-		VSB_printf(vxp->sb, "end of input");
+		VSB_cat(vxp->sb, "end of input");
 	else
 		VSB_printf(vxp->sb, "'%.*s'", PF(t));
 }
@@ -119,7 +122,7 @@ vxp_NextToken(struct vxp *vxp)
 	AN(vxp->t);
 	vxp->t = VTAILQ_NEXT(vxp->t, list);
 	if (vxp->t == NULL) {
-		VSB_printf(vxp->sb,
+		VSB_cat(vxp->sb,
 		    "Ran out of input, something is missing or"
 		    " maybe unbalanced parenthesis\n");
 		vxp->err = 1;
