@@ -260,8 +260,7 @@ vxp_expr_str(struct vxp *vxp, struct vex_rhs **prhs)
 static void
 vxp_expr_regex(struct vxp *vxp, struct vex_rhs **prhs)
 {
-	const char *errptr;
-	int erroff;
+	int err, erroff;
 
 	/* XXX: Caseless option */
 
@@ -279,10 +278,11 @@ vxp_expr_regex(struct vxp *vxp, struct vex_rhs **prhs)
 	(*prhs)->type = VEX_REGEX;
 	(*prhs)->val_string = strdup(vxp->t->dec);
 	(*prhs)->val_regex = VRE_compile(vxp->t->dec, vxp->vre_options,
-	    &errptr, &erroff);
+	    &err, &erroff, 1);
 	if ((*prhs)->val_regex == NULL) {
-		AN(errptr);
-		VSB_printf(vxp->sb, "Regular expression error: %s ", errptr);
+		VSB_cat(vxp->sb, "Regular expression error: ");
+		AZ(VRE_error(vxp->sb, err));
+		VSB_putc(vxp->sb, ' ');
 		vxp_ErrWhere(vxp, vxp->t, erroff);
 		return;
 	}
@@ -573,31 +573,37 @@ vxp_Parse(struct vxp *vxp)
 void
 vex_Free(struct vex **pvex)
 {
+	struct vex *vex;
+	struct vex_lhs *lhs;
+	struct vex_rhs *rhs;
 
-	if ((*pvex)->lhs != NULL) {
-		if ((*pvex)->lhs->tags != NULL)
-			vbit_destroy((*pvex)->lhs->tags);
-		if ((*pvex)->lhs->prefix != NULL)
-			free((*pvex)->lhs->prefix);
-		FREE_OBJ((*pvex)->lhs);
+	TAKE_OBJ_NOTNULL(vex, pvex, VEX_MAGIC);
+
+	if (vex->lhs) {
+		CAST_OBJ_NOTNULL(lhs, vex->lhs, VEX_LHS_MAGIC);
+		if (lhs->tags)
+			vbit_destroy(lhs->tags);
+		if (lhs->prefix)
+			free(lhs->prefix);
+		FREE_OBJ(lhs);
 	}
-	if ((*pvex)->rhs != NULL) {
-		if ((*pvex)->rhs->val_string)
-			free((*pvex)->rhs->val_string);
-		if ((*pvex)->rhs->val_regex)
-			VRE_free(&(*pvex)->rhs->val_regex);
-		FREE_OBJ((*pvex)->rhs);
+	if (vex->rhs) {
+		CAST_OBJ_NOTNULL(rhs, vex->rhs, VEX_RHS_MAGIC);
+		if (rhs->val_string)
+			free(rhs->val_string);
+		if (rhs->val_regex)
+			VRE_free(&rhs->val_regex);
+		FREE_OBJ(rhs);
 	}
-	if ((*pvex)->a != NULL) {
-		vex_Free(&(*pvex)->a);
-		AZ((*pvex)->a);
+	if (vex->a) {
+		vex_Free(&vex->a);
+		AZ(vex->a);
 	}
-	if ((*pvex)->b != NULL) {
-		vex_Free(&(*pvex)->b);
-		AZ((*pvex)->b);
+	if (vex->b) {
+		vex_Free(&vex->b);
+		AZ(vex->b);
 	}
-	FREE_OBJ(*pvex);
-	*pvex = NULL;
+	FREE_OBJ(vex);
 }
 
 #ifdef VXP_DEBUG
