@@ -169,18 +169,22 @@ def varproto(s):
         varprotos[s] = True
 
 class vardef(object):
-    def __init__(self, nam, typ, rd, wr, wu, vlo, vhi):
+    def __init__(self, nam, typ, rd, wr, wu, al, vlo, vhi):
         self.nam = nam
         self.typ = typ
         self.rd = rd
         self.wr = wr
         self.uns = wu
+        self.al = al
         self.vlo = vlo
         self.vhi = vhi
 
-        self.emit()
+        if al is None:
+            self.emit_var()
+        else:
+            self.emit_alias()
 
-    def emit(self):
+    def emit_var(self):
         fh.write("\n")
         fo.write("\n")
         cnam = self.nam.replace(".", "_")
@@ -217,9 +221,9 @@ class vardef(object):
             fo.write('\tsym->lname = "VRT_l_%s(ctx, ";\n' % cnam)
             s = "void VRT_l_%s(VRT_CTX, " % cnam
             if self.typ == "STRING":
-                s += ctyp.c + ", ...)"
+                s += ctyp.c + ", VCL_STRANDS)"
             elif self.typ == "BODY":
-                s += "enum lbody_e, " + ctyp.c + ", ...)"
+                s += "enum lbody_e, const char *, " + ctyp.c + ")"
             else:
                 s += "VCL_" + self.typ + ")"
             varproto(s)
@@ -234,6 +238,11 @@ class vardef(object):
         restrict(fo, self.uns)
         fo.write(";\n")
 
+    def emit_alias(self):
+        var_aliases.append("\tsym = VCC_MkSymAlias(tl, \"%s\", \"%s\");\n" %
+                (self.nam, self.al))
+        var_aliases.append("\tAN(sym);\n")
+
 def parse_vcl(x):
     vlo, vhi = (0, 99)
     x = x.split()
@@ -260,6 +269,7 @@ def parse_var(ln):
     vr = []
     vw = []
     vu = []
+    va = None
     while True:
         l = ln.pop(0)
         if l == "":
@@ -281,9 +291,12 @@ def parse_var(ln):
             for i in j[2:]:
                 vu.append(i.strip(",."))
             continue
+        if j[0] == "Alias" and j[1] == "of:":
+            va = j[2]
+            continue
         break
     if vn[:8] != "storage.":
-        vardef(vn, vt, vr, vw, vu, vlo, vhi)
+        vardef(vn, vt, vr, vw, vu, va, vlo, vhi)
 
 def parse_var_doc(fn):
     l = []
@@ -327,7 +340,6 @@ class vcltype(object):
 
 
 vcltype("STRINGS", "void", True)
-vcltype("STRING_LIST", "void*", True)
 vcltype("SUB", "void*", True)
 
 fi = open(join(srcroot, "include/vrt.h"))
@@ -756,11 +768,14 @@ vcc_Var_Init(struct vcc *tl)
     struct symbol *sym;
 """)
 
+var_aliases = []
 parse_var_doc(join(srcroot, "doc/sphinx/reference/vcl_var.rst"))
+for al in var_aliases:
+    fo.write(al)
 fo.write("}\n")
 
 for i in stv_variables:
-    fh.write(vcltypes[i[1]].c + " VRT_stevedore_" + i[0] + "(VCL_STEVEDORE);\n")
+    fh.write("VCL_" + i[1] + " VRT_stevedore_" + i[0] + "(VCL_STEVEDORE);\n")
 
 fo.write("\n/* VCL type identifiers */\n")
 
