@@ -41,18 +41,35 @@
 
 #include "vcc_compile.h"
 #include <vtcp.h>
+#include <vtree.h>
 #include <vsa.h>
 
 #define ACL_MAXADDR	(sizeof(struct in6_addr) + 1)
 
+VRBT_HEAD(acl_tree, acl_e);
+
+struct acl {
+	unsigned		magic;
+#define VCC_ACL_MAGIC		0xb9fb3cd0
+
+	int			flag_log;
+	int			flag_pedantic;
+	int			flag_table;
+
+	struct acl_tree		acl_tree;
+};
+
 struct acl_e {
-	VTAILQ_ENTRY(acl_e)	list;
+	unsigned		magic;
+#define VCC_ACL_E_MAGIC	0xcac81e23
+	VRBT_ENTRY(acl_e)	branch;
 	unsigned char		data[ACL_MAXADDR];
 	unsigned		mask;
 	unsigned		not;
 	unsigned		para;
+	unsigned		overlapped;
 	char			*addr;
-	char			*fixed;
+	const char		*fixed;
 	struct token		*t_addr;
 	struct token		*t_mask;
 };
@@ -67,17 +84,28 @@ struct acl_e {
 			return (1);					\
 	} while (0)
 
-static int
-vcl_acl_cmp(struct acl_e *ae1, struct acl_e *ae2)
+static void
+vcl_acl_free(struct acl_e **aep)
 {
-	unsigned char *p1, *p2;
+	struct acl_e *a;
+
+	TAKE_OBJ_NOTNULL(a, aep, VCC_ACL_E_MAGIC);
+	free(a->addr);
+	FREE_OBJ(a);
+}
+
+static int
+vcl_acl_cmp(const struct acl_e *ae1, const struct acl_e *ae2)
+{
+	const unsigned char *p1, *p2;
 	unsigned m;
 
+	CHECK_OBJ_NOTNULL(ae1, VCC_ACL_E_MAGIC);
+	CHECK_OBJ_NOTNULL(ae2, VCC_ACL_E_MAGIC);
+
 	p1 = ae1->data;
 	p2 = ae2->data;
-	m = ae1->mask;
-	if (ae2->mask < m)
-		m = ae2->mask;
+	m = vmin_t(unsigned, ae1->mask, ae2->mask);
 	for (; m >= 8; m -= 8) {
 		CMP(*p1, *p2);
 		p1++;
@@ -94,6 +122,39 @@ vcl_acl_cmp(struct acl_e *ae1, struct acl_e *ae2)
 	return (0);
 }
 
+static int
+vcl_acl_disjoint(const struct acl_e *ae1, const struct acl_e *ae2)
+{
+	const unsigned char *p1, *p2;
+	unsigned m;
+
+	CHECK_OBJ_NOTNULL(ae1, VCC_ACL_E_MAGIC);
+	CHECK_OBJ_NOTNULL(ae2, VCC_ACL_E_MAGIC);
+
+	p1 = ae1->data;
+	p2 = ae2->data;
+	m = ae1->mask;
+	if (ae2->mask < m)
+		m = ae2->mask;
+	for (; m >= 8; m -= 8) {
+		CMP(*p1, *p2);
+		p1++;
+		p2++;
+	}
+	if (m) {
+		m = 0xff00 >> m;
+		m &= 0xff;
+		CMP(*p1 & m, *p2 & m);
+	}
+	return (0);
+}
+
+VRBT_GENERATE_INSERT_COLOR(acl_tree, acl_e, branch, static)
+VRBT_GENERATE_INSERT(acl_tree, acl_e, branch, vcl_acl_cmp, static)
+VRBT_GENERATE_MINMAX(acl_tree, acl_e, branch, static)
+VRBT_GENERATE_NEXT(acl_tree, acl_e, branch, static)
+VRBT_GENERATE_PREV(acl_tree, acl_e, branch, static)
+
 static char *
 vcc_acl_chk(struct vcc *tl, const struct acl_e *ae, const int l,
     unsigned char *p, int fam)
@@ -102,6 +163,7 @@ vcc_acl_chk(struct vcc *tl, const struct acl_e *ae, const int l,
 	char h[VTCP_ADDRBUFSIZE];
 	char t[VTCP_ADDRBUFSIZE + 10];
 	char s[vsa_suckaddr_len];
+	char *r = NULL;
 	struct suckaddr *sa;
 	unsigned m;
 	int ll, ret = 0;
@@ -138,23 +200,20 @@ vcc_acl_chk(struct vcc *tl, const struct acl_e *ae, const int l,
 	AN(sa);
 	VTCP_name(sa, h, sizeof h, NULL, 0);
 	bprintf(t, "%s/%d", h, ae->mask);
-	VSB_cat(tl->sb, "Address/Netmask mismatch, ");
-	if (tl->acl_pedantic != 0)
-		VSB_printf(tl->sb, "need be %s\n", t);
-	else
-		VSB_printf(tl->sb, "changed to %s\n", t);
-	vcc_ErrWhere(tl, ae->t_addr);
-	if (tl->acl_pedantic == 0)
-		vcc_Warn(tl);
-	return (strdup(t));
+	if (tl->acl->flag_pedantic != 0) {
+		VSB_cat(tl->sb, "Non-zero bits in masked part, ");
+		VSB_printf(tl->sb, "(maybe use %s ?)\n", t);
+		vcc_ErrWhere(tl, ae->t_addr);
+	}
+	REPLACE(r, t);
+	return (r);
 }
 
 static void
 vcc_acl_add_entry(struct vcc *tl, const struct acl_e *ae, int l,
     unsigned char *u, int fam)
 {
-	struct acl_e *ae2, *aen;
-	int i;
+	struct acl_e *aen, *ae2;
 
 	if (fam == PF_INET && ae->mask > 32) {
 		VSB_printf(tl->sb,
@@ -173,9 +232,11 @@ vcc_acl_add_entry(struct vcc *tl, const struct acl_e *ae, int l,
 	}
 
 	/* Make a copy from the template */
-	aen = TlAlloc(tl, sizeof *ae2);
+	ALLOC_OBJ(aen, VCC_ACL_E_MAGIC);
 	AN(aen);
 	*aen = *ae;
+	aen->addr = strdup(ae->addr);
+	AN(aen->addr);
 
 	aen->fixed = vcc_acl_chk(tl, ae, l, u, fam);
 
@@ -187,40 +248,14 @@ vcc_acl_add_entry(struct vcc *tl, const struct acl_e *ae, int l,
 	assert(l + 1UL <= sizeof aen->data);
 	memcpy(aen->data + 1L, u, l);
 
-	VTAILQ_FOREACH(ae2, &tl->acl, list) {
-		i = vcl_acl_cmp(aen, ae2);
-		if (i == 0) {
-			/*
-			 * If the two rules agree, silently ignore it
-			 * XXX: is that counter intuitive ?
-			 */
-			if (aen->not == ae2->not) {
-				free(aen);
-				return;
-			}
-			VSB_cat(tl->sb, "Conflicting ACL entries:\n");
-			vcc_ErrWhere(tl, ae2->t_addr);
-			VSB_cat(tl->sb, "vs:\n");
-			vcc_ErrWhere(tl, aen->t_addr);
-			free(aen);
-			return;
-		}
-		/*
-		 * We could eliminate pointless rules here, for instance in:
-		 *	"10.1.0.1";
-		 *	"10.1";
-		 * The first rule is clearly pointless, as the second one
-		 * covers it.
-		 *
-		 * We do not do this however, because the shmlog may
-		 * be used to gather statistics.
-		 */
-		if (i < 0) {
-			VTAILQ_INSERT_BEFORE(ae2, aen, list);
-			return;
-		}
+	ae2 = VRBT_INSERT(acl_tree, &tl->acl->acl_tree, aen);
+	if (ae2 != NULL && ae2->not != aen->not) {
+		VSB_cat(tl->sb, "Conflicting ACL entries:\n");
+		vcc_ErrWhere(tl, ae2->t_addr);
+		VSB_cat(tl->sb, "vs:\n");
+		vcc_ErrWhere(tl, aen->t_addr);
+		vcl_acl_free(&aen);
 	}
-	VTAILQ_INSERT_TAIL(&tl->acl, aen, list);
 }
 
 static void
@@ -232,6 +267,7 @@ vcc_acl_try_getaddrinfo(struct vcc *tl, struct acl_e *ae)
 	unsigned char *u, i4, i6;
 	int error;
 
+	CHECK_OBJ_NOTNULL(ae, VCC_ACL_E_MAGIC);
 	memset(&hint, 0, sizeof hint);
 	hint.ai_family = PF_UNSPEC;
 	hint.ai_socktype = SOCK_STREAM;
@@ -330,6 +366,7 @@ vcc_acl_try_netnotation(struct vcc *tl, struct acl_e *ae)
 	unsigned u;
 	const char *p;
 
+	CHECK_OBJ_NOTNULL(ae, VCC_ACL_E_MAGIC);
 	memset(b, 0, sizeof b);
 	p = ae->addr;
 	for (i = 0; i < 4; i++) {
@@ -354,11 +391,10 @@ vcc_acl_try_netnotation(struct vcc *tl, struct acl_e *ae)
 static void
 vcc_acl_entry(struct vcc *tl)
 {
-	struct acl_e *ae;
+	struct acl_e ae[1];
 	char *sl, *e;
 
-	ae = TlAlloc(tl, sizeof *ae);
-	AN(ae);
+	INIT_OBJ(ae, VCC_ACL_E_MAGIC);
 
 	if (tl->t->tok == '!') {
 		ae->not = 1;
@@ -377,8 +413,7 @@ vcc_acl_entry(struct vcc *tl)
 
 	ExpectErr(tl, CSTR);
 	ae->t_addr = tl->t;
-	ae->addr = strdup(ae->t_addr->dec);
-	AN(ae->addr);
+	ae->addr = ae->t_addr->dec;
 	vcc_NextToken(tl);
 
 	if (strchr(ae->t_addr->dec, '/') != NULL) {
@@ -390,14 +425,12 @@ vcc_acl_entry(struct vcc *tl)
 		if (*e != '\0') {
 			VSB_cat(tl->sb, ".../mask is not numeric.\n");
 			vcc_ErrWhere(tl, ae->t_addr);
-			free(ae);
 			return;
 		}
 		ae->t_mask = ae->t_addr;
 		if (tl->t->tok == '/') {
 			VSB_cat(tl->sb, "/mask only allowed once.\n");
 			vcc_ErrWhere(tl, tl->t);
-			free(ae);
 			return;
 		}
 	} else if (tl->t->tok == '/') {
@@ -417,26 +450,130 @@ vcc_acl_entry(struct vcc *tl)
 	ERRCHK(tl);
 }
 
+/*********************************************************************
+ * Emit the tokens making up an entry as C-strings
+ */
+
+static void
+vcc_acl_emit_tokens(const struct vcc *tl, const struct acl_e *ae)
+{
+	struct token *t;
+	const char *sep = "";
+
+	CHECK_OBJ_NOTNULL(ae, VCC_ACL_E_MAGIC);
+	t = ae->t_addr;
+	do {
+		if (t->tok == CSTR) {
+			Fh(tl, 0, "%s\"\\\"\" ", sep);
+			EncToken(tl->fh, t);
+			Fh(tl, 0, " \"\\\"\"");
+		} else if (t == ae->t_mask) {
+			Fh(tl, 0, " \"%u\"", ae->mask - 8);
+		} else {
+			Fh(tl, 0, "%s\"%.*s\"", sep, PF(t));
+		}
+		if (t == ae->t_mask)
+			break;
+		t = vcc_PeekTokenFrom(tl, t);
+		AN(t);
+		sep = " ";
+	} while (ae->t_mask != NULL);
+	if (ae->fixed)
+		Fh(tl, 0, "\" fixed: %s\"", ae->fixed);
+}
+
+/*********************************************************************
+ * Emit ACL on table format
+ */
+
+static unsigned
+vcc_acl_emit_tables(const struct vcc *tl, unsigned n, const char *name)
+{
+	struct acl_e *ae;
+	unsigned rv = sizeof(ae->data) + 3;
+	unsigned nn = 0;
+	size_t sz;
+
+	Fh(tl, 0, "\nstatic unsigned char acl_tbl_%s[%u*%u] = {\n",
+	    name, n, rv);
+	VRBT_FOREACH(ae, acl_tree, &tl->acl->acl_tree) {
+		if (ae->overlapped)
+			continue;
+		Fh(tl, 0, "\t0x%02x,", ae->not ? 0 : 1);
+		Fh(tl, 0, "0x%02x,", (ae->mask >> 3) - 1);
+		Fh(tl, 0, "0x%02x,", (0xff00 >> (ae->mask & 7)) & 0xff);
+		for (sz = 0; sz < sizeof(ae->data); sz++)
+			Fh(tl, 0, "0x%02x,", ae->data[sz]);
+		for (; sz < rv - 3; sz++)
+			Fh(tl, 0, "0,");
+		Fh(tl, 0, "\n");
+		nn++;
+	}
+	assert(n == nn);
+	Fh(tl, 0, "};\n");
+	if (tl->acl->flag_log) {
+		Fh(tl, 0, "\nstatic const char *acl_str_%s[%d] = {\n",
+		    name, n);
+		VRBT_FOREACH(ae, acl_tree, &tl->acl->acl_tree) {
+			if (ae->overlapped)
+				continue;
+			Fh(tl, 0, "\t");
+			Fh(tl, 0, "\"%sMATCH %s \" ",
+			    ae->not ? "NEG_" : "", name);
+			vcc_acl_emit_tokens(tl, ae);
+			Fh(tl, 0, ",\n");
+		}
+		Fh(tl, 0, "};\n");
+	}
+	return (rv);
+}
+
 /*********************************************************************
  * Emit a function to match the ACL we have collected
  */
 
 static void
-vcc_acl_emit(struct vcc *tl, const char *name, const char *rname, int anon)
+vcc_acl_emit(struct vcc *tl, const struct symbol *sym)
 {
-	struct acl_e *ae;
+	struct acl_e *ae, *ae2;
 	int depth, l, m, i;
 	unsigned at[ACL_MAXADDR];
-	struct token *t;
-	struct inifin *ifp;
+	struct inifin *ifp = NULL;
 	struct vsb *func;
+	unsigned n, no, nw = 0;
 
 	func = VSB_new_auto();
 	AN(func);
-	VSB_printf(func, "match_acl_%s_", anon ? "anon" : "named");
-	VCC_PrintCName(func, name, NULL);
+	VSB_printf(func, "match_acl_");
+	VCC_PrintCName(func, sym->name, NULL);
 	AZ(VSB_finish(func));
 
+	depth = -1;
+	at[0] = 256;
+	ae2 = NULL;
+	n = no = 0;
+	VRBT_FOREACH_REVERSE(ae, acl_tree, &tl->acl->acl_tree) {
+		n++;
+		if (ae2 == NULL) {
+			ae2 = ae;
+		} else if (vcl_acl_disjoint(ae, ae2)) {
+			ae2 = ae;
+		} else {
+			no++;
+			ae->overlapped = 1;
+		}
+	}
+
+	Fh(tl, 0, "/* acl_n_%s n %u no %u */\n", sym->name, n, no);
+	if (n - no < (1<<1))
+		no = n;
+	else if (!tl->acl->flag_table)
+		no = n;
+
+	if (no < n)
+		nw = vcc_acl_emit_tables(tl, n - no, sym->name);
+
+
 	Fh(tl, 0, "\nstatic int v_matchproto_(acl_match_f)\n");
 	Fh(tl, 0, "%s(VRT_CTX, const VCL_IP p)\n", VSB_data(func));
 	Fh(tl, 0, "{\n");
@@ -445,17 +582,20 @@ vcc_acl_emit(struct vcc *tl, const char *name, const char *rname, int anon)
 	Fh(tl, 0, "\n");
 	Fh(tl, 0, "\tfam = VRT_VSA_GetPtr(ctx, p, &a);\n");
 	Fh(tl, 0, "\tif (fam < 0) {\n");
-	Fh(tl, 0, "\t\tVPI_acl_log(ctx, \"NO_FAM %s\");\n", name);
+	Fh(tl, 0, "\t\tVRT_fail(ctx,");
+	Fh(tl, 0, " \"ACL %s: no protocol family\");\n", sym->name);
 	Fh(tl, 0, "\t\treturn(0);\n");
 	Fh(tl, 0, "\t}\n\n");
-	if (!tl->err_unref && !anon) {
+	if (!tl->err_unref) {
 		ifp = New_IniFin(tl);
 		VSB_printf(ifp->ini,
-			"\tif (0) %s(0, 0);\n", VSB_data(func));
+			"\t(void)%s;\n", VSB_data(func));
 	}
-	depth = -1;
-	at[0] = 256;
-	VTAILQ_FOREACH(ae, &tl->acl, list) {
+
+	VRBT_FOREACH(ae, acl_tree, &tl->acl->acl_tree) {
+
+		if (no < n && !ae->overlapped)
+			continue;
 
 		/* Find how much common prefix we have */
 		for (l = 0; l <= depth && l * 8 < (int)ae->mask - 7; l++) {
@@ -498,25 +638,10 @@ vcc_acl_emit(struct vcc *tl, const char *name, const char *rname, int anon)
 
 		i = ((int)ae->mask + 7) / 8;
 
-		if (!anon) {
+		if (tl->acl->flag_log) {
 			Fh(tl, 0, "\t%*sVPI_acl_log(ctx, \"%sMATCH %s \" ",
-			    -i, "", ae->not ? "NEG_" : "", name);
-			t = ae->t_addr;
-			do {
-				if (t->tok == CSTR) {
-					Fh(tl, 0, " \"\\\"\" ");
-					EncToken(tl->fh, t);
-					Fh(tl, 0, " \"\\\"\" ");
-				} else
-					Fh(tl, 0, " \"%.*s\"", PF(t));
-				if (t == ae->t_mask)
-					break;
-				t = VTAILQ_NEXT(t, list);
-				AN(t);
-			} while (ae->t_mask != NULL);
-			if (ae->fixed)
-				Fh(tl, 0, "\" fixed: %s\"",
-				   ae->fixed);
+			    -i, "", ae->not ? "NEG_" : "", sym->name);
+			vcc_acl_emit_tokens(tl, ae);
 			Fh(tl, 0, ");\n");
 		}
 
@@ -527,18 +652,34 @@ vcc_acl_emit(struct vcc *tl, const char *name, const char *rname, int anon)
 	for (; 0 <= depth; depth--)
 		Fh(tl, 0, "\t%*.*s}\n", depth, depth, "");
 
-	/* Deny by default */
-	if (!anon)
-		Fh(tl, 0, "\tVPI_acl_log(ctx, \"NO_MATCH %s\");\n", name);
-	Fh(tl, 0, "\treturn (0);\n}\n");
+	if (no < n) {
+		Fh(tl, 0, "\treturn(\n\t    VPI_acl_table(ctx,\n");
+		Fh(tl, 0, "\t\tp,\n");
+		Fh(tl, 0, "\t\t%u, %u,\n", n - no, nw);
+		Fh(tl, 0, "\t\tacl_tbl_%s,\n", sym->name);
+		if (tl->acl->flag_log)
+			Fh(tl, 0, "\t\tacl_str_%s,\n", sym->name);
+		else
+			Fh(tl, 0, "\t\tNULL,\n");
+		Fh(tl, 0, "\t\t\"NO MATCH %s\"\n\t    )\n\t);\n", sym->name);
+	} else {
+		/* Deny by default */
+		if (tl->acl->flag_log)
+			Fh(tl, 0, "\tVPI_acl_log(ctx, \"NO_MATCH %s\");\n",
+			    sym->name);
+		Fh(tl, 0, "\treturn(0);\n");
+	}
+	Fh(tl, 0, "}\n");
 
-	if (!anon) {
-		/* Emit the struct that will be referenced */
-		Fh(tl, 0, "\nconst struct vrt_acl %s[] = {{\n", rname);
-		Fh(tl, 0, "\t.magic = VRT_ACL_MAGIC,\n");
-		Fh(tl, 0, "\t.match = &%s,\n", VSB_data(func));
-		Fh(tl, 0, "\t.name = \"%s\",\n", name);
-		Fh(tl, 0, "}};\n\n");
+	/* Emit the struct that will be referenced */
+	Fh(tl, 0, "\nstatic const struct vrt_acl %s[] = {{\n", sym->rname);
+	Fh(tl, 0, "\t.magic = VRT_ACL_MAGIC,\n");
+	Fh(tl, 0, "\t.match = &%s,\n", VSB_data(func));
+	Fh(tl, 0, "\t.name = \"%s\",\n", sym->name);
+	Fh(tl, 0, "}};\n\n");
+	if (!tl->err_unref) {
+		AN(ifp);
+		VSB_printf(ifp->ini, "\t(void)%s;", sym->rname);
 	}
 	VSB_destroy(&func);
 }
@@ -547,16 +688,46 @@ void
 vcc_ParseAcl(struct vcc *tl)
 {
 	struct symbol *sym;
+	int sign;
+	struct acl acl[1];
 
+	INIT_OBJ(acl, VCC_ACL_MAGIC);
+	tl->acl = acl;
+	acl->flag_pedantic = 1;
 	vcc_NextToken(tl);
-	VTAILQ_INIT(&tl->acl);
+	VRBT_INIT(&acl->acl_tree);
 
 	vcc_ExpectVid(tl, "ACL");
 	ERRCHK(tl);
-	sym = VCC_HandleSymbol(tl, ACL, ACL_SYMBOL_PREFIX);
+	sym = VCC_HandleSymbol(tl, ACL);
 	ERRCHK(tl);
 	AN(sym);
 
+	while (1) {
+		sign = vcc_IsFlag(tl);
+		if (tl->err) {
+			VSB_cat(tl->sb,
+			    "Valid ACL flags are `log` and `table`:\n");
+			return;
+		}
+		if (sign < 0)
+			break;
+		if (vcc_IdIs(tl->t, "log")) {
+			acl->flag_log = sign;
+			vcc_NextToken(tl);
+		} else if (vcc_IdIs(tl->t, "pedantic")) {
+			acl->flag_pedantic = sign;
+			vcc_NextToken(tl);
+		} else if (vcc_IdIs(tl->t, "table")) {
+			acl->flag_table = sign;
+			vcc_NextToken(tl);
+		} else {
+			VSB_cat(tl->sb, "Unknown ACL flag:\n");
+			vcc_ErrWhere(tl, tl->t);
+			return;
+		}
+	}
+
 	SkipToken(tl, '{');
 
 	while (tl->t->tok != '}') {
@@ -566,5 +737,5 @@ vcc_ParseAcl(struct vcc *tl)
 	}
 	SkipToken(tl, '}');
 
-	vcc_acl_emit(tl, sym->name, sym->rname, 0);
+	vcc_acl_emit(tl, sym);
 }
