@@ -316,7 +316,7 @@ vcc_ParseProbe(struct vcc *tl)
 		vcc_ParseProbeSpec(tl, NULL, &p);
 		tl->default_probe = p;
 	} else {
-		sym = VCC_HandleSymbol(tl, PROBE, "vgc_probe");
+		sym = VCC_HandleSymbol(tl, PROBE);
 		ERRCHK(tl);
 		AN(sym);
 		vcc_ParseProbeSpec(tl, sym, NULL);
@@ -353,8 +353,8 @@ vcc_ParseHostDef(struct vcc *tl, const struct token *t_be, const char *vgcname)
 		vcc_NextToken(tl);
 		SkipToken(tl, ';');
 		ifp = New_IniFin(tl);
-		VSB_printf(ifp->ini, "\t(void)%s;\n", vgcname);
-		VSB_printf(ifp->fin, "\t\t(void)%s;\n", vgcname);
+		VSB_printf(ifp->ini, "\t(void)%s;", vgcname);
+		VSB_printf(ifp->fin, "\t\t(void)%s;", vgcname);
 		return;
 	}
 
@@ -572,8 +572,10 @@ vcc_ParseHostDef(struct vcc *tl, const struct token *t_be, const char *vgcname)
 	ifp = New_IniFin(tl);
 	VSB_printf(ifp->ini,
 	    "\t%s =\n\t    VRT_new_backend_clustered(ctx, vsc_cluster,\n"
-	    "\t\t&vgc_dir_priv_%s);",
+	    "\t\t&vgc_dir_priv_%s);\n",
 	    vgcname, vgcname);
+	VSB_printf(ifp->ini,
+	    "\tif (%s)\n\t\tVRT_StaticDirector(%s);", vgcname, vgcname);
 	VSB_printf(ifp->fin, "\t\tVRT_delete_backend(ctx, &%s);", vgcname);
 }
 
@@ -612,7 +614,7 @@ vcc_ParseBackend(struct vcc *tl)
 		dn = "vgc_backend_default";
 		tl->default_director = dn;
 	} else {
-		sym = VCC_HandleSymbol(tl, BACKEND, "vgc_backend");
+		sym = VCC_HandleSymbol(tl, BACKEND);
 		ERRCHK(tl);
 		AN(sym);
 		dn = sym->rname;
