@@ -64,6 +64,7 @@
 
 #include "libvcc.h"
 #include "vfil.h"
+#include "vct.h"
 
 static const struct method method_tab[] = {
 	{ "none", 0U, 0},
@@ -82,13 +83,15 @@ vcc_vcl_met2c(struct vsb *vsb, unsigned method)
 	int d = 0;
 
 	//lint -e{774} Boolean within 'if' always evaluates to False
-#define VCL_MET_MAC(l,U,t,b)			\
-	if (method & VCL_MET_##U) {		\
-		if (d)				\
-			VSB_putc(vsb, '|');	\
-		VSB_cat(vsb, "VCL_MET_" #U);	\
-		d = 1;				\
-	}
+#define VCL_MET_MAC(l,U,t,b)				\
+	do {						\
+		if (method & VCL_MET_##U) {		\
+			if (d)				\
+				VSB_putc(vsb, '|');	\
+			VSB_cat(vsb, "VCL_MET_" #U);	\
+			d = 1;				\
+		}					\
+	} while (0);
 #include "tbl/vcl_returns.h"
 	AN(d);
 }
@@ -191,7 +194,7 @@ vcc_EmitProc(struct vcc *tl, struct proc *p)
 		maskcmp = "&";
 	}
 
-	if ((mask & VCL_MET_TASK_H) && (mask & ~VCL_MET_TASK_H) == 0)
+	if (dyn == 0 && (p->calledfrom & VCL_MET_TASK_H) == p->calledfrom)
 		cc_adv = "v_dont_optimize ";
 	else
 		cc_adv = "";
@@ -199,8 +202,9 @@ vcc_EmitProc(struct vcc *tl, struct proc *p)
 	nsub = tl->nsub++;
 
 	Fh(tl, 1, "vcl_func_f %s;\n", VSB_data(p->cname));
-	Fh(tl, 1, "const struct vcl_sub sub_%s[1] = {{\n",
-	   VSB_data(p->cname));
+	Fh(tl, 1, "extern const struct vcl_sub sub_%s[1];\n",
+	    VSB_data(p->cname));
+	Fh(tl, 1, "const struct vcl_sub sub_%s[1] = {{\n", VSB_data(p->cname));
 	Fh(tl, 1, "\t.magic\t\t= VCL_SUB_MAGIC,\n");
 	Fh(tl, 1, "\t.methods\t= 0x%x,\n", p->okmask);
 	Fh(tl, 1, "\t.name\t\t= \"%.*s\",\n", PF(p->name));
@@ -209,6 +213,7 @@ vcc_EmitProc(struct vcc *tl, struct proc *p)
 	Fh(tl, 1, "\t.n\t\t= %d,\n", nsub);
 	Fh(tl, 1, "\t.nref\t\t= %d,\n", p->sym->nref);
 	Fh(tl, 1, "\t.called\t\t= %d\n", p->called);
+	Fh(tl, 1, "\t// calledfrom\t  0x%x\n", p->calledfrom);
 	Fh(tl, 1, "}};\n");
 
 	if (dyn) {
@@ -601,158 +606,6 @@ EmitStruct(const struct vcc *tl)
 	Fc(tl, 0, "};\n");
 }
 
-/*--------------------------------------------------------------------*/
-
-static struct source *
-vcc_new_source(const char *src, const char *name)
-{
-	struct source *sp;
-
-	AN(src);
-	AN(name);
-	sp = calloc(1, sizeof *sp);
-	AN(sp);
-	REPLACE(sp->name, name);
-	sp->b = src;
-	sp->e = strchr(src, '\0');
-	return (sp);
-}
-
-/*--------------------------------------------------------------------*/
-
-static void
-vcc_destroy_source(struct source **spp)
-{
-	struct source *sp;
-
-	AN(spp);
-	sp = *spp;
-	*spp = NULL;
-
-	AN(sp);
-	free(sp->name);
-	free(sp->freeit);
-	free(sp);
-}
-
-/*--------------------------------------------------------------------*/
-
-static struct source *
-vcc_file_source(const struct vcc *tl, const char *fn)
-{
-	char *f, *fnp;
-	struct source *sp;
-
-	if (!tl->unsafe_path && strchr(fn, '/') != NULL) {
-		VSB_printf(tl->sb, "VCL filename '%s' is unsafe.\n", fn);
-		return (NULL);
-	}
-	f = NULL;
-	if (VFIL_searchpath(tl->vcl_path, NULL, &f, fn, &fnp) || f == NULL) {
-		VSB_printf(tl->sb, "Cannot read file '%s' (%s)\n",
-		    fnp != NULL ? fnp : fn, strerror(errno));
-		free(fnp);
-		return (NULL);
-	}
-	sp = vcc_new_source(f, fnp);
-	free(fnp);
-	sp->freeit = f;
-	return (sp);
-}
-
-/*--------------------------------------------------------------------*/
-
-static void
-vcc_resolve_includes(struct vcc *tl)
-{
-	struct token *t, *t1, *t2;
-	struct source *sp;
-	const struct source *sp1;
-	struct vsb *vsb;
-	const char *p;
-
-	VTAILQ_FOREACH(t, &tl->tokens, list) {
-		if (t->tok != ID || !vcc_IdIs(t, "include"))
-			continue;
-
-		t1 = VTAILQ_NEXT(t, list);
-		AN(t1);			/* There's always an EOI */
-		if (t1->tok != CSTR) {
-			VSB_cat(tl->sb,
-			    "include not followed by string constant.\n");
-			vcc_ErrWhere(tl, t1);
-			return;
-		}
-		t2 = VTAILQ_NEXT(t1, list);
-		AN(t2);			/* There's always an EOI */
-
-		if (t2->tok != ';') {
-			VSB_cat(tl->sb,
-			    "include <string> not followed by semicolon.\n");
-			vcc_ErrWhere(tl, t1);
-			return;
-		}
-
-		if (t1->dec[0] == '.' && t1->dec[1] == '/') {
-			/*
-			 * Nested include filenames, starting with "./" are
-			 * resolved relative to the VCL file which contains
-			 * the include directive.
-			 */
-			if (t1->src->name[0] != '/') {
-				VSB_cat(tl->sb,
-				    "include \"./xxxxx\"; needs absolute "
-				    "filename of including file.\n");
-				vcc_ErrWhere(tl, t1);
-				return;
-			}
-			vsb = VSB_new_auto();
-			AN(vsb);
-			p = strrchr(t1->src->name, '/');
-			AN(p);
-			VSB_bcat(vsb, t1->src->name, p - t1->src->name);
-			VSB_cat(vsb, t1->dec + 1);
-			AZ(VSB_finish(vsb));
-			sp = vcc_file_source(tl, VSB_data(vsb));
-			VSB_destroy(&vsb);
-		} else {
-			sp = vcc_file_source(tl, t1->dec);
-		}
-		if (sp == NULL) {
-			vcc_ErrWhere(tl, t1);
-			return;
-		}
-
-		for (sp1 = t->src; sp1 != NULL; sp1 = sp1->parent)
-			if (!strcmp(sp1->name, sp->name))
-				break;
-		if (sp1 != NULL) {
-			VSB_printf(tl->sb,
-			    "Recursive include of \"%s\"\n\n", sp->name);
-			vcc_ErrWhere(tl, t1);
-			for (sp1 = t->src; sp1 != NULL; sp1 = sp1->parent) {
-				if (sp1->parent_tok)
-					vcc_ErrWhere(tl, sp1->parent_tok);
-			}
-			vcc_destroy_source(&sp);
-			return;
-		}
-		sp->parent = t->src;
-		sp->parent_tok = t1;
-		VTAILQ_INSERT_TAIL(&tl->sources, sp, list);
-		sp->idx = tl->nsources++;
-		tl->t = t2;
-		vcc_Lexer(tl, sp, 0);
-
-		VTAILQ_REMOVE(&tl->tokens, t, list);
-		VTAILQ_REMOVE(&tl->tokens, t1, list);
-		VTAILQ_REMOVE(&tl->tokens, t2, list);
-		if (!tl->err)
-			vcc_resolve_includes(tl);
-		return;
-	}
-}
-
 /*--------------------------------------------------------------------
  * Compile the VCL code from the given source and return the C-source
  */
@@ -786,24 +639,19 @@ vcc_CompileSource(struct vcc *tl, struct source *sp, const char *jfile)
 	/* Register and lex the main source */
 	if (sp != NULL) {
 		AN(vcc_builtin);
-		VTAILQ_INSERT_TAIL(&tl->sources, sp, list);
-		sp->idx = tl->nsources++;
-		vcc_Lexer(tl, sp, 0);
+		vcc_lex_source(tl, sp, 0);
 		if (tl->err)
 			return (NULL);
 	}
 
 	/* Register and lex the builtin VCL */
-	sp = vcc_new_source(tl->builtin_vcl, "Builtin");
+	sp = vcc_new_source(tl->builtin_vcl, "builtin", "<builtin>");
 	assert(sp != NULL);
-	VTAILQ_INSERT_TAIL(&tl->sources, sp, list);
-	sp->idx = tl->nsources++;
-	vcc_Lexer(tl, sp, 1);
+	vcc_lex_source(tl, sp, 1);
 	if (tl->err)
 		return (NULL);
 
 	/* Expand and lex any includes in the token string */
-	vcc_resolve_includes(tl);
 	if (tl->err)
 		return (NULL);
 
@@ -945,7 +793,7 @@ VCC_Compile(struct vcc *tl, struct vsb **sb,
 	}
 
 	if (vclsrc != NULL)
-		sp = vcc_new_source(vclsrc, vclsrcfile);
+		sp = vcc_new_source(vclsrc, "vcl.inline", vclsrcfile);
 	else
 		sp = vcc_file_source(tl, vclsrcfile);
 
@@ -973,6 +821,7 @@ VCC_New(void)
 	struct vcc *tl;
 	struct symbol *sym;
 	struct proc *p;
+	struct vsb *vsb;
 	int i;
 
 	ALLOC_OBJ(tl, VCC_MAGIC);
@@ -1002,7 +851,30 @@ VCC_New(void)
 		    SYM_MAIN, SYM_SUB, VCL_LOW, VCL_HIGH);
 		p = vcc_NewProc(tl, sym);
 		p->method = &method_tab[i];
-		VSB_printf(p->cname, "VGC_function_%s", p->method->name);
+
+		// see also VCC_GlobalSymbol()
+		vsb = VSB_new_auto();
+		AN(vsb);
+		VSB_printf(vsb, "%s_%s", SUB->global_pfx, p->method->name);
+		AZ(VSB_finish(vsb));
+
+		AZ(VSB_bcat(p->cname, VSB_data(vsb), VSB_len(vsb)));
+
+		sym->lname = strdup(VSB_data(vsb));
+		AN(sym->lname);
+
+		VSB_clear(vsb);
+		VSB_printf(vsb, "sub_%s", sym->lname);
+		AZ(VSB_finish(vsb));
+
+		sym->rname = strdup(VSB_data(vsb));
+		AN(sym->rname);
+		VSB_destroy(&vsb);
+
+		sym->type = SUB;
+		sym->kind = VCC_HandleKind(SUB);
+		AZ(VCT_invalid_name(sym->rname, NULL));
+		sym->eval = vcc_Eval_Sub;
 	}
 	tl->sb = VSB_new_auto();
 	AN(tl->sb);
