@@ -48,7 +48,7 @@ struct expr {
 	uint8_t		constant;
 #define EXPR_VAR	(1<<0)
 #define EXPR_CONST	(1<<1)
-#define EXPR_STR_CONST	(1<<2)		// Last STRING_LIST elem is "..."
+#define EXPR_STR_CONST	(1<<2)		// Last string elem is "..."
 	struct token	*t1, *t2;
 	struct symbol	*instance;
 	int		nstr;
@@ -83,6 +83,8 @@ vcc_utype(vcc_type_t t)
 
 static void vcc_expr0(struct vcc *tl, struct expr **e, vcc_type_t fmt);
 static void vcc_expr_cor(struct vcc *tl, struct expr **e, vcc_type_t fmt);
+static void vcc_expr_typecheck(struct vcc *tl, struct expr **e, vcc_type_t fmt,
+    struct token *t1);
 
 static struct expr *
 vcc_new_expr(vcc_type_t fmt)
@@ -116,7 +118,7 @@ vcc_delete_expr(struct expr *e)
 {
 	if (e == NULL)
 		return;
-	CHECK_OBJ_NOTNULL(e, EXPR_MAGIC);
+	CHECK_OBJ(e, EXPR_MAGIC);
 	VSB_destroy(&e->vsb);
 	FREE_OBJ(e);
 }
@@ -146,6 +148,19 @@ vcc_delete_expr(struct expr *e)
  * XXX: check line lengths in edit, should pass indent in for this
  */
 
+static void
+vcc_strands_edit(const struct expr *e1, const struct expr *e2)
+{
+
+	if (e2->nstr == 1) {
+		VSB_printf(e1->vsb, "TOSTRAND(%s)", VSB_data(e2->vsb));
+		return;
+	}
+
+	VSB_printf(e1->vsb, "TOSTRANDS(%d,\v+\n%s\v-)",
+	    e2->nstr, VSB_data(e2->vsb));
+}
+
 static struct expr *
 vcc_expr_edit(struct vcc *tl, vcc_type_t fmt, const char *p, struct expr *e1,
     struct expr *e2)
@@ -174,22 +189,21 @@ vcc_expr_edit(struct vcc *tl, vcc_type_t fmt, const char *p, struct expr *e1,
 			e3 = (*p == 'S' ? e1 : e2);
 			AN(e3);
 			assert(e1->fmt == STRINGS);
-			if (e3->nstr > 1)
+			if (e3->nstr > 1) {
 				VSB_cat(e->vsb,
-				    "\nVRT_CollectString(ctx,\v+\n");
-			VSB_cat(e->vsb, VSB_data(e3->vsb));
-			if (e3->nstr > 1)
+				    "\nVRT_STRANDS_string(ctx,\v+\n");
+				vcc_strands_edit(e, e3);
 				VSB_cat(e->vsb,
-				    ",\nvrt_magic_string_end)\v-\n");
+				    "\v-\n)\n");
+			} else {
+				VSB_cat(e->vsb, VSB_data(e3->vsb));
+			}
 			break;
 		case 'T':
 		case 't':
 			e3 = (*p == 'T' ? e1 : e2);
 			AN(e3);
-			VSB_printf(e->vsb,
-			    "&(struct strands){.n = %d, .p = "
-			    "(const char *[%d]){\n%s\n}}",
-			    e3->nstr, e3->nstr, VSB_data(e3->vsb));
+			vcc_strands_edit(e, e3);
 			break;
 		case '1':
 			VSB_cat(e->vsb, VSB_data(e1->vsb));
@@ -223,8 +237,10 @@ vcc_expr_fmt(struct vsb *d, int ind, const struct expr *e1)
 	char *p;
 	int i;
 
-	for (i = 0; i < ind; i++)
-		VSB_cat(d, " ");
+	if (!e1->fmt->noindent) {
+		for (i = 0; i < ind; i++)
+			VSB_putc(d, ' ');
+	}
 	p = VSB_data(e1->vsb);
 	while (*p != '\0') {
 		if (*p == '\n') {
@@ -232,13 +248,13 @@ vcc_expr_fmt(struct vsb *d, int ind, const struct expr *e1)
 			if (*++p == '\0')
 				break;
 			for (i = 0; i < ind; i++)
-				VSB_cat(d, " ");
+				VSB_putc(d, ' ');
 		} else if (*p != '\v') {
 			VSB_putc(d, *p++);
 		} else {
 			switch (*++p) {
-			case '+': ind += 2; break;
-			case '-': ind -= 2; break;
+			case '+': ind += INDENT; break;
+			case '-': ind -= INDENT; break;
 			default:  WRONG("Illegal format in VCC expression");
 			}
 			p++;
@@ -288,15 +304,9 @@ vcc_expr_tostring(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 		(*e)->constant = constant;
 		(*e)->nstr = 1;
 	} else {
-		if ((*e)->fmt == BLOB)
-			VSB_cat(tl->sb,
-			    "Wrong use of BLOB value.\n"
-			    "BLOBs can only be used as arguments to VMOD"
-			    " functions.\n");
-		else
-			VSB_printf(tl->sb,
-			    "Cannot convert %s to STRING.\n",
-			    vcc_utype((*e)->fmt));
+		VSB_printf(tl->sb,
+		    "Cannot convert %s to STRING.\n",
+		    vcc_utype((*e)->fmt));
 		vcc_ErrWhere2(tl, (*e)->t1, tl->t);
 	}
 }
@@ -328,6 +338,29 @@ vcc_Eval_Handle(struct vcc *tl, struct expr **e, struct token *t,
 	}
 }
 
+void v_matchproto_(sym_expr_t)
+vcc_Eval_Sub(struct vcc *tl, struct expr **e, struct token *t,
+    struct symbol *sym, vcc_type_t type)
+{
+
+	(void)t;
+	(void)tl;
+	AN(sym->rname);
+	AZ(type->stringform);
+
+	assert (sym->type == SUB);
+
+	if (type == SUB) {
+		*e = vcc_mk_expr(sym->type, "%s", sym->rname);
+		(*e)->constant = EXPR_CONST;
+		return;
+	}
+
+	VSB_printf(tl->sb, "Symbol '%s' can only be used as a %s expression\n",
+	    sym->name, sym->type->name);
+	vcc_ErrWhere(tl, tl->t);
+}
+
 /*--------------------------------------------------------------------
  */
 
@@ -472,7 +505,7 @@ vcc_func(struct vcc *tl, struct expr **e, const void *priv,
 	int n;
 
 	CAST_OBJ_NOTNULL(vv, priv, VJSN_VAL_MAGIC);
-	assert(vv->type == VJSN_ARRAY);
+	assert(vjsn_is_array(vv));
 	vv = VTAILQ_FIRST(&vv->children);
 	rfmt = VCC_Type(VTAILQ_FIRST(&vv->children)->value);
 	AN(rfmt);
@@ -506,7 +539,7 @@ vcc_func(struct vcc *tl, struct expr **e, const void *priv,
 	}
 	VTAILQ_INIT(&head);
 	for (;vv != NULL; vv = VTAILQ_NEXT(vv, list)) {
-		assert(vv->type == VJSN_ARRAY);
+		assert(vjsn_is_array(vv));
 		fa = calloc(1, sizeof *fa);
 		AN(fa);
 		fa->cname = cfunc;
@@ -535,7 +568,7 @@ vcc_func(struct vcc *tl, struct expr **e, const void *priv,
 				}
 			}
 		}
-		if (sa != NULL && vvp != NULL && vvp->type == VJSN_TRUE) {
+		if (sa != NULL && vvp != NULL && vjsn_is_true(vvp)) {
 			fa->optional = 1;
 			vvp = VTAILQ_NEXT(vvp, list);
 		}
@@ -678,6 +711,38 @@ vcc_Eval_SymFunc(struct vcc *tl, struct expr **e, struct token *t,
 	}
 }
 
+/*--------------------------------------------------------------------
+ */
+
+static void
+vcc_number(struct vcc *tl, struct expr **e, vcc_type_t fmt, const char *sign)
+{
+	VCL_INT vi;
+	struct expr *e1;
+	struct token *t;
+
+	assert(fmt != VOID);
+	if (fmt == BYTES) {
+		vcc_ByteVal(tl, &vi);
+		ERRCHK(tl);
+		e1 = vcc_mk_expr(BYTES, "%ju", (intmax_t)vi);
+	} else {
+		t = tl->t;
+		vcc_NextToken(tl);
+		if (tl->t->tok == ID) {
+			e1 = vcc_mk_expr(DURATION, "%s%.3f * %g",
+			    sign, t->num, vcc_DurationUnit(tl));
+			ERRCHK(tl);
+		} else if (fmt == REAL || t->tok == FNUM) {
+			e1 = vcc_mk_expr(REAL, "%s%.3f", sign, t->num);
+		} else {
+			e1 = vcc_mk_expr(INT, "%s%.0f", sign, t->num);
+		}
+	}
+	e1->constant = EXPR_CONST;
+	*e = e1;
+}
+
 /*--------------------------------------------------------------------
  * SYNTAX:
  *    Expr5:
@@ -686,6 +751,7 @@ vcc_Eval_SymFunc(struct vcc *tl, struct expr **e, struct token *t,
  *	CNUM
  *	FNUM
  *	CSTR
+ *	CBLOB
  */
 
 static void
@@ -693,9 +759,8 @@ vcc_expr5(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 {
 	struct expr *e1, *e2;
 	const char *ip, *sign;
-	struct token *t;
+	struct token *t, *t1;
 	struct symbol *sym;
-	double d;
 
 	sign = "";
 	*e = NULL;
@@ -713,8 +778,21 @@ vcc_expr5(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	switch (tl->t->tok) {
 	case ID:
 		t = tl->t;
-		sym = VCC_SymbolGet(tl, SYM_MAIN, SYM_NONE, SYMTAB_PARTIAL,
-		    XREF_REF);
+		t1 = vcc_PeekToken(tl);
+		AN(t1);
+		sym = VCC_SymbolGet(tl, SYM_MAIN, SYM_NONE,
+		    SYMTAB_PARTIAL_NOERR, XREF_REF);
+		if (sym == NULL && fmt->global_pfx != NULL && t1->tok != '.') {
+			sym = VCC_SymbolGet(tl, SYM_MAIN, SYM_NONE,
+			    SYMTAB_CREATE, XREF_REF);
+			ERRCHK(tl);
+			AN(sym);
+			VCC_GlobalSymbol(sym, fmt);
+		}
+		ERRCHK(tl);
+		if (sym == NULL)
+			AZ(VCC_SymbolGet(tl, SYM_MAIN, SYM_NONE,
+			    SYMTAB_PARTIAL, XREF_REF));
 		ERRCHK(tl);
 		AN(sym);
 		if (sym->kind == SYM_INSTANCE) {
@@ -809,26 +887,7 @@ vcc_expr5(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 		/* FALLTHROUGH */
 	case FNUM:
 	case CNUM:
-		assert(fmt != VOID);
-		if (fmt == BYTES) {
-			vcc_ByteVal(tl, &d);
-			ERRCHK(tl);
-			e1 = vcc_mk_expr(BYTES, "%.1f", d);
-		} else {
-			t = tl->t;
-			vcc_NextToken(tl);
-			if (tl->t->tok == ID) {
-				e1 = vcc_mk_expr(DURATION, "(%s%.*s) * %g",
-				    sign, PF(t), vcc_DurationUnit(tl));
-				ERRCHK(tl);
-			} else if (fmt == REAL || t->tok == FNUM) {
-				e1 = vcc_mk_expr(REAL, "%s%.*s", sign, PF(t));
-			} else {
-				e1 = vcc_mk_expr(INT, "%s%.*s", sign, PF(t));
-			}
-		}
-		e1->constant = EXPR_CONST;
-		*e = e1;
+		vcc_number(tl, e, fmt, sign);
 		return;
 	case CBLOB:
 		e1 = vcc_new_expr(BLOB);
@@ -1089,12 +1148,16 @@ cmp_simple(struct vcc *tl, struct expr **e, const struct cmps *cp)
 static void v_matchproto_(cmp_f)
 cmp_regexp(struct vcc *tl, struct expr **e, const struct cmps *cp)
 {
+	struct token *t1;
 	struct expr *e2;
 	char buf[128];
 
 	*e = vcc_expr_edit(tl, STRING, "\vS", *e, NULL);
 	vcc_NextToken(tl);
-	vcc_expr0(tl, &e2, REGEX);
+	t1 = tl->t;
+	vcc_expr4(tl, &e2, REGEX);
+	ERRCHK(tl);
+	vcc_expr_typecheck(tl, &e2, REGEX, t1);
 	ERRCHK(tl);
 	bprintf(buf, "%sVRT_re_match(ctx, \v1, \v2)", cp->emit);
 	*e = vcc_expr_edit(tl, BOOL, buf, *e, e2);
@@ -1103,17 +1166,18 @@ cmp_regexp(struct vcc *tl, struct expr **e, const struct cmps *cp)
 static void v_matchproto_(cmp_f)
 cmp_acl(struct vcc *tl, struct expr **e, const struct cmps *cp)
 {
-	struct symbol *sym;
+	struct token *t1;
+	struct expr *e2;
 	char buf[256];
 
 	vcc_NextToken(tl);
-	vcc_ExpectVid(tl, "ACL");
-	sym = VCC_SymbolGet(tl, SYM_MAIN, SYM_ACL, SYMTAB_CREATE, XREF_REF);
+	t1 = tl->t;
+	vcc_expr4(tl, &e2, ACL);
 	ERRCHK(tl);
-	AN(sym);
-	VCC_GlobalSymbol(sym, ACL, ACL_SYMBOL_PREFIX);
-	bprintf(buf, "%sVRT_acl_match(ctx, %s, \v1)", cp->emit, sym->rname);
-	*e = vcc_expr_edit(tl, BOOL, buf, *e, NULL);
+	vcc_expr_typecheck(tl, &e2, ACL, t1);
+	ERRCHK(tl);
+	bprintf(buf, "%sVRT_acl_match(ctx, \v1, \v2)", cp->emit);
+	*e = vcc_expr_edit(tl, BOOL, buf, e2, *e);
 }
 
 static void v_matchproto_(cmp_f)
@@ -1351,8 +1415,6 @@ vcc_expr0(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	t1 = tl->t;
 	if (fmt->stringform)
 		vcc_expr_cor(tl, e, STRINGS);
-	else if (fmt == REGEX)
-		vcc_expr4(tl, e, REGEX);
 	else
 		vcc_expr_cor(tl, e, fmt);
 	ERRCHK(tl);
@@ -1371,10 +1433,20 @@ vcc_expr0(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 		return;
 	}
 
+	if (fmt == BODY && !(*e)->fmt->bodyform)
+		vcc_expr_tostring(tl, e, STRINGS);
+
+	if (fmt == BODY && (*e)->fmt->bodyform) {
+		if ((*e)->fmt == STRINGS)
+			*e = vcc_expr_edit(tl, BODY, "STRING, 0, \vT", *e, NULL);
+		else if ((*e)->fmt == BLOB)
+			*e = vcc_expr_edit(tl, BODY, "BLOB, 0, \v1", *e, NULL);
+		else
+			WRONG("Unhandled bodyform");
+	}
+
 	if ((*e)->fmt == STRINGS && fmt->stringform) {
-		if (fmt == STRING_LIST)
-			(*e)->fmt = STRING_LIST;
-		else if (fmt == STRING)
+		if (fmt == STRING)
 			*e = vcc_expr_edit(tl, STRING, "\vS", *e, NULL);
 		else if (fmt == STRANDS)
 			*e = vcc_expr_edit(tl, STRANDS, "\vT", (*e), NULL);
@@ -1382,20 +1454,26 @@ vcc_expr0(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 			WRONG("Unhandled stringform");
 	}
 
-	if ((*e)->fmt == STRING_LIST)
-		*e = vcc_expr_edit(tl, STRING_LIST,
-		    "\n\v1,\nvrt_magic_string_end", *e, NULL);
-
 	if (fmt == BOOL) {
 		vcc_expr_tobool(tl, e);
 		ERRCHK(tl);
 	}
 
+	vcc_expr_typecheck(tl, e, fmt, t1);
+}
+
+static void
+vcc_expr_typecheck(struct vcc *tl, struct expr **e, vcc_type_t fmt,
+    struct token *t1)
+{
+
+	assert(fmt != VOID);
+	assert(fmt != STRINGS);
+
 	if (fmt != (*e)->fmt)  {
 		VSB_printf(tl->sb, "Expression has type %s, expected %s\n",
 		    vcc_utype((*e)->fmt), vcc_utype(fmt));
 		vcc_ErrWhere2(tl, t1, tl->t);
-		return;
 	}
 }
 
