@@ -4,6 +4,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -59,47 +61,20 @@
 #include <string.h>
 
 #include "vcc_compile.h"
-#include "libvcc.h"		// VCC_Stevedore() proto
-
-/*--------------------------------------------------------------------
- *
- */
-
-static struct stvars {
-	const char	*name;
-	vcc_type_t	fmt;
-} stvars[] = {
-#define VRTSTVVAR(nm, vtype, ctype, dval)	{ #nm, vtype },
-#include "tbl/vrt_stv_var.h"
-#undef VRTSTVVAR
-	{ NULL,			BOOL }
-};
 
 void
 vcc_stevedore(struct vcc *vcc, const char *stv_name)
 {
 	struct symbol *sym;
-	struct stvars *sv;
 	char buf[1024];
 
 	CHECK_OBJ_NOTNULL(vcc, VCC_MAGIC);
 	bprintf(buf, "storage.%s", stv_name);
-	sym = VCC_Symbol(vcc, NULL, buf, NULL, SYM_VAR, 1);
+	sym = VCC_MkSym(vcc, buf, SYM_MAIN, SYM_VAR, VCL_LOW, VCL_41);
 	AN(sym);
-	sym->fmt = STEVEDORE;
+	sym->type = STEVEDORE;
 	sym->eval = vcc_Eval_Var;
 	bprintf(buf, "VRT_stevedore(\"%s\")", stv_name);
 	sym->rname = TlDup(vcc, buf);
 	sym->r_methods = ~0;
-
-	for (sv = stvars; sv->name != NULL; sv++) {
-		bprintf(buf, "storage.%s.%s", stv_name, sv->name);
-		sym = VCC_Symbol(vcc, NULL, buf, NULL, SYM_VAR, 1);
-		AN(sym);
-		sym->fmt = sv->fmt;
-		sym->eval = vcc_Eval_Var;
-		bprintf(buf, "VRT_Stv_%s(\"%s\")", sv->name, stv_name);
-		sym->rname = TlDup(vcc, buf);
-		sym->r_methods = ~0;
-	}
 }
