@@ -263,18 +263,33 @@ vcc_ErrWhere(struct vcc *tl, const struct token *t)
 
 /*--------------------------------------------------------------------*/
 
+struct token *
+vcc_PeekTokenFrom(const struct vcc *tl, const struct token *t)
+{
+	struct token *t2;
+
+	CHECK_OBJ_NOTNULL(tl, VCC_MAGIC);
+	AN(t);
+	assert(t->tok != EOI);
+	t2 = VTAILQ_NEXT(t, list);
+	AN(t2);
+	return (t2);
+}
+
+struct token *
+vcc_PeekToken(const struct vcc *tl)
+{
+
+	CHECK_OBJ_NOTNULL(tl, VCC_MAGIC);
+	return (vcc_PeekTokenFrom(tl, tl->t));
+}
+
 void
 vcc_NextToken(struct vcc *tl)
 {
 
-	tl->t = VTAILQ_NEXT(tl->t, list);
-	if (tl->t == NULL) {
-		VSB_cat(tl->sb,
-		    "Ran out of input, something is missing or"
-		    " maybe unbalanced (...) or {...}\n");
-		tl->err = 1;
-		return;
-	}
+	CHECK_OBJ_NOTNULL(tl, VCC_MAGIC);
+	tl->t = vcc_PeekTokenFrom(tl, tl->t);
 }
 
 void
@@ -310,29 +325,46 @@ vcc_IdIs(const struct token *t, const char *p)
  * Check that we have a Varnish identifier
  */
 
+void
+vcc_PrintTokens(const struct vcc *tl,
+    const struct token *tb, const struct token *te)
+{
+
+	CHECK_OBJ_NOTNULL(tl, VCC_MAGIC);
+	AN(tb);
+	AN(te);
+	while (tb != te) {
+		VSB_printf(tl->sb, "%.*s", PF(tb));
+		tb = vcc_PeekTokenFrom(tl, tb);
+		AN(tb);
+	}
+}
+
 void
 vcc_ExpectVid(struct vcc *tl, const char *what)
 {
 	const char *bad = NULL;
-	struct token *t2, *t3;
+	struct token *t2;
 
 	ExpectErr(tl, ID);
 	ERRCHK(tl);
 
-	t2 = VTAILQ_NEXT(tl->t, list);
+	t2 = vcc_PeekToken(tl);
+	AN(t2);
 	while (t2->tok == '.') {
 		bad = ".";
-		t2 = VTAILQ_NEXT(t2, list);
+		t2 = vcc_PeekTokenFrom(tl, t2);
+		AN(t2);
 		if (t2->tok != ID)
 			break;
-		t2 = VTAILQ_NEXT(t2, list);
+		t2 = vcc_PeekTokenFrom(tl, t2);
+		AN(t2);
 	}
 	if (bad == NULL)
 		bad = VCT_invalid_name(tl->t->b, tl->t->e);
 	if (bad != NULL) {
 		VSB_printf(tl->sb, "Name of %s, '", what);
-		for (t3 = tl->t; t3 != t2; t3 = VTAILQ_NEXT(t3, list))
-			VSB_printf(tl->sb, "%.*s", PF(t3));
+		vcc_PrintTokens(tl, tl->t, t2);
 		VSB_printf(tl->sb,
 		    "', contains illegal character '%c'\n", *bad);
 		vcc_ErrWhere2(tl, tl->t, t2);
@@ -344,20 +376,19 @@ vcc_ExpectVid(struct vcc *tl, const char *what)
  * Decode a string
  */
 
-static int
-vcc_decstr(struct vcc *tl)
+static void
+vcc_decstr(struct vcc *tl, unsigned sep)
 {
 	char *q;
 	unsigned int l;
 
 	assert(tl->t->tok == CSTR);
-	l = (tl->t->e - tl->t->b) - 2;
+	l = pdiff(tl->t->b + sep, tl->t->e - sep);
 	tl->t->dec = TlAlloc(tl, l + 1);
-	assert(tl->t->dec != NULL);
+	AN(tl->t->dec);
 	q = tl->t->dec;
-	memcpy(q, tl->t->b + 1, l);
+	memcpy(q, tl->t->b + sep, l);
 	q[l] = '\0';
-	return (0);
 }
 
 /*--------------------------------------------------------------------
@@ -366,7 +397,7 @@ vcc_decstr(struct vcc *tl)
 
 static void
 vcc_addtoken(struct vcc *tl, unsigned tok,
-    const struct source *sp, const char *b, const char *e)
+    struct source *sp, const char *b, const char *e)
 {
 	struct token *t;
 
@@ -376,19 +407,114 @@ vcc_addtoken(struct vcc *tl, unsigned tok,
 	t->b = b;
 	t->e = e;
 	t->src = sp;
-	if (tl->t != NULL)
-		VTAILQ_INSERT_AFTER(&tl->tokens, tl->t, t, list);
-	else
-		VTAILQ_INSERT_TAIL(&tl->tokens, t, list);
+	VTAILQ_INSERT_TAIL(&sp->src_tokens, t, src_list);
 	tl->t = t;
 }
 
+/*--------------------------------------------------------------------
+ * Find a delimited token
+ */
+
+static const struct delim_def {
+	const char	*name;
+	const char	*b;
+	const char	*e;
+	unsigned	len;	/* NB: must be the same for both delimiters */
+	unsigned	crlf;
+	unsigned	tok;
+} delim_defs[] = {
+#define DELIM_DEF(nm, l, r, c, t)		\
+	{ nm, l, r, sizeof (l) - 1, c, t }
+	DELIM_DEF("long-string", "\"\"\"", "\"\"\"", 1, CSTR),	/* """...""" */
+	DELIM_DEF("long-string", "{\"", "\"}", 1, CSTR),	/*  {"..."}  */
+	DELIM_DEF("string", "\"", "\"", 0, CSTR),		/*   "..."   */
+	DELIM_DEF("inline C source", "C{", "}C", 1, CSRC),	/*  C{...}C  */
+#undef DELIM_DEF
+	{ NULL }
+};
+
+static unsigned
+vcc_delim_token(struct vcc *tl, struct source *sp, const char *p,
+    const char **qp)
+{
+	const struct delim_def *dd;
+	const char *q, *r;
+
+	for (dd = delim_defs; dd->name != NULL; dd++)
+		if (!strncmp(p, dd->b, dd->len))
+			break;
+
+	if (dd->name == NULL)
+		return (0);
+
+	q = strstr(p + dd->len, dd->e);
+	if (q != NULL && !dd->crlf) {
+		r = strpbrk(p + dd->len, "\r\n");
+		if (r != NULL && r < q)
+			q = NULL;
+	}
+
+	if (q == NULL) {
+		vcc_addtoken(tl, EOI, sp, p, p + dd->len);
+		VSB_printf(tl->sb, "Unterminated %s, starting at\n", dd->name);
+		vcc_ErrWhere(tl, tl->t);
+		return (0);
+	}
+
+	assert(q < sp->e);
+	vcc_addtoken(tl, dd->tok, sp, p, q + dd->len);
+	if (dd->tok == CSTR)
+		vcc_decstr(tl, dd->len);
+	*qp = q + dd->len;
+	return (1);
+}
+
+/*--------------------------------------------------------------------
+ * Lex a number, either CNUM or FNUM.
+ * We enforce the RFC8941 restrictions on number of digits here.
+ */
+
+static const char *
+vcc_lex_number(struct vcc *tl, struct source *sp, const char *p)
+{
+	const char *q, *r;
+	char *s;
+
+	for (q = p; q < sp->e; q++)
+		if (!vct_isdigit(*q))
+			break;
+	if (*q != '.') {
+		vcc_addtoken(tl, CNUM, sp, p, q);
+		if (q - p > 15) {
+			VSB_cat(tl->sb, "Too many digits for integer.\n");
+			vcc_ErrWhere(tl, tl->t);
+			return (NULL);
+		}
+		tl->t->num = strtod(p, &s);
+		assert(s == tl->t->e);
+		return (q);
+	}
+	r = ++q;
+	for (; r < sp->e; r++)
+		if (!vct_isdigit(*r))
+			break;
+	vcc_addtoken(tl, FNUM, sp, p, r);
+	if (q - p > 13 || r - q > 3) {
+		VSB_cat(tl->sb, "Too many digits for real.\n");
+		vcc_ErrWhere(tl, tl->t);
+		return(NULL);
+	}
+	tl->t->num = strtod(p, &s);
+	assert(s == tl->t->e);
+	return (r);
+}
+
 /*--------------------------------------------------------------------
  * Lexical analysis and token generation
  */
 
 void
-vcc_Lexer(struct vcc *tl, const struct source *sp, int eoi)
+vcc_Lexer(struct vcc *tl, struct source *sp)
 {
 	const char *p, *q, *r;
 	unsigned u;
@@ -443,50 +569,6 @@ vcc_Lexer(struct vcc *tl, const struct source *sp, int eoi)
 			continue;
 		}
 
-		/* Recognize inline C-code */
-		if (*p == 'C' && p[1] == '{') {
-			for (q = p + 2; q < sp->e; q++) {
-				if (*q == '}' && q[1] == 'C') {
-					vcc_addtoken(tl, CSRC, sp, p, q + 2);
-					break;
-				}
-			}
-			if (q < sp->e) {
-				p = q + 2;
-				continue;
-			}
-			vcc_addtoken(tl, EOI, sp, p, p + 2);
-			VSB_cat(tl->sb,
-			    "Unterminated inline C source, starting at\n");
-			vcc_ErrWhere(tl, tl->t);
-			return;
-		}
-
-		/* Recognize long-strings */
-		if (*p == '{' && p[1] == '"') {
-			for (q = p + 2; q < sp->e; q++) {
-				if (*q == '"' && q[1] == '}') {
-					vcc_addtoken(tl, CSTR, sp, p, q + 2);
-					break;
-				}
-			}
-			if (q < sp->e) {
-				p = q + 2;
-				u = tl->t->e - tl->t->b;
-				u -= 4;		/* {" ... "} */
-				tl->t->dec = TlAlloc(tl, u + 1 );
-				AN(tl->t->dec);
-				memcpy(tl->t->dec, tl->t->b + 2, u);
-				tl->t->dec[u] = '\0';
-				continue;
-			}
-			vcc_addtoken(tl, EOI, sp, p, p + 2);
-			VSB_cat(tl->sb,
-			    "Unterminated long-string, starting at\n");
-			vcc_ErrWhere(tl, tl->t);
-			return;
-		}
-
 		/* Recognize BLOB (= SF-binary) */
 		if (*p == ':') {
 			vsb = VSB_new_auto();
@@ -528,7 +610,8 @@ vcc_Lexer(struct vcc *tl, const struct source *sp, int eoi)
 			AZ(VSB_finish(vsb));
 
 			bprintf(namebuf, "blob_%u", tl->unique++);
-			Fh(tl, 0, "\nconst unsigned char %s_data[%zd] = {\n",
+			Fh(tl, 0,
+			    "\nstatic const unsigned char %s_data[%zd] = {\n",
 			    namebuf, VSB_len(vsb));
 			for (u = 0; u < VSB_len(vsb); u++) {
 				Fh(tl, 0, "\t0x%02x,", VSB_data(vsb)[u] & 0xff);
@@ -538,7 +621,8 @@ vcc_Lexer(struct vcc *tl, const struct source *sp, int eoi)
 			if ((u & 7) != 7)
 				Fh(tl, 0, "\n");
 			Fh(tl, 0, "};\n");
-			Fh(tl, 0, "\nconst struct vrt_blob %s[1] = {{\n",
+			Fh(tl, 0,
+			    "\nstatic const struct vrt_blob %s[1] = {{\n",
 			    namebuf);
 			Fh(tl, 0, "\t.len =\t%zd,\n", VSB_len(vsb));
 			Fh(tl, 0, "\t.blob =\t%s_data,\n", namebuf);
@@ -549,6 +633,13 @@ vcc_Lexer(struct vcc *tl, const struct source *sp, int eoi)
 			continue;
 		}
 
+		/* Match delimited tokens */
+		if (vcc_delim_token(tl, sp, p, &q) != 0) {
+			p = q;
+			continue;
+		}
+		ERRCHK(tl);
+
 		/* Match for the fixed tokens (see generate.py) */
 		u = vcl_fixed_token(p, &q);
 		if (u != 0) {
@@ -557,28 +648,6 @@ vcc_Lexer(struct vcc *tl, const struct source *sp, int eoi)
 			continue;
 		}
 
-		/* Match strings */
-		if (*p == '"') {
-			for (q = p + 1; q < sp->e; q++) {
-				if (*q == '"') {
-					q++;
-					break;
-				}
-				if (*q == '\r' || *q == '\n') {
-					vcc_addtoken(tl, EOI, sp, p, q);
-					VSB_cat(tl->sb,
-					    "Unterminated string at\n");
-					vcc_ErrWhere(tl, tl->t);
-					return;
-				}
-			}
-			vcc_addtoken(tl, CSTR, sp, p, q);
-			if (vcc_decstr(tl))
-				return;
-			p = q;
-			continue;
-		}
-
 		/* Match Identifiers */
 		if (vct_isident1(*p)) {
 			for (q = p; q < sp->e; q++)
@@ -591,19 +660,9 @@ vcc_Lexer(struct vcc *tl, const struct source *sp, int eoi)
 
 		/* Match numbers { [0-9]+ } */
 		if (vct_isdigit(*p)) {
-			for (q = p; q < sp->e; q++)
-				if (!vct_isdigit(*q))
-					break;
-			if (*q != '.') {
-				vcc_addtoken(tl, CNUM, sp, p, q);
-				p = q;
-				continue;
-			}
-			for (++q; q < sp->e; q++)
-				if (!vct_isdigit(*q))
-					break;
-			vcc_addtoken(tl, FNUM, sp, p, q);
-			p = q;
+			p = vcc_lex_number(tl, sp, p);
+			if (p == NULL)
+				return;
 			continue;
 		}
 		vcc_addtoken(tl, EOI, sp, p, p + 1);
@@ -611,6 +670,5 @@ vcc_Lexer(struct vcc *tl, const struct source *sp, int eoi)
 		vcc_ErrWhere(tl, tl->t);
 		return;
 	}
-	if (eoi)
-		vcc_addtoken(tl, EOI, sp, sp->e, sp->e);
+	vcc_addtoken(tl, EOI, sp, sp->e, sp->e);
 }
