@@ -36,12 +36,15 @@
 
 #include "vcc_compile.h"
 
+#include "vct.h"
+
 /*--------------------------------------------------------------------*/
 
 void v_matchproto_(sym_wildcard_t)
 vcc_Var_Wildcard(struct vcc *tl, struct symbol *parent, struct symbol *sym)
 {
 	struct vsb *vsb;
+	const char *p;
 
 	assert(parent->type == HEADER);
 
@@ -52,6 +55,16 @@ vcc_Var_Wildcard(struct vcc *tl, struct symbol *parent, struct symbol *sym)
 		return;
 	}
 
+	for (p = sym->name; *p != '\0'; p++) {
+		if (!vct_istchar(*p)) {
+			VSB_cat(tl->sb, "Invalid character '");
+			VSB_quote(tl->sb, p, 1, VSB_QUOTE_PLAIN);
+			VSB_cat(tl->sb, "' in header name.\n");
+			tl->err = 1;
+			return;
+		}
+	}
+
 	AN(sym);
 	sym->noref = 1;
 	sym->kind = SYM_VAR;
@@ -80,8 +93,7 @@ vcc_Var_Wildcard(struct vcc *tl, struct symbol *parent, struct symbol *sym)
 	AZ(VSB_finish(vsb));
 	sym->lname = TlDup(tl, VSB_data(vsb));
 	VSB_clear(vsb);
-	VSB_printf(vsb, "VRT_SetHdr(ctx, %s, vrt_magic_string_unset)",
-	    sym->rname);
+	VSB_printf(vsb, "VRT_UnsetHdr(ctx, %s)", sym->rname);
 	AZ(VSB_finish(vsb));
 	sym->uname = TlDup(tl, VSB_data(vsb));
 	VSB_destroy(&vsb);
