@@ -77,16 +77,52 @@ static void vcc_VmodObject(struct vcc *tl, struct symbol *sym);
 static void vcc_VmodSymbols(struct vcc *tl, const struct symbol *sym);
 
 static void
-func_sym(struct vcc *tl, vcc_kind_t kind, const struct symbol *psym,
-    const struct vjsn_val *v)
+alias_sym(struct vcc *tl, const struct symbol *psym, const struct vjsn_val *v)
 {
+	char *alias = NULL, *func = NULL;
 	struct symbol *sym;
 	struct vsb *buf;
 
 	buf = VSB_new_auto();
 	AN(buf);
 
+	VCC_SymName(buf, psym);
+	VSB_printf(buf, ".%s", v->value);
+	AZ(VSB_finish(buf));
+	REPLACE(alias, VSB_data(buf));
+
+	v = VTAILQ_NEXT(v, list);
+	assert(vjsn_is_string(v));
+
 	VSB_clear(buf);
+	VCC_SymName(buf, psym);
+	VSB_printf(buf, ".%s", v->value);
+	AZ(VSB_finish(buf));
+	REPLACE(func, VSB_data(buf));
+
+	sym = VCC_MkSymAlias(tl, alias, func);
+	AN(sym);
+	assert(sym->kind == SYM_FUNC || sym->kind == SYM_METHOD);
+	VSB_destroy(&buf);
+	free(alias);
+	free(func);
+}
+
+static void
+func_sym(struct vcc *tl, vcc_kind_t kind, const struct symbol *psym,
+    const struct vjsn_val *v)
+{
+	struct symbol *sym;
+	struct vsb *buf;
+
+	if (kind == SYM_ALIAS) {
+		alias_sym(tl, psym, v);
+		return;
+	}
+
+	buf = VSB_new_auto();
+	AN(buf);
+
 	VCC_SymName(buf, psym);
 	VSB_printf(buf, ".%s", v->value);
 	AZ(VSB_finish(buf));
@@ -108,15 +144,15 @@ func_sym(struct vcc *tl, vcc_kind_t kind, const struct symbol *psym,
 
 	v = VTAILQ_NEXT(v, list);
 
-	assert(v->type == VJSN_ARRAY);
+	assert(vjsn_is_array(v));
 	sym->action = vcc_Act_Call;
 	sym->vmod_name = psym->vmod_name;
 	sym->eval = vcc_Eval_SymFunc;
 	sym->eval_priv = v;
 	v = VTAILQ_FIRST(&v->children);
-	assert(v->type == VJSN_ARRAY);
+	assert(vjsn_is_array(v));
 	v = VTAILQ_FIRST(&v->children);
-	assert(v->type == VJSN_STRING);
+	assert(vjsn_is_string(v));
 	sym->type = VCC_Type(v->value);
 	AN(sym->type);
 	sym->r_methods = VCL_MET_TASK_ALL;
@@ -134,9 +170,9 @@ vcc_json_always(struct vcc *tl, const struct vjsn *vj, const char *vmod_name)
 	ifp = NULL;
 
 	VTAILQ_FOREACH(vv, &vj->value->children, list) {
-		assert(vv->type == VJSN_ARRAY);
+		assert(vjsn_is_array(vv));
 		vv2 = VTAILQ_FIRST(&vv->children);
-		assert(vv2->type == VJSN_STRING);
+		assert(vjsn_is_string(vv2));
 		if (!strcmp(vv2->value, "$VMOD")) {
 			vmod_syntax =
 			    strtod(VTAILQ_NEXT(vv2, list)->value, NULL);
@@ -158,6 +194,7 @@ vcc_json_always(struct vcc *tl, const struct vjsn *vj, const char *vmod_name)
 			    vv2->value, vmod_name);
 			VSB_printf(ifp->event, "%s(ctx, &vmod_priv_%s, ev)",
 			    vv2->value, vmod_name);
+		} else if (!strcmp(vv2->value, "$ALIAS")) {
 		} else if (!strcmp(vv2->value, "$FUNC")) {
 		} else if (!strcmp(vv2->value, "$OBJ")) {
 		} else {
@@ -237,6 +274,7 @@ vcc_vmod_kind(const char *type)
 	VMOD_KIND("$OBJ", SYM_OBJECT);
 	VMOD_KIND("$METHOD", SYM_METHOD);
 	VMOD_KIND("$FUNC", SYM_FUNC);
+	VMOD_KIND("$ALIAS", SYM_ALIAS);
 #undef VMOD_KIND
 	return (SYM_NONE);
 }
@@ -281,12 +319,12 @@ vcc_VmodSymbols(struct vcc *tl, const struct symbol *sym)
 	}
 
 	for (; vv != NULL; vv = VTAILQ_NEXT(vv, list)) {
-		if (vv->type != VJSN_ARRAY)
+		if (!vjsn_is_array(vv))
 			continue;
 		vv1 = VTAILQ_FIRST(&vv->children);
-		assert(vv1->type == VJSN_STRING);
+		assert(vjsn_is_string(vv1));
 		vv2 = VTAILQ_NEXT(vv1, list);
-		if (vv2->type != VJSN_STRING)
+		if (!vjsn_is_string(vv2))
 			continue;
 
 		kind = vcc_vmod_kind(vv1->value);
@@ -315,7 +353,8 @@ vcc_ParseImport(struct vcc *tl)
 
 	ExpectErr(tl, ID);		/* "vmod_name" */
 	mod = tl->t;
-	tmod = VTAILQ_NEXT(mod, list);
+	tmod = vcc_PeekTokenFrom(tl, mod);
+	AN(tmod);
 	if (tmod->tok == ID && vcc_IdIs(tmod, "as")) {
 		vcc_NextToken(tl);		/* "vmod_name" */
 		vcc_NextToken(tl);		/* "as" */
@@ -474,7 +513,7 @@ vcc_Act_New(struct vcc *tl, struct token *t, struct symbol *sym)
 	ExpectErr(tl, ID);
 	vcc_ExpectVid(tl, "VCL object");
 	ERRCHK(tl);
-	isym = VCC_HandleSymbol(tl, INSTANCE, "vo");
+	isym = VCC_HandleSymbol(tl, INSTANCE);
 	ERRCHK(tl);
 	AN(isym);
 	isym->noref = 1;
@@ -498,9 +537,9 @@ vcc_Act_New(struct vcc *tl, struct token *t, struct symbol *sym)
 
 	vv = VTAILQ_NEXT(vv, list);
 	// vv = flags
-	assert(vv->type == VJSN_OBJECT);
+	assert(vjsn_is_object(vv));
 	VTAILQ_FOREACH(vf, &vv->children, list)
-		if (!strcmp(vf->name, "NULL_OK") && vf->type == VJSN_TRUE)
+		if (!strcmp(vf->name, "NULL_OK") && vjsn_is_true(vf))
 			null_ok = 1;
 	if (!null_ok)
 		VTAILQ_INSERT_TAIL(&tl->sym_objects, isym, sideways);
@@ -513,7 +552,7 @@ vcc_Act_New(struct vcc *tl, struct token *t, struct symbol *sym)
 
 	vf = VTAILQ_FIRST(&vv->children);
 	vv = VTAILQ_NEXT(vv, list);
-	assert(vf->type == VJSN_STRING);
+	assert(vjsn_is_string(vf));
 	assert(!strcmp(vf->value, "$INIT"));
 
 	vf = VTAILQ_NEXT(vf, list);
@@ -529,7 +568,7 @@ vcc_Act_New(struct vcc *tl, struct token *t, struct symbol *sym)
 	isym->def_e = tl->t;
 
 	vf = VTAILQ_FIRST(&vv->children);
-	assert(vf->type == VJSN_STRING);
+	assert(vjsn_is_string(vf));
 	assert(!strcmp(vf->value, "$FINI"));
 
 	vf = VTAILQ_NEXT(vf, list);
