@@ -70,7 +70,7 @@ extern void mylog(struct vsl_log *vsl, enum VSL_tag_e tag,
 /**********************************************************************/
 
 static enum vfp_status v_matchproto_(vfp_pull_f)
-xyzzy_rot13_pull(struct vfp_ctx *vc, struct vfp_entry *vfe, void *p,
+xyzzy_vfp_rot13_pull(struct vfp_ctx *vc, struct vfp_entry *vfe, void *p,
     ssize_t *lp)
 {
 	enum vfp_status vp;
@@ -91,9 +91,9 @@ xyzzy_rot13_pull(struct vfp_ctx *vc, struct vfp_entry *vfe, void *p,
 	return (vp);
 }
 
-static const struct vfp xyzzy_rot13 = {
+static const struct vfp xyzzy_vfp_rot13 = {
 	.name = "rot13",
-	.pull = xyzzy_rot13_pull,
+	.pull = xyzzy_vfp_rot13_pull,
 };
 
 /**********************************************************************/
@@ -102,10 +102,12 @@ static const struct vfp xyzzy_rot13 = {
 #define ROT13_BUFSZ 8
 
 static int v_matchproto_(vdp_init_f)
-xyzzy_rot13_init(struct vdp_ctx *vdc, void **priv, struct objcore *oc)
+xyzzy_vfp_rot13_init(VRT_CTX, struct vdp_ctx *vdc, void **priv, struct objcore *oc)
 {
 	(void)vdc;
 	(void)oc;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	AN(priv);
 	*priv = malloc(ROT13_BUFSZ);
 	if (*priv == NULL)
@@ -114,7 +116,7 @@ xyzzy_rot13_init(struct vdp_ctx *vdc, void **priv, struct objcore *oc)
 }
 
 static int v_matchproto_(vdp_bytes_f)
-xyzzy_rot13_bytes(struct vdp_ctx *vdx, enum vdp_action act, void **priv,
+xyzzy_vfp_rot13_bytes(struct vdp_ctx *vdx, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
 	char *q;
@@ -152,7 +154,7 @@ xyzzy_rot13_bytes(struct vdp_ctx *vdx, enum vdp_action act, void **priv,
 }
 
 static int v_matchproto_(vdp_fini_f)
-xyzzy_rot13_fini(struct vdp_ctx *vdc, void **priv)
+xyzzy_vfp_rot13_fini(struct vdp_ctx *vdc, void **priv)
 {
 	(void)vdc;
 	AN(priv);
@@ -163,9 +165,9 @@ xyzzy_rot13_fini(struct vdp_ctx *vdc, void **priv)
 
 static const struct vdp xyzzy_vdp_rot13 = {
 	.name  = "rot13",
-	.init  = xyzzy_rot13_init,
-	.bytes = xyzzy_rot13_bytes,
-	.fini  = xyzzy_rot13_fini,
+	.init  = xyzzy_vfp_rot13_init,
+	.bytes = xyzzy_vfp_rot13_bytes,
+	.fini  = xyzzy_vfp_rot13_fini,
 };
 
 /**********************************************************************
@@ -242,10 +244,10 @@ priv_ ## name ## _fini(VRT_CTX, void *ptr)				\
 									\
 static const struct vmod_priv_methods					\
 xyzzy_test_priv_ ## name ## _methods[1] = {{				\
-		.magic = VMOD_PRIV_METHODS_MAGIC,			\
-		.type = "debug_test_priv_" #name,			\
-		.fini = priv_ ## name ## _fini				\
-	}};
+	.magic = VMOD_PRIV_METHODS_MAGIC,				\
+	.type = "debug_test_priv_" #name,				\
+	.fini = priv_ ## name ## _fini					\
+}};
 PRIV_FINI(call, 0)
 PRIV_FINI(task, 1)
 PRIV_FINI(top, 1)
@@ -422,13 +424,12 @@ priv_vcl_fini(VRT_CTX, void *priv)
 	AZ(priv_vcl->vclref_discard);
 	AZ(priv_vcl->vclref_cold);
 	FREE_OBJ(priv_vcl);
-	AZ(priv_vcl);
 }
 
 static const struct vmod_priv_methods priv_vcl_methods[1] = {{
-		.magic = VMOD_PRIV_METHODS_MAGIC,
-		.type = "debug_priv_vcl_fini",
-		.fini = priv_vcl_fini
+	.magic = VMOD_PRIV_METHODS_MAGIC,
+	.type = "debug_priv_vcl_fini",
+	.fini = priv_vcl_fini
 }};
 
 static int
@@ -452,9 +453,19 @@ event_load(VRT_CTX, struct vmod_priv *priv)
 	priv->priv = priv_vcl;
 	priv->methods = priv_vcl_methods;
 
-	VRT_AddVFP(ctx, &xyzzy_rot13);
-
+	VRT_AddVFP(ctx, &xyzzy_vfp_rot13);
 	VRT_AddVDP(ctx, &xyzzy_vdp_rot13);
+
+	// This should fail
+	AN(VRT_AddFilter(ctx, &xyzzy_vfp_rot13, &xyzzy_vdp_rot13));
+	// Reset the error, we know what we're doing.
+	*ctx->handling = 0;
+
+	VRT_RemoveVFP(ctx, &xyzzy_vfp_rot13);
+	VRT_RemoveVDP(ctx, &xyzzy_vdp_rot13);
+
+	AZ(VRT_AddFilter(ctx, &xyzzy_vfp_rot13, &xyzzy_vdp_rot13));
+
 	VRT_AddVDP(ctx, &xyzzy_vdp_pedantic);
 	return (0);
 }
@@ -518,15 +529,10 @@ event_warm(VRT_CTX, const struct vmod_priv *priv)
 {
 	struct priv_vcl *priv_vcl;
 	char buf[32];
-	const char *p[2];
-	struct strands msg[1];
+	const char *vcl_name = VCL_Name(ctx->vcl);
 
 	// Using VSLs for coverage
-	msg->n = 2;
-	msg->p = p;
-	p[0] = VCL_Name(ctx->vcl);
-	p[1] = ": VCL_EVENT_WARM";
-	VSLs(SLT_Debug, 0, msg);
+	VSLs(SLT_Debug, 0, TOSTRANDS(2, vcl_name, ": VCL_EVENT_WARM"));
 
 	AN(ctx->msg);
 	if (cache_param->max_esi_depth == 42) {
@@ -622,8 +628,7 @@ event_discard(VRT_CTX, void *priv)
 
 	AZ(ctx->msg);
 
-	VRT_RemoveVFP(ctx, &xyzzy_rot13);
-	VRT_RemoveVDP(ctx, &xyzzy_vdp_rot13);
+	VRT_RemoveFilter(ctx, &xyzzy_vfp_rot13, &xyzzy_vdp_rot13);
 	VRT_RemoveVDP(ctx, &xyzzy_vdp_pedantic);
 
 	if (--loads)
@@ -665,17 +670,6 @@ xyzzy_vcl_discard_delay(VRT_CTX, struct vmod_priv *priv, VCL_DURATION delay)
 	priv_vcl->vcl_discard_delay = delay;
 }
 
-VCL_BOOL v_matchproto_(td_debug_match_acl)
-xyzzy_match_acl(VRT_CTX, VCL_ACL acl, VCL_IP ip)
-{
-
-	CHECK_OBJ_ORNULL(ctx, VRT_CTX_MAGIC);
-	AN(acl);
-	assert(VSA_Sane(ip));
-
-	return (VRT_acl_match(ctx, acl, ip));
-}
-
 VCL_VOID v_matchproto_(td_debug_test_probe)
 xyzzy_test_probe(VRT_CTX, VCL_PROBE probe, VCL_PROBE same)
 {
@@ -758,11 +752,10 @@ VCL_VOID
 xyzzy_concat__fini(struct xyzzy_debug_concat **concatp)
 {
 	struct xyzzy_debug_concat *concat;
-	void *p;
+
 
 	TAKE_OBJ_NOTNULL(concat, concatp, CONCAT_MAGIC);
-	p = TRUST_ME(concat->s);
-	free(p);
+	free(TRUST_ME(concat->s));
 	FREE_OBJ(concat);
 }
 
@@ -782,7 +775,7 @@ xyzzy_concatenate(VRT_CTX, VCL_STRANDS s)
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	r = VRT_StrandsWS(ctx->ws, NULL, s);
 	if (r != NULL && *r != '\0')
-		WS_Assert_Allocated(ctx->ws, r, strlen(r) + 1);
+		AN(WS_Allocated(ctx->ws, r, strlen(r) + 1));
 	return (r);
 }
 
@@ -792,34 +785,42 @@ xyzzy_collect(VRT_CTX, VCL_STRANDS s)
 	VCL_STRING r;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	r = VRT_CollectStrands(ctx, s);
+	r = VRT_STRANDS_string(ctx, s);
 	if (r != NULL && *r != '\0')
-		WS_Assert_Allocated(ctx->ws, r, strlen(r) + 1);
+		AN(WS_Allocated(ctx->ws, r, strlen(r) + 1));
 	return (r);
 }
 
 /* cf. VRT_SetHdr() */
 VCL_VOID
-xyzzy_sethdr(VRT_CTX, VCL_HEADER hs, VCL_STRANDS s)
+xyzzy_sethdr(VRT_CTX, VCL_HEADER hdr, VCL_STRANDS s)
 {
 	struct http *hp;
 	const char *b;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	AN(hs);
-	AN(hs->what);
-	hp = VRT_selecthttp(ctx, hs->where);
+	if (hdr == NULL) {
+		VRT_fail(ctx, "debug.sethdr(): header argument is NULL");
+		return;
+	}
+	hp = VRT_selecthttp(ctx, hdr->where);
+	if (hp == NULL) {
+		VRT_fail(ctx, "debug.sethdr(): header argument "
+		    "can not be used here");
+		return;
+	}
+	AN(hdr->what);
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
 	if (s->n == 0) {
-		http_Unset(hp, hs->what);
+		http_Unset(hp, hdr->what);
 	} else {
-		b = VRT_StrandsWS(hp->ws, hs->what + 1, s);
+		b = VRT_StrandsWS(hp->ws, hdr->what + 1, s);
 		if (b == NULL) {
-			VSLb(ctx->vsl, SLT_LostHeader, "%s", hs->what + 1);
+			VSLb(ctx->vsl, SLT_LostHeader, "%s", hdr->what + 1);
 		} else {
 			if (*b != '\0')
-				WS_Assert_Allocated(hp->ws, b, strlen(b) + 1);
-			http_Unset(hp, hs->what);
+				AN(WS_Allocated(hp->ws, b, strlen(b) + 1));
+			http_Unset(hp, hdr->what);
 			http_SetHeader(hp, b);
 		}
 	}
@@ -1063,15 +1064,17 @@ xyzzy_get_ip(VRT_CTX)
 
 //lint -save -e9075 external symbol '...' defined without a prior declaration
 
+extern const struct vmod_data Vmod_wrong0_Data;
 const struct vmod_data Vmod_wrong0_Data = {
-	.vrt_major =    0,
-	.vrt_minor =    0,
+	.vrt_major =	0,
+	.vrt_minor =	0,
 };
 
 //lint -save -e835 A zero has been given as left argument to operatorp'+'
+extern const struct vmod_data Vmod_wrong1_Data;
 const struct vmod_data Vmod_wrong1_Data = {
-	.vrt_major =    VRT_MAJOR_VERSION,
-	.vrt_minor =    VRT_MINOR_VERSION + 1,
+	.vrt_major =	VRT_MAJOR_VERSION,
+	.vrt_minor =	VRT_MINOR_VERSION + 1,
 };
 //lint -restore
 
@@ -1079,9 +1082,10 @@ static const struct foo {
 	int bar;
 } foo_struct[1];
 
+extern const struct vmod_data Vmod_wrong2_Data;
 const struct vmod_data Vmod_wrong2_Data = {
-	.vrt_major =    VRT_MAJOR_VERSION,
-	.vrt_minor =    VRT_MINOR_VERSION,
+	.vrt_major =	VRT_MAJOR_VERSION,
+	.vrt_minor =	VRT_MINOR_VERSION,
 	.name =		"wrongN",
 	.func =		foo_struct,
 	.func_len =	sizeof foo_struct,
@@ -1089,9 +1093,10 @@ const struct vmod_data Vmod_wrong2_Data = {
 	.proto =	"blablabla",
 };
 
+extern const struct vmod_data Vmod_wrong3_Data;
 const struct vmod_data Vmod_wrong3_Data = {
-	.vrt_major =    VRT_MAJOR_VERSION,
-	.vrt_minor =    VRT_MINOR_VERSION,
+	.vrt_major =	VRT_MAJOR_VERSION,
+	.vrt_minor =	VRT_MINOR_VERSION,
 	.name =		"wrongN",
 	.func =		foo_struct,
 	.func_len =	sizeof foo_struct,
@@ -1208,9 +1213,9 @@ fail_f(VRT_CTX, void *priv)
 }
 
 static const struct vmod_priv_methods xyzzy_fail_task_fini_methods[1] = {{
-		.magic = VMOD_PRIV_METHODS_MAGIC,
-		.type = "debug_fail_task_fini",
-		.fini = fail_f
+	.magic = VMOD_PRIV_METHODS_MAGIC,
+	.type = "debug_fail_task_fini",
+	.fini = fail_f
 }};
 
 VCL_VOID v_matchproto_(td_xyzzy_debug_fail_task_fini)
@@ -1365,9 +1370,7 @@ xyzzy_caller__fini(struct VPFX(debug_caller) **callerp)
 
 	if (callerp == NULL || *callerp == NULL)
 		return;
-	CHECK_OBJ(*callerp, DEBUG_CALLER_MAGIC);
-	caller = *callerp;
-	*callerp = NULL;
+	TAKE_OBJ_NOTNULL(caller, callerp, DEBUG_CALLER_MAGIC);
 	FREE_OBJ(caller);
 }
 
