@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2009-2016 UPLEX - Nils Goroll Systemoptimierung
+ * Copyright 2009-2016 UPLEX - Nils Goroll Systemoptimierung
  * All rights reserved.
  *
  * Authors: Nils Goroll <nils.goroll@uplex.de>
@@ -100,9 +100,9 @@ shard_change_fini(VRT_CTX, void * priv)
 }
 
 static const struct vmod_priv_methods shard_change_priv_methods[1] = {{
-		.magic = VMOD_PRIV_METHODS_MAGIC,
-		.type = "vmod_directors_shard_cfg",
-		.fini = shard_change_fini
+	.magic = VMOD_PRIV_METHODS_MAGIC,
+	.type = "vmod_directors_shard_cfg",
+	.fini = shard_change_fini
 }};
 
 static struct shard_change *
@@ -195,7 +195,8 @@ shard_change_task_backend(VRT_CTX, struct sharddir *shardd,
 		return (NULL);
 	}
 
-	b->backend = be;
+	b->backend = NULL;
+	VRT_Assign_Backend(&b->backend, be);
 	b->ident = ident != NULL && *ident != '\0' ? ident : NULL;
 	b->rampup = rampup;
 
@@ -270,8 +271,6 @@ shardcfg_hashcircle(struct sharddir *shardd)
 	const char *ident;
 	const int len = 12; // log10(UINT32_MAX) + 2;
 	char s[len];
-	struct strands ss[1];
-	const char *ssp[2];
 
 	CHECK_OBJ_NOTNULL(shardd, SHARDDIR_MAGIC);
 	AZ(shardd->hashcircle);
@@ -284,10 +283,7 @@ shardcfg_hashcircle(struct sharddir *shardd)
 	rmax = (UINT32_MAX - 1) / shardd->n_backend;
 	for (b = backends; b < backends + shardd->n_backend; b++) {
 		CHECK_OBJ_NOTNULL(b->backend, DIRECTOR_MAGIC);
-		r = b->replicas;
-		if (r > rmax)
-			r = rmax;
-		n_points += r;
+		n_points += vmin_t(uint32_t, b->replicas, rmax);
 	}
 
 	assert(n_points < UINT32_MAX);
@@ -303,18 +299,13 @@ shardcfg_hashcircle(struct sharddir *shardd)
 		AN(ident);
 		assert(ident[0] != '\0');
 
-		r = b->replicas;
-		if (r > rmax)
-			r = rmax;
+		r = vmin_t(uint32_t, b->replicas, rmax);
 
 		for (j = 0; j < r; j++) {
 			assert(snprintf(s, len, "%d", j) < len);
-			ss->n = 2;
-			ssp[0] = ident;
-			ssp[1] = s;
-			ss->p = ssp;
 			assert (i < n_points);
-			shardd->hashcircle[i].point = VRT_HashStrands32(ss);
+			shardd->hashcircle[i].point =
+			    VRT_HashStrands32(TOSTRANDS(2, ident, s));
 			shardd->hashcircle[i].host = h;
 			i++;
 		}
@@ -343,6 +334,7 @@ shardcfg_backend_free(struct shard_backend *f)
 {
 	if (f->freeptr)
 		free (f->freeptr);
+	VRT_Assign_Backend(&f->backend, NULL);
 	memset(f, 0, sizeof(*f));
 }
 
@@ -421,8 +413,7 @@ shardcfg_backend_expand(const struct backend_reconfig *re)
 
 	CHECK_OBJ_NOTNULL(re->shardd, SHARDDIR_MAGIC);
 
-	if (min < 16)
-		min = 16;
+	min = vmax_t(unsigned, min, 16);
 
 	if (re->shardd->l_backend < min)
 		re->shardd->l_backend = min;
@@ -477,8 +468,7 @@ shardcfg_backend_clear(struct sharddir *shardd)
 
 
 static void
-shardcfg_backend_del(struct backend_reconfig *re,
-    const struct shard_backend *spec)
+shardcfg_backend_del(struct backend_reconfig *re, struct shard_backend *spec)
 {
 	unsigned i, max = re->shardd->n_backend + re->hole_n;
 	struct shard_backend * const bb = re->shardd->backend;
@@ -493,10 +483,10 @@ shardcfg_backend_del(struct backend_reconfig *re,
 		re->shardd->n_backend--;
 		if (i < re->shardd->n_backend + re->hole_n) {
 			(re->hole_n)++;
-			if (i < re->hole_i)
-				re->hole_i = i;
+			re->hole_i = vmin(re->hole_i, i);
 		}
 	}
+	VRT_Assign_Backend(&spec->backend, NULL);
 }
 
 static void
